@@ -13,11 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING
+
 from rucio.api import permission
 from rucio.common import exception
+from rucio.common.config import convert_to_any_type
 from rucio.core import config
 from rucio.db.sqla.session import read_session, transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 """
 ConfigParser compatible interface.
 
@@ -27,7 +33,7 @@ ConfigParser compatible interface.
 
 
 @read_session
-def sections(issuer=None, vo='def', session=None):
+def sections(issuer=None, vo='def', *, session: "Session"):
     """
     Return a list of the sections available.
 
@@ -44,7 +50,7 @@ def sections(issuer=None, vo='def', session=None):
 
 
 @transactional_session
-def add_section(section, issuer=None, vo='def', session=None):
+def add_section(section, issuer=None, vo='def', *, session: "Session"):
     """
     Add a section to the configuration.
 
@@ -61,7 +67,7 @@ def add_section(section, issuer=None, vo='def', session=None):
 
 
 @read_session
-def has_section(section, issuer=None, vo='def', session=None):
+def has_section(section, issuer=None, vo='def', *, session: "Session"):
     """
     Indicates whether the named section is present in the configuration.
 
@@ -79,7 +85,7 @@ def has_section(section, issuer=None, vo='def', session=None):
 
 
 @read_session
-def options(section, issuer=None, vo='def', session=None):
+def options(section, issuer=None, vo='def', *, session: "Session"):
     """
     Returns a list of options available in the specified section.
 
@@ -97,7 +103,7 @@ def options(section, issuer=None, vo='def', session=None):
 
 
 @read_session
-def has_option(section, option, issuer=None, vo='def', session=None):
+def has_option(section, option, issuer=None, vo='def', *, session: "Session"):
     """
     Check if the given section exists and contains the given option.
 
@@ -116,7 +122,7 @@ def has_option(section, option, issuer=None, vo='def', session=None):
 
 
 @read_session
-def get(section, option, issuer=None, vo='def', session=None):
+def get(section, option, issuer=None, vo='def', *, session: "Session"):
     """
     Get an option value for the named section. Value can be auto-coerced to int, float, and bool; string otherwise.
 
@@ -134,11 +140,11 @@ def get(section, option, issuer=None, vo='def', session=None):
     kwargs = {'issuer': issuer, 'section': section, 'option': option}
     if not permission.has_permission(issuer=issuer, vo=vo, action='config_get', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%s cannot retrieve option %s from section %s' % (issuer, option, section))
-    return config.get(section, option, session=session)
+    return config.get(section, option, session=session, convert_type_fnc=convert_to_any_type)
 
 
 @read_session
-def items(section, issuer=None, vo='def', session=None):
+def items(section, issuer=None, vo='def', *, session: "Session"):
     """
     Return a list of (option, value) pairs for each option in the given section. Values are auto-coerced as in get().
 
@@ -153,11 +159,11 @@ def items(section, issuer=None, vo='def', session=None):
     kwargs = {'issuer': issuer, 'section': section}
     if not permission.has_permission(issuer=issuer, vo=vo, action='config_items', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%s cannot retrieve options and values from section %s' % (issuer, section))
-    return config.items(section, session=session)
+    return config.items(section, session=session, convert_type_fnc=convert_to_any_type)
 
 
 @transactional_session
-def set(section, option, value, issuer=None, vo='def', session=None):
+def set(section, option, value, issuer=None, vo='def', *, session: "Session"):
     """
     Set the given option to the specified value.
 
@@ -176,7 +182,7 @@ def set(section, option, value, issuer=None, vo='def', session=None):
 
 
 @transactional_session
-def remove_section(section, issuer=None, vo='def', session=None):
+def remove_section(section, issuer=None, vo='def', *, session: "Session"):
     """
     Remove the specified option from the specified section.
 
@@ -194,7 +200,7 @@ def remove_section(section, issuer=None, vo='def', session=None):
 
 
 @transactional_session
-def remove_option(section, option, issuer=None, vo='def', session=None):
+def remove_option(section, option, issuer=None, vo='def', *, session: "Session"):
     """
     Remove the specified section from the configuration.
 
