@@ -28,12 +28,12 @@ from rucio.db.sqla.constants import DIDType
 from rucio.db.sqla.session import read_session, stream_session, transactional_session
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Optional
+    from typing import Any, Optional
     from sqlalchemy.orm import Session
 
 
 @stream_session
-def list_dids(scope, filters, did_type='collection', ignore_case=False, limit=None, offset=None, long=False, recursive=False, vo='def', session=None):
+def list_dids(scope, filters, did_type='collection', ignore_case=False, limit=None, offset=None, long=False, recursive=False, vo='def', *, session: "Session"):
     """
     List dids in a scope.
 
@@ -64,40 +64,8 @@ def list_dids(scope, filters, did_type='collection', ignore_case=False, limit=No
         yield api_update_return_dict(d, session=session)
 
 
-@stream_session
-def list_dids_extended(scope, filters, did_type='collection', ignore_case=False, limit=None, offset=None, long=False, recursive=False, vo='def', session=None):
-    """
-    List dids in a scope.
-
-    :param scope: The scope name.
-    :param pattern: The wildcard pattern.
-    :param did_type:  The type of the did: all(container, dataset, file), collection(dataset or container), dataset, container
-    :param ignore_case: Ignore case distinctions.
-    :param limit: The maximum number of DIDs returned.
-    :param offset: Offset number.
-    :param long: Long format option to display more information for each DID.
-    :param recursive: Recursively list DIDs content.
-    :param session: The database session in use.
-    """
-    validate_schema(name='did_filters', obj=filters, vo=vo)
-    scope = InternalScope(scope, vo=vo)
-
-    # replace account and scope in filters with internal representation
-    for or_group in filters:
-        if 'account' in or_group:
-            or_group['account'] = InternalAccount(or_group['account'], vo=vo)
-        if 'scope' in or_group:
-            or_group['account'] = InternalScope(or_group['scope'], vo=vo)
-
-    result = did.list_dids_extended(scope=scope, filters=filters, did_type=did_type, ignore_case=ignore_case,
-                                    limit=limit, offset=offset, long=long, recursive=recursive, session=session)
-
-    for d in result:
-        yield api_update_return_dict(d, session=session)
-
-
 @transactional_session
-def add_did(scope, name, did_type, issuer, account=None, statuses={}, meta={}, rules=[], lifetime=None, dids=[], rse=None, vo='def', session=None):
+def add_did(scope, name, did_type, issuer, account=None, statuses={}, meta={}, rules=[], lifetime=None, dids=[], rse=None, vo='def', *, session: "Session"):
     """
     Add data did.
 
@@ -157,7 +125,7 @@ def add_did(scope, name, did_type, issuer, account=None, statuses={}, meta={}, r
 
 
 @transactional_session
-def add_dids(dids, issuer, vo='def', session=None):
+def add_dids(dids, issuer, vo='def', *, session: "Session"):
     """
     Bulk Add did.
 
@@ -189,7 +157,7 @@ def add_dids(dids, issuer, vo='def', session=None):
 
 
 @transactional_session
-def attach_dids(scope, name, attachment, issuer, vo='def', session=None):
+def attach_dids(scope, name, attachment, issuer, vo='def', *, session: "Session"):
     """
     Append content to data did.
 
@@ -230,7 +198,7 @@ def attach_dids(scope, name, attachment, issuer, vo='def', session=None):
 
 
 @transactional_session
-def attach_dids_to_dids(attachments, issuer, ignore_duplicate=False, vo='def', session=None):
+def attach_dids_to_dids(attachments, issuer, ignore_duplicate=False, vo='def', *, session: "Session"):
     """
     Append content to dids.
 
@@ -265,7 +233,7 @@ def attach_dids_to_dids(attachments, issuer, ignore_duplicate=False, vo='def', s
 
 
 @transactional_session
-def detach_dids(scope, name, dids, issuer, vo='def', session=None):
+def detach_dids(scope, name, dids, issuer, vo='def', *, session: "Session"):
     """
     Detach data identifier
 
@@ -288,7 +256,7 @@ def detach_dids(scope, name, dids, issuer, vo='def', session=None):
 
 
 @stream_session
-def list_new_dids(did_type=None, thread=None, total_threads=None, chunk_size=1000, vo='def', session=None):
+def list_new_dids(did_type=None, thread=None, total_threads=None, chunk_size=1000, vo='def', *, session: "Session"):
     """
     List recent identifiers.
 
@@ -306,7 +274,7 @@ def list_new_dids(did_type=None, thread=None, total_threads=None, chunk_size=100
 
 
 @transactional_session
-def set_new_dids(dids, new_flag=True, vo='def', session=None):
+def set_new_dids(dids, new_flag=True, vo='def', *, session: "Session"):
     """
     Set/reset the flag new
 
@@ -323,7 +291,7 @@ def set_new_dids(dids, new_flag=True, vo='def', session=None):
 
 
 @stream_session
-def list_content(scope, name, vo='def', session=None):
+def list_content(scope, name, vo='def', *, session: "Session"):
     """
     List data identifier contents.
 
@@ -341,7 +309,7 @@ def list_content(scope, name, vo='def', session=None):
 
 
 @stream_session
-def list_content_history(scope, name, vo='def', session=None):
+def list_content_history(scope, name, vo='def', *, session: "Session"):
     """
     List data identifier contents history.
 
@@ -360,7 +328,7 @@ def list_content_history(scope, name, vo='def', session=None):
 
 
 @stream_session
-def list_files(scope, name, long, vo='def', session=None):
+def list_files(scope, name, long, vo='def', *, session: "Session"):
     """
     List data identifier file contents.
 
@@ -380,7 +348,7 @@ def list_files(scope, name, long, vo='def', session=None):
 
 
 @stream_session
-def scope_list(scope, name=None, recursive=False, vo='def', session=None):
+def scope_list(scope, name=None, recursive=False, vo='def', *, session: "Session"):
     """
     List data identifiers in a scope.
 
@@ -404,7 +372,7 @@ def scope_list(scope, name=None, recursive=False, vo='def', session=None):
 
 
 @read_session
-def get_did(scope: str, name: str, dynamic_depth: "Optional[DIDType]" = None, vo: str = 'def', session: "Optional[Session]" = None) -> "Dict[str, Any]":
+def get_did(scope: str, name: str, dynamic_depth: "Optional[DIDType]" = None, vo: str = 'def', *, session: "Session") -> "dict[str, Any]":
     """
     Retrieve a single data did.
 
@@ -425,7 +393,7 @@ def get_did(scope: str, name: str, dynamic_depth: "Optional[DIDType]" = None, vo
 
 
 @transactional_session
-def set_metadata(scope, name, key, value, issuer, recursive=False, vo='def', session=None):
+def set_metadata(scope, name, key, value, issuer, recursive=False, vo='def', *, session: "Session"):
     """
     Add metadata to data did.
 
@@ -451,7 +419,7 @@ def set_metadata(scope, name, key, value, issuer, recursive=False, vo='def', ses
 
 
 @transactional_session
-def set_metadata_bulk(scope, name, meta, issuer, recursive=False, vo='def', session=None):
+def set_metadata_bulk(scope, name, meta, issuer, recursive=False, vo='def', *, session: "Session"):
     """
     Add metadata to data did.
 
@@ -477,7 +445,7 @@ def set_metadata_bulk(scope, name, meta, issuer, recursive=False, vo='def', sess
 
 
 @transactional_session
-def set_dids_metadata_bulk(dids, issuer, recursive=False, vo='def', session=None):
+def set_dids_metadata_bulk(dids, issuer, recursive=False, vo='def', *, session: "Session"):
     """
     Add metadata to a list of data identifiers.
 
@@ -502,7 +470,7 @@ def set_dids_metadata_bulk(dids, issuer, recursive=False, vo='def', session=None
 
 
 @read_session
-def get_metadata(scope, name, plugin='DID_COLUMN', vo='def', session=None):
+def get_metadata(scope, name, plugin='DID_COLUMN', vo='def', *, session: "Session"):
     """
     Get data identifier metadata
 
@@ -519,7 +487,7 @@ def get_metadata(scope, name, plugin='DID_COLUMN', vo='def', session=None):
 
 
 @stream_session
-def get_metadata_bulk(dids, inherit=False, vo='def', session=None):
+def get_metadata_bulk(dids, inherit=False, vo='def', *, session: "Session"):
     """
     Get metadata for a list of dids
     :param dids:               A list of dids.
@@ -537,7 +505,7 @@ def get_metadata_bulk(dids, inherit=False, vo='def', session=None):
 
 
 @transactional_session
-def delete_metadata(scope, name, key, vo='def', session=None):
+def delete_metadata(scope, name, key, vo='def', *, session: "Session"):
     """
     Delete a key from the metadata column
 
@@ -553,7 +521,7 @@ def delete_metadata(scope, name, key, vo='def', session=None):
 
 
 @transactional_session
-def set_status(scope, name, issuer, vo='def', session=None, **kwargs):
+def set_status(scope, name, issuer, vo='def', *, session: "Session", **kwargs):
     """
     Set data identifier status
 
@@ -574,7 +542,7 @@ def set_status(scope, name, issuer, vo='def', session=None, **kwargs):
 
 
 @stream_session
-def get_dataset_by_guid(guid, vo='def', session=None):
+def get_dataset_by_guid(guid, vo='def', *, session: "Session"):
     """
     Get the parent datasets for a given GUID.
     :param guid: The GUID.
@@ -592,7 +560,7 @@ def get_dataset_by_guid(guid, vo='def', session=None):
 
 
 @stream_session
-def list_parent_dids(scope, name, vo='def', session=None):
+def list_parent_dids(scope, name, vo='def', *, session: "Session"):
     """
     List parent datasets and containers of a did.
 
@@ -611,7 +579,7 @@ def list_parent_dids(scope, name, vo='def', session=None):
 
 
 @transactional_session
-def create_did_sample(input_scope, input_name, output_scope, output_name, issuer, nbfiles, vo='def', session=None):
+def create_did_sample(input_scope, input_name, output_scope, output_name, issuer, nbfiles, vo='def', *, session: "Session"):
     """
     Create a sample from an input collection.
 
@@ -639,7 +607,7 @@ def create_did_sample(input_scope, input_name, output_scope, output_name, issuer
 
 
 @transactional_session
-def resurrect(dids, issuer, vo='def', session=None):
+def resurrect(dids, issuer, vo='def', *, session: "Session"):
     """
     Resurrect DIDs.
 
@@ -660,7 +628,7 @@ def resurrect(dids, issuer, vo='def', session=None):
 
 
 @stream_session
-def list_archive_content(scope, name, vo='def', session=None):
+def list_archive_content(scope, name, vo='def', *, session: "Session"):
     """
     List archive contents.
 
@@ -678,7 +646,7 @@ def list_archive_content(scope, name, vo='def', session=None):
 
 
 @transactional_session
-def add_did_to_followed(scope, name, account, session=None, vo='def'):
+def add_did_to_followed(scope, name, account, *, session: "Session", vo='def'):
     """
     Mark a did as followed by the given account
 
@@ -693,7 +661,7 @@ def add_did_to_followed(scope, name, account, session=None, vo='def'):
 
 
 @transactional_session
-def add_dids_to_followed(dids, account, session=None, vo='def'):
+def add_dids_to_followed(dids, account, *, session: "Session", vo='def'):
     """
     Bulk mark datasets as followed
 
@@ -706,7 +674,7 @@ def add_dids_to_followed(dids, account, session=None, vo='def'):
 
 
 @stream_session
-def get_users_following_did(name, scope, session=None, vo='def'):
+def get_users_following_did(name, scope, *, session: "Session", vo='def'):
     """
     Return list of users following a did
 
@@ -722,7 +690,7 @@ def get_users_following_did(name, scope, session=None, vo='def'):
 
 
 @transactional_session
-def remove_did_from_followed(scope, name, account, issuer, session=None, vo='def'):
+def remove_did_from_followed(scope, name, account, issuer, *, session: "Session", vo='def'):
     """
     Mark a did as not followed
 
@@ -742,7 +710,7 @@ def remove_did_from_followed(scope, name, account, issuer, session=None, vo='def
 
 
 @transactional_session
-def remove_dids_from_followed(dids, account, issuer, session=None, vo='def'):
+def remove_dids_from_followed(dids, account, issuer, *, session: "Session", vo='def'):
     """
     Bulk mark datasets as not followed
 
