@@ -13,23 +13,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import uuid
 import datetime
+from typing import TYPE_CHECKING
 
 from rucio.api import permission
-from rucio.db.sqla.constants import BadFilesStatus
-from rucio.db.sqla.session import read_session, stream_session, transactional_session
-from rucio.core import replica
-from rucio.core.rse import get_rse_id, get_rse_name
 from rucio.common import exception
+from rucio.common.constants import SuspiciousAvailability
 from rucio.common.schema import validate_schema
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import api_update_return_dict, invert_dict
-from rucio.common.constants import SuspiciousAvailability
+from rucio.core import replica
+from rucio.core.rse import get_rse_id, get_rse_name
+from rucio.db.sqla.constants import BadFilesStatus
+from rucio.db.sqla.session import read_session, stream_session, transactional_session
+
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
 
 
 @read_session
-def get_bad_replicas_summary(rse_expression=None, from_date=None, to_date=None, vo='def', session=None):
+def get_bad_replicas_summary(rse_expression=None, from_date=None, to_date=None, vo='def', *, session: "Session"):
     """
     List the bad file replicas summary. Method used by the rucio-ui.
     :param rse_expression: The RSE expression.
@@ -43,7 +46,7 @@ def get_bad_replicas_summary(rse_expression=None, from_date=None, to_date=None,
 
 
 @read_session
-def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=None, older_than=None, limit=None, list_pfns=False, vo='def', session=None):
+def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=None, older_than=None, limit=None, list_pfns=False, vo='def', *, session: "Session"):
     """
     List the bad file replicas history states. Method used by the rucio-ui.
     :param state: The state of the file (SUSPICIOUS or BAD).
@@ -64,7 +67,7 @@ def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=No
 
 
 @transactional_session
-def declare_bad_file_replicas(replicas, reason, issuer, vo='def', force=False, session=None):
+def declare_bad_file_replicas(replicas, reason, issuer, vo='def', force=False, *, session: "Session"):
     """
     Declare a list of bad replicas.
 
@@ -81,21 +84,23 @@ def declare_bad_file_replicas(replicas, reason, issuer, vo='def', force=False, s
     if not replicas:
         return {}
 
-    kwargs = {}
-    rse_map = {}                # RSE name -> RSE id
-    if not permission.has_permission(issuer=issuer, vo=vo, action='declare_bad_file_replicas', kwargs=kwargs, session=session):
-        raise exception.AccessDenied('Account %s can not declare bad replicas' % (issuer))
-
-    issuer = InternalAccount(issuer, vo=vo)
+    as_pfns = isinstance(replicas[0], str)
 
     # make sure all elements are either strings or dicts, without mixing
-    type_ = type(replicas[0])
-    if any(not isinstance(r, type_) for r in replicas):
+    if any(isinstance(r, str) != as_pfns for r in replicas):
         raise exception.InvalidType('The replicas must be specified either as a list of PFNs (strings) or list of dicts')
 
+    rse_map = {}                # RSE name -> RSE id
+
     replicas_lst = replicas
-    if type_ is dict:
-        replicas_lst = []           # need to create new list to convert replica["scope"] from strings to InternalScope objects
+    rse_ids_to_check = set()        # to check for permission to declare bad replicas
+    if as_pfns:
+        scheme, rses_for_replicas, unknowns = replica.get_pfn_to_rse(replicas, vo=vo, session=session)
+        if unknowns:
+            raise exception.ReplicaNotFound("Not all replicas found")
+        rse_ids_to_check = set(rses_for_replicas.keys())
+    else:
+        replicas_lst = []
         for r in replicas:
             if "name" not in r or "scope" not in r or ("rse" not in r and "rse_id" not in r):
                 raise exception.InvalidType('The replica dictionary must include scope and either rse (name) or rse_id')
@@ -105,14 +110,24 @@ def declare_bad_file_replicas(replicas, reason, issuer, vo='def', force=False, s
                 rse = r["rse"]
                 rse_map[rse] = rse_id = get_rse_id(rse=rse, vo=vo, session=session)
             replicas_lst.append({
-                "scope": scope,
                 "rse_id": rse_id,
+                "scope": scope,
                 "name": r["name"]
             })
+            rse_ids_to_check.add(rse_id)
 
     rse_id_to_name = invert_dict(rse_map)   # RSE id -> RSE name
 
-    undeclared = replica.declare_bad_file_replicas(replicas_lst, reason=reason, issuer=issuer, status=BadFilesStatus.BAD,
+    for rse_id in rse_ids_to_check:
+        if not permission.has_permission(issuer=issuer, vo=vo, action='declare_bad_file_replicas',
+                                         kwargs={"rse_id": rse_id},
+                                         session=session):
+            raise exception.AccessDenied('Account %s can not declare bad replicas in RSE %s' %
+                                         (issuer, rse_id_to_name.get(rse_id, rse_id)))
+
+    undeclared = replica.declare_bad_file_replicas(replicas_lst, reason=reason,
+                                                   issuer=InternalAccount(issuer, vo=vo),
+                                                   status=BadFilesStatus.BAD,
                                                    force=force, session=session)
     out = {}
     for rse_id, ulist in undeclared.items():
@@ -124,7 +139,6 @@ def declare_bad_file_replicas(replicas, reason, issuer, vo='def', force=False, s
                 rse_name = rse_id_to_name[rse_id]
             else:
                 try:
-                    uuid.UUID(rse_id)
                     rse_name = get_rse_name(rse_id=rse_id, session=session)
                 except (ValueError, exception.RSENotFound):
                     rse_name = str(rse_id)
@@ -134,7 +148,7 @@ def declare_bad_file_replicas(replicas, reason, issuer, vo='def', force=False, s
 
 
 @transactional_session
-def declare_suspicious_file_replicas(pfns, reason, issuer, vo='def', session=None):
+def declare_suspicious_file_replicas(pfns, reason, issuer, vo='def', *, session: "Session"):
     """
     Declare a list of bad replicas.
 
@@ -163,7 +177,7 @@ def declare_suspicious_file_replicas(pfns, reason, issuer, vo='def', session=Non
 
 
 @stream_session
-def get_did_from_pfns(pfns, rse, vo='def', session=None):
+def get_did_from_pfns(pfns, rse, vo='def', *, session: "Session"):
     """
     Get the DIDs associated to a PFN on one given RSE
 
@@ -188,7 +202,7 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
                   client_location=None, domain=None, signature_lifetime=None,
                   resolve_archives=True, resolve_parents=False,
                   nrandom=None, updated_after=None,
-                  issuer=None, vo='def', session=None):
+                  issuer=None, vo='def', *, session: "Session"):
     """
     List file replicas for a list of data identifiers.
 
@@ -242,7 +256,7 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
 
 
 @transactional_session
-def add_replicas(rse, files, issuer, ignore_availability=False, vo='def', session=None):
+def add_replicas(rse, files, issuer, ignore_availability=False, vo='def', *, session: "Session"):
     """
     Bulk add file replicas.
 
@@ -277,7 +291,7 @@ def add_replicas(rse, files, issuer, ignore_availability=False, vo='def', sessio
 
 
 @transactional_session
-def delete_replicas(rse, files, issuer, ignore_availability=False, vo='def', session=None):
+def delete_replicas(rse, files, issuer, ignore_availability=False, vo='def', *, session: "Session"):
     """
     Bulk delete file replicas.
 
@@ -307,7 +321,7 @@ def delete_replicas(rse, files, issuer, ignore_availability=False, vo='def', ses
 
 
 @transactional_session
-def update_replicas_states(rse, files, issuer, vo='def', session=None):
+def update_replicas_states(rse, files, issuer, vo='def', *, session: "Session"):
     """
     Update File replica information and state.
 
@@ -336,7 +350,7 @@ def update_replicas_states(rse, files, issuer, vo='def', session=None):
 
 
 @stream_session
-def list_dataset_replicas(scope, name, deep=False, vo='def', session=None):
+def list_dataset_replicas(scope, name, deep=False, vo='def', *, session: "Session"):
     """
     :param scope: The scope of the dataset.
     :param name: The name of the dataset.
@@ -357,7 +371,7 @@ def list_dataset_replicas(scope, name, deep=False, vo='def', session=None):
 
 
 @stream_session
-def list_dataset_replicas_bulk(dids, vo='def', session=None):
+def list_dataset_replicas_bulk(dids, vo='def', *, session: "Session"):
     """
     :param dids: The list of did dictionaries with scope and name.
     :param vo: The VO to act on.
@@ -386,7 +400,7 @@ def list_dataset_replicas_bulk(dids, vo='def', session=None):
 
 
 @stream_session
-def list_dataset_replicas_vp(scope, name, deep=False, vo='def', session=None):
+def list_dataset_replicas_vp(scope, name, deep=False, vo='def', *, session: "Session"):
     """
     :param scope: The scope of the dataset.
     :param name: The name of the dataset.
@@ -405,7 +419,7 @@ def list_dataset_replicas_vp(scope, name, deep=False, vo='def', session=None):
 
 
 @stream_session
-def list_datasets_per_rse(rse, filters={}, limit=None, vo='def', session=None):
+def list_datasets_per_rse(rse, filters={}, limit=None, vo='def', *, session: "Session"):
     """
     :param scope: The scope of the dataset.
     :param name: The name of the dataset.
@@ -425,7 +439,7 @@ def list_datasets_per_rse(rse, filters={}, limit=None, vo='def', session=None):
 
 
 @transactional_session
-def add_bad_pfns(pfns, issuer, state, reason=None, expires_at=None, vo='def', session=None):
+def add_bad_pfns(pfns, issuer, state, reason=None, expires_at=None, vo='def', *, session: "Session"):
     """
     Add bad PFNs.
 
@@ -452,7 +466,7 @@ def add_bad_pfns(pfns, issuer, state, reason=None, expires_at=None, vo='def', se
 
 
 @transactional_session
-def add_bad_dids(dids, rse, issuer, state, reason=None, expires_at=None, vo='def', session=None):
+def add_bad_dids(dids, rse, issuer, state, reason=None, expires_at=None, vo='def', *, session: "Session"):
     """
     Add bad replica entries for DIDs.
 
@@ -478,7 +492,7 @@ def add_bad_dids(dids, rse, issuer, state, reason=None, expires_at=None, vo='def
 
 
 @read_session
-def get_suspicious_files(rse_expression, younger_than=None, nattempts=None, vo='def', session=None):
+def get_suspicious_files(rse_expression, younger_than=None, nattempts=None, vo='def', *, session: "Session"):
     """
     List the list of suspicious files on a list of RSEs
     :param rse_expression: The RSE expression where the suspicious files are located
@@ -493,7 +507,7 @@ def get_suspicious_files(rse_expression, younger_than=None, nattempts=None, vo='
 
 
 @transactional_session
-def set_tombstone(rse, scope, name, issuer, vo='def', session=None):
+def set_tombstone(rse, scope, name, issuer, vo='def', *, session: "Session"):
     """
     Sets a tombstone on one replica.
 
