@@ -24,6 +24,7 @@ import logging
 import operator
 import os
 import re
+
 from tabulate import tabulate
 
 from rucio.common.dumper import DUMPS_CACHE_DIR
@@ -70,7 +71,7 @@ class DataModel(object):
         self.date = None
 
     @classmethod
-    def get_fieldnames(cls):
+    def get_fieldnames(cls) -> list[str]:
         """
         Get the field names
         """
