@@ -1059,3 +1059,34 @@ class MetadataSchemaMismatchError(RucioException):
         super(MetadataSchemaMismatchError, self).__init__(*args, **kwargs)
         self._message = 'The external table does not match the expected table schema.'
         self.error_code = 102
+
+
+class PolicyPackageVersionError(RucioException):
+    """
+    Policy package is not compatible with this version of Rucio.
+    """
+    def __init__(self, package, *args, **kwargs):
+        super(PolicyPackageVersionError, self).__init__(*args, **kwargs)
+        self._message = 'Policy package %s is not compatible with this Rucio version' % package
+        self.error_code = 103
+
+
+class InvalidSourceReplicaExpression(RucioException):
+    """
+    Source Replica Expression Considered Invalid
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(InvalidSourceReplicaExpression, self).__init__(*args, **kwargs)
+        self._message = 'Provided Source Replica expression is considered invalid.'
+        self.error_code = 104
+
+
+class DeprecationError(RucioException):
+    """
+    Function has been deprecated.
+    """
+    def __init__(self, *args, **kwargs):
+        super(DeprecationError, self).__init__(*args, **kwargs)
+        self._message = 'Command or function has been deprecated.'
+        self.error_code = 105
