@@ -1,4 +1,5 @@
-# Copyright 2019 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,22 +12,16 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2019
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2019
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 from jsonschema import validate, ValidationError
 
 from rucio.common.exception import InvalidObject
 
+ACCOUNT_LENGTH = 25
 
 ACCOUNT = {"description": "Account name",
            "type": "string",
-           "pattern": "^[a-z0-9-_]{1,30}$"}
+           "pattern": "^[a-z0-9-_]{1,%s}$" % ACCOUNT_LENGTH}
 
 ACCOUNTS = {"description": "Array of accounts",
             "type": "array",
@@ -41,12 +36,12 @@ ACCOUNT_TYPE = {"description": "Account type",
 
 ACTIVITY = {"description": "Activity name",
             "type": "string",
-            "enum": ["Data Brokering", "Data Consolidation", "Data rebalancing",
-                     "Debug", "Express", "Functional Test", "Group Subscriptions",
-                     "Production Input", "Production Output",
+            "enum": ["Data Consolidation", "Data Rebalancing",
+                     "Functional Test", "Functional Test WebDAV", "Recovery",
+                     "Production Input", "Production Output", "Production Merge",
                      "Analysis Input", "Analysis Output", "Staging",
-                     "T0 Export", "T0 Tape", "Upload/Download (Job)",
-                     "Upload/Download (User)", "User Subscriptions"]}
+                     "Raw Export", "Upload/Download (Job)", "Upload/Download (User)",
+                     "User Merge", "User Transfers", "Calibration Transfers"]}
 
 SCOPE_LENGTH = 25
 
@@ -62,7 +57,7 @@ NAME_LENGTH = 250
 
 NAME = {"description": "Data Identifier name",
         "type": "string",
-        "pattern": r"^[\/A-Za-z0-9][A-Za-z0-9\\.\\-\\_\/]{1,%s}$" % NAME_LENGTH}
+        "pattern": r"^/[A-Za-z0-9\.\-\_\+\/]{1,%s}$" % NAME_LENGTH}
 
 R_NAME = {"description": "Data Identifier name",
           "type": "string",
@@ -77,6 +72,9 @@ ASK_APPROVAL = {"description": "Rule approval request",
 ASYNCHRONOUS = {"description": "Asynchronous rule creation",
                 "type": ["boolean", "null"]}
 
+DELAY_INJECTION = {"description": "Time (in seconds) to wait before starting applying the rule. Implies asynchronous rule creation.",
+                   "type": ["integer", "null"]}
+
 PURGE_REPLICAS = {"description": "Rule purge replica status",
                   "type": "boolean"}
 
@@ -93,7 +91,7 @@ RSE_ATTRIBUTE = {"description": "RSE attribute",
 
 DEFAULT_RSE_ATTRIBUTE = {"description": "Default RSE attribute",
                          "type": "string",
-                         "pattern": r'([A-Z0-9]+([_-][A-Z0-9]+)*)'}
+                         "pattern": r'([A-Za-z0-9]+([_-][A-Za-z0-9]+)*)'}
 
 REPLICA_STATE = {"description": "Replica state",
                  "type": "string",
@@ -185,6 +183,7 @@ RULE = {"description": "Replication rule",
                        "comment": COMMENT,
                        "ask_approval": ASK_APPROVAL,
                        "asynchronous": ASYNCHRONOUS,
+                       "delay_injection": DELAY_INJECTION,
                        "priority": PRIORITY,
                        'split_container': SPLIT_CONTAINER,
                        'meta': METADATA},
@@ -232,10 +231,8 @@ DID = {"description": "Data Identifier(DID)",
        "required": ["scope", "name"],
        "additionalProperties": False}
 
-DID_FILTERS = {"description": "Filters dictionary to list DIDs",
-               "type": "object",
-               "properties": {"created_before": DATE,
-                              "created_afted": DATE},
+DID_FILTERS = {"description": "Array to filter DIDs by metadata",
+               "type": "array",
                "additionalProperties": True}
 
 R_DID = {"description": "Data Identifier(DID)",
@@ -271,7 +268,7 @@ ATTACHMENT = {"description": "Attachement",
                              "name": NAME,
                              "rse": {"description": "RSE name",
                                      "type": ["string", "null"],
-                                     "pattern": "^([A-Z0-9]+([_-][A-Z0-9]+)*)$"},
+                                     "pattern": "^([A-Z])+((([A-Za-z0-9]+))*(-)?)*$"},
                              "dids": DIDS},
               "required": ["dids"],
               "additionalProperties": False}
@@ -348,6 +345,26 @@ ACCOUNT_ATTRIBUTE = {"description": "Account attribute",
 # SCOPE_NAME_REGEXP = '/(.*)/(.*)'
 SCOPE_NAME_REGEXP = '/([^/]*)(?=/)(.*)'
 
+DISTANCE = {"description": "RSE distance",
+            "type": "object",
+            "properties": {
+                "src_rse_id": {"type": "string"},
+                "dest_rse_id": {"type": "string"},
+                "ranking": {"type": "integer"}
+            },
+            "required": ["src_rse_id", "dest_rse_id", "ranking"],
+            "additionalProperties": True}
+
+IMPORT = {"description": "import data into rucio.",
+          "type": "object",
+          "properties": {
+              "rses": {
+                  "type": "object"
+              },
+              "distances": {
+                  "type": "object"
+              }
+          }}
 
 SCHEMAS = {'account': ACCOUNT,
            'account_type': ACCOUNT_TYPE,
@@ -371,7 +388,8 @@ SCHEMAS = {'account': ACCOUNT,
            'subscription_filter': SUBSCRIPTION_FILTER,
            'cache_add_replicas': CACHE_ADD_REPLICAS,
            'cache_delete_replicas': CACHE_DELETE_REPLICAS,
-           'account_attribute': ACCOUNT_ATTRIBUTE}
+           'account_attribute': ACCOUNT_ATTRIBUTE,
+           'import': IMPORT}
 
 
 def validate_schema(name, obj):
@@ -385,4 +403,4 @@ def validate_schema(name, obj):
         if obj:
             validate(obj, SCHEMAS.get(name, {}))
     except ValidationError as error:  # NOQA, pylint: disable=W0612
-        raise InvalidObject("Problem validating %(name)s : %(error)s" % locals())
+        raise InvalidObject(f'Problem validating {name}: {error}')
