@@ -23,32 +23,33 @@ import io
 import itertools
 import json
 import logging
-import mmap
 import os
 import os.path
 import re
+import signal
 import socket
 import subprocess
 import tempfile
 import threading
 import time
-import zlib
 from collections import OrderedDict
+from configparser import NoOptionError, NoSectionError
 from enum import Enum
-from functools import partial
+from functools import partial, wraps
+from io import StringIO
+from itertools import zip_longest
+from typing import TYPE_CHECKING
+from urllib.parse import urlparse, urlencode, quote, parse_qsl, urlunparse
 from uuid import uuid4 as uuid
 from xml.etree import ElementTree
 
+import mmap
 import requests
-from io import StringIO
-from itertools import zip_longest
-from urllib.parse import urlparse, urlencode, quote, parse_qsl, urlunparse
-from configparser import NoOptionError, NoSectionError
+import zlib
 
 from rucio.common.config import config_get, config_has_section
 from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError, RucioException, \
-    DuplicateCriteriaInDIDFilter, DIDFilterSyntaxError, InvalidAlgorithmName
-
+    DuplicateCriteriaInDIDFilter, DIDFilterSyntaxError, InvalidAlgorithmName, PolicyPackageVersionError
 from rucio.common.extra import import_extras
 from rucio.common.types import InternalAccount, InternalScope
 
@@ -60,6 +61,13 @@ if EXTRA_MODULES['paramiko']:
     except Exception:
         EXTRA_MODULES['paramiko'] = False
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+    from typing import TypeVar
+
+    T = TypeVar('T')
+
+
 # HTTP code dictionary. Not complete. Can be extended if needed.
 codes = {
     # Informational.
@@ -426,7 +434,7 @@ def parse_response(data):
     return json.loads(data, object_hook=datetime_parser)
 
 
-def execute(cmd, blocking=True):
+def execute(cmd) -> tuple[int, str, str]:
     """
     Executes a command in a subprocess. Returns a tuple
     of (exitcode, out, err), where out is the string output
@@ -442,12 +450,10 @@ def execute(cmd, blocking=True):
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
 
-    if blocking:
-        result = process.communicate()
-        (out, err) = result
-        exitcode = process.returncode
-        return exitcode, out.decode(encoding='utf-8'), err.decode(encoding='utf-8')
-    return process
+    result = process.communicate()
+    (out, err) = result
+    exitcode = process.returncode
+    return exitcode, out.decode(encoding='utf-8'), err.decode(encoding='utf-8')
 
 
 def rse_supported_protocol_operations():
@@ -1360,21 +1366,19 @@ def run_cmd_process(cmd, timeout=3600):
     :return: stdout xor stderr, and errorcode
     """
 
-    time_start = datetime.datetime.now().second
     process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, preexec_fn=os.setsid, universal_newlines=True)
 
-    running_time = 0
-    while process.poll() is None and running_time < timeout:
-        time_now = datetime.datetime.now().second
-        running_time = int(time_now - time_start)
-        time.sleep(3)
-    if process.poll() is None:
-        process.terminate()
-        time.sleep(3)
-    if process.poll() is None:
-        process.kill()
+    try:
+        stdout, stderr = process.communicate(timeout=timeout)
+    except subprocess.TimeoutExpired:
+        try:
+            # Kill the whole process group since we're using shell=True.
+            os.killpg(os.getpgid(process.pid), signal.SIGTERM)
+            stdout, stderr = process.communicate(timeout=3)
+        except subprocess.TimeoutExpired:
+            os.killpg(os.getpgid(process.pid), signal.SIGKILL)
+            stdout, stderr = process.communicate()
 
-    stdout, stderr = process.communicate()
     if not stderr:
         stderr = ''
     if not stdout:
@@ -1663,7 +1667,6 @@ class PriorityQueue:
     def __init__(self):
         self.heap = []
         self.container = {}
-        self.empty_slots = []
 
     def __len__(self):
         return len(self.heap)
@@ -1764,6 +1767,7 @@ def register_policy_package_algorithms(algorithm_type, dictionary):
                 package = os.environ[env_name]
             else:
                 package = config.config_get('policy', 'package' + ('' if not vo else '-' + vo))
+            check_policy_package_version(package)
             module = importlib.import_module(package)
             if hasattr(module, 'get_algorithms'):
                 all_algorithms = module.get_algorithms()
@@ -1818,6 +1822,29 @@ def register_policy_package_algorithms(algorithm_type, dictionary):
                 try_importing_policy(algorithm_type, dictionary, vo['vo'])
 
 
+def check_policy_package_version(package):
+    import importlib
+    from rucio.version import version_string
+    '''
+    Checks that the Rucio version supported by the policy package is compatible
+    with this version. Raises an exception if not.
+    :param package: the fully qualified name of the policy package
+    '''
+    try:
+        module = importlib.import_module(package)
+    except ImportError:
+        # package not found. Will be picked up elsewhere
+        return
+    if not hasattr(module, 'SUPPORTED_VERSION'):
+        # package is not versioned
+        return
+    supported_version = module.SUPPORTED_VERSION if isinstance(module.SUPPORTED_VERSION, list) else [module.SUPPORTED_VERSION]
+    components = 2 if version_string().startswith("1.") else 1
+    current_version = ".".join(version_string().split(".")[:components])
+    if current_version not in supported_version:
+        raise PolicyPackageVersionError(package)
+
+
 class Availability:
     """
     This util class acts as a translator between the availability stored as
@@ -1888,3 +1915,32 @@ class Availability:
         delete_value = (self.delete or self.delete is None) * 1
 
         return read_value + write_value + delete_value
+
+
+def retrying(
+        retry_on_exception: "Callable[[Exception], bool]",
+        wait_fixed: int,
+        stop_max_attempt_number: int
+) -> "Callable[[Callable[..., T]], Callable[..., T]]":
+    """
+    Decorator which retries a function multiple times on certain types of exceptions.
+    :param retry_on_exception: Function which takes an exception as argument and returns True if we must retry on this exception
+    :param wait_fixed: the amount of time to wait in-between two tries
+    :param stop_max_attempt_number: maximum number of allowed attempts
+    """
+    def _decorator(fn):
+        @wraps(fn)
+        def _wrapper(*args, **kwargs):
+            attempt = 0
+            while True:
+                attempt += 1
+                try:
+                    return fn(*args, **kwargs)
+                except Exception as e:
+                    if attempt >= stop_max_attempt_number:
+                        raise
+                    if not retry_on_exception(e):
+                        raise
+                time.sleep(wait_fixed / 1000.0)
+        return _wrapper
+    return _decorator
