@@ -12,21 +12,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import datetime
+from typing import TYPE_CHECKING
 
+from sqlalchemy import literal, insert, select
 from sqlalchemy.orm.exc import NoResultFound
 
-from rucio.common.exception import CounterNotFound
-import rucio.core.account
-import rucio.core.rse
-
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.session import read_session, transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 MAX_COUNTERS = 10
 
 
 @transactional_session
-def add_counter(rse_id, account, session=None):
+def add_counter(rse_id, account, *, session: "Session"):
     """
     Creates the specified counter for a rse_id and account.
 
@@ -39,37 +41,7 @@ def add_counter(rse_id, account, session=None):
 
 
 @transactional_session
-def create_counters_for_new_account(account, session=None):
-    """
-    Creates all the Account counters when creating a account.
-
-    :param account: The account.
-    :param session: The database session in use.models.RSECounter
-    """
-
-    vo = account.vo
-    for rse in rucio.core.rse.list_rses(session=session):
-        if rse['vo'] == vo:
-            add_counter(rse_id=rse['id'], account=account, session=session)
-
-
-@transactional_session
-def create_counters_for_new_rse(rse_id, session=None):
-    """
-    Creates all the Account counters when creating a new rse.
-
-    :param rse_id:  The rse_id.
-    :param session: The database session in use.models.RSECounter
-    """
-
-    vo = rucio.core.rse.get_rse_vo(rse_id, session=session)
-    for account in rucio.core.account.list_accounts(session=session):
-        if account['account'].vo == vo:
-            add_counter(rse_id=rse_id, account=account['account'], session=session)
-
-
-@transactional_session
-def increase(rse_id, account, files, bytes_, session=None):
+def increase(rse_id, account, files, bytes_, *, session: "Session"):
     """
     Increments the specified counter by the specified amount.
 
@@ -83,7 +55,7 @@ def increase(rse_id, account, files, bytes_, session=None):
 
 
 @transactional_session
-def decrease(rse_id, account, files, bytes_, session=None):
+def decrease(rse_id, account, files, bytes_, *, session: "Session"):
     """
     Decreases the specified counter by the specified amount.
 
@@ -97,7 +69,7 @@ def decrease(rse_id, account, files, bytes_, session=None):
 
 
 @transactional_session
-def del_counter(rse_id, account, session=None):
+def del_counter(rse_id, account, *, session: "Session"):
     """
     Resets the specified counter and initializes it by the specified amounts.
 
@@ -110,7 +82,7 @@ def del_counter(rse_id, account, session=None):
 
 
 @read_session
-def get_updated_account_counters(total_workers, worker_number, session=None):
+def get_updated_account_counters(total_workers, worker_number, *, session: "Session"):
     """
     Get updated rse_counters.
 
@@ -133,7 +105,7 @@ def get_updated_account_counters(total_workers, worker_number, session=None):
 
 
 @transactional_session
-def update_account_counter(account, rse_id, session=None):
+def update_account_counter(account, rse_id, *, session: "Session"):
     """
     Read the updated_account_counters and update the account_counter.
 
@@ -159,7 +131,7 @@ def update_account_counter(account, rse_id, session=None):
 
 
 @transactional_session
-def update_account_counter_history(account, rse_id, session=None):
+def update_account_counter_history(account, rse_id, *, session: "Session"):
     """
     Read the AccountUsage and update the AccountUsageHistory.
 
@@ -167,21 +139,33 @@ def update_account_counter_history(account, rse_id, session=None):
     :param rse_id:   The rse_id to update.
     :param session:  Database session in use.
     """
-    AccountUsageHistory = models.AccountUsageHistory
-    try:
-        counter = session.query(models.AccountUsage).filter_by(rse_id=rse_id, account=account).one()
-        AccountUsageHistory(rse_id=rse_id, account=account, files=counter.files, bytes=counter.bytes).save(session=session)
-    except NoResultFound:
-        raise CounterNotFound('No usage can be found for account %s on RSE %s' % (account, rucio.core.rse.get_rse_name(rse_id=rse_id, session=session)))
+    counter = session.query(models.AccountUsage).filter_by(rse_id=rse_id, account=account).one_or_none()
+    if counter:
+        models.AccountUsageHistory(rse_id=rse_id, account=account, files=counter.files, bytes=counter.bytes).save(session=session)
+    else:
+        models.AccountUsageHistory(rse_id=rse_id, account=account, files=0, bytes=0).save(session=session)
 
 
 @transactional_session
-def fill_account_counter_history_table(session=None):
+def fill_account_counter_history_table(*, session: "Session"):
     """
-    Fill the account usage history table with the current usage.
+    Make a snapshot of current counters
 
     :param session:  Database session in use.
     """
-    AccountUsageHistory = models.AccountUsageHistory
-    for usage in session.query(models.AccountUsage).all():
-        AccountUsageHistory(rse_id=usage['rse_id'], account=usage['account'], files=usage['files'], bytes=usage['bytes']).save(session=session)
+
+    select_counters_stmt = select(
+        models.AccountUsage.rse_id,
+        models.AccountUsage.account,
+        models.AccountUsage.files,
+        models.AccountUsage.bytes,
+        literal(datetime.datetime.utcnow()),
+    )
+
+    stmt = insert(
+        models.AccountUsageHistory
+    ).from_select(
+        ['rse_id', 'account', 'files', 'bytes', 'updated_at'],
+        select_counters_stmt
+    )
+    session.execute(stmt)
