@@ -16,19 +16,21 @@
 import datetime
 import hashlib
 import random
+import re
 import sys
 import traceback
 from base64 import b64decode
+from typing import TYPE_CHECKING
 
 import paramiko
-from dogpile.cache.api import NO_VALUE
 from dogpile.cache import make_region
-from sqlalchemy import and_, or_, select, delete
+from dogpile.cache.api import NO_VALUE
+from sqlalchemy import delete, null, or_, select
 
 from rucio.common.cache import make_region_memcached
 from rucio.common.config import config_get_bool
 from rucio.common.exception import CannotAuthenticate, RucioException
-from rucio.common.utils import chunks, generate_uuid
+from rucio.common.utils import chunks, generate_uuid, date_to_str
 from rucio.core.account import account_exists
 from rucio.core.oidc import validate_jwt
 from rucio.db.sqla import filter_thread_work
@@ -36,10 +38,46 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import IdentityType
 from rucio.db.sqla.session import read_session, transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+    from typing import Any, Union
+
+
+def strip_x509_proxy_attributes(dn: str) -> str:
+    """Strip X509 proxy attributes from a DN.
+
+    When an X509 VOMS proxy certificate is produced, an additional Common Name
+    attribute is added to the subject of the original certificate.  Its value
+    can take different forms.  For proxy versions 3 and later (the default), the
+    value is a numeric.  For previous versions, the value is exclusively one of
+    'limited proxy' or 'proxy', depending on how it was produced (the most
+    trustworthy documentation on this seems to be the VOMS source code itself;
+    refer to the file sslutils.c).  Note that this addition might happen more
+    than once (e.g. if a limited proxy is used to produce a full proxy).
+
+    By default, the Apache server will return the DN in an RFC-compliant format,
+    which can look like this:
+        CN=John Doe,OU=Users,DC=example,DC=com
+    However, in case the LegacyDNStringFormat of mod_ssl is enabled, then it can
+    look like this instead:
+        /DC=com/DC=example/OU=Users/CN=John Doe
+    In the first case, the Common Name attributes added by VOMS are prepended,
+    whereas in the second case, they are appended.
+
+    The motivation for stripping these attributes is to avoid having to store
+    multiple DNs in the database (as different identities).
+    """
+    if dn.startswith('/'):
+        regexp = r'(/CN=(limited proxy|proxy|[0-9]+))+$'
+    else:
+        regexp = r'^(CN=(limited proxy|proxy|[0-9]+),)+'
+
+    return re.sub(regexp, '', dn)
+
 
 def token_key_generator(namespace, fni, **kwargs):
     """ :returns: generate key function """
-    def generate_key(token, session=None):
+    def generate_key(token, *, session: "Session"):
         """ :returns: token """
         return token
     return generate_key
@@ -52,7 +90,7 @@ else:
 
 
 @transactional_session
-def get_auth_token_user_pass(account, username, password, appid, ip=None, session=None):
+def get_auth_token_user_pass(account, username, password, appid, ip=None, *, session: "Session"):
     """
     Authenticate a Rucio account temporarily via username and password.
 
@@ -72,8 +110,13 @@ def get_auth_token_user_pass(account, username, password, appid, ip=None, sessio
     if not account_exists(account, session=session):
         return None
 
-    result = session.query(models.Identity).filter_by(identity=username,
-                                                      identity_type=IdentityType.USERPASS).first()
+    query = select(
+        models.Identity
+    ).where(
+        models.Identity.identity == username,
+        models.Identity.identity_type == IdentityType.USERPASS
+    )
+    result = session.execute(query).scalar()
 
     db_salt = result['salt']
     db_password = result['password']
@@ -83,9 +126,15 @@ def get_auth_token_user_pass(account, username, password, appid, ip=None, sessio
         return None
 
     # get account identifier
-    result = session.query(models.IdentityAccountAssociation).filter_by(identity=username,
-                                                                        identity_type=IdentityType.USERPASS,
-                                                                        account=account).first()
+    query = select(
+        models.IdentityAccountAssociation
+    ).where(
+        models.IdentityAccountAssociation.identity == username,
+        models.IdentityAccountAssociation.identity_type == IdentityType.USERPASS,
+        models.IdentityAccountAssociation.account == account
+    )
+    result = session.execute(query).scalar()
+
     db_account = result['account']
 
     # remove expired tokens
@@ -93,7 +142,7 @@ def get_auth_token_user_pass(account, username, password, appid, ip=None, sessio
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
-    token = '%(account)s-%(username)s-%(appid)s-%(tuid)s' % locals()
+    token = f'{account}-{username}-{appid}-{tuid}'
     new_token = models.Token(account=db_account, identity=username, token=token, ip=ip)
     new_token.save(session=session)
 
@@ -101,7 +150,7 @@ def get_auth_token_user_pass(account, username, password, appid, ip=None, sessio
 
 
 @transactional_session
-def get_auth_token_x509(account, dn, appid, ip=None, session=None):
+def get_auth_token_x509(account, dn, appid, ip=None, *, session: "Session"):
     """
     Authenticate a Rucio account temporarily via an x509 certificate.
 
@@ -125,7 +174,7 @@ def get_auth_token_x509(account, dn, appid, ip=None, session=None):
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
-    token = '%(account)s-%(dn)s-%(appid)s-%(tuid)s' % locals()
+    token = f'{account}-{dn}-{appid}-{tuid}'
     new_token = models.Token(account=account, identity=dn, token=token, ip=ip)
     new_token.save(session=session)
 
@@ -133,7 +182,7 @@ def get_auth_token_x509(account, dn, appid, ip=None, session=None):
 
 
 @transactional_session
-def get_auth_token_gss(account, gsstoken, appid, ip=None, session=None):
+def get_auth_token_gss(account, gsstoken, appid, ip=None, *, session: "Session"):
     """
     Authenticate a Rucio account temporarily via a GSS token.
 
@@ -157,7 +206,7 @@ def get_auth_token_gss(account, gsstoken, appid, ip=None, session=None):
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
-    token = '%(account)s-%(gsstoken)s-%(appid)s-%(tuid)s' % locals()
+    token = f'{account}-{gsstoken}-{appid}-{tuid}'
     new_token = models.Token(account=account, token=token, ip=ip)
     new_token.save(session=session)
 
@@ -165,7 +214,7 @@ def get_auth_token_gss(account, gsstoken, appid, ip=None, session=None):
 
 
 @transactional_session
-def get_auth_token_ssh(account, signature, appid, ip=None, session=None):
+def get_auth_token_ssh(account, signature, appid, ip=None, *, session: "Session"):
     """
     Authenticate a Rucio account temporarily via SSH key exchange.
 
@@ -187,13 +236,23 @@ def get_auth_token_ssh(account, signature, appid, ip=None, session=None):
         return None
 
     # get all active challenge tokens for the requested account
-    active_challenge_tokens = session.query(models.Token).filter(models.Token.expired_at >= datetime.datetime.utcnow(),
-                                                                 models.Token.account == account,
-                                                                 models.Token.token.like('challenge-%')).all()
+    query = select(
+        models.Token
+    ).where(
+        models.Token.expired_at >= datetime.datetime.utcnow(),
+        models.Token.account == account,
+        models.Token.token.like('challenge-%')
+    )
+    active_challenge_tokens = session.execute(query).scalars().all()
 
     # get all identities for the requested account
-    identities = session.query(models.IdentityAccountAssociation).filter_by(identity_type=IdentityType.SSH,
-                                                                            account=account).all()
+    query = select(
+        models.IdentityAccountAssociation
+    ).where(
+        models.IdentityAccountAssociation.identity_type == IdentityType.SSH,
+        models.IdentityAccountAssociation.account == account
+    )
+    identities = session.execute(query).scalars().all()
 
     # no challenge tokens found
     if not active_challenge_tokens:
@@ -221,7 +280,7 @@ def get_auth_token_ssh(account, signature, appid, ip=None, session=None):
 
     # create new rucio-auth-token for account
     tuid = generate_uuid()  # NOQA
-    token = '%(account)s-ssh:pubkey-%(appid)s-%(tuid)s' % locals()
+    token = f'{account}-ssh:pubkey-{appid}-{tuid}'
     new_token = models.Token(account=account, token=token, ip=ip)
     new_token.save(session=session)
 
@@ -229,7 +288,7 @@ def get_auth_token_ssh(account, signature, appid, ip=None, session=None):
 
 
 @transactional_session
-def get_ssh_challenge_token(account, appid, ip=None, session=None):
+def get_ssh_challenge_token(account, appid, ip=None, *, session: "Session"):
     """
     Prepare a challenge token for subsequent SSH public key authentication.
 
@@ -255,7 +314,7 @@ def get_ssh_challenge_token(account, appid, ip=None, session=None):
     expiration = datetime.datetime.utcnow() + datetime.timedelta(seconds=10)
     expiration_unix = expiration.strftime("%s")
 
-    challenge_token = 'challenge-%(crypto_rand)s-%(account)s-%(expiration_unix)s' % locals()
+    challenge_token = f'challenge-{crypto_rand}-{account}-{expiration_unix}'
 
     new_challenge_token = models.Token(account=account, token=challenge_token, ip=ip,
                                        expired_at=expiration)
@@ -265,7 +324,7 @@ def get_ssh_challenge_token(account, appid, ip=None, session=None):
 
 
 @transactional_session
-def get_auth_token_saml(account, saml_nameid, appid, ip=None, session=None):
+def get_auth_token_saml(account, saml_nameid, appid, ip=None, *, session: "Session"):
     """
     Authenticate a Rucio account temporarily via SAML.
 
@@ -288,7 +347,7 @@ def get_auth_token_saml(account, saml_nameid, appid, ip=None, session=None):
     __delete_expired_tokens_account(account=account, session=session)
 
     tuid = generate_uuid()  # NOQA
-    token = '%(account)s-%(saml_nameid)s-%(appid)s-%(tuid)s' % locals()
+    token = f'{account}-{saml_nameid}-{appid}-{tuid}'
     new_token = models.Token(account=account, identity=saml_nameid, token=token, ip=ip)
     new_token.save(session=session)
 
@@ -296,7 +355,7 @@ def get_auth_token_saml(account, saml_nameid, appid, ip=None, session=None):
 
 
 @transactional_session
-def redirect_auth_oidc(auth_code, fetchtoken=False, session=None):
+def redirect_auth_oidc(auth_code, fetchtoken=False, *, session: "Session"):
     """
     Finds the Authentication URL in the Rucio DB oauth_requests table
     and redirects user's browser to this URL.
@@ -313,26 +372,31 @@ def redirect_auth_oidc(auth_code, fetchtoken=False, session=None):
 
     """
     try:
-        redirect_result = session.query(models.OAuthRequest.redirect_msg).filter_by(access_msg=auth_code).first()
+        query = select(
+            models.OAuthRequest.redirect_msg
+        ).where(
+            models.OAuthRequest.access_msg == auth_code
+        )
+        redirect_result = session.execute(query).scalar()
 
         if not redirect_result:
             return None
 
-        if 'http' not in redirect_result[0] and fetchtoken:
+        if 'http' not in redirect_result and fetchtoken:
             # in this case the function check if the value is a valid token
-            vdict = validate_auth_token(redirect_result[0], session=session)
+            vdict = validate_auth_token(redirect_result, session=session)
             if vdict:
-                return redirect_result[0]
+                return redirect_result
             return None
-        elif 'http' in redirect_result[0] and not fetchtoken:
+        elif 'http' in redirect_result and not fetchtoken:
             # return redirection URL
-            return redirect_result[0]
+            return redirect_result
     except:
         raise CannotAuthenticate(traceback.format_exc())
 
 
 @transactional_session
-def delete_expired_tokens(total_workers, worker_number, limit=1000, session=None):
+def delete_expired_tokens(total_workers, worker_number, limit=1000, *, session: "Session"):
     """
     Delete expired tokens.
 
@@ -347,23 +411,43 @@ def delete_expired_tokens(total_workers, worker_number, limit=1000, session=None
     # get expired tokens
     try:
         # delete all expired tokens except tokens which have refresh token that is still valid
-        query = session.query(models.Token.token).filter(and_(models.Token.expired_at <= datetime.datetime.utcnow()))\
-                                                 .filter(or_(models.Token.refresh_expired_at.__eq__(None),
-                                                             models.Token.refresh_expired_at <= datetime.datetime.utcnow()))\
-                                                 .order_by(models.Token.expired_at)
+        query = select(
+            models.Token.token
+        ).where(
+            models.Token.expired_at <= datetime.datetime.utcnow(),
+            or_(
+                models.Token.refresh_expired_at == null(),
+                models.Token.refresh_expired_at <= datetime.datetime.utcnow()
+            )
+        ).order_by(
+            models.Token.expired_at
+        )
 
         query = filter_thread_work(session=session, query=query, total_threads=total_workers, thread_id=worker_number, hash_variable='token')
 
         # limiting the number of tokens deleted at once
         query = query.limit(limit)
+        # Oracle does not support chaining order_by(), limit(), and
+        # with_for_update(). Use a nested query to overcome this.
+        if session.bind.dialect.name == 'oracle':
+            query = select(
+                models.Token.token
+            ).where(
+                models.Token.token.in_(query)
+            ).with_for_update(
+                skip_locked=True
+            )
+        else:
+            query = query.with_for_update(skip_locked=True)
         # remove expired tokens
         deleted_tokens = 0
-        for items in session.execute(query).partitions(10):
-            tokens = tuple(map(lambda row: row.token, items))
-            deleted_tokens += session.query(models.Token) \
-                                     .filter(models.Token.token.in_(tokens)) \
-                                     .with_for_update(skip_locked=True) \
-                                     .delete(synchronize_session='fetch')
+        for tokens in session.execute(query).scalars().partitions(10):
+            query = delete(
+                models.Token
+            ).where(
+                models.Token.token.in_(tokens)
+            )
+            deleted_tokens += session.execute(query).rowcount
 
     except Exception as error:
         raise RucioException(error.args)
@@ -372,7 +456,7 @@ def delete_expired_tokens(total_workers, worker_number, limit=1000, session=None
 
 
 @read_session
-def query_token(token, session=None):
+def query_token(token, *, session: "Session"):
     """
     Validate an authentication token using the database. This method will only be called
     if no entry could be found in the according cache.
@@ -388,25 +472,24 @@ def query_token(token, session=None):
               if successful, None otherwise.
     """
     # Query the DB to validate token
-    ret = session.query(models.Token.account,
-                        models.Token.identity,
-                        models.Token.expired_at,
-                        models.Token.audience,
-                        models.Token.oidc_scope).\
-        filter(models.Token.token == token,
-               models.Token.expired_at > datetime.datetime.utcnow()).\
-        all()
-    if ret:
-        return {'account': ret[0][0],
-                'identity': ret[0][1],
-                'lifetime': ret[0][2],
-                'audience': ret[0][3],
-                'authz_scope': ret[0][4]}
+    query = select(
+        models.Token.account,
+        models.Token.identity,
+        models.Token.expired_at.label('lifetime'),
+        models.Token.audience,
+        models.Token.oidc_scope.label('authz_scope')
+    ).where(
+        models.Token.token == token,
+        models.Token.expired_at > datetime.datetime.utcnow()
+    )
+    result = session.execute(query).first()
+    if result:
+        return result._asdict()
     return None
 
 
 @transactional_session
-def validate_auth_token(token, session=None):
+def validate_auth_token(token: str, *, session: "Session") -> "dict[str, Any]":
     """
     Validate an authentication token.
 
@@ -417,17 +500,18 @@ def validate_auth_token(token, session=None):
                            lifetime: <token lifetime>,
                            audience: <audience>,
                            authz_scope: <authz_scope> }
-              if successful, None otherwise.
+              if successful
+    :raises: CannotAuthenticate if unsuccessful
     """
     if not token:
-        return None
+        raise CannotAuthenticate("No token was passed!")
 
     # Be gentle with bash variables, there can be whitespace
     token = token.strip()
     cache_key = token.replace(' ', '')
 
     # Check if token ca be found in cache region
-    value = TOKENREGION.get(cache_key)
+    value: "Union[NO_VALUE, dict[str, Any]]" = TOKENREGION.get(cache_key)
     if value is NO_VALUE:  # no cached entry found
         value = query_token(token, session=session)
         if not value:
@@ -435,15 +519,14 @@ def validate_auth_token(token, session=None):
             # & save it in Rucio if scope and audience are correct
             if len(token.split(".")) == 3:
                 value = validate_jwt(token, session=session)
-                if not value:
-                    return None
             else:
-                return None
+                raise CannotAuthenticate(traceback.format_exc())
         # save token in the cache
         TOKENREGION.set(cache_key, value)
-    if value.get('lifetime', datetime.datetime(1970, 1, 1)) < datetime.datetime.utcnow():  # check if expired
+    lifetime = value.get('lifetime', datetime.datetime(1970, 1, 1))
+    if lifetime < datetime.datetime.utcnow():  # check if expired
         TOKENREGION.delete(cache_key)
-        return None
+        raise CannotAuthenticate(f"Token found but expired since {date_to_str(lifetime)}.")
     return value
 
 
@@ -452,25 +535,29 @@ def token_dictionary(token: models.Token):
 
 
 @transactional_session
-def __delete_expired_tokens_account(account, session=None):
+def __delete_expired_tokens_account(account, *, session: "Session"):
     """"
     Deletes expired tokens from the database.
 
     :param account: Account to delete expired tokens.
     :param session: The database session in use.
     """
-    stmt_select = select(models.Token) \
-        .where(and_(models.Token.expired_at < datetime.datetime.utcnow(),
-                    models.Token.account == account)) \
-        .with_for_update(skip_locked=True)
-    result_select = session.execute(stmt_select)
+    select_query = select(
+        models.Token.token
+    ).where(
+        models.Token.expired_at < datetime.datetime.utcnow(),
+        models.Token.account == account
+    ).with_for_update(
+        skip_locked=True
+    )
+    tokens = session.execute(select_query).scalars().all()
 
-    tokens = []
-    for t in result_select.columns('token'):
-        tokens.append(t.token)
-
-    for t in chunks(tokens, 100):
-        stmt_delete = delete(models.Token) \
-            .where(models.Token.token.in_(t)) \
-            .prefix_with("/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */")
-        session.execute(stmt_delete)
+    for chunk in chunks(tokens, 100):
+        delete_query = delete(
+            models.Token
+        ).prefix_with(
+            "/*+ INDEX(TOKENS_ACCOUNT_EXPIRED_AT_IDX) */"
+        ).where(
+            models.Token.token.in_(chunk)
+        )
+        session.execute(delete_query)
