@@ -13,6 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from collections.abc import Callable
+from typing import TYPE_CHECKING, Any, Optional, TypeVar
+
 from dogpile.cache.api import NoValue
 from sqlalchemy import func
 
@@ -21,6 +24,11 @@ from rucio.common.exception import ConfigNotFound
 from rucio.db.sqla import models
 from rucio.db.sqla.session import read_session, transactional_session
 
+T = TypeVar('T')
+
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 REGION = make_region_memcached(expiration_time=900)
 
@@ -48,7 +56,12 @@ def _value_cache_key(section, option):
 
 
 @read_session
-def sections(use_cache=True, expiration_time=900, session=None):
+def sections(
+        *,
+        use_cache: bool = True,
+        expiration_time: int = 900,
+        session: "Session"
+):
     """
     Return a list of the sections available.
 
@@ -70,7 +83,7 @@ def sections(use_cache=True, expiration_time=900, session=None):
 
 
 @transactional_session
-def add_section(section, session=None):
+def add_section(section: str, *, session: "Session"):
     """
     Add a section to the configuration.
     :param session: The database session in use.
@@ -81,7 +94,13 @@ def add_section(section, session=None):
 
 
 @read_session
-def has_section(section, use_cache=True, expiration_time=900, session=None):
+def has_section(
+        section: str,
+        *,
+        use_cache: bool = True,
+        expiration_time: int = 900,
+        session: "Session"
+) -> bool:
     """
     Indicates whether the named section is present in the configuration.
 
@@ -103,7 +122,13 @@ def has_section(section, use_cache=True, expiration_time=900, session=None):
 
 
 @read_session
-def options(section, use_cache=True, expiration_time=900, session=None):
+def options(
+        section: str,
+        *,
+        use_cache: bool = True,
+        expiration_time: int = 900,
+        session: "Session"
+) -> list[str]:
     """
     Returns a list of options available in the specified section.
 
@@ -125,7 +150,14 @@ def options(section, use_cache=True, expiration_time=900, session=None):
 
 
 @read_session
-def has_option(section, option, use_cache=True, expiration_time=900, session=None):
+def has_option(
+        section,
+        option,
+        *,
+        use_cache=True,
+        expiration_time=900,
+        session: "Session"
+):
     """
     Check if the given section exists and contains the given option.
 
@@ -148,7 +180,16 @@ def has_option(section, option, use_cache=True, expiration_time=900, session=Non
 
 
 @read_session
-def get(section, option, default=None, use_cache=True, expiration_time=900, session=None):
+def get(
+        section: str,
+        option: str,
+        *,
+        default: Optional[T] = None,
+        use_cache: bool = True,
+        expiration_time: int = 900,
+        convert_type_fnc: Callable[[str], T],
+        session: "Session"
+) -> T:
     """
     Get an option value for the named section. Value can be auto-coerced to string, int, float, bool, None.
 
@@ -160,6 +201,7 @@ def get(section, option, default=None, use_cache=True, expiration_time=900, sess
     :param default: The default value if no value is found.
     :param use_cache: Boolean if the cache should be used.
     :param expiration_time: Time after that the cached value gets ignored.
+    :param convert_type_fnc: A function used to parse the string config value into the desired destination type
     :param session: The database session in use.
     :returns: The auto-coerced value.
     """
@@ -170,26 +212,34 @@ def get(section, option, default=None, use_cache=True, expiration_time=900, sess
     if isinstance(value, NoValue):
         tmp = session.query(models.Config.value).filter_by(section=section, opt=option).first()
         if tmp is not None:
-            value = __convert_type(tmp[0])
+            value = convert_type_fnc(tmp[0])
             write_to_cache(value_key, tmp[0])
         elif default is None:
             raise ConfigNotFound
         else:
             value = default
-            write_to_cache(value_key, str(value))  # Also write default to cache
+            write_to_cache(value_key, value)  # Also write default to cache
     else:
-        value = __convert_type(value)
+        value = convert_type_fnc(value)
     return value
 
 
 @read_session
-def items(section, use_cache=True, expiration_time=900, session=None):
+def items(
+        section: str,
+        use_cache: bool = True,
+        expiration_time: int = 900,
+        *,
+        convert_type_fnc: Callable[[str], T],
+        session: "Session"
+) -> list[tuple[str, T]]:
     """
     Return a list of (option, value) pairs for each option in the given section. Values are auto-coerced as in get().
 
     :param section: The name of the section.
     :param use_cache: Boolean if the cache should be used.
     :param expiration_time: Time after that the cached value gets ignored.
+    :param convert_type_fnc: A function used to parse the string config value into the desired destination type
     :param session: The database session in use.
     :returns: [('option', auto-coerced value), ...]
     """
@@ -200,11 +250,17 @@ def items(section, use_cache=True, expiration_time=900, session=None):
     if isinstance(items, NoValue):
         items = session.query(models.Config.opt, models.Config.value).filter_by(section=section).all()
         write_to_cache(items_key, items)
-    return [(item[0], __convert_type(item[1])) for item in items]
+    return [(item[0], convert_type_fnc(item[1])) for item in items]
 
 
 @transactional_session
-def set(section, option, value, session=None):
+def set(
+        section: str,
+        option: str,
+        value: Any,
+        *,
+        session: "Session"
+):
     """
     Set the given option to the specified value. If the option doesn't exist, it is created.
 
@@ -240,7 +296,7 @@ def set(section, option, value, session=None):
 
 
 @transactional_session
-def remove_section(section, session=None):
+def remove_section(section: str, *, session: "Session") -> bool:
     """
     Remove the specified section from the specified section.
 
@@ -267,7 +323,7 @@ def remove_section(section, session=None):
 
 
 @transactional_session
-def remove_option(section, option, session=None):
+def remove_option(section: str, option: str, *, session: "Session") -> bool:
     """
     Remove the specified option from the configuration.
 
@@ -297,24 +353,6 @@ def remove_option(section, option, session=None):
         return True
 
 
-def __convert_type(value):
-    '''
-    __convert_type
-    '''
-    if value.lower() in ['true', 'yes', 'on']:
-        return True
-    elif value.lower() in ['false', 'no', 'off']:
-        return False
-
-    for conv in (int, float):
-        try:
-            return conv(value)
-        except:
-            pass
-
-    return value
-
-
 def read_from_cache(key, expiration_time=900):
     """
     Try to read a value from a cache.
