@@ -13,8 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from datetime import datetime, timedelta
 import operator
+from datetime import datetime, timedelta
+from typing import TYPE_CHECKING
 
 from sqlalchemy import update, inspect
 from sqlalchemy.exc import CompileError, InvalidRequestError
@@ -30,6 +31,10 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType
 from rucio.db.sqla.session import stream_session, read_session, transactional_session
 
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
+
 
 class DidColumnMeta(DidMetaPlugin):
     """
@@ -40,7 +45,7 @@ class DidColumnMeta(DidMetaPlugin):
         self.plugin_name = "DID_COLUMN"
 
     @read_session
-    def get_metadata(self, scope, name, session=None):
+    def get_metadata(self, scope, name, *, session: "Session"):
         """
         Get data identifier metadata.
 
@@ -51,25 +56,32 @@ class DidColumnMeta(DidMetaPlugin):
         try:
             row = session.query(models.DataIdentifier).filter_by(scope=scope, name=name).\
                 with_hint(models.DataIdentifier, "INDEX(DIDS DIDS_PK)", 'oracle').one()
-            d = {}
-            for column in row.__table__.columns:
-                d[column.name] = getattr(row, column.name)
-            return d
+            return row.to_dict()
         except NoResultFound:
-            raise exception.DataIdentifierNotFound("Data identifier '%(scope)s:%(name)s' not found" % locals())
+            raise exception.DataIdentifierNotFound(f"Data identifier '{scope}:{name}' not found")
 
-    def set_metadata(self, scope, name, key, value, recursive=False, session=None):
+    @transactional_session
+    def set_metadata(self, scope, name, key, value, recursive=False, *, session: "Session"):
         self.set_metadata_bulk(scope=scope, name=name, metadata={key: value}, recursive=recursive, session=session)
 
     @transactional_session
-    def set_metadata_bulk(self, scope, name, metadata, recursive=False, session=None):
+    def set_metadata_bulk(self, scope, name, metadata, recursive=False, *, session: "Session"):
         did_query = session.query(models.DataIdentifier).with_hint(models.DataIdentifier, "INDEX(DIDS DIDS_PK)", 'oracle').filter_by(scope=scope, name=name)
         if did_query.one_or_none() is None:
             raise exception.DataIdentifierNotFound("Data identifier '%s:%s' not found" % (scope, name))
 
         remainder = {}
         for key, value in metadata.items():
-            if key == 'lifetime':
+            if key == 'eol_at' and isinstance(value, str):
+                try:
+                    eol_at = datetime.strptime(value, '%Y-%M-%d')
+                    rowcount = did_query.update({'eol_at': eol_at}, synchronize_session='fetch')
+                except TypeError as error:
+                    raise exception.InvalidValueForKey(error)
+                if not rowcount:
+                    # check for did presence
+                    raise exception.UnsupportedOperation('%s for %s:%s cannot be updated' % (key, scope, name))
+            elif key == 'lifetime':
                 try:
                     expired_at = None
                     if value is not None:
@@ -159,7 +171,7 @@ class DidColumnMeta(DidMetaPlugin):
 
     @stream_session
     def list_dids(self, scope, filters, did_type='collection', ignore_case=False, limit=None,
-                  offset=None, long=False, recursive=False, ignore_dids=None, session=None):
+                  offset=None, long=False, recursive=False, ignore_dids=None, *, session: "Session"):
         """
         Search data identifiers.
 
@@ -260,7 +272,7 @@ class DidColumnMeta(DidMetaPlugin):
                     ignore_dids.add(did_full)
                     yield did.name
 
-    def delete_metadata(self, scope, name, key, session=None):
+    def delete_metadata(self, scope, name, key, *, session: "Optional[Session]" = None):
         """
         Deletes the metadata stored for the given key.
 
@@ -270,7 +282,7 @@ class DidColumnMeta(DidMetaPlugin):
         """
         raise NotImplementedError('The DidColumnMeta plugin does not currently support deleting metadata.')
 
-    def manages_key(self, key, session=None):
+    def manages_key(self, key, *, session: "Optional[Session]" = None):
         # Build list of which keys are managed by this plugin.
         #
         all_did_table_columns = []
