@@ -18,6 +18,7 @@ import fnmatch
 import operator
 from datetime import datetime, timedelta, date
 from importlib import import_module
+from typing import TYPE_CHECKING
 
 import sqlalchemy
 from sqlalchemy import cast, or_, and_
@@ -28,6 +29,10 @@ from rucio.common.utils import parse_did_filter_from_string_fe
 from rucio.db.sqla.constants import DIDType
 from rucio.db.sqla.session import read_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
+
 # lookup table converting keyword suffixes to pythonic operators.
 OPERATORS_CONVERSION_LUT = {
     "gte": operator.ge,
@@ -260,7 +265,9 @@ class FilterEngine:
             else:
                 return value
         try:
-            value = ast.literal_eval(value)     # will catch float, int and bool
+            operators = ('+', '-', '*', '/')
+            if not any(operator in value for operator in operators):    # fix for lax ast literal_eval in earlier python versions
+                value = ast.literal_eval(value)                         # will catch float, int and bool
         except (ValueError, SyntaxError):
             pass
         return value
@@ -283,7 +290,7 @@ class FilterEngine:
             for and_group in or_group:
                 key, oper, value = and_group
                 if isinstance(value, str) and any([char in value for char in ['*', '%']]):   # wildcards
-                    if value in ('*', '%', u'*', u'%'):                                      # match wildcard exactly == no filtering on key
+                    if value in ('*', '%', '*', '%'):                                        # match wildcard exactly == no filtering on key
                         continue
                     else:                                                                    # partial match with wildcard == like || notlike
                         if oper == operator.eq:
@@ -295,10 +302,8 @@ class FilterEngine:
                         elif oper == operator.ne:
                             expression = {
                                 key: {
-                                    {
-                                        '$not': {
-                                            '$regex': fnmatch.translate(value)              # translate partial wildcard expression to regex
-                                        }
+                                    '$not': {
+                                        '$regex': fnmatch.translate(value)                  # translate partial wildcard expression to regex
                                     }
                                 }
                             }
@@ -346,7 +351,7 @@ class FilterEngine:
                 else:
                     is_in_json_column = True
                 if isinstance(value, str) and any([char in value for char in ['*', '%']]):  # wildcards
-                    if value in ('*', '%', u'*', u'%'):                                     # match wildcard exactly == no filtering on key
+                    if value in ('*', '%', '*', '%'):                                       # match wildcard exactly == no filtering on key
                         continue
                     else:                                                                   # partial match with wildcard == like || notlike
                         if oper == operator.eq:
@@ -396,7 +401,7 @@ class FilterEngine:
         return ' OR '.join(or_expressions)
 
     @read_session
-    def create_sqla_query(self, session=None, additional_model_attributes=[], additional_filters={}, json_column=None):
+    def create_sqla_query(self, *, session: "Session", additional_model_attributes=[], additional_filters={}, json_column=None):
         """
         Returns a database query that fully describes the filters.
 
@@ -424,7 +429,7 @@ class FilterEngine:
                 key, oper, value = and_group
                 if isinstance(key, sqlalchemy.orm.attributes.InstrumentedAttribute):                # -> this key filters on a table column.
                     if isinstance(value, str) and any([char in value for char in ['*', '%']]):      # wildcards
-                        if value in ('*', '%', u'*', u'%'):                                         # match wildcard exactly == no filtering on key
+                        if value in ('*', '%', '*', '%'):                                           # match wildcard exactly == no filtering on key
                             continue
                         else:                                                                       # partial match with wildcard == like || notlike
                             if oper == operator.eq:
@@ -438,7 +443,7 @@ class FilterEngine:
                 elif json_column:                                                                   # -> this key filters on the content of a json column
                     if session.bind.dialect.name == 'oracle':
                         if isinstance(value, str) and any([char in value for char in ['*', '%']]):  # wildcards
-                            if value in ('*', '%', u'*', u'%'):                                     # match wildcard exactly == no filtering on key
+                            if value in ('*', '%', '*', '%'):                                       # match wildcard exactly == no filtering on key
                                 continue
                             else:                                                                   # partial match with wildcard == like || notlike
                                 if oper == operator.eq:
@@ -457,7 +462,7 @@ class FilterEngine:
                                 raise exception.FilterEngineGenericError(e)
                     else:
                         if isinstance(value, str) and any([char in value for char in ['*', '%']]):  # wildcards
-                            if value in ('*', '%', u'*', u'%'):                                     # match wildcard exactly == no filtering on key
+                            if value in ('*', '%', '*', '%'):                                       # match wildcard exactly == no filtering on key
                                 continue
                             else:                                                                   # partial match with wildcard == like || notlike
                                 if oper == operator.eq:
