@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import operator
+from typing import TYPE_CHECKING
 
 import pymongo
 
@@ -23,6 +24,10 @@ from rucio.common.types import InternalScope
 from rucio.core.did_meta_plugins.did_meta_plugin_interface import DidMetaPlugin
 from rucio.core.did_meta_plugins.filter_engine import FilterEngine
 
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
+
 IMMUTABLE_KEYS = [
     '_id',              # index key
     'scope',            # generated on insert
@@ -37,7 +42,7 @@ class MongoDidMeta(DidMetaPlugin):
         if host is None:
             host = config.config_get('metadata', 'mongo_service_host')
         if port is None:
-            port = config.config_get('metadata', 'mongo_service_port')
+            port = config.config_get_int('metadata', 'mongo_service_port')
         if db is None:
             db = config.config_get('metadata', 'mongo_db')
         if collection is None:
@@ -52,7 +57,7 @@ class MongoDidMeta(DidMetaPlugin):
     def drop_database(self):
         self.client.drop_database(self.db.name)
 
-    def get_metadata(self, scope, name, session=None):
+    def get_metadata(self, scope, name, *, session: "Optional[Session]" = None):
         """
         Get data identifier metadata.
 
@@ -72,10 +77,10 @@ class MongoDidMeta(DidMetaPlugin):
                 doc.pop(key)
 
         if not doc:
-            raise exception.DataIdentifierNotFound("No metadata found for did '%(scope)s:%(name)s'" % locals())
+            raise exception.DataIdentifierNotFound(f"No metadata found for did '{scope}:{name}'")
         return doc
 
-    def set_metadata(self, scope, name, key, value, recursive=False, session=None):
+    def set_metadata(self, scope, name, key, value, recursive=False, *, session: "Optional[Session]" = None):
         """
         Set single metadata key.
 
@@ -86,22 +91,22 @@ class MongoDidMeta(DidMetaPlugin):
         :param recursive: recurse into DIDs (not supported)
         :param session: The database session in use
         """
-        self.set_metadata_bulk(scope=scope, name=name, meta={key: value}, recursive=recursive, session=session)
+        self.set_metadata_bulk(scope=scope, name=name, metadata={key: value}, recursive=recursive, session=session)
 
-    def set_metadata_bulk(self, scope, name, meta, recursive=False, session=None):
+    def set_metadata_bulk(self, scope, name, metadata, recursive=False, *, session: "Optional[Session]" = None):
         """
         Bulk set metadata keys.
 
         :param scope: the scope of did
         :param name: the name of the did
-        :param meta: dictionary of metadata keypairs to be added
+        :param metadata: dictionary of metadata keypairs to be added
         :param recursive: recurse into DIDs (not supported)
         :param session: The database session in use
         """
         # pop immutable keys
         for key in IMMUTABLE_KEYS:
-            if key in meta:
-                meta.pop(key)
+            if key in metadata:
+                metadata.pop(key)
 
         # set first document with did == _id
         self.col.update_one(
@@ -109,7 +114,7 @@ class MongoDidMeta(DidMetaPlugin):
                 "_id": "{}:{}".format(scope.internal, name)
             },
             {
-                '$set': meta,
+                '$set': metadata,
                 '$setOnInsert': {
                     'scope': "{}".format(scope.external),
                     'vo': "{}".format(scope.vo),
@@ -119,7 +124,7 @@ class MongoDidMeta(DidMetaPlugin):
             upsert=True
         )
 
-    def delete_metadata(self, scope, name, key, session=None):
+    def delete_metadata(self, scope, name, key, *, session: "Optional[Session]" = None):
         """
         Delete a key from metadata.
 
@@ -134,7 +139,7 @@ class MongoDidMeta(DidMetaPlugin):
             raise exception.DataIdentifierNotFound(e)
 
     def list_dids(self, scope, filters, did_type='collection', ignore_case=False, limit=None,
-                  offset=None, long=False, recursive=False, ignore_dids=None, session=None):
+                  offset=None, long=False, recursive=False, ignore_dids=None, *, session: "Optional[Session]" = None):
         if not ignore_dids:
             ignore_dids = set()
 
@@ -182,7 +187,7 @@ class MongoDidMeta(DidMetaPlugin):
                     ignore_dids.add(did_full)
                     yield did['name']
 
-    def manages_key(self, key, session=None):
+    def manages_key(self, key, *, session: "Optional[Session]" = None):
         return True
 
     def get_plugin_name(self):
