@@ -13,17 +13,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING
+
+from rucio.common.config import config_get
 from rucio.common.exception import RSEOperationNotSupported
 from rucio.common.types import InternalAccount
 from rucio.core import rse as rse_module, distance as distance_module, account as account_module, identity as identity_module
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import RSEType, AccountType, IdentityType
 from rucio.db.sqla.session import transactional_session
-from rucio.common.config import config_get
+
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
 
 
 @transactional_session
-def import_rses(rses, rse_sync_method='edit', attr_sync_method='edit', protocol_sync_method='edit', vo='def', session=None):
+def import_rses(rses, rse_sync_method='edit', attr_sync_method='edit', protocol_sync_method='edit', vo='def', *, session: "Session"):
     new_rses = []
     for rse_name in rses:
         rse = rses[rse_name]
@@ -51,7 +56,8 @@ def import_rses(rses, rse_sync_method='edit', attr_sync_method='edit', protocol_
                                         city=rse.get('city'), region_code=rse.get('region_code'), country_name=rse.get('country_name'),
                                         staging_area=rse.get('staging_area'), continent=rse.get('continent'), time_zone=rse.get('time_zone'),
                                         ISP=rse.get('ISP'), rse_type=rse.get('rse_type'), latitude=rse.get('latitude'),
-                                        longitude=rse.get('longitude'), ASN=rse.get('ASN'), availability=rse.get('availability'),
+                                        longitude=rse.get('longitude'), ASN=rse.get('ASN'), availability_read=rse.get('availability_read'),
+                                        availability_write=rse.get('availability_write'), availability_delete=rse.get('availability_delete'),
                                         session=session)
 
         new_rses.append(rse_id)
@@ -89,7 +95,7 @@ def import_rses(rses, rse_sync_method='edit', attr_sync_method='edit', protocol_
 
         # Limits
         old_limits = rse_module.get_rse_limits(rse_id=rse_id, session=session)
-        for limit_name in ['MaxBeingDeletedFiles', 'MinFreeSpace']:
+        for limit_name in ['MinFreeSpace']:
             limit = rse.get(limit_name)
             if limit:
                 if limit_name in old_limits:
@@ -130,30 +136,27 @@ def import_rses(rses, rse_sync_method='edit', attr_sync_method='edit', protocol_
 
 
 @transactional_session
-def import_distances(distances, vo='def', session=None):
+def import_distances(distances, vo='def', *, session: "Session"):
     for src_rse_name in distances:
         src = rse_module.get_rse_id(rse=src_rse_name, vo=vo, session=session)
         for dest_rse_name in distances[src_rse_name]:
             dest = rse_module.get_rse_id(rse=dest_rse_name, vo=vo, session=session)
-            distance = distances[src_rse_name][dest_rse_name]
-            if 'src_rse_id' in distance:
-                del distance['src_rse_id']
-            if 'dest_rse_id' in distance:
-                del distance['dest_rse_id']
+            distance_dict = distances[src_rse_name][dest_rse_name]
+            if 'src_rse_id' in distance_dict:
+                del distance_dict['src_rse_id']
+            if 'dest_rse_id' in distance_dict:
+                del distance_dict['dest_rse_id']
 
             old_distance = distance_module.get_distances(src_rse_id=src, dest_rse_id=dest, session=session)
+            new_distance = distance_dict.get('distance', distance_dict.get('ranking'))
             if old_distance:
-                distance_module.update_distances(src_rse_id=src, dest_rse_id=dest, parameters=distance, session=session)
+                distance_module.update_distances(src_rse_id=src, dest_rse_id=dest, distance=new_distance, session=session)
             else:
-                distance_module.add_distance(src_rse_id=src, dest_rse_id=dest, ranking=distance.get('ranking'),
-                                             agis_distance=distance.get('agis_distance'), geoip_distance=distance.get('geoip_distance'),
-                                             active=distance.get('active'), submitted=distance.get('submitted'),
-                                             transfer_speed=distance.get('transfer_speed'), finished=distance.get('finished'),
-                                             failed=distance.get('failed'), session=session)
+                distance_module.add_distance(src_rse_id=src, dest_rse_id=dest, distance=new_distance, session=session)
 
 
 @transactional_session
-def import_identities(identities, account_name, old_identities, old_identity_account, account_email, session=None):
+def import_identities(identities, account_name, old_identities, old_identity_account, account_email, *, session: "Session"):
     for identity in identities:
         identity['type'] = IdentityType[identity['type'].upper()]
 
@@ -182,7 +185,7 @@ def import_identities(identities, account_name, old_identities, old_identity_acc
 
 
 @transactional_session
-def import_accounts(accounts, vo='def', session=None):
+def import_accounts(accounts, vo='def', *, session: "Session"):
     vo_filter = {'account': InternalAccount(account='*', vo=vo)}
     old_accounts = {account['account']: account for account in account_module.list_accounts(filter_=vo_filter, session=session)}
     missing_accounts = [account for account in accounts if account['account'] not in old_accounts]
@@ -219,7 +222,7 @@ def import_accounts(accounts, vo='def', session=None):
 
 
 @transactional_session
-def import_data(data, vo='def', session=None):
+def import_data(data, vo='def', *, session: "Session"):
     """
     Import data to add and update records in Rucio.
 
