@@ -14,14 +14,13 @@
 # limitations under the License.
 
 import json
-
 from typing import TYPE_CHECKING
 
-from sqlalchemy import or_, delete, update
+from sqlalchemy import or_, delete, update, insert
 from sqlalchemy.exc import IntegrityError
 
-from rucio.common.constants import HermesService, MAX_MESSAGE_LENGTH
 from rucio.common.config import config_get_list
+from rucio.common.constants import HermesService, MAX_MESSAGE_LENGTH
 from rucio.common.exception import InvalidObject, RucioException
 from rucio.common.utils import APIEncoder, chunks
 from rucio.db.sqla import filter_thread_work
@@ -29,15 +28,15 @@ from rucio.db.sqla.models import Message, MessageHistory
 from rucio.db.sqla.session import transactional_session
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, List, Optional
+    from typing import Any, Optional
     from sqlalchemy.orm import Session
 
-    MessageType = Dict[str, Any]
-    MessagesListType = List[MessageType]
+    MessageType = dict[str, Any]
+    MessagesListType = list[MessageType]
 
 
 @transactional_session
-def add_messages(messages: "MessagesListType", session: "Optional[Session]" = None) -> None:
+def add_messages(messages: "MessagesListType", *, session: "Session") -> None:
     """
     Add a list of messages to be submitted asynchronously to a message broker.
 
@@ -73,13 +72,13 @@ def add_messages(messages: "MessagesListType", session: "Optional[Session]" = No
                     msg['payload'] = 'nolimit'
                 msgs.append(msg)
             except TypeError as err:  # noqa: F841
-                raise InvalidObject('Invalid JSON for payload: %(err)s' % locals())
+                raise InvalidObject(f'Invalid JSON for payload: {err}')
     for messages_chunk in chunks(msgs, 1000):
-        session.bulk_insert_mappings(Message, messages_chunk)
+        session.execute(insert(Message), messages_chunk)
 
 
 @transactional_session
-def add_message(event_type: str, payload: dict, session: "Optional[Session]" = None) -> None:
+def add_message(event_type: str, payload: dict, *, session: "Session") -> None:
     """
     Add a message to be submitted asynchronously to a message broker.
 
@@ -99,7 +98,7 @@ def retrieve_messages(bulk: int = 1000,
                       event_type: "Optional[str]" = None,
                       lock: bool = False,
                       old_mode: bool = True,
-                      session: "Optional[Session]" = None) -> "MessagesListType":
+                      *, session: "Session") -> "MessagesListType":
     """
     Retrieve up to $bulk messages.
 
@@ -175,7 +174,7 @@ def retrieve_messages(bulk: int = 1000,
 
 
 @transactional_session
-def delete_messages(messages: "MessagesListType", session: "Optional[Session]" = None) -> None:
+def delete_messages(messages: "MessagesListType", *, session: "Session") -> None:
     """
     Delete all messages with the given IDs, and archive them to the history.
 
@@ -195,13 +194,13 @@ def delete_messages(messages: "MessagesListType", session: "Optional[Session]" =
                 execution_options(synchronize_session=False)
             session.execute(stmt)
 
-            session.bulk_insert_mappings(MessageHistory, messages)
+            session.execute(insert(MessageHistory), messages)
     except IntegrityError as e:
         raise RucioException(e.args)
 
 
 @transactional_session
-def truncate_messages(session: "Optional[Session]" = None) -> None:
+def truncate_messages(*, session: "Session") -> None:
     """
     Delete all stored messages. This is for internal purposes only.
 
@@ -215,7 +214,7 @@ def truncate_messages(session: "Optional[Session]" = None) -> None:
 
 
 @transactional_session
-def update_messages_services(messages: "MessagesListType", services: str, session: "Optional[Session]" = None) -> None:
+def update_messages_services(messages: "MessagesListType", services: str, *, session: "Session") -> None:
     """
     Update the services for all messages with the given IDs.
 
