@@ -14,6 +14,7 @@
 # limitations under the License.
 
 from re import match
+from typing import TYPE_CHECKING
 
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm.exc import NoResultFound
@@ -26,9 +27,12 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType, KeyType
 from rucio.db.sqla.session import read_session, transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 @transactional_session
-def add_key(key, key_type, value_type=None, value_regexp=None, session=None):
+def add_key(key, key_type, value_type=None, value_regexp=None, *, session: "Session"):
     """
     Adds a new allowed key.
 
@@ -41,7 +45,7 @@ def add_key(key, key_type, value_type=None, value_regexp=None, session=None):
 
     # Check if value_type is supported
     if value_type and value_type not in [str(t) for t in AUTHORIZED_VALUE_TYPES]:
-        raise UnsupportedValueType('The type \'%(value_type)s\' is not supported for values!' % locals())
+        raise UnsupportedValueType(f"The type '{value_type}' is not supported for values!")
 
     # Convert key_type
     if isinstance(key_type, str):
@@ -72,12 +76,12 @@ def add_key(key, key_type, value_type=None, value_regexp=None, session=None):
            or match('.*IntegrityError.*duplicate key value violates unique constraint.*', error.args[0]) \
            or match('.*UniqueViolation.*duplicate key value violates unique constraint.*', error.args[0]) \
            or match('.*IntegrityError.*columns? key.*not unique.*', error.args[0]):
-            raise Duplicate('key \'%(key)s\' already exists!' % locals())
+            raise Duplicate(f"key '{key}' already exists!")
         raise
 
 
 @transactional_session
-def del_key(key, session=None):
+def del_key(key, *, session: "Session"):
     """
     Deletes a key.
 
@@ -88,7 +92,7 @@ def del_key(key, session=None):
 
 
 @read_session
-def list_keys(session=None):
+def list_keys(*, session: "Session"):
     """
     Lists all keys.
 
@@ -104,7 +108,7 @@ def list_keys(session=None):
 
 
 @transactional_session
-def add_value(key, value, session=None):
+def add_value(key, value, *, session: "Session"):
     """
     Adds a new value to a key.
 
@@ -123,13 +127,13 @@ def add_value(key, value, session=None):
            or match('.*IntegrityError.*duplicate key value violates unique constraint.*', error.args[0]) \
            or match('.*UniqueViolation.*duplicate key value violates unique constraint.*', error.args[0]) \
            or match('.*IntegrityError.*columns? key.*value.*not unique.*', error.args[0]):
-            raise Duplicate('key-value \'%(key)s-%(value)s\' already exists!' % locals())
+            raise Duplicate(f"key-value '{key}-{value}' already exists!")
         if match('.*IntegrityError.*foreign key constraints? failed.*', error.args[0]):
-            raise KeyNotFound("key '%(key)s' does not exist!" % locals())
+            raise KeyNotFound(f"key '{key}' does not exist!")
         if match('.*IntegrityError.*ORA-02291: integrity constraint.*DID_MAP_KEYS_FK.*violated.*', error.args[0]):
-            raise KeyNotFound("key '%(key)s' does not exist!" % locals())
+            raise KeyNotFound(f"key '{key}' does not exist!")
         if error.args[0] == "(IntegrityError) (1452, 'Cannot add or update a child row: a foreign key constraint fails (`rucio`.`did_key_map`, CONSTRAINT `DID_MAP_KEYS_FK` FOREIGN KEY (`key`) REFERENCES `did_keys` (`key`))')":
-            raise KeyNotFound("key '%(key)s' does not exist!" % locals())
+            raise KeyNotFound(f"key '{key}' does not exist!")
 
         raise RucioException(error.args)
 
@@ -146,7 +150,7 @@ def add_value(key, value, session=None):
 
 
 @read_session
-def list_values(key, session=None):
+def list_values(key, *, session: "Session"):
     """
     Lists all values for a key.
 
@@ -163,7 +167,7 @@ def list_values(key, session=None):
 
 
 @read_session
-def validate_meta(meta, did_type, session=None):
+def validate_meta(meta, did_type, *, session: "Session"):
     """
     Validates metadata for a did.
 
