@@ -14,17 +14,17 @@
 # limitations under the License.
 
 import json
+import logging
 import random
 import subprocess
-import logging
 import traceback
 from datetime import datetime, timedelta
-from math import floor
+from typing import Any, TYPE_CHECKING, Optional
 from urllib.parse import urlparse, parse_qs
-from typing import Dict, Tuple, Any
 
 from jwkest.jws import JWS
 from jwkest.jwt import JWT
+from math import floor
 from oic import rndstr
 from oic.oauth2.message import CCAccessTokenRequest
 from oic.oic import Client, Grant, Token, REQUEST2ENDPOINT
@@ -32,22 +32,28 @@ from oic.oic.message import (AccessTokenResponse, AuthorizationResponse,
                              Message, RegistrationResponse)
 from oic.utils import time_util
 from oic.utils.authn.client import CLIENT_AUTHN_METHOD
-from sqlalchemy import and_
+from sqlalchemy import delete, select, update
 from sqlalchemy.sql.expression import true
 
+from rucio.common import types
 from rucio.common.config import config_get, config_get_int
 from rucio.common.exception import (CannotAuthenticate, CannotAuthorize,
                                     RucioException)
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.utils import all_oidc_req_claims_present, build_url, val_to_space_sep_str
-from rucio.common import types
 from rucio.core.account import account_exists
 from rucio.core.identity import exist_identity_account, get_default_account
-from rucio.core.monitor import record_counter, Timer
+from rucio.core.monitor import MetricManager
 from rucio.db.sqla import filter_thread_work
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import IdentityType
 from rucio.db.sqla.session import read_session, transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
+
+METRICS = MetricManager(module=__name__)
 
 # worokaround for a bug in pyoidc (as of Dec 2019)
 REQUEST2ENDPOINT['CCAccessTokenRequest'] = 'token_endpoint'
@@ -69,7 +75,7 @@ LEEWAY_SECS = 120
 # --> check 'profile' info (requested profile scope)
 
 
-def __get_rucio_oidc_clients(keytimeout: int = 43200) -> Tuple[Dict, Dict]:
+def __get_rucio_oidc_clients(keytimeout: int = 43200) -> tuple[dict, dict]:
     """
     Creates a Rucio OIDC Client instances per Identity Provider (IdP)
     according to etc/idpsecrets.json configuration file.
@@ -141,7 +147,7 @@ def __initialize_oidc_clients() -> None:
 __initialize_oidc_clients()
 
 
-def __get_init_oidc_client(token_object: models.Token = None, token_type: str = None, **kwargs) -> Dict[Any, Any]:
+def __get_init_oidc_client(token_object: models.Token = None, token_type: str = None, **kwargs) -> dict[Any, Any]:
     """
     Get an OIDC client object, (re-)initialised with parameters corresponding
     to authorization flows used to get a token. For special cases - token refresh,
@@ -236,7 +242,7 @@ def __get_init_oidc_client(token_object: models.Token = None, token_type: str =
 
 
 @transactional_session
-def get_auth_oidc(account: str, session=None, **kwargs) -> str:
+def get_auth_oidc(account: str, *, session: "Session", **kwargs) -> str:
     """
     Assembles the authorization request of the Rucio Client tailored to the Rucio user
     & Identity Provider. Saves authentication session parameters in the oauth_requests
@@ -298,7 +304,7 @@ def get_auth_oidc(account: str, session=None, **kwargs) -> str:
             return None
 
     try:
-        timer = Timer()
+        stopwatch = Stopwatch()
         # redirect_url needs to be specified & one of those defined
         # in the Rucio OIDC Client configuration
         redirect_to = "auth/oidc_code"
@@ -346,7 +352,7 @@ def get_auth_oidc(account: str, session=None, **kwargs) -> str:
             auth_url = build_url('https://' + auth_server.netloc, path='{}auth/oidc_redirect'.format(
                 auth_server.path.split('auth/')[0].lstrip('/')), params=access_msg)
 
-        timer.record('IdP_authentication.request')
+        METRICS.timer('IdP_authentication.request').observe(stopwatch.elapsed)
         return auth_url
 
     except Exception as error:
@@ -354,7 +360,7 @@ def get_auth_oidc(account: str, session=None, **kwargs) -> str:
 
 
 @transactional_session
-def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
+def get_token_oidc(auth_query_string: str, ip: str = None, *, session: "Session"):
     """
     After Rucio User got redirected to Rucio /auth/oidc_token (or /auth/oidc_code)
     REST endpoints with authz code and session state encoded within the URL.
@@ -369,23 +375,29 @@ def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
               (no auto, auto, polling).
     """
     try:
-        timer = Timer()
+        stopwatch = Stopwatch()
         parsed_authquery = parse_qs(auth_query_string)
         state = parsed_authquery["state"][0]
         code = parsed_authquery["code"][0]
         # getting oauth request params from the oauth_requests DB Table
-        oauth_req_params = session.query(models.OAuthRequest).filter_by(state=state).first()
+        query = select(
+            models.OAuthRequest
+        ).where(
+            models.OAuthRequest.state == state
+        )
+        oauth_req_params = session.execute(query).scalar()
         if oauth_req_params is None:
             raise CannotAuthenticate("User related Rucio OIDC session could not keep "
                                      + "track of responses from outstanding requests.")  # NOQA: W503
-        req_url = urlparse(oauth_req_params.redirect_msg)
+        req_url = urlparse(oauth_req_params.redirect_msg or '')
         issuer = req_url.scheme + "://" + req_url.netloc
         req_params = parse_qs(req_url.query)
-        for key in req_params:
-            req_params[key] = val_to_space_sep_str(req_params[key])
+        client_params = {}
+        for key in list(req_params):
+            client_params[key] = val_to_space_sep_str(req_params[key])
 
-        oidc_client = __get_init_oidc_client(issuer=issuer, code=code, **req_params)['client']
-        record_counter(name='IdP_authentication.code_granted')
+        oidc_client = __get_init_oidc_client(issuer=issuer, code=code, **client_params)['client']
+        METRICS.counter(name='IdP_authentication.code_granted').inc()
         # exchange access code for a access token
         oidc_tokens = oidc_client.do_access_token_request(state=state,
                                                           request_args={"code": code},
@@ -415,7 +427,7 @@ def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
         if not exist_identity_account(jwt_row_dict['identity'], IdentityType.OIDC, jwt_row_dict['account'], session=session):
             raise CannotAuthenticate("OIDC identity '%s' of the '%s' account is unknown to Rucio."
                                      % (jwt_row_dict['identity'], str(jwt_row_dict['account'])))
-        record_counter(name='IdP_authentication.success')
+        METRICS.counter(name='success').inc()
         # get access token expiry timestamp
         jwt_row_dict['lifetime'] = datetime.utcnow() + timedelta(seconds=oidc_tokens['expires_in'])
         # get audience and scope info from the token
@@ -443,10 +455,12 @@ def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
         if 'refresh_token' in oidc_tokens:
             extra_dict['refresh_token'] = oidc_tokens['refresh_token']
             extra_dict['refresh'] = True
+            extra_dict['refresh_lifetime'] = REFRESH_LIFETIME_H
             try:
-                extra_dict['refresh_lifetime'] = int(oauth_req_params.refresh_lifetime)
+                if oauth_req_params.refresh_lifetime is not None:
+                    extra_dict['refresh_lifetime'] = int(oauth_req_params.refresh_lifetime)
             except Exception:
-                extra_dict['refresh_lifetime'] = REFRESH_LIFETIME_H
+                pass
             try:
                 values = __get_keyvalues_from_claims(oidc_tokens['refresh_token'], ['exp'])
                 exp = values['exp']
@@ -456,9 +470,9 @@ def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
                 extra_dict['refresh_expired_at'] = datetime.utcnow() + timedelta(hours=REFRESH_LIFETIME_H)
 
         new_token = __save_validated_token(oidc_tokens['access_token'], jwt_row_dict, extra_dict=extra_dict, session=session)
-        record_counter(name='IdP_authorization.access_token.saved')
+        METRICS.counter(name='IdP_authorization.access_token.saved').inc()
         if 'refresh_token' in oidc_tokens:
-            record_counter(name='IdP_authorization.refresh_token.saved')
+            METRICS.counter(name='IdP_authorization.refresh_token.saved').inc()
         # In case authentication via browser was requested,
         # we save the token in the oauth_requests table
         if oauth_req_params.access_msg:
@@ -466,17 +480,28 @@ def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
             if 'http' not in oauth_req_params.access_msg:
                 if '_polling' not in oauth_req_params.access_msg:
                     fetchcode = rndstr(50)
-                    session.query(models.OAuthRequest).filter(models.OAuthRequest.state == state)\
-                           .update({models.OAuthRequest.access_msg: fetchcode,
-                                    models.OAuthRequest.redirect_msg: new_token['token']})
+                    query = update(
+                        models.OAuthRequest
+                    ).where(
+                        models.OAuthRequest.state == state
+                    ).values({
+                        models.OAuthRequest.access_msg: fetchcode,
+                        models.OAuthRequest.redirect_msg: new_token['token']
+                    })
                 # If Rucio Client was requested to poll the Rucio Auth server
                 # for a token automatically, we save the token under a access_msg.
                 else:
-                    session.query(models.OAuthRequest).filter(models.OAuthRequest.state == state)\
-                           .update({models.OAuthRequest.access_msg: oauth_req_params.access_msg,
-                                    models.OAuthRequest.redirect_msg: new_token['token']})
+                    query = update(
+                        models.OAuthRequest
+                    ).where(
+                        models.OAuthRequest.state == state
+                    ).values({
+                        models.OAuthRequest.access_msg: oauth_req_params.access_msg,
+                        models.OAuthRequest.redirect_msg: new_token['token']
+                    })
+                session.execute(query)
                 session.commit()
-            timer.record('IdP_authorization')
+            METRICS.timer('IdP_authorization').observe(stopwatch.elapsed)
             if '_polling' in oauth_req_params.access_msg:
                 return {'polling': True}
             elif 'http' in oauth_req_params.access_msg:
@@ -484,19 +509,19 @@ def get_token_oidc(auth_query_string: str, ip: str = None, session=None):
             else:
                 return {'fetchcode': fetchcode}
         else:
-            timer.record('IdP_authorization')
+            METRICS.timer('IdP_authorization').observe(stopwatch.elapsed)
             return {'token': new_token}
 
     except Exception:
         # TO-DO catch different exceptions - InvalidGrant etc. ...
-        record_counter(name='IdP_authorization.access_token.exception')
+        METRICS.counter(name='IdP_authorization.access_token.exception').inc()
         logging.debug(traceback.format_exc())
         return None
         # raise CannotAuthenticate(traceback.format_exc())
 
 
 @transactional_session
-def __get_admin_token_oidc(account: types.InternalAccount, req_scope, req_audience, issuer, session=None):
+def __get_admin_token_oidc(account: types.InternalAccount, req_scope, req_audience, issuer, *, session: "Session"):
     """
     Get a token for Rucio application to act on behalf of itself.
     client_credential flow is used for this purpose.
@@ -530,14 +555,14 @@ def __get_admin_token_oidc(account: types.InternalAccount, req_scope, req_audien
                                          response=AccessTokenResponse)
         if 'error' in oidc_tokens:
             raise CannotAuthorize(oidc_tokens['error'])
-        record_counter(name='IdP_authentication.rucio_admin_token_granted')
+        METRICS.counter(name='IdP_authentication.rucio_admin_token_granted').inc()
         # save the access token in the Rucio DB
         if 'access_token' in oidc_tokens:
             validate_dict = __get_rucio_jwt_dict(oidc_tokens['access_token'], account=account, session=session)
             if validate_dict:
-                record_counter(name='IdP_authentication.success')
+                METRICS.counter(name='IdP_authentication.success').inc()
                 new_token = __save_validated_token(oidc_tokens['access_token'], validate_dict, extra_dict={}, session=session)
-                record_counter(name='IdP_authorization.access_token.saved')
+                METRICS.counter(name='IdP_authentication.access_token.saved').inc()
                 return new_token
             else:
                 logging.debug("Rucio could not get a valid admin token from the Identity Provider.")
@@ -548,14 +573,14 @@ def __get_admin_token_oidc(account: types.InternalAccount, req_scope, req_audien
 
     except Exception:
         # TO-DO catch different exceptions - InvalidGrant etc. ...
-        record_counter(name='IdP_authorization.access_token.exception')
+        METRICS.counter(name='IdP_authorization.access_token.exception').inc()
         logging.debug(traceback.format_exc())
         return None
         # raise CannotAuthenticate(traceback.format_exc())
 
 
 @read_session
-def __get_admin_account_for_issuer(session=None):
+def __get_admin_account_for_issuer(*, session: "Session"):
     """ Gets admin account for the IdP issuer
     :returns : dictionary { 'issuer_1': (account, identity), ... }
     """
@@ -569,14 +594,19 @@ def __get_admin_account_for_issuer(session=None):
     issuer_account_dict = {}
     for issuer in OIDC_ADMIN_CLIENTS:
         admin_identity = oidc_identity_string(OIDC_ADMIN_CLIENTS[issuer].client_id, issuer)
-        admin_account = session.query(models.IdentityAccountAssociation)\
-                               .filter_by(identity_type=IdentityType.OIDC, identity=admin_identity).first()
-        issuer_account_dict[issuer] = (admin_account.account, admin_identity)
+        query = select(
+            models.IdentityAccountAssociation.account
+        ).where(
+            models.IdentityAccountAssociation.identity_type == IdentityType.OIDC,
+            models.IdentityAccountAssociation.identity == admin_identity
+        )
+        admin_account = session.execute(query).scalar()
+        issuer_account_dict[issuer] = (admin_account, admin_identity)
     return issuer_account_dict
 
 
 @transactional_session
-def get_token_for_account_operation(account: str, req_audience: str = None, req_scope: str = None, admin: bool = False, session=None):
+def get_token_for_account_operation(account: str, req_audience: str = None, req_scope: str = None, admin: bool = False, *, session: "Session"):
     """
     Looks-up a JWT token with the required scope and audience claims with the account OIDC issuer.
     If tokens are found, and none contains the requested audience and scope a new token is requested
@@ -597,16 +627,24 @@ def get_token_for_account_operation(account: str, req_audience: str = None, req_
             req_audience = EXPECTED_OIDC_AUDIENCE
 
         # get all identities for the corresponding account
-        identities_list = session.query(models.IdentityAccountAssociation.identity) \
-                                 .filter(models.IdentityAccountAssociation.identity_type == IdentityType.OIDC,
-                                         models.IdentityAccountAssociation.account == account).all()
-        identities = []
-        for identity in identities_list:
-            identities.append(identity[0])
+        query = select(
+            models.IdentityAccountAssociation.identity
+        ).where(
+            models.IdentityAccountAssociation.identity_type == IdentityType.OIDC,
+            models.IdentityAccountAssociation.account == account
+        )
+        identities = session.execute(query).scalars().all()
         # get all active/valid OIDC tokens
-        account_tokens = session.query(models.Token).filter(models.Token.identity.in_(identities),
-                                                            models.Token.account == account,
-                                                            models.Token.expired_at > datetime.utcnow()).with_for_update(skip_locked=True).all()
+        query = select(
+            models.Token
+        ).where(
+            models.Token.identity.in_(identities),
+            models.Token.account == account,
+            models.Token.expired_at > datetime.utcnow()
+        ).with_for_update(
+            skip_locked=True
+        )
+        account_tokens = session.execute(query).scalars().all()
 
         # for Rucio Admin account we ask IdP for a token via client_credential grant
         # for each user account OIDC identity there is an OIDC issuer that must be, by construction,
@@ -644,8 +682,13 @@ def get_token_for_account_operation(account: str, req_audience: str = None, req_
             if 'openid' in req_scope:
                 req_scope = req_scope.replace("openid", "").strip()
             # checking if there is not already a token to use
-            admin_account_tokens = session.query(models.Token).filter(models.Token.account == account,
-                                                                      models.Token.expired_at > datetime.utcnow()).all()
+            query = select(
+                models.Token
+            ).where(
+                models.Token.account == account,
+                models.Token.expired_at > datetime.utcnow()
+            )
+            admin_account_tokens = session.execute(query).scalars().all()
             for admin_token in admin_account_tokens:
                 if hasattr(admin_token, 'audience') and hasattr(admin_token, 'oidc_scope') and\
                    all_oidc_req_claims_present(admin_token.oidc_scope, admin_token.audience, req_scope, req_audience):
@@ -682,9 +725,14 @@ def get_token_for_account_operation(account: str, req_audience: str = None, req_
                 admin_acc_idt_tuple = admin_iss_acc_idt_dict[admin_issuer]
                 admin_account = admin_acc_idt_tuple[0]
                 admin_identity = admin_acc_idt_tuple[1]
-                admin_account_tokens = session.query(models.Token).filter(models.Token.identity == admin_identity,
-                                                                          models.Token.account == admin_account,
-                                                                          models.Token.expired_at > datetime.utcnow()).all()
+                query = select(
+                    models.Token
+                ).where(
+                    models.Token.identity == admin_identity,
+                    models.Token.account == admin_account,
+                    models.Token.expired_at > datetime.utcnow()
+                )
+                admin_account_tokens = session.execute(query).scalars().all()
                 for admin_token in admin_account_tokens:
                     if hasattr(admin_token, 'audience') and hasattr(admin_token, 'oidc_scope') and\
                        all_oidc_req_claims_present(admin_token.oidc_scope, admin_token.audience, req_scope, req_audience):
@@ -734,8 +782,9 @@ def get_token_for_account_operation(account: str, req_audience: str = None, req_
         return None
 
 
+@METRICS.time_it
 @transactional_session
-def __exchange_token_oidc(subject_token_object: models.Token, session=None, **kwargs):
+def __exchange_token_oidc(subject_token_object: models.Token, *, session: "Session", **kwargs):
     """
     Exchanged an access_token for a new one with different scope &/ audience
     providing that the scope specified is registered with IdP for the Rucio OIDC Client
@@ -764,9 +813,6 @@ def __exchange_token_oidc(subject_token_object: models.Token, session=None, **kw
     if not grant_type:
         grant_type = EXCHANGE_GRANT_TYPE
     try:
-        timer = Timer()
-
-        record_counter(name='IdP_authentication.code_granted')
         oidc_dict = __get_init_oidc_client(token_object=subject_token_object, token_type="subject_token")
         oidc_client = oidc_dict['client']
         args = {"subject_token": subject_token_object.token,
@@ -805,10 +851,9 @@ def __exchange_token_oidc(subject_token_object: models.Token, session=None, **kw
                 extra_dict['refresh_expired_at'] = datetime.utcnow() + timedelta(hours=REFRESH_LIFETIME_H)
 
         new_token = __save_validated_token(oidc_tokens['access_token'], jwt_row_dict, extra_dict=extra_dict, session=session)
-        record_counter(name='IdP_authorization.access_token.saved')
+        METRICS.counter(name='IdP_authorization.access_token.saved').inc()
         if 'refresh_token' in oidc_tokens:
-            record_counter(name='IdP_authorization.refresh_token.saved')
-        timer.record('IdP_authorization.token_exchange')
+            METRICS.counter(name='IdP_authorization.refresh_token.saved').inc()
         return new_token
 
     except Exception:
@@ -818,28 +863,35 @@ def __exchange_token_oidc(subject_token_object: models.Token, session=None, **kw
 
 
 @transactional_session
-def __change_refresh_state(token: str, refresh: bool = False, session=None):
+def __change_refresh_state(token: str, refresh: bool = False, *, session: "Session"):
     """
     Changes token refresh state to True/False.
 
     :param token:      the access token for which the refresh value should be changed.
     """
     try:
+        query = update(
+            models.Token
+        ).where(
+            models.Token.token == token
+        )
         if refresh:
             # update refresh column for a token to True
-            session.query(models.Token).filter(models.Token.token == token)\
-                                       .update({models.Token.refresh: True})
+            query = query.values({
+                models.Token.refresh: True
+            })
         else:
-            session.query(models.Token).filter(models.Token.token == token)\
-                                       .update({models.Token.refresh: False,
-                                                models.Token.refresh_expired_at: datetime.utcnow()})
-        session.commit()
+            query = query.values({
+                models.Token.refresh: False,
+                models.Token.refresh_expired_at: datetime.utcnow()
+            })
+        session.execute(query)
     except Exception as error:
         raise RucioException(error.args) from error
 
 
 @transactional_session
-def refresh_cli_auth_token(token_string: str, account: str, session=None):
+def refresh_cli_auth_token(token_string: str, account: str, *, session: "Session"):
     """
     Checks if there is active refresh token and if so returns
     either active token with expiration timestamp or requests a new
@@ -850,11 +902,16 @@ def refresh_cli_auth_token(token_string: str, account: str, session=None):
     :return: tuple of (access token, expiration epoch), None otherswise
     """
     # only validated tokens are in the DB, check presence of token_string
-    account_token = session.query(models.Token) \
-                           .filter(models.Token.token == token_string,
-                                   models.Token.account == account,
-                                   models.Token.expired_at > datetime.utcnow()) \
-                           .with_for_update(skip_locked=True).first()
+    query = select(
+        models.Token
+    ).where(
+        models.Token.token == token_string,
+        models.Token.account == account,
+        models.Token.expired_at > datetime.utcnow()
+    ).with_for_update(
+        skip_locked=True
+    )
+    account_token = session.execute(query).scalar()
 
     # if token does not exist in the DB, return None
     if account_token is None:
@@ -886,12 +943,17 @@ def refresh_cli_auth_token(token_string: str, account: str, session=None):
     else:
         # find account token with the same scope,
         # audience and has a valid refresh token
-        new_token = session.query(models.Token) \
-                           .filter(models.Token.refresh == true(),
-                                   models.Token.refresh_expired_at > datetime.utcnow(),
-                                   models.Token.account == account,
-                                   models.Token.expired_at > datetime.utcnow()) \
-                           .with_for_update(skip_locked=True).first()
+        query = select(
+            models.Token
+        ).where(
+            models.Token.refresh == true(),
+            models.Token.refresh_expired_at > datetime.utcnow(),
+            models.Token.account == account,
+            models.Token.expired_at > datetime.utcnow()
+        ).with_for_update(
+            skip_locked=True
+        )
+        new_token = session.execute(query).scalar()
         if new_token is None:
             return None
 
@@ -908,7 +970,7 @@ def refresh_cli_auth_token(token_string: str, account: str, session=None):
 
 
 @transactional_session
-def refresh_jwt_tokens(total_workers: int, worker_number: int, refreshrate: int = 3600, limit: int = 1000, session=None):
+def refresh_jwt_tokens(total_workers: int, worker_number: int, refreshrate: int = 3600, limit: int = 1000, *, session: "Session"):
     """
     Refreshes tokens which expired or will expire before (now + refreshrate)
     next run of this function and which have valid refresh token.
@@ -924,22 +986,31 @@ def refresh_jwt_tokens(total_workers: int, worker_number: int, refreshrate: int
     try:
         # get tokens for refresh that expire in the next <refreshrate> seconds
         expiration_future = datetime.utcnow() + timedelta(seconds=refreshrate)
-        query = session.query(models.Token.token) \
-                       .filter(and_(models.Token.refresh == true(),
-                                    models.Token.refresh_expired_at > datetime.utcnow(),
-                                    models.Token.expired_at < expiration_future))\
-                       .order_by(models.Token.expired_at)
+        query = select(
+            models.Token
+        ).where(
+            models.Token.refresh == true(),
+            models.Token.refresh_expired_at > datetime.utcnow(),
+            models.Token.expired_at < expiration_future
+        ).order_by(
+            models.Token.expired_at
+        )
         query = filter_thread_work(session=session, query=query, total_threads=total_workers, thread_id=worker_number, hash_variable='token')
-
         # limiting the number of tokens for refresh
         query = query.limit(limit)
-        filtered_tokens = []
-        for items in session.execute(query).partitions(10):
-            tokens = tuple(map(lambda row: row.token, items))
-            filtered_tokens += session.query(models.Token) \
-                                      .filter(models.Token.token.in_(tokens)) \
-                                      .with_for_update(skip_locked=True) \
-                                      .all()
+        # Oracle does not support chaining order_by(), limit(), and
+        # with_for_update(). Use a nested query to overcome this.
+        if session.bind.dialect.name == 'oracle':
+            query = select(
+                models.Token
+            ).where(
+                models.Token.token.in_(query.with_only_columns(models.Token.token))
+            ).with_for_update(
+                skip_locked=True
+            )
+        else:
+            query = query.with_for_update(skip_locked=True)
+        filtered_tokens = session.execute(query).scalars().all()
 
         # refreshing these tokens
         for token in filtered_tokens:
@@ -953,8 +1024,9 @@ def refresh_jwt_tokens(total_workers: int, worker_number: int, refreshrate: int
     return nrefreshed
 
 
+@METRICS.time_it
 @transactional_session
-def __refresh_token_oidc(token_object: models.Token, session=None):
+def __refresh_token_oidc(token_object: models.Token, *, session: "Session"):
     """
     Requests new access and refresh tokens from the Identity Provider.
     Assumption: The Identity Provider issues refresh tokens for one time use only and
@@ -968,8 +1040,6 @@ def __refresh_token_oidc(token_object: models.Token, session=None):
         constraints. Otherwise, throws an an Exception.
     """
     try:
-        timer = Timer()
-        record_counter(name='IdP_authorization.refresh_token.request')
         jwt_row_dict, extra_dict = {}, {}
         jwt_row_dict['account'] = token_object.account
         jwt_row_dict['identity'] = token_object.identity
@@ -994,7 +1064,7 @@ def __refresh_token_oidc(token_object: models.Token, session=None):
         oidc_tokens = oidc_client.do_access_token_refresh(state=state, skew=LEEWAY_SECS)
         if 'error' in oidc_tokens:
             raise CannotAuthorize(oidc_tokens['error'])
-        record_counter(name='IdP_authorization.refresh_token.refreshed')
+        METRICS.counter(name='IdP_authorization.refresh_token.refreshed').inc()
         # get audience and scope information
         if 'scope' in oidc_tokens and 'audience' in oidc_tokens:
             jwt_row_dict['authz_scope'] = val_to_space_sep_str(oidc_tokens['scope'])
@@ -1020,21 +1090,20 @@ def __refresh_token_oidc(token_object: models.Token, session=None):
                 # 4 day expiry period by default
                 extra_dict['refresh_expired_at'] = datetime.utcnow() + timedelta(hours=REFRESH_LIFETIME_H)
             new_token = __save_validated_token(oidc_tokens['access_token'], jwt_row_dict, extra_dict=extra_dict, session=session)
-            record_counter(name='IdP_authorization.access_token.saved')
-            record_counter(name='IdP_authorization.refresh_token.saved')
+            METRICS.counter(name='IdP_authorization.access_token.saved').inc()
+            METRICS.counter(name='IdP_authorization.refresh_token.saved').inc()
         else:
             raise CannotAuthorize("OIDC identity '%s' of the '%s' account is did not " % (token_object.identity, token_object.account)
                                   + "succeed requesting a new access and refresh tokens.")  # NOQA: W503
-        timer.record('IdP_authorization.refresh_token')
         return new_token
 
     except Exception as error:
-        record_counter(name='IdP_authorization.refresh_token.exception')
+        METRICS.counter(name='IdP_authorization.refresh_token.exception').inc()
         raise CannotAuthorize(traceback.format_exc()) from error
 
 
 @transactional_session
-def delete_expired_oauthrequests(total_workers: int, worker_number: int, limit: int = 1000, session=None):
+def delete_expired_oauthrequests(total_workers: int, worker_number: int, limit: int = 1000, *, session: "Session"):
     """
     Delete expired OAuth request parameters.
 
@@ -1048,20 +1117,37 @@ def delete_expired_oauthrequests(total_workers: int, worker_number: int, limit:
 
     try:
         # get expired OAuth request parameters
-        query = session.query(models.OAuthRequest.state).filter(models.OAuthRequest.expired_at < datetime.utcnow())\
-                       .order_by(models.OAuthRequest.expired_at)
-
+        query = select(
+            models.OAuthRequest.state
+        ).where(
+            models.OAuthRequest.expired_at < datetime.utcnow()
+        ).order_by(
+            models.OAuthRequest.expired_at
+        )
         query = filter_thread_work(session=session, query=query, total_threads=total_workers, thread_id=worker_number, hash_variable='state')
-
         # limiting the number of oauth requests deleted at once
         query = query.limit(limit)
+        # Oracle does not support chaining order_by(), limit(), and
+        # with_for_update(). Use a nested query to overcome this.
+        if session.bind.dialect.name == 'oracle':
+            query = select(
+                models.OAuthRequest.state
+            ).where(
+                models.OAuthRequest.state.in_(query)
+            ).with_for_update(
+                skip_locked=True
+            )
+        else:
+            query = query.with_for_update(skip_locked=True)
+
         ndeleted = 0
-        for items in session.execute(query).partitions(10):
-            states = tuple(map(lambda row: row.state, items))
-            ndeleted += session.query(models.OAuthRequest) \
-                               .filter(models.OAuthRequest.state.in_(states)) \
-                               .with_for_update(skip_locked=True) \
-                               .delete(synchronize_session='fetch')
+        for states in session.execute(query).scalars().partitions(10):
+            query = delete(
+                models.OAuthRequest
+            ).where(
+                models.OAuthRequest.state.in_(states)
+            )
+            ndeleted += session.execute(query).rowcount
         return ndeleted
     except Exception as error:
         raise RucioException(error.args) from error
@@ -1091,7 +1177,7 @@ def __get_keyvalues_from_claims(token: str, keys=None):
 
 
 @read_session
-def __get_rucio_jwt_dict(jwt: str, account=None, session=None):
+def __get_rucio_jwt_dict(jwt: str, account=None, *, session: "Session"):
     """
     Get a Rucio token dictionary from token claims.
     Check token expiration and find default Rucio
@@ -1135,7 +1221,7 @@ def __get_rucio_jwt_dict(jwt: str, account=None, session=None):
 
 
 @transactional_session
-def __save_validated_token(token, valid_dict, extra_dict=None, session=None):
+def __save_validated_token(token, valid_dict, extra_dict=None, *, session: "Session"):
     """
     Save JWT token to the Rucio DB.
 
@@ -1169,7 +1255,7 @@ def __save_validated_token(token, valid_dict, extra_dict=None, session=None):
 
 
 @transactional_session
-def validate_jwt(json_web_token: str, session=None):
+def validate_jwt(json_web_token: str, *, session: "Session") -> dict[str, Any]:
     """
     Verifies signature and validity of a JSON Web Token.
     Gets the issuer public keys from the oidc_client
@@ -1183,7 +1269,8 @@ def validate_jwt(json_web_token: str, session=None):
                            lifetime: <token lifetime>,
                            audience: <audience>,
                            authz_scope: <authz_scope> }
-              if successful, None otherwise.
+              if successful.
+    :raises: CannotAuthenticate if unsuccessful
     """
 
     if not OIDC_CLIENTS:
@@ -1195,9 +1282,9 @@ def validate_jwt(json_web_token: str, session=None):
     try:
 
         # getting issuer from the token payload
-        token_dict = __get_rucio_jwt_dict(json_web_token, session=session)
+        token_dict: Optional[dict[str, Any]] = __get_rucio_jwt_dict(json_web_token, session=session)
         if not token_dict:
-            return None
+            raise CannotAuthenticate(traceback.format_exc())
         issuer = token_dict['identity'].split(", ")[1].split("=")[1]
         oidc_client = OIDC_CLIENTS[issuer]
         issuer_keys = oidc_client.keyjar.get_issuer_keys(issuer)
@@ -1217,7 +1304,7 @@ def validate_jwt(json_web_token: str, session=None):
                 token_dict['authz_scope'] = inspect_claims['scope']
             except:
                 pass
-        record_counter(name='JSONWebToken.valid')
+        METRICS.counter(name='JSONWebToken.valid').inc()
         # if token is valid and coming from known issuer --> check aud and scope and save it if unknown
         if token_dict['authz_scope'] and token_dict['audience']:
             if all_oidc_req_claims_present(token_dict['authz_scope'], token_dict['audience'], EXPECTED_OIDC_SCOPE, EXPECTED_OIDC_AUDIENCE):
@@ -1225,16 +1312,16 @@ def validate_jwt(json_web_token: str, session=None):
                 __save_validated_token(json_web_token, token_dict, session=session)
             else:
                 logging.debug("Token audience [%s] or scope [%s] verification failed.", token_dict['audience'], token_dict['authz_scope'])
-                return None
+                raise CannotAuthenticate(traceback.format_exc())
         else:
             logging.debug("Token audience or scope not present.")
-            return None
-        record_counter(name='JSONWebToken.saved')
+            raise CannotAuthenticate(traceback.format_exc())
+        METRICS.counter(name='JSONWebToken.saved').inc()
         return token_dict
     except Exception:
-        record_counter(name='JSONWebToken.invalid')
+        METRICS.counter(name='JSONWebToken.invalid').inc()
         logging.debug(traceback.format_exc())
-        return None
+        raise CannotAuthenticate(traceback.format_exc())
 
 
 def oidc_identity_string(sub: str, iss: str):
