@@ -13,12 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from os import environ
-
-from configparser import NoOptionError, NoSectionError
-from rucio.common import config, exception
-
 import importlib
+from configparser import NoOptionError, NoSectionError
+from os import environ
+from typing import TYPE_CHECKING
+
+from rucio.common import config, exception
+from rucio.common.utils import check_policy_package_version
+
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
 
 # dictionary of permission modules for each VO
 permission_modules = {}
@@ -48,9 +53,11 @@ if not multivo:
     if config.config_has_section('policy'):
         try:
             if 'RUCIO_POLICY_PACKAGE' in environ:
-                POLICY = environ['RUCIO_POLICY_PACKAGE'] + ".permission"
+                POLICY = environ['RUCIO_POLICY_PACKAGE']
             else:
-                POLICY = config.config_get('policy', 'package', check_config_table=False) + ".permission"
+                POLICY = config.config_get('policy', 'package', check_config_table=False)
+            check_policy_package_version(POLICY)
+            POLICY = POLICY + ".permission"
         except (NoOptionError, NoSectionError):
             # fall back to old system for now
             POLICY = 'rucio.core.permission.' + FALLBACK_POLICY.lower()
@@ -71,9 +78,11 @@ def load_permission_for_vo(vo):
         try:
             env_name = 'RUCIO_POLICY_PACKAGE_' + vo.upper()
             if env_name in environ:
-                POLICY = environ[env_name] + ".permission"
+                POLICY = environ[env_name]
             else:
-                POLICY = config.config_get('policy', 'package-' + vo) + ".permission"
+                POLICY = config.config_get('policy', 'package-' + vo)
+            check_policy_package_version(POLICY)
+            POLICY = POLICY + ".permission"
         except (NoOptionError, NoSectionError):
             # fall back to old system for now
             try:
@@ -92,7 +101,7 @@ def load_permission_for_vo(vo):
     permission_modules[vo] = module
 
 
-def has_permission(issuer, action, kwargs, session=None):
+def has_permission(issuer, action, kwargs, *, session: "Optional[Session]" = None):
     if issuer.vo not in permission_modules:
         load_permission_for_vo(issuer.vo)
     return permission_modules[issuer.vo].has_permission(issuer, action, kwargs, session=session)
