@@ -15,23 +15,24 @@
 
 from typing import TYPE_CHECKING
 
-from rucio.common.config import config_get
-from rucio.core.did import get_metadata
-from rucio.core.lifetime_exception import list_exceptions
 import rucio.core.scope
-from rucio.core.account import has_account_attribute
+from rucio.common.config import config_get
+from rucio.common.types import InternalScope, InternalAccount
+from rucio.core.account import has_account_attribute, list_account_attributes
+from rucio.core.did import get_metadata
 from rucio.core.identity import exist_identity_account
+from rucio.core.lifetime_exception import list_exceptions
+from rucio.core.rse import list_rse_attributes
+from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import get_rule
 from rucio.db.sqla.constants import IdentityType
 
-
 if TYPE_CHECKING:
     from typing import Optional
     from sqlalchemy.orm import Session
-    from rucio.common.types import InternalAccount
 
 
-def has_permission(issuer: "InternalAccount", action: str, kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def has_permission(issuer: "InternalAccount", action: str, kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account has the specified permission to
     execute an action with parameters.
@@ -80,6 +81,7 @@ def has_permission(issuer: "InternalAccount", action: str, kwargs: dict, session
             'attach_dids_to_dids': perm_attach_dids_to_dids,
             'create_did_sample': perm_create_did_sample,
             'set_metadata': perm_set_metadata,
+            'set_metadata_bulk': perm_set_metadata_bulk,
             'set_status': perm_set_status,
             'queue_requests': perm_queue_requests,
             'set_rse_usage': perm_set_rse_usage,
@@ -108,7 +110,7 @@ def has_permission(issuer: "InternalAccount", action: str, kwargs: dict, session
             'list_heartbeats': perm_list_heartbeats,
             'resurrect': perm_resurrect,
             'update_lifetime_exceptions': perm_update_lifetime_exceptions,
-            'get_ssh_challenge_token': perm_get_ssh_challenge_token,
+            'get_auth_token_ssh': perm_get_auth_token_ssh,
             'get_signed_url': perm_get_signed_url,
             'add_bad_pfns': perm_add_bad_pfns,
             'del_account_identity': perm_del_account_identity,
@@ -123,7 +125,19 @@ def _is_root(issuer):
     return issuer.external == 'root'
 
 
-def perm_default(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def _perm_country(issuer: "InternalAccount", rses: list, roles: list, *, session: "Optional[Session]" = None) -> bool:
+    admin_in_country = []
+    for kv in list_account_attributes(account=issuer, session=session):
+        if kv['key'].startswith('country-') and kv['value'] == 'admin':
+            admin_in_country.append(kv['key'].partition('-')[2])
+    if admin_in_country:
+        for rse in rses:
+            if list_rse_attributes(rse_id=rse['id'], session=session).get('country') in admin_in_country:
+                return True
+    return False
+
+
+def perm_default(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Default permission.
 
@@ -135,7 +149,7 @@ def perm_default(issuer: "InternalAccount", kwargs: dict, session: "Optional[Ses
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rse(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_rse(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add a RSE.
 
@@ -148,7 +162,7 @@ def perm_add_rse(issuer: "InternalAccount", kwargs: dict, session: "Optional[Ses
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_update_rse(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_rse(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can update a RSE.
 
@@ -161,7 +175,7 @@ def perm_update_rse(issuer: "InternalAccount", kwargs: dict, session: "Optional[
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_add_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_rule(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add a replication rule.
 
@@ -183,7 +197,7 @@ def perm_add_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Se
     return perm_default(issuer, kwargs, session=session) or has_account_attribute(account=issuer, key='rule_admin', session=session)
 
 
-def perm_add_subscription(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_subscription(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add a subscription.
 
@@ -196,7 +210,7 @@ def perm_add_subscription(issuer: "InternalAccount", kwargs: dict, session: "Opt
         or has_account_attribute(account=issuer, key='subscription_admin', session=session)
 
 
-def perm_add_rse_attribute(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_rse_attribute(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add a RSE attribute.
 
@@ -209,7 +223,7 @@ def perm_add_rse_attribute(issuer: "InternalAccount", kwargs: dict, session: "Op
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_del_rse_attribute(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_rse_attribute(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete a RSE attribute.
 
@@ -222,7 +236,7 @@ def perm_del_rse_attribute(issuer: "InternalAccount", kwargs: dict, session: "Op
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_del_rse(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_rse(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete a RSE.
 
@@ -235,7 +249,7 @@ def perm_del_rse(issuer: "InternalAccount", kwargs: dict, session: "Optional[Ses
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_add_account(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_account(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add an account.
 
@@ -248,7 +262,7 @@ def perm_add_account(issuer: "InternalAccount", kwargs: dict, session: "Optional
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_del_account(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_account(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can del an account.
 
@@ -261,7 +275,7 @@ def perm_del_account(issuer: "InternalAccount", kwargs: dict, session: "Optional
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_update_account(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_account(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can update an account.
 
@@ -274,7 +288,7 @@ def perm_update_account(issuer: "InternalAccount", kwargs: dict, session: "Optio
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_add_scope(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_scope(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add a scope to an account.
 
@@ -287,7 +301,7 @@ def perm_add_scope(issuer: "InternalAccount", kwargs: dict, session: "Optional[S
         or has_account_attribute(account=issuer, key='scope_admin', session=session)
 
 
-def perm_get_auth_token_user_pass(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_auth_token_user_pass(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -301,7 +315,7 @@ def perm_get_auth_token_user_pass(issuer: "InternalAccount", kwargs: dict, sessi
     return False
 
 
-def perm_get_auth_token_gss(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_auth_token_gss(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -315,7 +329,7 @@ def perm_get_auth_token_gss(issuer: "InternalAccount", kwargs: dict, session: "O
     return False
 
 
-def perm_get_auth_token_x509(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_auth_token_x509(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -329,7 +343,7 @@ def perm_get_auth_token_x509(issuer: "InternalAccount", kwargs: dict, session: "
     return False
 
 
-def perm_get_auth_token_saml(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_auth_token_saml(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -343,7 +357,7 @@ def perm_get_auth_token_saml(issuer: "InternalAccount", kwargs: dict, session: "
     return False
 
 
-def perm_add_account_identity(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_account_identity(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add an identity to an account.
 
@@ -356,7 +370,7 @@ def perm_add_account_identity(issuer: "InternalAccount", kwargs: dict, session:
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_del_account_identity(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_account_identity(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete an identity to an account.
 
@@ -369,7 +383,7 @@ def perm_del_account_identity(issuer: "InternalAccount", kwargs: dict, session:
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_del_identity(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_identity(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete an identity.
 
@@ -382,7 +396,7 @@ def perm_del_identity(issuer: "InternalAccount", kwargs: dict, session: "Optiona
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_add_did(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_did(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add an data identifier to a scope.
 
@@ -402,13 +416,17 @@ def perm_add_did(issuer: "InternalAccount", kwargs: dict, session: "Optional[Ses
         if not perm_add_rule(issuer, kwargs=kwargs_rule, session=session):
             return False
 
+    scope = kwargs['scope']
+    if isinstance(kwargs['scope'], str):
+        scope = InternalScope(kwargs['scope'])
     return perm_default(issuer, kwargs, session=session)\
         or has_account_attribute(account=issuer, key='did_admin', session=session)\
         or has_account_attribute(account=issuer, key='production_account', session=session)\
-        or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
+        or rucio.core.scope.is_scope_owner(scope=scope, account=issuer, session=session)\
+        or (kwargs.get('name', False) and kwargs['name'].startswith('/belle/scout'))
 
 
-def perm_add_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_dids(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can bulk add data identifiers.
 
@@ -424,7 +442,7 @@ def perm_add_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional[Se
     return True
 
 
-def perm_attach_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_attach_dids(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -438,7 +456,7 @@ def perm_attach_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_attach_dids_to_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_attach_dids_to_dids(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -459,7 +477,7 @@ def perm_attach_dids_to_dids(issuer: "InternalAccount", kwargs: dict, session: "
         return True
 
 
-def perm_create_did_sample(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_create_did_sample(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can create a sample of a data identifier collection.
 
@@ -474,7 +492,7 @@ def perm_create_did_sample(issuer: "InternalAccount", kwargs: dict, session: "Op
         or kwargs['scope'].external == 'mock'
 
 
-def perm_del_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_rule(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an issuer can delete a replication rule.
 
@@ -483,12 +501,24 @@ def perm_del_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Se
     :param session: The DB session to use
     :returns: True if account is allowed to call the API call, otherwise False
     """
+    rule = get_rule(rule_id=kwargs['rule_id'], session=session)
+    rses = parse_expression(rule['rse_expression'], filter_={'vo': issuer.vo}, session=session)
+    # Check if user is a country admin
+    if _perm_country(issuer=issuer, rses=rses, roles=['admin', ], session=session):
+        return True
+
+    # DELETERS can delete the rule
+    for rse in rses:
+        rse_attr = list_rse_attributes(rse_id=rse['id'], session=session)
+        if rse_attr.get('rule_deleters'):
+            if issuer.external in rse_attr.get('rule_deleters').split(','):
+                return True
     return perm_default(issuer, kwargs, session=session)\
         or has_account_attribute(account=issuer, key='rule_admin', session=session)\
         or get_rule(kwargs['rule_id'], session=session)['account'] == issuer
 
 
-def perm_update_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_rule(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an issuer can update a replication rule.
 
@@ -499,10 +529,10 @@ def perm_update_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional
     """
     return perm_default(issuer, kwargs, session=session)\
         or has_account_attribute(account=issuer, key='rule_admin', session=session)\
-        or (kwargs.get('rule_id') and get_rule(kwargs['rule_id'], session=session)['account'] == issuer)
+        or (kwargs.get('rule_id', False) and get_rule(kwargs['rule_id'], session=session)['account'] == issuer)
 
 
-def perm_approve_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_approve_rule(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an issuer can approve a replication rule.
 
@@ -515,7 +545,7 @@ def perm_approve_rule(issuer: "InternalAccount", kwargs: dict, session: "Optiona
         or has_account_attribute(account=issuer, key='rule_admin', session=session)
 
 
-def perm_reduce_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_reduce_rule(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an issuer can reduce a replication rule.
 
@@ -528,7 +558,7 @@ def perm_reduce_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional
         or has_account_attribute(account=issuer, key='rule_admin', session=session)
 
 
-def perm_move_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_move_rule(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an issuer can move a replication rule.
 
@@ -542,7 +572,7 @@ def perm_move_rule(issuer: "InternalAccount", kwargs: dict, session: "Optional[S
         or get_rule(kwargs['rule_id'], session=session)['account'] == issuer
 
 
-def perm_update_subscription(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_subscription(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can update a subscription.
 
@@ -555,7 +585,7 @@ def perm_update_subscription(issuer: "InternalAccount", kwargs: dict, session: "
         or has_account_attribute(account=issuer, key='subscription_admin', session=session)
 
 
-def perm_detach_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_detach_dids(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can detach an data identifier from the other data identifier.
 
@@ -569,7 +599,7 @@ def perm_detach_dids(issuer: "InternalAccount", kwargs: dict, session: "Optional
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_set_metadata(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_set_metadata_bulk(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set a metadata on a data identifier.
 
@@ -585,7 +615,23 @@ def perm_set_metadata(issuer: "InternalAccount", kwargs: dict, session: "Optiona
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_set_status(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_set_metadata(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
+    """
+    Checks if an account can set a metadata on a data identifier.
+
+    :param issuer: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :param session: The DB session to use
+    :returns: True if account is allowed, otherwise False
+    """
+    meta = get_metadata(kwargs['scope'], kwargs['name'], session=session)
+    return perm_default(issuer, kwargs, session=session)\
+        or has_account_attribute(account=issuer, key='did_admin', session=session)\
+        or meta.get('account', '') == issuer\
+        or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
+
+
+def perm_set_status(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set status on an data identifier.
 
@@ -601,7 +647,7 @@ def perm_set_status(issuer: "InternalAccount", kwargs: dict, session: "Optional[
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_add_protocol(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_protocol(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add a protocol to an RSE.
 
@@ -614,7 +660,7 @@ def perm_add_protocol(issuer: "InternalAccount", kwargs: dict, session: "Optiona
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_del_protocol(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_protocol(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete protocols from an RSE.
 
@@ -627,7 +673,7 @@ def perm_del_protocol(issuer: "InternalAccount", kwargs: dict, session: "Optiona
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_update_protocol(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_protocol(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can update protocols of an RSE.
 
@@ -640,7 +686,7 @@ def perm_update_protocol(issuer: "InternalAccount", kwargs: dict, session: "Opti
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_declare_bad_file_replicas(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_declare_bad_file_replicas(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can declare bad file replicas.
 
@@ -652,7 +698,7 @@ def perm_declare_bad_file_replicas(issuer: "InternalAccount", kwargs: dict, sess
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_declare_suspicious_file_replicas(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_declare_suspicious_file_replicas(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can declare suspicious file replicas.
 
@@ -664,7 +710,7 @@ def perm_declare_suspicious_file_replicas(issuer: "InternalAccount", kwargs: dic
     return True
 
 
-def perm_add_replicas(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_replicas(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add replicas.
 
@@ -673,11 +719,15 @@ def perm_add_replicas(issuer: "InternalAccount", kwargs: dict, session: "Optiona
     :param session: The DB session to use
     :returns: True if account is allowed, otherwise False
     """
+    rses = [{'id': kwargs['rse_id']}]
+    if str(kwargs.get('rse', '')).endswith('LOCAL-SE')\
+            and _perm_country(issuer=issuer, rses=rses, roles=['admin', 'user'], session=session):
+        return True
     return str(kwargs.get('rse', '')).endswith('TMP-SE')\
         or perm_default(issuer, kwargs, session=session)
 
 
-def perm_skip_availability_check(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_skip_availability_check(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can skip the availabity check to add/delete file replicas.
 
@@ -689,7 +739,7 @@ def perm_skip_availability_check(issuer: "InternalAccount", kwargs: dict, sessio
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_delete_replicas(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_delete_replicas(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete replicas.
 
@@ -701,7 +751,7 @@ def perm_delete_replicas(issuer: "InternalAccount", kwargs: dict, session: "Opti
     return False
 
 
-def perm_update_replicas_states(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_replicas_states(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete replicas.
 
@@ -713,7 +763,7 @@ def perm_update_replicas_states(issuer: "InternalAccount", kwargs: dict, session
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_queue_requests(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_queue_requests(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can submit transfer or deletion requests on destination RSEs for data identifiers.
 
@@ -725,7 +775,7 @@ def perm_queue_requests(issuer: "InternalAccount", kwargs: dict, session: "Optio
     return _is_root(issuer)
 
 
-def perm_query_request(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_query_request(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can query a request.
 
@@ -737,7 +787,7 @@ def perm_query_request(issuer: "InternalAccount", kwargs: dict, session: "Option
     return _is_root(issuer)
 
 
-def perm_get_request_by_did(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_request_by_did(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can get a request by DID.
 
@@ -749,7 +799,7 @@ def perm_get_request_by_did(issuer: "InternalAccount", kwargs: dict, session: "O
     return True
 
 
-def perm_cancel_request(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_cancel_request(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can cancel a request.
 
@@ -761,7 +811,7 @@ def perm_cancel_request(issuer: "InternalAccount", kwargs: dict, session: "Optio
     return _is_root(issuer)
 
 
-def perm_get_next(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_next(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can retrieve the next request matching the request type and state.
 
@@ -773,7 +823,7 @@ def perm_get_next(issuer: "InternalAccount", kwargs: dict, session: "Optional[Se
     return _is_root(issuer)
 
 
-def perm_set_rse_usage(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_set_rse_usage(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set RSE usage information.
 
@@ -786,7 +836,7 @@ def perm_set_rse_usage(issuer: "InternalAccount", kwargs: dict, session: "Option
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_set_rse_limits(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_set_rse_limits(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set RSE limits.
 
@@ -799,7 +849,7 @@ def perm_set_rse_limits(issuer: "InternalAccount", kwargs: dict, session: "Optio
         or has_account_attribute(account=issuer, key='rse_admin', session=session)
 
 
-def perm_set_local_account_limit(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_set_local_account_limit(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set an account limit.
 
@@ -808,11 +858,14 @@ def perm_set_local_account_limit(issuer: "InternalAccount", kwargs: dict, sessio
     :param session: The DB session to use
     :returns: True if account is allowed, otherwise False
     """
+    rses = [{'id': kwargs['rse_id']}]
+    if _perm_country(issuer=issuer, rses=rses, roles=['admin', ], session=session):
+        return True
     return perm_default(issuer, kwargs, session=session)\
         or (has_account_attribute(account=issuer, key='rse_admin', session=session) and has_account_attribute(account=issuer, key='account_admin', session=session))
 
 
-def perm_set_global_account_limit(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_set_global_account_limit(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set a global account limit.
 
@@ -825,7 +878,7 @@ def perm_set_global_account_limit(issuer: "InternalAccount", kwargs: dict, sessi
         or (has_account_attribute(account=issuer, key='rse_admin', session=session) and has_account_attribute(account=issuer, key='account_admin', session=session))
 
 
-def perm_delete_local_account_limit(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_delete_local_account_limit(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete an account limit.
 
@@ -838,7 +891,7 @@ def perm_delete_local_account_limit(issuer: "InternalAccount", kwargs: dict, ses
         or (has_account_attribute(account=issuer, key='rse_admin', session=session) and has_account_attribute(account=issuer, key='account_admin', session=session))
 
 
-def perm_delete_global_account_limit(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_delete_global_account_limit(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can delete a global account limit.
 
@@ -851,7 +904,7 @@ def perm_delete_global_account_limit(issuer: "InternalAccount", kwargs: dict, se
         or (has_account_attribute(account=issuer, key='rse_admin', session=session) and has_account_attribute(account=issuer, key='account_admin', session=session))
 
 
-def perm_config(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_config(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can read/write the configuration.
 
@@ -864,7 +917,7 @@ def perm_config(issuer: "InternalAccount", kwargs: dict, session: "Optional[Sess
         or has_account_attribute(account=issuer, key='config_admin', session=session)
 
 
-def perm_get_local_account_usage(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_local_account_usage(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can get the account usage of an account.
 
@@ -879,7 +932,7 @@ def perm_get_local_account_usage(issuer: "InternalAccount", kwargs: dict, sessio
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_get_global_account_usage(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_global_account_usage(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can get the account usage of an account.
 
@@ -894,7 +947,7 @@ def perm_get_global_account_usage(issuer: "InternalAccount", kwargs: dict, sessi
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_add_account_attribute(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_account_attribute(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add attributes to accounts.
 
@@ -907,7 +960,7 @@ def perm_add_account_attribute(issuer: "InternalAccount", kwargs: dict, session:
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_del_account_attribute(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_del_account_attribute(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can add attributes to accounts.
 
@@ -920,7 +973,7 @@ def perm_del_account_attribute(issuer: "InternalAccount", kwargs: dict, session:
         or has_account_attribute(account=issuer, key='account_admin', session=session)
 
 
-def perm_list_heartbeats(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_list_heartbeats(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can list heartbeats.
 
@@ -932,7 +985,7 @@ def perm_list_heartbeats(issuer: "InternalAccount", kwargs: dict, session: "Opti
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_resurrect(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_resurrect(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can resurrect DIDS.
 
@@ -944,7 +997,7 @@ def perm_resurrect(issuer: "InternalAccount", kwargs: dict, session: "Optional[S
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_update_lifetime_exceptions(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_update_lifetime_exceptions(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can approve/reject Lifetime Model exceptions.
 
@@ -960,9 +1013,9 @@ def perm_update_lifetime_exceptions(issuer: "InternalAccount", kwargs: dict, ses
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_get_ssh_challenge_token(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_auth_token_ssh(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
-    Checks if an account can request a challenge token.
+    Checks if an account can request an ssh token.
 
     :param issuer: Account identifier which issues the command.
     :param kwargs: List of arguments for the action.
@@ -972,7 +1025,7 @@ def perm_get_ssh_challenge_token(issuer: "InternalAccount", kwargs: dict, sessio
     return True
 
 
-def perm_get_signed_url(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_get_signed_url(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can request a signed URL.
 
@@ -985,7 +1038,7 @@ def perm_get_signed_url(issuer: "InternalAccount", kwargs: dict, session: "Optio
         or has_account_attribute(account=issuer, key='sign-gcs', session=session)
 
 
-def perm_add_bad_pfns(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_add_bad_pfns(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can declare bad PFNs.
 
@@ -997,7 +1050,7 @@ def perm_add_bad_pfns(issuer: "InternalAccount", kwargs: dict, session: "Optiona
     return perm_default(issuer, kwargs, session=session)
 
 
-def perm_remove_did_from_followed(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_remove_did_from_followed(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can remove did from followed table.
 
@@ -1010,7 +1063,7 @@ def perm_remove_did_from_followed(issuer: "InternalAccount", kwargs: dict, sessi
         or kwargs['account'] == issuer
 
 
-def perm_remove_dids_from_followed(issuer: "InternalAccount", kwargs: dict, session: "Optional[Session]" = None) -> bool:
+def perm_remove_dids_from_followed(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can bulk remove dids from followed table.
 
