@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING
+
 import rucio.core.scope
 from rucio.core.account import has_account_attribute
 from rucio.core.identity import exist_identity_account
@@ -22,15 +24,13 @@ from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import get_rule
 from rucio.db.sqla.constants import IdentityType
 
-try:
-    # Python 2: "unicode" is built-in
-    unicode
-except NameError:
-    unicode = str
-    basestring = str
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
+    from rucio.common.types import InternalAccount
 
 
-def has_permission(issuer, action, kwargs, session=None):
+def has_permission(issuer, action, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account has the specified permission to
     execute an action with parameters.
@@ -109,7 +109,7 @@ def has_permission(issuer, action, kwargs, session=None):
             'list_heartbeats': perm_list_heartbeats,
             'resurrect': perm_resurrect,
             'update_lifetime_exceptions': perm_update_lifetime_exceptions,
-            'get_ssh_challenge_token': perm_get_ssh_challenge_token,
+            'get_auth_token_ssh': perm_get_auth_token_ssh,
             'get_signed_url': perm_get_signed_url,
             'add_bad_pfns': perm_add_bad_pfns,
             'del_account_identity': perm_del_account_identity,
@@ -129,7 +129,7 @@ def _is_root(issuer):
     return issuer.external == 'root'
 
 
-def perm_default(issuer, kwargs, session=None):
+def perm_default(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Default permission.
 
@@ -141,7 +141,7 @@ def perm_default(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rse(issuer, kwargs, session=None):
+def perm_add_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE.
 
@@ -153,7 +153,7 @@ def perm_add_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_rse(issuer, kwargs, session=None):
+def perm_update_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a RSE.
 
@@ -165,7 +165,7 @@ def perm_update_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rule(issuer, kwargs, session=None):
+def perm_add_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a replication rule.
 
@@ -181,7 +181,7 @@ def perm_add_rule(issuer, kwargs, session=None):
     if _is_root(issuer) and repr(kwargs['account']).startswith('sync_'):
         return True
 
-    if isinstance(repr(issuer), basestring) and repr(issuer).startswith('sync_'):
+    if isinstance(repr(issuer), str) and repr(issuer).startswith('sync_'):  # noqa
         return True
 
     # Anyone can use _Temp RSEs if a lifetime is set and under a month
@@ -202,7 +202,7 @@ def perm_add_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_subscription(issuer, kwargs, session=None):
+def perm_add_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a subscription.
 
@@ -216,7 +216,7 @@ def perm_add_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_rse_attribute(issuer, kwargs, session=None):
+def perm_add_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE attribute.
 
@@ -230,7 +230,7 @@ def perm_add_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse_attribute(issuer, kwargs, session=None):
+def perm_del_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE attribute.
 
@@ -244,7 +244,7 @@ def perm_del_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse(issuer, kwargs, session=None):
+def perm_del_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE.
 
@@ -256,7 +256,7 @@ def perm_del_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_account(issuer, kwargs, session=None):
+def perm_add_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an account.
 
@@ -268,7 +268,7 @@ def perm_add_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_del_account(issuer, kwargs, session=None):
+def perm_del_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can del an account.
 
@@ -280,7 +280,7 @@ def perm_del_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_update_account(issuer, kwargs, session=None):
+def perm_update_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update an account.
 
@@ -292,7 +292,7 @@ def perm_update_account(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_scope(issuer, kwargs, session=None):
+def perm_add_scope(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a scop to a account.
 
@@ -301,10 +301,10 @@ def perm_add_scope(issuer, kwargs, session=None):
     :param session: The DB session to use
     :returns: True if account is allowed, otherwise False
     """
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
+def perm_get_auth_token_user_pass(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -318,7 +318,7 @@ def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_gss(issuer, kwargs, session=None):
+def perm_get_auth_token_gss(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -332,7 +332,7 @@ def perm_get_auth_token_gss(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_x509(issuer, kwargs, session=None):
+def perm_get_auth_token_x509(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -346,7 +346,7 @@ def perm_get_auth_token_x509(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_saml(issuer, kwargs, session=None):
+def perm_get_auth_token_saml(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -360,7 +360,7 @@ def perm_get_auth_token_saml(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_identity(issuer, kwargs, session=None):
+def perm_add_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an identity to an account.
 
@@ -370,10 +370,10 @@ def perm_add_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_identity(issuer, kwargs, session=None):
+def perm_del_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity to an account.
 
@@ -383,10 +383,10 @@ def perm_del_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_identity(issuer, kwargs, session=None):
+def perm_del_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity.
 
@@ -399,7 +399,7 @@ def perm_del_identity(issuer, kwargs, session=None):
     return _is_root(issuer) or issuer.external in kwargs.get('accounts')
 
 
-def perm_add_did(issuer, kwargs, session=None):
+def perm_add_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an data identifier to a scope.
 
@@ -414,7 +414,7 @@ def perm_add_did(issuer, kwargs, session=None):
             if rule['account'] != issuer:
                 return False
 
-    if kwargs['scope'].external != u'cms':
+    if kwargs['scope'].external != 'cms':
         if kwargs['type'] == 'DATASET':
             if '/USER#' not in kwargs['name']:
                 return False
@@ -425,10 +425,10 @@ def perm_add_did(issuer, kwargs, session=None):
     return (_is_root(issuer)
             or has_account_attribute(account=issuer, key='admin', session=session)  # NOQA: W503
             or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)  # NOQA: W503
-            or kwargs['scope'].external == u'mock')  # NOQA: W503
+            or kwargs['scope'].external == 'mock')  # NOQA: W503
 
 
-def perm_add_dids(issuer, kwargs, session=None):
+def perm_add_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk add data identifiers.
 
@@ -447,7 +447,7 @@ def perm_add_dids(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_attach_dids(issuer, kwargs, session=None):
+def perm_attach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -462,7 +462,7 @@ def perm_attach_dids(issuer, kwargs, session=None):
             or kwargs['scope'].external == 'mock')  # NOQA: W503
 
 
-def perm_attach_dids_to_dids(issuer, kwargs, session=None):
+def perm_attach_dids_to_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -483,7 +483,7 @@ def perm_attach_dids_to_dids(issuer, kwargs, session=None):
         return True
 
 
-def perm_create_did_sample(issuer, kwargs, session=None):
+def perm_create_did_sample(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can create a sample of a data identifier collection.
 
@@ -498,7 +498,7 @@ def perm_create_did_sample(issuer, kwargs, session=None):
                       or kwargs['scope'].external == 'mock')  # NOQA: W503
 
 
-def perm_del_rule(issuer, kwargs, session=None):
+def perm_del_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can delete a replication rule.
 
@@ -515,7 +515,7 @@ def perm_del_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_rule(issuer, kwargs, session=None):
+def perm_update_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can update a replication rule.
 
@@ -529,7 +529,7 @@ def perm_update_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_approve_rule(issuer, kwargs, session=None):
+def perm_approve_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can approve a replication rule.
 
@@ -554,7 +554,7 @@ def perm_approve_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_reduce_rule(issuer, kwargs, session=None):
+def perm_reduce_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can reduce a replication rule.
 
@@ -568,7 +568,7 @@ def perm_reduce_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_move_rule(issuer, kwargs, session=None):
+def perm_move_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can move a replication rule.
 
@@ -582,7 +582,7 @@ def perm_move_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_subscription(issuer, kwargs, session=None):
+def perm_update_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a subscription.
 
@@ -597,7 +597,7 @@ def perm_update_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_detach_dids(issuer, kwargs, session=None):
+def perm_detach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can detach an data identifier from the other data identifier.
 
@@ -609,7 +609,7 @@ def perm_detach_dids(issuer, kwargs, session=None):
     return perm_attach_dids(issuer, kwargs, session=session)
 
 
-def perm_set_metadata(issuer, kwargs, session=None):
+def perm_set_metadata(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a metadata on a data identifier.
 
@@ -622,7 +622,7 @@ def perm_set_metadata(issuer, kwargs, session=None):
             or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session))  # NOQA: W503
 
 
-def perm_set_status(issuer, kwargs, session=None):
+def perm_set_status(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set status on an data identifier.
 
@@ -639,7 +639,7 @@ def perm_set_status(issuer, kwargs, session=None):
             or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session))  # NOQA: W503
 
 
-def perm_add_protocol(issuer, kwargs, session=None):
+def perm_add_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a protocol to an RSE.
 
@@ -651,7 +651,7 @@ def perm_add_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_protocol(issuer, kwargs, session=None):
+def perm_del_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete protocols from an RSE.
 
@@ -663,7 +663,7 @@ def perm_del_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_protocol(issuer, kwargs, session=None):
+def perm_update_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update protocols of an RSE.
 
@@ -675,7 +675,7 @@ def perm_update_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_qos_policy(issuer, kwargs, session=None):
+def perm_add_qos_policy(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add QoS policies to an RSE.
 
@@ -687,7 +687,7 @@ def perm_add_qos_policy(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_qos_policy(issuer, kwargs, session=None):
+def perm_delete_qos_policy(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete QoS policies from an RSE.
 
@@ -699,7 +699,7 @@ def perm_delete_qos_policy(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
+def perm_declare_bad_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad file replicas.
 
@@ -711,7 +711,7 @@ def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
+def perm_declare_suspicious_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare suspicious file replicas.
 
@@ -723,7 +723,7 @@ def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
     return True
 
 
-def perm_add_replicas(issuer, kwargs, session=None):
+def perm_add_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add replicas.
 
@@ -740,7 +740,7 @@ def perm_add_replicas(issuer, kwargs, session=None):
     return is_root or is_temp or is_admin
 
 
-def perm_skip_availability_check(issuer, kwargs, session=None):
+def perm_skip_availability_check(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can skip the availabity check to add/delete file replicas.
 
@@ -752,7 +752,7 @@ def perm_skip_availability_check(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_replicas(issuer, kwargs, session=None):
+def perm_delete_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -767,7 +767,7 @@ def perm_delete_replicas(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_replicas_states(issuer, kwargs, session=None):
+def perm_update_replicas_states(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -779,7 +779,7 @@ def perm_update_replicas_states(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_queue_requests(issuer, kwargs, session=None):
+def perm_queue_requests(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can submit transfer or deletion requests on destination RSEs for data identifiers.
 
@@ -791,7 +791,7 @@ def perm_queue_requests(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_get_request_by_did(issuer, kwargs, session=None):
+def perm_get_request_by_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get a request by DID.
 
@@ -803,7 +803,7 @@ def perm_get_request_by_did(issuer, kwargs, session=None):
     return True
 
 
-def perm_cancel_request(issuer, kwargs, session=None):
+def perm_cancel_request(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can cancel a request.
 
@@ -815,7 +815,7 @@ def perm_cancel_request(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_get_next(issuer, kwargs, session=None):
+def perm_get_next(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can retrieve the next request matching the request type and state.
 
@@ -827,7 +827,7 @@ def perm_get_next(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_usage(issuer, kwargs, session=None):
+def perm_set_rse_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE usage information.
 
@@ -839,7 +839,7 @@ def perm_set_rse_usage(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_limits(issuer, kwargs, session=None):
+def perm_set_rse_limits(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE limits.
 
@@ -851,7 +851,7 @@ def perm_set_rse_limits(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_set_local_account_limit(issuer, kwargs, session=None):
+def perm_set_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set an account limit.
 
@@ -880,7 +880,7 @@ def perm_set_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_set_global_account_limit(issuer, kwargs, session=None):
+def perm_set_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a global account limit.
 
@@ -903,7 +903,7 @@ def perm_set_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_global_account_limit(issuer, kwargs, session=None):
+def perm_delete_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a global account limit.
 
@@ -927,7 +927,7 @@ def perm_delete_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_local_account_limit(issuer, kwargs, session=None):
+def perm_delete_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an account limit.
 
@@ -954,7 +954,7 @@ def perm_delete_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_config(issuer, kwargs, session=None):
+def perm_config(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can read/write the configuration.
 
@@ -966,7 +966,7 @@ def perm_config(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_local_account_usage(issuer, kwargs, session=None):
+def perm_get_local_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -984,7 +984,7 @@ def perm_get_local_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_attribute(issuer, kwargs, session=None):
+def perm_add_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -996,7 +996,7 @@ def perm_add_account_attribute(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_attribute(issuer, kwargs, session=None):
+def perm_del_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -1008,7 +1008,7 @@ def perm_del_account_attribute(issuer, kwargs, session=None):
     return perm_add_account_attribute(issuer, kwargs, session=session)
 
 
-def perm_list_heartbeats(issuer, kwargs, session=None):
+def perm_list_heartbeats(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list heartbeats.
 
@@ -1020,7 +1020,7 @@ def perm_list_heartbeats(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_resurrect(issuer, kwargs, session=None):
+def perm_resurrect(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can resurrect DIDS.
 
@@ -1032,7 +1032,7 @@ def perm_resurrect(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
+def perm_update_lifetime_exceptions(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can approve/reject Lifetime Model exceptions.
 
@@ -1044,9 +1044,9 @@ def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
+def perm_get_auth_token_ssh(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
-    Checks if an account can request a challenge token.
+    Checks if an account can request an ssh token.
 
     :param issuer: Account identifier which issues the command.
     :param kwargs: List of arguments for the action.
@@ -1056,7 +1056,7 @@ def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
     return True
 
 
-def perm_get_signed_url(issuer, kwargs, session=None):
+def perm_get_signed_url(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can request a signed URL.
 
@@ -1067,7 +1067,7 @@ def perm_get_signed_url(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_add_bad_pfns(issuer, kwargs, session=None):
+def perm_add_bad_pfns(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad PFNs.
 
@@ -1079,7 +1079,7 @@ def perm_add_bad_pfns(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_remove_did_from_followed(issuer, kwargs, session=None):
+def perm_remove_did_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can remove did from followed table.
 
@@ -1094,7 +1094,7 @@ def perm_remove_did_from_followed(issuer, kwargs, session=None):
             or kwargs['scope'].external == 'mock')  # NOQA: W503
 
 
-def perm_remove_dids_from_followed(issuer, kwargs, session=None):
+def perm_remove_dids_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk remove dids from followed table.
 
@@ -1110,7 +1110,7 @@ def perm_remove_dids_from_followed(issuer, kwargs, session=None):
     return True
 
 
-def perm_add_vo(issuer, kwargs, session=None):
+def perm_add_vo(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a VO.
     :param issuer: Account identifier which issues the command.
@@ -1121,7 +1121,7 @@ def perm_add_vo(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_list_vos(issuer, kwargs, session=None):
+def perm_list_vos(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list a VO.
     :param issuer: Account identifier which issues the command.
@@ -1133,7 +1133,7 @@ def perm_list_vos(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_recover_vo_root_identity(issuer, kwargs, session=None):
+def perm_recover_vo_root_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can recover identities for VOs.
     :param issuer: Account identifier which issues the command.
@@ -1144,7 +1144,7 @@ def perm_recover_vo_root_identity(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_update_vo(issuer, kwargs, session=None):
+def perm_update_vo(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a VO.
     :param issuer: Account identifier which issues the command.
@@ -1155,7 +1155,7 @@ def perm_update_vo(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_access_rule_vo(issuer, kwargs, session=None):
+def perm_access_rule_vo(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if we're at the same VO as the rule_id's
     :param issuer: Account identifier which issues the command.
