@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING
+
 import rucio.core.scope
 from rucio.core.account import list_account_attributes, has_account_attribute
 from rucio.core.identity import exist_identity_account
@@ -21,8 +23,13 @@ from rucio.core.rse import list_rse_attributes
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.db.sqla.constants import IdentityType
 
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
+    from rucio.common.types import InternalAccount
 
-def has_permission(issuer, action, kwargs, session=None):
+
+def has_permission(issuer, action, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account has the specified permission to
     execute an action with parameters.
@@ -101,7 +108,7 @@ def has_permission(issuer, action, kwargs, session=None):
             'list_heartbeats': perm_list_heartbeats,
             'resurrect': perm_resurrect,
             'update_lifetime_exceptions': perm_update_lifetime_exceptions,
-            'get_ssh_challenge_token': perm_get_ssh_challenge_token,
+            'get_auth_token_ssh': perm_get_auth_token_ssh,
             'get_signed_url': perm_get_signed_url,
             'add_bad_pfns': perm_add_bad_pfns,
             'del_account_identity': perm_del_account_identity,
@@ -117,7 +124,7 @@ def _is_root(issuer):
     return issuer.external == 'root'
 
 
-def perm_default(issuer, kwargs, session=None):
+def perm_default(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Default permission.
 
@@ -129,7 +136,7 @@ def perm_default(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rse(issuer, kwargs, session=None):
+def perm_add_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE.
 
@@ -141,7 +148,7 @@ def perm_add_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_rse(issuer, kwargs, session=None):
+def perm_update_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a RSE.
 
@@ -153,7 +160,7 @@ def perm_update_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rule(issuer, kwargs, session=None):
+def perm_add_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a replication rule.
 
@@ -169,7 +176,7 @@ def perm_add_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_subscription(issuer, kwargs, session=None):
+def perm_add_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a subscription.
 
@@ -183,7 +190,7 @@ def perm_add_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_rse_attribute(issuer, kwargs, session=None):
+def perm_add_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE attribute.
 
@@ -197,7 +204,7 @@ def perm_add_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse_attribute(issuer, kwargs, session=None):
+def perm_del_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE attribute.
 
@@ -211,7 +218,7 @@ def perm_del_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse(issuer, kwargs, session=None):
+def perm_del_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE.
 
@@ -223,7 +230,7 @@ def perm_del_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_account(issuer, kwargs, session=None):
+def perm_add_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an account.
 
@@ -235,7 +242,7 @@ def perm_add_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_del_account(issuer, kwargs, session=None):
+def perm_del_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can del an account.
 
@@ -247,7 +254,7 @@ def perm_del_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_update_account(issuer, kwargs, session=None):
+def perm_update_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update an account.
 
@@ -259,7 +266,7 @@ def perm_update_account(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_scope(issuer, kwargs, session=None):
+def perm_add_scope(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a scop to a account.
 
@@ -268,10 +275,10 @@ def perm_add_scope(issuer, kwargs, session=None):
     :param session: The DB session to use
     :returns: True if account is allowed, otherwise False
     """
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
+def perm_get_auth_token_user_pass(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -285,7 +292,7 @@ def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_gss(issuer, kwargs, session=None):
+def perm_get_auth_token_gss(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -299,7 +306,7 @@ def perm_get_auth_token_gss(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_x509(issuer, kwargs, session=None):
+def perm_get_auth_token_x509(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -313,7 +320,7 @@ def perm_get_auth_token_x509(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_saml(issuer, kwargs, session=None):
+def perm_get_auth_token_saml(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -327,7 +334,7 @@ def perm_get_auth_token_saml(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_identity(issuer, kwargs, session=None):
+def perm_add_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an identity to an account.
 
@@ -337,10 +344,10 @@ def perm_add_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_identity(issuer, kwargs, session=None):
+def perm_del_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity to an account.
 
@@ -350,10 +357,10 @@ def perm_del_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_identity(issuer, kwargs, session=None):
+def perm_del_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity.
 
@@ -366,7 +373,7 @@ def perm_del_identity(issuer, kwargs, session=None):
     return _is_root(issuer) or issuer.external in kwargs.get('accounts')
 
 
-def perm_add_did(issuer, kwargs, session=None):
+def perm_add_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an data identifier to a scope.
 
@@ -384,10 +391,10 @@ def perm_add_did(issuer, kwargs, session=None):
     return _is_root(issuer)\
         or has_account_attribute(account=issuer, key='admin', session=session)\
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)\
-        or kwargs['scope'].external == u'mock'
+        or kwargs['scope'].external == 'mock'
 
 
-def perm_add_dids(issuer, kwargs, session=None):
+def perm_add_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk add data identifiers.
 
@@ -406,7 +413,7 @@ def perm_add_dids(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_attach_dids(issuer, kwargs, session=None):
+def perm_attach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -421,7 +428,7 @@ def perm_attach_dids(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_attach_dids_to_dids(issuer, kwargs, session=None):
+def perm_attach_dids_to_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -442,7 +449,7 @@ def perm_attach_dids_to_dids(issuer, kwargs, session=None):
         return True
 
 
-def perm_create_did_sample(issuer, kwargs, session=None):
+def perm_create_did_sample(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can create a sample of a data identifier collection.
 
@@ -457,7 +464,7 @@ def perm_create_did_sample(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_del_rule(issuer, kwargs, session=None):
+def perm_del_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can delete a replication rule.
 
@@ -471,7 +478,7 @@ def perm_del_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_rule(issuer, kwargs, session=None):
+def perm_update_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can update a replication rule.
 
@@ -485,7 +492,7 @@ def perm_update_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_approve_rule(issuer, kwargs, session=None):
+def perm_approve_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can approve a replication rule.
 
@@ -499,7 +506,7 @@ def perm_approve_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_reduce_rule(issuer, kwargs, session=None):
+def perm_reduce_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can reduce a replication rule.
 
@@ -513,7 +520,7 @@ def perm_reduce_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_move_rule(issuer, kwargs, session=None):
+def perm_move_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can move a replication rule.
 
@@ -527,7 +534,7 @@ def perm_move_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_subscription(issuer, kwargs, session=None):
+def perm_update_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a subscription.
 
@@ -542,7 +549,7 @@ def perm_update_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_detach_dids(issuer, kwargs, session=None):
+def perm_detach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can detach an data identifier from the other data identifier.
 
@@ -554,7 +561,7 @@ def perm_detach_dids(issuer, kwargs, session=None):
     return perm_attach_dids(issuer, kwargs, session=session)
 
 
-def perm_set_metadata(issuer, kwargs, session=None):
+def perm_set_metadata(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a metadata on a data identifier.
 
@@ -566,7 +573,7 @@ def perm_set_metadata(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_set_status(issuer, kwargs, session=None):
+def perm_set_status(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set status on an data identifier.
 
@@ -582,7 +589,7 @@ def perm_set_status(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_add_protocol(issuer, kwargs, session=None):
+def perm_add_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a protocol to an RSE.
 
@@ -594,7 +601,7 @@ def perm_add_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_protocol(issuer, kwargs, session=None):
+def perm_del_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete protocols from an RSE.
 
@@ -606,7 +613,7 @@ def perm_del_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_protocol(issuer, kwargs, session=None):
+def perm_update_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update protocols of an RSE.
 
@@ -618,7 +625,7 @@ def perm_update_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_qos_policy(issuer, kwargs, session=None):
+def perm_add_qos_policy(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add QoS policies to an RSE.
 
@@ -630,7 +637,7 @@ def perm_add_qos_policy(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_qos_policy(issuer, kwargs, session=None):
+def perm_delete_qos_policy(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete QoS policies from an RSE.
 
@@ -642,7 +649,7 @@ def perm_delete_qos_policy(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
+def perm_declare_bad_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad file replicas.
 
@@ -654,7 +661,7 @@ def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
+def perm_declare_suspicious_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare suspicious file replicas.
 
@@ -666,7 +673,7 @@ def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
     return True
 
 
-def perm_add_replicas(issuer, kwargs, session=None):
+def perm_add_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add replicas.
 
@@ -683,7 +690,7 @@ def perm_add_replicas(issuer, kwargs, session=None):
         or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_skip_availability_check(issuer, kwargs, session=None):
+def perm_skip_availability_check(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can skip the availabity check to add/delete file replicas.
 
@@ -695,7 +702,7 @@ def perm_skip_availability_check(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_replicas(issuer, kwargs, session=None):
+def perm_delete_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -707,7 +714,7 @@ def perm_delete_replicas(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_replicas_states(issuer, kwargs, session=None):
+def perm_update_replicas_states(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -719,7 +726,7 @@ def perm_update_replicas_states(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_queue_requests(issuer, kwargs, session=None):
+def perm_queue_requests(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can submit transfer or deletion requests on destination RSEs for data identifiers.
 
@@ -731,7 +738,7 @@ def perm_queue_requests(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_get_request_by_did(issuer, kwargs, session=None):
+def perm_get_request_by_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get a request by DID.
 
@@ -743,7 +750,7 @@ def perm_get_request_by_did(issuer, kwargs, session=None):
     return True
 
 
-def perm_cancel_request(issuer, kwargs, session=None):
+def perm_cancel_request(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can cancel a request.
 
@@ -755,7 +762,7 @@ def perm_cancel_request(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_get_next(issuer, kwargs, session=None):
+def perm_get_next(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can retrieve the next request matching the request type and state.
 
@@ -767,7 +774,7 @@ def perm_get_next(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_usage(issuer, kwargs, session=None):
+def perm_set_rse_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE usage information.
 
@@ -779,7 +786,7 @@ def perm_set_rse_usage(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_limits(issuer, kwargs, session=None):
+def perm_set_rse_limits(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE limits.
 
@@ -791,7 +798,7 @@ def perm_set_rse_limits(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_set_local_account_limit(issuer, kwargs, session=None):
+def perm_set_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set an account limit.
 
@@ -812,7 +819,7 @@ def perm_set_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_set_global_account_limit(issuer, kwargs, session=None):
+def perm_set_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a global account limit.
 
@@ -835,7 +842,7 @@ def perm_set_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_local_account_limit(issuer, kwargs, session=None):
+def perm_delete_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an account limit.
 
@@ -856,7 +863,7 @@ def perm_delete_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_global_account_limit(issuer, kwargs, session=None):
+def perm_delete_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a global account limit.
 
@@ -880,7 +887,7 @@ def perm_delete_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_config(issuer, kwargs, session=None):
+def perm_config(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can read/write the configuration.
 
@@ -892,7 +899,7 @@ def perm_config(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_local_account_usage(issuer, kwargs, session=None):
+def perm_get_local_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -910,7 +917,7 @@ def perm_get_local_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_global_account_usage(issuer, kwargs, session=None):
+def perm_get_global_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -929,7 +936,7 @@ def perm_get_global_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_attribute(issuer, kwargs, session=None):
+def perm_add_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -941,7 +948,7 @@ def perm_add_account_attribute(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_attribute(issuer, kwargs, session=None):
+def perm_del_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -953,7 +960,7 @@ def perm_del_account_attribute(issuer, kwargs, session=None):
     return perm_add_account_attribute(issuer, kwargs, session=session)
 
 
-def perm_list_heartbeats(issuer, kwargs, session=None):
+def perm_list_heartbeats(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list heartbeats.
 
@@ -965,7 +972,7 @@ def perm_list_heartbeats(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_resurrect(issuer, kwargs, session=None):
+def perm_resurrect(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can resurrect DIDS.
 
@@ -977,7 +984,7 @@ def perm_resurrect(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
+def perm_update_lifetime_exceptions(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can approve/reject Lifetime Model exceptions.
 
@@ -992,7 +999,7 @@ def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
+def perm_get_auth_token_ssh(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can request a challenge token.
 
@@ -1003,7 +1010,7 @@ def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
     return True
 
 
-def perm_get_signed_url(issuer, kwargs, session=None):
+def perm_get_signed_url(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can request a signed URL.
 
@@ -1014,7 +1021,7 @@ def perm_get_signed_url(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='sign-gcs', session=session)
 
 
-def perm_add_bad_pfns(issuer, kwargs, session=None):
+def perm_add_bad_pfns(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad PFNs.
 
@@ -1026,7 +1033,7 @@ def perm_add_bad_pfns(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_remove_did_from_followed(issuer, kwargs, session=None):
+def perm_remove_did_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can remove did from followed table.
 
@@ -1041,7 +1048,7 @@ def perm_remove_did_from_followed(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_remove_dids_from_followed(issuer, kwargs, session=None):
+def perm_remove_dids_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk remove dids from followed table.
 
@@ -1057,7 +1064,7 @@ def perm_remove_dids_from_followed(issuer, kwargs, session=None):
     return True
 
 
-def perm_export(issuer, kwargs, session=None):
+def perm_export(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can export the RSE info.
 
