@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING
+
 import rucio.core.scope
 from rucio.core.account import list_account_attributes, has_account_attribute
 from rucio.core.identity import exist_identity_account
@@ -21,8 +23,13 @@ from rucio.core.rse import list_rse_attributes
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.db.sqla.constants import IdentityType
 
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
+    from rucio.common.types import InternalAccount
 
-def has_permission(issuer, action, kwargs, session=None):
+
+def has_permission(issuer, action, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account has the specified permission to
     execute an action with parameters.
@@ -73,6 +80,7 @@ def has_permission(issuer, action, kwargs, session=None):
             'attach_dids_to_dids': perm_attach_dids_to_dids,
             'create_did_sample': perm_create_did_sample,
             'set_metadata': perm_set_metadata,
+            'set_metadata_bulk': perm_set_metadata_bulk,
             'set_status': perm_set_status,
             'queue_requests': perm_queue_requests,
             'set_rse_usage': perm_set_rse_usage,
@@ -104,7 +112,7 @@ def has_permission(issuer, action, kwargs, session=None):
             'list_heartbeats': perm_list_heartbeats,
             'resurrect': perm_resurrect,
             'update_lifetime_exceptions': perm_update_lifetime_exceptions,
-            'get_ssh_challenge_token': perm_get_ssh_challenge_token,
+            'get_auth_token_ssh': perm_get_auth_token_ssh,
             'get_signed_url': perm_get_signed_url,
             'add_bad_pfns': perm_add_bad_pfns,
             'del_account_identity': perm_del_account_identity,
@@ -120,7 +128,7 @@ def _is_root(issuer):
     return issuer.external == 'root'
 
 
-def perm_default(issuer, kwargs, session=None):
+def perm_default(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Default permission.
 
@@ -132,7 +140,7 @@ def perm_default(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rse(issuer, kwargs, session=None):
+def perm_add_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE.
 
@@ -144,7 +152,7 @@ def perm_add_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_rse(issuer, kwargs, session=None):
+def perm_update_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a RSE.
 
@@ -156,7 +164,7 @@ def perm_update_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rule(issuer, kwargs, session=None):
+def perm_add_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a replication rule.
 
@@ -172,7 +180,7 @@ def perm_add_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_subscription(issuer, kwargs, session=None):
+def perm_add_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a subscription.
 
@@ -186,7 +194,7 @@ def perm_add_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_rse_attribute(issuer, kwargs, session=None):
+def perm_add_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE attribute.
 
@@ -200,7 +208,7 @@ def perm_add_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse_attribute(issuer, kwargs, session=None):
+def perm_del_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE attribute.
 
@@ -214,7 +222,7 @@ def perm_del_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse(issuer, kwargs, session=None):
+def perm_del_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE.
 
@@ -226,7 +234,7 @@ def perm_del_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_account(issuer, kwargs, session=None):
+def perm_add_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an account.
 
@@ -238,7 +246,7 @@ def perm_add_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_del_account(issuer, kwargs, session=None):
+def perm_del_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can del an account.
 
@@ -250,7 +258,7 @@ def perm_del_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_update_account(issuer, kwargs, session=None):
+def perm_update_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update an account.
 
@@ -262,7 +270,7 @@ def perm_update_account(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_scope(issuer, kwargs, session=None):
+def perm_add_scope(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a scop to a account.
 
@@ -271,10 +279,10 @@ def perm_add_scope(issuer, kwargs, session=None):
     :param session: The DB session to use
     :returns: True if account is allowed, otherwise False
     """
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
+def perm_get_auth_token_user_pass(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -288,7 +296,7 @@ def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_gss(issuer, kwargs, session=None):
+def perm_get_auth_token_gss(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -302,7 +310,7 @@ def perm_get_auth_token_gss(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_x509(issuer, kwargs, session=None):
+def perm_get_auth_token_x509(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -316,7 +324,7 @@ def perm_get_auth_token_x509(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_saml(issuer, kwargs, session=None):
+def perm_get_auth_token_saml(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -330,7 +338,7 @@ def perm_get_auth_token_saml(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_identity(issuer, kwargs, session=None):
+def perm_add_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an identity to an account.
 
@@ -340,10 +348,10 @@ def perm_add_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_identity(issuer, kwargs, session=None):
+def perm_del_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity to an account.
 
@@ -353,10 +361,10 @@ def perm_del_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_identity(issuer, kwargs, session=None):
+def perm_del_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity.
 
@@ -369,7 +377,7 @@ def perm_del_identity(issuer, kwargs, session=None):
     return _is_root(issuer) or issuer.external in kwargs.get('accounts')
 
 
-def perm_add_did(issuer, kwargs, session=None):
+def perm_add_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an data identifier to a scope.
 
@@ -387,10 +395,10 @@ def perm_add_did(issuer, kwargs, session=None):
     return _is_root(issuer)\
         or has_account_attribute(account=issuer, key='admin', session=session)\
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)\
-        or kwargs['scope'].external == u'mock'
+        or kwargs['scope'].external == 'mock'
 
 
-def perm_add_dids(issuer, kwargs, session=None):
+def perm_add_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk add data identifiers.
 
@@ -409,7 +417,7 @@ def perm_add_dids(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_attach_dids(issuer, kwargs, session=None):
+def perm_attach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -424,7 +432,7 @@ def perm_attach_dids(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_attach_dids_to_dids(issuer, kwargs, session=None):
+def perm_attach_dids_to_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -445,7 +453,7 @@ def perm_attach_dids_to_dids(issuer, kwargs, session=None):
         return True
 
 
-def perm_create_did_sample(issuer, kwargs, session=None):
+def perm_create_did_sample(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can create a sample of a data identifier collection.
 
@@ -460,7 +468,7 @@ def perm_create_did_sample(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_del_rule(issuer, kwargs, session=None):
+def perm_del_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can delete a replication rule.
 
@@ -474,7 +482,7 @@ def perm_del_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_rule(issuer, kwargs, session=None):
+def perm_update_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can update a replication rule.
 
@@ -488,7 +496,7 @@ def perm_update_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_approve_rule(issuer, kwargs, session=None):
+def perm_approve_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can approve a replication rule.
 
@@ -502,7 +510,7 @@ def perm_approve_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_reduce_rule(issuer, kwargs, session=None):
+def perm_reduce_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can reduce a replication rule.
 
@@ -516,7 +524,7 @@ def perm_reduce_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_move_rule(issuer, kwargs, session=None):
+def perm_move_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can move a replication rule.
 
@@ -530,7 +538,7 @@ def perm_move_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_subscription(issuer, kwargs, session=None):
+def perm_update_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a subscription.
 
@@ -545,7 +553,7 @@ def perm_update_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_detach_dids(issuer, kwargs, session=None):
+def perm_detach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can detach an data identifier from the other data identifier.
 
@@ -557,7 +565,7 @@ def perm_detach_dids(issuer, kwargs, session=None):
     return perm_attach_dids(issuer, kwargs, session=session)
 
 
-def perm_set_metadata(issuer, kwargs, session=None):
+def perm_set_metadata_bulk(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
     Checks if an account can set a metadata on a data identifier.
 
@@ -569,7 +577,19 @@ def perm_set_metadata(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_set_status(issuer, kwargs, session=None):
+def perm_set_metadata(issuer, kwargs, *, session: "Optional[Session]" = None):
+    """
+    Checks if an account can set a metadata on a data identifier.
+
+    :param issuer: Account identifier which issues the command.
+    :param kwargs: List of arguments for the action.
+    :param session: The DB session to use
+    :returns: True if account is allowed, otherwise False
+    """
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
+
+
+def perm_set_status(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set status on an data identifier.
 
@@ -585,7 +605,7 @@ def perm_set_status(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_add_protocol(issuer, kwargs, session=None):
+def perm_add_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a protocol to an RSE.
 
@@ -597,7 +617,7 @@ def perm_add_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_protocol(issuer, kwargs, session=None):
+def perm_del_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete protocols from an RSE.
 
@@ -609,7 +629,7 @@ def perm_del_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_protocol(issuer, kwargs, session=None):
+def perm_update_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update protocols of an RSE.
 
@@ -621,7 +641,7 @@ def perm_update_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_qos_policy(issuer, kwargs, session=None):
+def perm_add_qos_policy(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add QoS policies to an RSE.
 
@@ -633,7 +653,7 @@ def perm_add_qos_policy(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_qos_policy(issuer, kwargs, session=None):
+def perm_delete_qos_policy(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete QoS policies from an RSE.
 
@@ -645,7 +665,7 @@ def perm_delete_qos_policy(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
+def perm_declare_bad_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad file replicas.
 
@@ -657,7 +677,7 @@ def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
+def perm_declare_suspicious_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare suspicious file replicas.
 
@@ -669,7 +689,7 @@ def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
     return True
 
 
-def perm_add_replicas(issuer, kwargs, session=None):
+def perm_add_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add replicas.
 
@@ -686,7 +706,7 @@ def perm_add_replicas(issuer, kwargs, session=None):
         or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_skip_availability_check(issuer, kwargs, session=None):
+def perm_skip_availability_check(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can skip the availabity check to add/delete file replicas.
 
@@ -698,7 +718,7 @@ def perm_skip_availability_check(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_replicas(issuer, kwargs, session=None):
+def perm_delete_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -710,7 +730,7 @@ def perm_delete_replicas(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_replicas_states(issuer, kwargs, session=None):
+def perm_update_replicas_states(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -722,7 +742,7 @@ def perm_update_replicas_states(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_queue_requests(issuer, kwargs, session=None):
+def perm_queue_requests(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can submit transfer or deletion requests on destination RSEs for data identifiers.
 
@@ -734,7 +754,7 @@ def perm_queue_requests(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_list_requests(issuer, kwargs, session=None):
+def perm_list_requests(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list requests.
 
@@ -746,7 +766,7 @@ def perm_list_requests(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_list_requests_history(issuer, kwargs, session=None):
+def perm_list_requests_history(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list historical requests.
 
@@ -758,7 +778,7 @@ def perm_list_requests_history(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_request_by_did(issuer, kwargs, session=None):
+def perm_get_request_by_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get a request by DID.
 
@@ -770,7 +790,7 @@ def perm_get_request_by_did(issuer, kwargs, session=None):
     return True
 
 
-def perm_get_request_history_by_did(issuer, kwargs, session=None):
+def perm_get_request_history_by_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get a historical request by DID.
 
@@ -782,7 +802,7 @@ def perm_get_request_history_by_did(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_cancel_request(issuer, kwargs, session=None):
+def perm_cancel_request(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can cancel a request.
 
@@ -794,7 +814,7 @@ def perm_cancel_request(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_get_next(issuer, kwargs, session=None):
+def perm_get_next(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can retrieve the next request matching the request type and state.
 
@@ -806,7 +826,7 @@ def perm_get_next(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_usage(issuer, kwargs, session=None):
+def perm_set_rse_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE usage information.
 
@@ -818,7 +838,7 @@ def perm_set_rse_usage(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_limits(issuer, kwargs, session=None):
+def perm_set_rse_limits(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE limits.
 
@@ -830,7 +850,7 @@ def perm_set_rse_limits(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_set_local_account_limit(issuer, kwargs, session=None):
+def perm_set_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set an account limit.
 
@@ -851,7 +871,7 @@ def perm_set_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_set_global_account_limit(issuer, kwargs, session=None):
+def perm_set_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a global account limit.
 
@@ -874,7 +894,7 @@ def perm_set_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_local_account_limit(issuer, kwargs, session=None):
+def perm_delete_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an account limit.
 
@@ -895,7 +915,7 @@ def perm_delete_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_global_account_limit(issuer, kwargs, session=None):
+def perm_delete_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a global account limit.
 
@@ -919,7 +939,7 @@ def perm_delete_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_config(issuer, kwargs, session=None):
+def perm_config(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can read/write the configuration.
 
@@ -931,7 +951,7 @@ def perm_config(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_local_account_usage(issuer, kwargs, session=None):
+def perm_get_local_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -949,7 +969,7 @@ def perm_get_local_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_global_account_usage(issuer, kwargs, session=None):
+def perm_get_global_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -968,7 +988,7 @@ def perm_get_global_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_attribute(issuer, kwargs, session=None):
+def perm_add_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -980,7 +1000,7 @@ def perm_add_account_attribute(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_attribute(issuer, kwargs, session=None):
+def perm_del_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -992,7 +1012,7 @@ def perm_del_account_attribute(issuer, kwargs, session=None):
     return perm_add_account_attribute(issuer, kwargs, session=session)
 
 
-def perm_list_heartbeats(issuer, kwargs, session=None):
+def perm_list_heartbeats(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list heartbeats.
 
@@ -1004,7 +1024,7 @@ def perm_list_heartbeats(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_resurrect(issuer, kwargs, session=None):
+def perm_resurrect(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can resurrect DIDS.
 
@@ -1016,7 +1036,7 @@ def perm_resurrect(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
+def perm_update_lifetime_exceptions(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can approve/reject Lifetime Model exceptions.
 
@@ -1031,9 +1051,9 @@ def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
+def perm_get_auth_token_ssh(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
-    Checks if an account can request a challenge token.
+    Checks if an account can request an ssh token.
 
     :param issuer: Account identifier which issues the command.
     :param session: The DB session to use
@@ -1042,7 +1062,7 @@ def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
     return True
 
 
-def perm_get_signed_url(issuer, kwargs, session=None):
+def perm_get_signed_url(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can request a signed URL.
 
@@ -1053,7 +1073,7 @@ def perm_get_signed_url(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_add_bad_pfns(issuer, kwargs, session=None):
+def perm_add_bad_pfns(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad PFNs.
 
@@ -1065,7 +1085,7 @@ def perm_add_bad_pfns(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_remove_did_from_followed(issuer, kwargs, session=None):
+def perm_remove_did_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can remove did from followed table.
 
@@ -1080,7 +1100,7 @@ def perm_remove_did_from_followed(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_remove_dids_from_followed(issuer, kwargs, session=None):
+def perm_remove_dids_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk remove dids from followed table.
 
@@ -1096,7 +1116,7 @@ def perm_remove_dids_from_followed(issuer, kwargs, session=None):
     return True
 
 
-def perm_export(issuer, kwargs, session=None):
+def perm_export(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can export the RSE info.
 
