@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING
+
 import rucio.core.scope
 from rucio.core.account import list_account_attributes, has_account_attribute
 from rucio.core.identity import exist_identity_account
@@ -22,8 +24,13 @@ from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import get_rule
 from rucio.db.sqla.constants import IdentityType
 
+if TYPE_CHECKING:
+    from typing import Optional
+    from sqlalchemy.orm import Session
+    from rucio.common.types import InternalAccount
 
-def has_permission(issuer, action, kwargs, session=None):
+
+def has_permission(issuer, action, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account has the specified permission to
     execute an action with parameters.
@@ -103,7 +110,7 @@ def has_permission(issuer, action, kwargs, session=None):
             'list_heartbeats': perm_list_heartbeats,
             'resurrect': perm_resurrect,
             'update_lifetime_exceptions': perm_update_lifetime_exceptions,
-            'get_ssh_challenge_token': perm_get_ssh_challenge_token,
+            'get_auth_token_ssh': perm_get_auth_token_ssh,
             'get_signed_url': perm_get_signed_url,
             'add_bad_pfns': perm_add_bad_pfns,
             'del_account_identity': perm_del_account_identity,
@@ -123,7 +130,7 @@ def _is_root(issuer):
     return issuer.external == 'root'
 
 
-def perm_default(issuer, kwargs, session=None):
+def perm_default(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Default permission.
 
@@ -135,7 +142,7 @@ def perm_default(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rse(issuer, kwargs, session=None):
+def perm_add_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE.
 
@@ -147,7 +154,7 @@ def perm_add_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_rse(issuer, kwargs, session=None):
+def perm_update_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a RSE.
 
@@ -159,7 +166,7 @@ def perm_update_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_rule(issuer, kwargs, session=None):
+def perm_add_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a replication rule.
 
@@ -175,7 +182,7 @@ def perm_add_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_subscription(issuer, kwargs, session=None):
+def perm_add_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a subscription.
 
@@ -189,7 +196,7 @@ def perm_add_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_rse_attribute(issuer, kwargs, session=None):
+def perm_add_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a RSE attribute.
 
@@ -203,7 +210,7 @@ def perm_add_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse_attribute(issuer, kwargs, session=None):
+def perm_del_rse_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE attribute.
 
@@ -217,7 +224,7 @@ def perm_del_rse_attribute(issuer, kwargs, session=None):
     return False
 
 
-def perm_del_rse(issuer, kwargs, session=None):
+def perm_del_rse(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a RSE.
 
@@ -229,7 +236,7 @@ def perm_del_rse(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_account(issuer, kwargs, session=None):
+def perm_add_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an account.
 
@@ -241,7 +248,7 @@ def perm_add_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_del_account(issuer, kwargs, session=None):
+def perm_del_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can del an account.
 
@@ -253,7 +260,7 @@ def perm_del_account(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_update_account(issuer, kwargs, session=None):
+def perm_update_account(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update an account.
 
@@ -265,7 +272,7 @@ def perm_update_account(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_add_scope(issuer, kwargs, session=None):
+def perm_add_scope(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a scop to a account.
 
@@ -274,10 +281,10 @@ def perm_add_scope(issuer, kwargs, session=None):
     :param session: The DB session to use
     :returns: True if account is allowed, otherwise False
     """
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
+def perm_get_auth_token_user_pass(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -291,7 +298,7 @@ def perm_get_auth_token_user_pass(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_gss(issuer, kwargs, session=None):
+def perm_get_auth_token_gss(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -305,7 +312,7 @@ def perm_get_auth_token_gss(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_x509(issuer, kwargs, session=None):
+def perm_get_auth_token_x509(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -319,7 +326,7 @@ def perm_get_auth_token_x509(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_auth_token_saml(issuer, kwargs, session=None):
+def perm_get_auth_token_saml(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if a user can request a token with user_pass for an account.
 
@@ -333,7 +340,7 @@ def perm_get_auth_token_saml(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_identity(issuer, kwargs, session=None):
+def perm_add_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an identity to an account.
 
@@ -343,10 +350,10 @@ def perm_add_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_identity(issuer, kwargs, session=None):
+def perm_del_account_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity to an account.
 
@@ -356,10 +363,10 @@ def perm_del_account_identity(issuer, kwargs, session=None):
     :returns: True if account is allowed, otherwise False
     """
 
-    return _is_root(issuer) or issuer == kwargs.get('account')
+    return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_identity(issuer, kwargs, session=None):
+def perm_del_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an identity.
 
@@ -372,7 +379,7 @@ def perm_del_identity(issuer, kwargs, session=None):
     return _is_root(issuer) or issuer.external in kwargs.get('accounts')
 
 
-def perm_add_did(issuer, kwargs, session=None):
+def perm_add_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add an data identifier to a scope.
 
@@ -390,10 +397,10 @@ def perm_add_did(issuer, kwargs, session=None):
     return _is_root(issuer)\
         or has_account_attribute(account=issuer, key='admin', session=session)\
         or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)\
-        or kwargs['scope'].external == u'mock'
+        or kwargs['scope'].external == 'mock'
 
 
-def perm_add_dids(issuer, kwargs, session=None):
+def perm_add_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk add data identifiers.
 
@@ -412,7 +419,7 @@ def perm_add_dids(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_attach_dids(issuer, kwargs, session=None):
+def perm_attach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -427,7 +434,7 @@ def perm_attach_dids(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_attach_dids_to_dids(issuer, kwargs, session=None):
+def perm_attach_dids_to_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can append an data identifier to the other data identifier.
 
@@ -448,7 +455,7 @@ def perm_attach_dids_to_dids(issuer, kwargs, session=None):
         return True
 
 
-def perm_create_did_sample(issuer, kwargs, session=None):
+def perm_create_did_sample(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can create a sample of a data identifier collection.
 
@@ -463,7 +470,7 @@ def perm_create_did_sample(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_del_rule(issuer, kwargs, session=None):
+def perm_del_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can delete a replication rule.
 
@@ -477,7 +484,7 @@ def perm_del_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_rule(issuer, kwargs, session=None):
+def perm_update_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can update a replication rule.
 
@@ -491,7 +498,7 @@ def perm_update_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_approve_rule(issuer, kwargs, session=None):
+def perm_approve_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can approve a replication rule.
 
@@ -505,7 +512,7 @@ def perm_approve_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_reduce_rule(issuer, kwargs, session=None):
+def perm_reduce_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can reduce a replication rule.
 
@@ -519,7 +526,7 @@ def perm_reduce_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_move_rule(issuer, kwargs, session=None):
+def perm_move_rule(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an issuer can move a replication rule.
 
@@ -533,7 +540,7 @@ def perm_move_rule(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_subscription(issuer, kwargs, session=None):
+def perm_update_subscription(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a subscription.
 
@@ -548,7 +555,7 @@ def perm_update_subscription(issuer, kwargs, session=None):
     return False
 
 
-def perm_detach_dids(issuer, kwargs, session=None):
+def perm_detach_dids(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can detach an data identifier from the other data identifier.
 
@@ -560,7 +567,7 @@ def perm_detach_dids(issuer, kwargs, session=None):
     return perm_attach_dids(issuer, kwargs, session=session)
 
 
-def perm_set_metadata(issuer, kwargs, session=None):
+def perm_set_metadata(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a metadata on a data identifier.
 
@@ -572,7 +579,7 @@ def perm_set_metadata(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_set_status(issuer, kwargs, session=None):
+def perm_set_status(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set status on an data identifier.
 
@@ -588,7 +595,7 @@ def perm_set_status(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session) or rucio.core.scope.is_scope_owner(scope=kwargs['scope'], account=issuer, session=session)
 
 
-def perm_add_protocol(issuer, kwargs, session=None):
+def perm_add_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a protocol to an RSE.
 
@@ -600,7 +607,7 @@ def perm_add_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_protocol(issuer, kwargs, session=None):
+def perm_del_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete protocols from an RSE.
 
@@ -612,7 +619,7 @@ def perm_del_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_protocol(issuer, kwargs, session=None):
+def perm_update_protocol(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update protocols of an RSE.
 
@@ -624,7 +631,7 @@ def perm_update_protocol(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
+def perm_declare_bad_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad file replicas.
 
@@ -636,7 +643,7 @@ def perm_declare_bad_file_replicas(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
+def perm_declare_suspicious_file_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare suspicious file replicas.
 
@@ -648,7 +655,7 @@ def perm_declare_suspicious_file_replicas(issuer, kwargs, session=None):
     return True
 
 
-def perm_add_replicas(issuer, kwargs, session=None):
+def perm_add_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add replicas.
 
@@ -665,7 +672,7 @@ def perm_add_replicas(issuer, kwargs, session=None):
         or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_skip_availability_check(issuer, kwargs, session=None):
+def perm_skip_availability_check(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can skip the availabity check to add/delete file replicas.
 
@@ -677,7 +684,7 @@ def perm_skip_availability_check(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_delete_replicas(issuer, kwargs, session=None):
+def perm_delete_replicas(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -689,7 +696,7 @@ def perm_delete_replicas(issuer, kwargs, session=None):
     return False
 
 
-def perm_update_replicas_states(issuer, kwargs, session=None):
+def perm_update_replicas_states(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete replicas.
 
@@ -701,7 +708,7 @@ def perm_update_replicas_states(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_queue_requests(issuer, kwargs, session=None):
+def perm_queue_requests(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can submit transfer or deletion requests on destination RSEs for data identifiers.
 
@@ -713,7 +720,7 @@ def perm_queue_requests(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_list_requests(issuer, kwargs, session=None):
+def perm_list_requests(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list requests.
 
@@ -725,7 +732,7 @@ def perm_list_requests(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_list_requests_history(issuer, kwargs, session=None):
+def perm_list_requests_history(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list historical requests.
 
@@ -737,7 +744,7 @@ def perm_list_requests_history(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_request_by_did(issuer, kwargs, session=None):
+def perm_get_request_by_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get a request by DID.
 
@@ -749,7 +756,7 @@ def perm_get_request_by_did(issuer, kwargs, session=None):
     return True
 
 
-def perm_get_request_history_by_did(issuer, kwargs, session=None):
+def perm_get_request_history_by_did(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get a historical request by DID.
 
@@ -761,7 +768,7 @@ def perm_get_request_history_by_did(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_cancel_request(issuer, kwargs, session=None):
+def perm_cancel_request(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can cancel a request.
 
@@ -773,7 +780,7 @@ def perm_cancel_request(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_get_next(issuer, kwargs, session=None):
+def perm_get_next(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can retrieve the next request matching the request type and state.
 
@@ -785,7 +792,7 @@ def perm_get_next(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_usage(issuer, kwargs, session=None):
+def perm_set_rse_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE usage information.
 
@@ -797,7 +804,7 @@ def perm_set_rse_usage(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_set_rse_limits(issuer, kwargs, session=None):
+def perm_set_rse_limits(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set RSE limits.
 
@@ -809,7 +816,7 @@ def perm_set_rse_limits(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_set_local_account_limit(issuer, kwargs, session=None):
+def perm_set_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set an account limit.
 
@@ -830,7 +837,7 @@ def perm_set_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_set_global_account_limit(issuer, kwargs, session=None):
+def perm_set_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can set a global account limit.
 
@@ -853,7 +860,7 @@ def perm_set_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_local_account_limit(issuer, kwargs, session=None):
+def perm_delete_local_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete an account limit.
 
@@ -874,7 +881,7 @@ def perm_delete_local_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_delete_global_account_limit(issuer, kwargs, session=None):
+def perm_delete_global_account_limit(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can delete a global account limit.
 
@@ -898,7 +905,7 @@ def perm_delete_global_account_limit(issuer, kwargs, session=None):
     return False
 
 
-def perm_config(issuer, kwargs, session=None):
+def perm_config(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can read/write the configuration.
 
@@ -910,7 +917,7 @@ def perm_config(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_local_account_usage(issuer, kwargs, session=None):
+def perm_get_local_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -928,7 +935,7 @@ def perm_get_local_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_get_global_account_usage(issuer, kwargs, session=None):
+def perm_get_global_account_usage(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can get the account usage of an account.
 
@@ -953,7 +960,7 @@ def perm_get_global_account_usage(issuer, kwargs, session=None):
     return False
 
 
-def perm_add_account_attribute(issuer, kwargs, session=None):
+def perm_add_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -965,7 +972,7 @@ def perm_add_account_attribute(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_del_account_attribute(issuer, kwargs, session=None):
+def perm_del_account_attribute(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add attributes to accounts.
 
@@ -977,7 +984,7 @@ def perm_del_account_attribute(issuer, kwargs, session=None):
     return perm_add_account_attribute(issuer, kwargs, session=session)
 
 
-def perm_list_heartbeats(issuer, kwargs, session=None):
+def perm_list_heartbeats(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list heartbeats.
 
@@ -989,7 +996,7 @@ def perm_list_heartbeats(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_resurrect(issuer, kwargs, session=None):
+def perm_resurrect(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can resurrect DIDS.
 
@@ -1001,7 +1008,7 @@ def perm_resurrect(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
+def perm_update_lifetime_exceptions(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can approve/reject Lifetime Model exceptions.
 
@@ -1016,9 +1023,9 @@ def perm_update_lifetime_exceptions(issuer, kwargs, session=None):
     return _is_root(issuer) or has_account_attribute(account=issuer, key='admin', session=session)
 
 
-def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
+def perm_get_auth_token_ssh(issuer: "InternalAccount", kwargs: dict, *, session: "Optional[Session]" = None) -> bool:
     """
-    Checks if an account can request a challenge token.
+    Checks if an account can request an ssh token.
 
     :param issuer: Account identifier which issues the command.
     :param session: The DB session to use
@@ -1027,7 +1034,7 @@ def perm_get_ssh_challenge_token(issuer, kwargs, session=None):
     return True
 
 
-def perm_get_signed_url(issuer, kwargs, session=None):
+def perm_get_signed_url(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can request a signed URL.
 
@@ -1038,7 +1045,7 @@ def perm_get_signed_url(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_add_bad_pfns(issuer, kwargs, session=None):
+def perm_add_bad_pfns(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can declare bad PFNs.
 
@@ -1050,7 +1057,7 @@ def perm_add_bad_pfns(issuer, kwargs, session=None):
     return _is_root(issuer)
 
 
-def perm_remove_did_from_followed(issuer, kwargs, session=None):
+def perm_remove_did_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can remove did from followed table.
 
@@ -1065,7 +1072,7 @@ def perm_remove_did_from_followed(issuer, kwargs, session=None):
         or kwargs['scope'].external == 'mock'
 
 
-def perm_remove_dids_from_followed(issuer, kwargs, session=None):
+def perm_remove_dids_from_followed(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can bulk remove dids from followed table.
 
@@ -1081,7 +1088,7 @@ def perm_remove_dids_from_followed(issuer, kwargs, session=None):
     return True
 
 
-def perm_add_vo(issuer, kwargs, session=None):
+def perm_add_vo(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can add a VO.
 
@@ -1093,7 +1100,7 @@ def perm_add_vo(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_list_vos(issuer, kwargs, session=None):
+def perm_list_vos(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can list a VO.
 
@@ -1105,7 +1112,7 @@ def perm_list_vos(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_recover_vo_root_identity(issuer, kwargs, session=None):
+def perm_recover_vo_root_identity(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can recover identities for VOs.
 
@@ -1117,7 +1124,7 @@ def perm_recover_vo_root_identity(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_update_vo(issuer, kwargs, session=None):
+def perm_update_vo(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if an account can update a VO.
 
@@ -1129,7 +1136,7 @@ def perm_update_vo(issuer, kwargs, session=None):
     return (issuer.internal == 'super_root')
 
 
-def perm_access_rule_vo(issuer, kwargs, session=None):
+def perm_access_rule_vo(issuer, kwargs, *, session: "Optional[Session]" = None):
     """
     Checks if we're at the same VO as the rule_id's
 
