@@ -14,17 +14,21 @@
 # limitations under the License.
 
 import datetime
+from typing import TYPE_CHECKING
 
 from sqlalchemy import and_, or_
-from sqlalchemy.sql.expression import false
+from sqlalchemy.sql.expression import false, insert
 
+from rucio.common.utils import chunks
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.session import read_session, transactional_session
-from rucio.common.utils import chunks
+
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
 
 
 @transactional_session
-def add_quarantined_replicas(rse_id, replicas, session=None):
+def add_quarantined_replicas(rse_id, replicas, *, session: "Session"):
     """
     Bulk add quarantined file replicas.
 
@@ -62,8 +66,8 @@ def add_quarantined_replicas(rse_id, replicas, session=None):
     quarantine_replicas = [(path, rseid) for path, rseid in quarantine_query]
     replicas = [replica for replica in replicas if (replica['path'], rse_id) not in quarantine_replicas]
 
-    session.bulk_insert_mappings(
-        models.QuarantinedReplica,
+    session.execute(
+        insert(models.QuarantinedReplica),
         [{'rse_id': rse_id,
           'path': file['path'],
           'scope': file.get('scope'),
@@ -73,7 +77,7 @@ def add_quarantined_replicas(rse_id, replicas, session=None):
 
 
 @transactional_session
-def delete_quarantined_replicas(rse_id, replicas, session=None):
+def delete_quarantined_replicas(rse_id, replicas, *, session: "Session"):
     """
     Delete file replicas.
 
@@ -92,17 +96,19 @@ def delete_quarantined_replicas(rse_id, replicas, session=None):
             filter(or_(*conditions)).\
             delete(synchronize_session=False)
 
-    session.\
-        bulk_insert_mappings(models.QuarantinedReplicaHistory,
-                             [{'rse_id': rse_id, 'path': replica['path'],
-                               'bytes': replica.get('bytes'),
-                               'created_at': replica.get('created_at'),
-                               'deleted_at': datetime.datetime.utcnow()}
-                              for replica in replicas])
+    if replicas:
+        session.execute(
+            insert(models.QuarantinedReplicaHistory),
+            [{'rse_id': rse_id, 'path': replica['path'],
+              'bytes': replica.get('bytes'),
+              'created_at': replica.get('created_at'),
+              'deleted_at': datetime.datetime.utcnow()}
+             for replica in replicas]
+        )
 
 
 @read_session
-def list_quarantined_replicas(rse_id, limit, worker_number=None, total_workers=None, session=None):
+def list_quarantined_replicas(rse_id, limit, worker_number=None, total_workers=None, *, session: "Session"):
     """
     List RSE Quarantined File replicas.
 
@@ -165,7 +171,7 @@ def list_quarantined_replicas(rse_id, limit, worker_number=None, total_workers=N
 
 
 @read_session
-def list_rses_with_quarantined_replicas(filters=None, session=None):
+def list_rses_with_quarantined_replicas(filters=None, *, session: "Session"):
     """
     List RSEs in the Quarantined Queues.
 
