@@ -15,15 +15,19 @@
 
 import abc
 import re
+from hashlib import sha256
+from typing import TYPE_CHECKING
 
 from dogpile.cache.api import NoValue
-from hashlib import sha256
 
 from rucio.common.cache import make_region_memcached
 from rucio.common.exception import InvalidRSEExpression, RSEWriteBlocked
 from rucio.core.rse import list_rses, get_rses_with_attribute, get_rse_attribute
 from rucio.db.sqla.session import transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 DEFAULT_RSE_ATTRIBUTE = r'([A-Za-z0-9]+([_-][A-Za-z0-9]+)*)'
 RSE_ATTRIBUTE = r'([A-Za-z0-9\._-]+[=<>][A-Za-z0-9_-]+)'
@@ -38,12 +42,12 @@ REGION = make_region_memcached(expiration_time=600)
 
 
 @transactional_session
-def parse_expression(expression, filter_=None, session=None):
+def parse_expression(expression, filter_=None, *, session: "Session"):
     """
     Parse a RSE expression and return the list of RSE dictionaries.
 
     :param expression:    RSE expression, e.g: 'CERN|BNL'.
-    :param filter_:        Availability filter (dictionary) used for the RSEs. e.g.: {'availability_write': True}
+    :param filter_:       Availability filter (dictionary) used for the RSEs. e.g.: {'availability_write': True}
     :param session:       Database session in use.
     :returns:             A list of rse dictionaries.
     :raises:              InvalidRSEExpression, RSENotFound, RSEWriteBlocked
@@ -96,7 +100,7 @@ def parse_expression(expression, filter_=None, session=None):
     if filter_:
         for rse in vo_result:
             if filter_.get('availability_write', False):
-                if rse.get('availability') & 2:
+                if rse.get('availability_write'):
                     final_result.append(rse)
         if not final_result:
             raise RSEWriteBlocked('RSE excluded; not available for writing.')
@@ -119,11 +123,11 @@ def __resolve_term_expression(expression):
     operator = None
     original_expression = expression
 
-    while(True):
+    while True:
         if len(expression) == 0:
             return (left_term, original_expression)
         elif expression[0] == "(":
-            if (left_term is None):
+            if left_term is None:
                 left_term, termexpression = __resolve_term_expression(__extract_term(expression))
                 expression = expression[len(termexpression) + 2:]
                 continue
