@@ -14,26 +14,29 @@
 # limitations under the License.
 
 import logging
-
 from datetime import datetime
+from typing import TYPE_CHECKING
 
-from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy import func
+from sqlalchemy.orm.exc import NoResultFound
 
-from rucio.common.config import config_get
-from rucio.common.exception import InsufficientTargetRSEs
-from rucio.core import account_counter, rse_counter, request as request_core
 import rucio.core.did
 import rucio.core.lock
 import rucio.core.replica
+from rucio.common.config import config_get_int
+from rucio.common.exception import InsufficientTargetRSEs
+from rucio.core import account_counter, rse_counter, request as request_core
 from rucio.core.rse import get_rse, get_rse_attribute, get_rse_name
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import LockState, RuleGrouping, ReplicaState, RequestType, DIDType, OBSOLETE
 from rucio.db.sqla.session import transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 @transactional_session
-def apply_rule_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], session=None):
+def apply_rule_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], *, session: "Session"):
     """
     Apply rule grouping to files.
 
@@ -93,7 +96,7 @@ def apply_rule_grouping(datasetfiles, locks, replicas, source_replicas, rseselec
 
 
 @transactional_session
-def repair_stuck_locks_and_apply_rule_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, session=None):
+def repair_stuck_locks_and_apply_rule_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, *, session: "Session"):
     """
     Apply rule grouping to files.
 
@@ -149,7 +152,7 @@ def repair_stuck_locks_and_apply_rule_grouping(datasetfiles, locks, replicas, so
 
 
 @transactional_session
-def create_transfer_dict(dest_rse_id, request_type, scope, name, rule, lock=None, bytes_=None, md5=None, adler32=None, ds_scope=None, ds_name=None, copy_pin_lifetime=None, activity=None, retry_count=None, session=None):
+def create_transfer_dict(dest_rse_id, request_type, scope, name, rule, lock=None, bytes_=None, md5=None, adler32=None, ds_scope=None, ds_name=None, copy_pin_lifetime=None, activity=None, retry_count=None, *, session: "Session"):
     """
     This method creates a transfer dictionary and returns it
 
@@ -193,7 +196,7 @@ def create_transfer_dict(dest_rse_id, request_type, scope, name, rule, lock=None
 
 
 @transactional_session
-def __apply_rule_to_files_none_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], session=None):
+def __apply_rule_to_files_none_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], *, session: "Session"):
     """
     Apply a rule to files with NONE grouping.
 
@@ -274,7 +277,7 @@ def __apply_rule_to_files_none_grouping(datasetfiles, locks, replicas, source_re
 
 
 @transactional_session
-def __apply_rule_to_files_all_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], session=None):
+def __apply_rule_to_files_all_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], *, session: "Session"):
     """
     Apply a rule to files with ALL grouping.
 
@@ -291,6 +294,7 @@ def __apply_rule_to_files_all_grouping(datasetfiles, locks, replicas, source_rep
     :raises:                   InsufficientQuota, InsufficientTargetRSEs
     :attention:                This method modifies the contents of the locks and replicas input parameters.
     """
+
     locks_to_create = {}            # {'rse_id': [locks]}
     replicas_to_create = {}         # {'rse_id': [replicas]}
     transfers_to_create = []        # [{'dest_rse_id':, 'scope':, 'name':, 'request_type':, 'metadata':}]
@@ -393,7 +397,7 @@ def __apply_rule_to_files_all_grouping(datasetfiles, locks, replicas, source_rep
 
 
 @transactional_session
-def __apply_rule_to_files_dataset_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], session=None):
+def __apply_rule_to_files_dataset_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, preferred_rse_ids=[], source_rses=[], *, session: "Session"):
     """
     Apply a rule to files with ALL grouping.
 
@@ -511,7 +515,7 @@ def __apply_rule_to_files_dataset_grouping(datasetfiles, locks, replicas, source
 
 
 @transactional_session
-def __repair_stuck_locks_with_none_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, session=None, logger=logging.log):
+def __repair_stuck_locks_with_none_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, *, session: "Session", logger=logging.log):
     """
     Apply a rule to files with NONE grouping.
 
@@ -618,7 +622,7 @@ def __repair_stuck_locks_with_none_grouping(datasetfiles, locks, replicas, sourc
 
 
 @transactional_session
-def __repair_stuck_locks_with_all_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, session=None, logger=logging.log):
+def __repair_stuck_locks_with_all_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, *, session: "Session", logger=logging.log):
     """
     Apply a rule to files with ALL grouping.
 
@@ -694,7 +698,7 @@ def __repair_stuck_locks_with_all_grouping(datasetfiles, locks, replicas, source
 
 
 @transactional_session
-def __repair_stuck_locks_with_dataset_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, session=None, logger=logging.log):
+def __repair_stuck_locks_with_dataset_grouping(datasetfiles, locks, replicas, source_replicas, rseselector, rule, source_rses, *, session: "Session", logger=logging.log):
     """
     Apply a rule to files with DATASET grouping.
 
@@ -802,7 +806,7 @@ def __is_retry_required(lock, activity):
 
 
 @transactional_session
-def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availability_write, locks_to_create, locks, source_rses, replicas_to_create, replicas, source_replicas, transfers_to_create, session=None, logger=logging.log):
+def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availability_write, locks_to_create, locks, source_rses, replicas_to_create, replicas, source_replicas, transfers_to_create, *, session: "Session", logger=logging.log):
     """
     This method creates a lock and if necessary a new replica and fills the corresponding dictionaries.
 
@@ -924,21 +928,6 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                                                 ds_name=dataset['name'],
                                                                 session=session))
                 return True
-            elif staging_required:  # include the copy_pin_lifetime in dictionary
-                transfers_to_create.append(create_transfer_dict(dest_rse_id=rse_id,
-                                                                request_type=RequestType.TRANSFER,
-                                                                scope=file['scope'],
-                                                                name=file['name'],
-                                                                rule=rule,
-                                                                lock=new_lock,
-                                                                bytes_=file['bytes'],
-                                                                md5=file['md5'],
-                                                                adler32=file['adler32'],
-                                                                ds_scope=dataset['scope'],
-                                                                ds_name=dataset['name'],
-                                                                copy_pin_lifetime=copy_pin_lifetime,
-                                                                session=session))
-                return True
             return False
         # Replica is not available at the rse yet -- COPYING
         else:
@@ -1000,7 +989,7 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                                             ds_name=dataset['name'],
                                                             session=session))
             return True
-        elif staging_required:  # include the copy_pin_lifetime in dictionary
+        elif staging_required:
             transfers_to_create.append(create_transfer_dict(dest_rse_id=rse_id,
                                                             request_type=RequestType.TRANSFER,
                                                             scope=file['scope'],
@@ -1012,7 +1001,6 @@ def __create_lock_and_replica(file, dataset, rule, rse_id, staging_area, availab
                                                             adler32=file['adler32'],
                                                             ds_scope=dataset['scope'],
                                                             ds_name=dataset['name'],
-                                                            copy_pin_lifetime=copy_pin_lifetime,
                                                             session=session))
             return True
         return False
@@ -1086,7 +1074,7 @@ def __create_replica(rse_id, scope, name, bytes_, state, md5, adler32, logger=lo
 
 
 @transactional_session
-def __update_lock_replica_and_create_transfer(lock, replica, rule, dataset, transfers_to_create, session=None, logger=logging.log):
+def __update_lock_replica_and_create_transfer(lock, replica, rule, dataset, transfers_to_create, *, session: "Session", logger=logging.log):
     """
     This method updates a lock and replica and fills the corresponding dictionaries.
 
@@ -1111,7 +1099,7 @@ def __update_lock_replica_and_create_transfer(lock, replica, rule, dataset, tran
     else:
         lock.repair_cnt += 1
 
-    if get_rse(rse_id=lock.rse_id, session=session).staging_area:
+    if get_rse(rse_id=lock.rse_id, session=session)['staging_area']:
         copy_pin_lifetime = rule.expires_at - datetime.utcnow()
         copy_pin_lifetime = copy_pin_lifetime.seconds + copy_pin_lifetime.days * 24 * 3600
         transfers_to_create.append(create_transfer_dict(dest_rse_id=lock.rse_id,
@@ -1144,7 +1132,7 @@ def __update_lock_replica_and_create_transfer(lock, replica, rule, dataset, tran
 
 
 @transactional_session
-def __set_replica_unavailable(replica, session=None):
+def __set_replica_unavailable(replica, *, session: "Session"):
     """
     This method updates a replica and sets it to UNAVAILABLE.
 
@@ -1185,7 +1173,7 @@ def __set_replica_unavailable(replica, session=None):
 
 
 @transactional_session
-def apply_rule(did, rule, rses, source_rses, rseselector, session=None, logger=logging.log):
+def apply_rule(did, rule, rses, source_rses, rseselector, *, session: "Session", logger=logging.log):
     """
     Apply a replication rule to one did.
 
@@ -1198,7 +1186,7 @@ def apply_rule(did, rule, rses, source_rses, rseselector, session=None, logger=l
     :param session:      the database session in use
     """
 
-    max_partition_size = config_get('rules', 'apply_rule_max_partition_size', default=2000, session=session)  # process dataset files in bunches of max this size
+    max_partition_size = config_get_int('rules', 'apply_rule_max_partition_size', default=2000, session=session)  # process dataset files in bunches of max this size
 
     # accounting counters
     rse_counters_files = {}
@@ -1300,7 +1288,7 @@ def apply_rule(did, rule, rses, source_rses, rseselector, session=None, logger=l
         # sort alphabetically for deterministic order
         try:
             datasets = sorted(datasets)
-        except:
+        except Exception:
             pass
 
         # prnt(datasets)
