@@ -16,10 +16,9 @@
 import datetime
 import logging
 import re
-from typing import TYPE_CHECKING
 from configparser import NoOptionError, NoSectionError
-
 from json import dumps
+from typing import TYPE_CHECKING
 
 from sqlalchemy import func
 from sqlalchemy.exc import IntegrityError, StatementError
@@ -33,11 +32,12 @@ from rucio.db.sqla.constants import SubscriptionState
 from rucio.db.sqla.session import transactional_session, stream_session, read_session
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Iterator, Optional, Callable
+    from collections.abc import Callable, Iterator
+    from typing import Any, Optional
     from sqlalchemy.orm import Session
     from rucio.common.types import InternalAccount
     LoggerFunction = Callable[..., Any]
-    SubscriptionType = Dict
+    SubscriptionType = dict
 
 
 @transactional_session
@@ -50,7 +50,7 @@ def add_subscription(name: str,
                      retroactive: "Optional[bool]" = False,
                      dry_run: "Optional[bool]" = False,
                      priority: "Optional[int]" = 3,
-                     session: "Optional[Session]" = None) -> str:
+                     *, session: "Session") -> str:
     """
     Adds a new subscription which will be verified against every new added file and dataset
 
@@ -131,7 +131,7 @@ def add_subscription(name: str,
 def update_subscription(name: str,
                         account: "InternalAccount",
                         metadata: dict,
-                        session: "Optional[Session]" = None) -> None:
+                        *, session: "Session") -> None:
     """
     Updates a subscription
 
@@ -173,8 +173,17 @@ def update_subscription(name: str,
     SubscriptionHistory = models.SubscriptionHistory
     try:
         subscription = session.query(models.Subscription).filter_by(account=account, name=name).one()
+
+        # To avoid update in the subscription history table whenever last processed field is changed
+        current_subscription_state = subscription.to_dict()
+        new_subscription_state = values.copy()
+
+        for key in ["updated_at", "last_processed"]:
+            new_subscription_state.pop(key, "None")
+            current_subscription_state.pop(key, "None")
+
         subscription.update(values)
-        if keep_history:
+        if keep_history and current_subscription_state != new_subscription_state:
             subscription_history = SubscriptionHistory(id=subscription.id,
                                                        name=subscription.name,
                                                        filter=subscription.filter,
@@ -191,14 +200,14 @@ def update_subscription(name: str,
                                                        created_at=subscription.created_at)
             subscription_history.save(session=session)
     except NoResultFound:
-        raise SubscriptionNotFound("Subscription for account '%(account)s' named '%(name)s' not found" % locals())
+        raise SubscriptionNotFound(f"Subscription for account '{account}' named '{name}' not found")
 
 
 @stream_session
 def list_subscriptions(name: "Optional[str]" = None,
                        account: "Optional[InternalAccount]" = None,
                        state: "Optional[SubscriptionState]" = None,
-                       session: "Optional[Session]" = None,
+                       *, session: "Session",
                        logger: "LoggerFunction" = logging.log) -> "Iterator[SubscriptionType]":
     """
     Returns a dictionary with the subscription information :
@@ -230,18 +239,16 @@ def list_subscriptions(name: "Optional[str]" = None,
     except IntegrityError as error:
         logger(logging.ERROR, str(error))
         raise RucioException(error.args)
-    result = {}
+    found = False
     for row in query:
-        result = {}
-        for column in row.__table__.columns:
-            result[column.name] = getattr(row, column.name)
-        yield result
-    if result == {}:
-        raise SubscriptionNotFound("Subscription for account '%(account)s' named '%(name)s' not found" % locals())
+        found = True
+        yield row.to_dict()
+    if not found:
+        raise SubscriptionNotFound(f"Subscription for account '{account}' named '{name}' not found")
 
 
 @transactional_session
-def delete_subscription(subscription_id: str, session: "Optional[Session]" = None) -> None:
+def delete_subscription(subscription_id: str, *, session: "Session") -> None:
     """
     Deletes a subscription
 
@@ -252,7 +259,7 @@ def delete_subscription(subscription_id: str, session: "Optional[Session]" = Non
 
 
 @stream_session
-def list_subscription_rule_states(name=None, account=None, session=None, logger=logging.log):
+def list_subscription_rule_states(name=None, account=None, *, session: "Session", logger=logging.log):
     """Returns a list of with the number of rules per state for a subscription.
 
     :param name:               Name of the subscription
@@ -264,7 +271,9 @@ def list_subscription_rule_states(name=None, account=None, session=None, logger=
     subscription = aliased(models.Subscription)
     rule = aliased(models.ReplicationRule)
     # count needs a label to allow conversion to dict (label name can be changed)
-    query = session.query(subscription.account, subscription.name, rule.state, func.count().label('count')).join(rule, subscription.id == rule.subscription_id)
+    query = session.query(
+        subscription.account, subscription.name, rule.state, func.count().label('count')).join(
+        rule, subscription.id == rule.subscription_id)
 
     try:
         if name:
@@ -288,7 +297,7 @@ def list_subscription_rule_states(name=None, account=None, session=None, logger=
 
 
 @read_session
-def get_subscription_by_id(subscription_id, session=None):
+def get_subscription_by_id(subscription_id, *, session: "Session"):
     """
     Get a specific subscription by id.
 
@@ -300,11 +309,7 @@ def get_subscription_by_id(subscription_id, session=None):
 
     try:
         subscription = session.query(models.Subscription).filter_by(id=subscription_id).one()
-        result = {}
-        for column in subscription.__table__.columns:
-            result[column.name] = getattr(subscription, column.name)
-        return result
-
+        return subscription.to_dict()
     except NoResultFound:
         raise SubscriptionNotFound('No subscription with the id %s found' % (subscription_id))
     except StatementError:
