@@ -14,19 +14,24 @@
 # limitations under the License.
 
 from datetime import datetime
+from typing import TYPE_CHECKING
 
 from sqlalchemy import and_, or_, func, delete
-from sqlalchemy.sql.expression import case
+from sqlalchemy.sql.expression import case, insert, update
 
 from rucio.core.did import attach_dids
 from rucio.core.replica import add_replica
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.session import read_session, transactional_session
+
 # from rucio.rse import rsemanager as rsemgr
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 @transactional_session
-def add_temporary_dids(dids, account, session=None):
+def add_temporary_dids(dids, account, *, session: "Session"):
     """
     Bulk add temporary data identifiers.
 
@@ -60,7 +65,8 @@ def add_temporary_dids(dids, account, session=None):
                                'offset': did.get('offset'),
                                'expired_at': datetime.utcnow()})
     try:
-        session.bulk_insert_mappings(models.TemporaryDataIdentifier, temporary_dids)
+        if temporary_dids:
+            session.execute(insert(models.TemporaryDataIdentifier), temporary_dids)
     except:
         raise
 
@@ -69,7 +75,7 @@ def add_temporary_dids(dids, account, session=None):
 def compose(scope, name, rse_id, bytes_, sources, account,
             md5=None, adler32=None, pfn=None, meta={}, rules=[],
             parent_scope=None, parent_name=None,
-            session=None):
+            *, session: "Session"):
     """
     Concatenates a list of existing dids into a new file replica
 
@@ -105,12 +111,13 @@ def compose(scope, name, rse_id, bytes_, sources, account,
         expired_dids.append({'scope': source['scope'],
                              'name': source['name'],
                              'expired_at': now})
-    session.bulk_update_mappings(models.TemporaryDataIdentifier, expired_dids)
+    if expired_dids:
+        session.execute(update(models.TemporaryDataIdentifier), expired_dids)
 
 
 @read_session
 def list_expired_temporary_dids(rse_id, limit, worker_number=None, total_workers=None,
-                                session=None):
+                                *, session: "Session"):
     """
     List expired temporary DIDs.
 
@@ -128,7 +135,7 @@ def list_expired_temporary_dids(rse_id, limit, worker_number=None, total_workers
                           models.TemporaryDataIdentifier.path,
                           models.TemporaryDataIdentifier.bytes).\
         with_hint(models.TemporaryDataIdentifier, "INDEX(tmp_dids TMP_DIDS_EXPIRED_AT_IDX)", 'oracle').\
-        filter(case([(models.TemporaryDataIdentifier.expired_at != is_none, models.TemporaryDataIdentifier.rse_id), ]) == rse_id)
+        filter(case((models.TemporaryDataIdentifier.expired_at != is_none, models.TemporaryDataIdentifier.rse_id)) == rse_id)
 
     query = filter_thread_work(session=session, query=query, total_threads=total_workers, thread_id=worker_number, hash_variable='name')
 
@@ -141,7 +148,7 @@ def list_expired_temporary_dids(rse_id, limit, worker_number=None, total_workers
 
 
 @transactional_session
-def delete_temporary_dids(dids, session=None):
+def delete_temporary_dids(dids, *, session: "Session"):
     """
     Delete temporary file replicas.
 
@@ -163,7 +170,7 @@ def delete_temporary_dids(dids, session=None):
 
 
 @read_session
-def get_count_of_expired_temporary_dids(rse_id, session=None):
+def get_count_of_expired_temporary_dids(rse_id, *, session: "Session"):
     """
     List expired temporary DIDs.
 
@@ -175,7 +182,7 @@ def get_count_of_expired_temporary_dids(rse_id, session=None):
     is_none = None
     count = session.query(func.count(models.TemporaryDataIdentifier.scope)).\
         with_hint(models.TemporaryDataIdentifier, "INDEX(tmp_dids TMP_DIDS_EXPIRED_AT_IDX)", 'oracle').\
-        filter(case([(models.TemporaryDataIdentifier.expired_at != is_none, models.TemporaryDataIdentifier.rse_id), ]) == rse_id).\
+        filter(case((models.TemporaryDataIdentifier.expired_at != is_none, models.TemporaryDataIdentifier.rse_id)) == rse_id).\
         one()
 
     return count[0] or 0
