@@ -17,26 +17,28 @@
 Core tracer module
 """
 
+import ipaddress
 import json
 import logging.handlers
 import random
 import socket
 
 import stomp
-import ipaddress
-from jsonschema import validate, ValidationError, draft7_format_checker
+from jsonschema import validate, ValidationError, Draft7Validator
 
 from rucio.common.config import config_get, config_get_int
 from rucio.common.exception import InvalidObject
 from rucio.common.logging import rucio_log_formatter
 from rucio.common.schema.generic import UUID, TIME_ENTRY, IPv4orIPv6
-from rucio.core.monitor import record_counter
+from rucio.core.monitor import MetricManager
+
+METRICS = MetricManager(module=__name__)
 
 CONFIG_COMMON_LOGLEVEL = getattr(logging, config_get('common', 'loglevel', raise_exception=False, default='DEBUG').upper())
 
 CONFIG_TRACE_LOGLEVEL = getattr(logging, config_get('trace', 'loglevel', raise_exception=False, default='DEBUG').upper())
 CONFIG_TRACE_LOGFORMAT = config_get('trace', 'logformat', raise_exception=False, default='%(message)s')
-CONFIG_TRACE_TRACEDIR = config_get('trace', 'tracedir', raise_exception=False, default='/var/log/rucio')
+CONFIG_TRACE_TRACEDIR = config_get('trace', 'tracedir', raise_exception=False, default='/var/log/rucio/trace')
 CONFIG_TRACE_MAXBYTES = config_get_int('trace', 'maxbytes', raise_exception=False, default=1000000000)
 CONFIG_TRACE_BACKUPCOUNT = config_get_int('trace', 'backupCount', raise_exception=False, default=10)
 
@@ -252,7 +254,7 @@ SCHEMAS = {
     'sfo2eos': SPECIAL_SCHEMA
 }
 
-FORMAT_CHECKER = draft7_format_checker
+FORMAT_CHECKER = Draft7Validator.FORMAT_CHECKER
 
 
 @FORMAT_CHECKER.checks(format="ipv4_or_ipv6")
@@ -267,8 +269,7 @@ def ip_format_checker(value: str) -> bool:
         result = False
     else:
         result = True
-    finally:
-        return result
+    return result
 
 
 logging.getLogger("stomp").setLevel(logging.CRITICAL)
@@ -291,6 +292,7 @@ def date_handler(obj):
     return obj.isoformat() if hasattr(obj, 'isoformat') else obj
 
 
+@METRICS.count_it
 def trace(payload):
     """
     Write a trace to the buffer log file and send it to active mq.
@@ -298,7 +300,6 @@ def trace(payload):
     :param payload: Python dictionary with trace report.
     """
 
-    record_counter('trace.trace')
     report = json.dumps(payload, default=date_handler)
     ROTATING_LOGGER.debug(report)
     t_conns = CONNS[:]
