@@ -34,22 +34,24 @@ from rucio.common.exception import (InvalidRSEExpression,
 from rucio.common.utils import construct_surl
 from rucio.core import did, message as message_core, request as request_core
 from rucio.core.account import list_accounts
-from rucio.core.monitor import record_counter
+from rucio.core.monitor import MetricManager
 from rucio.core.request import set_request_state, RequestWithSources, RequestSource
+from rucio.core.rse import RseData
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.db.sqla import models
-from rucio.db.sqla.constants import DIDType, RequestState, RequestType
-from rucio.db.sqla.session import read_session, transactional_session
+from rucio.db.sqla.constants import DIDType, RequestState, RequestType, TransferLimitDirection
+from rucio.db.sqla.session import read_session, transactional_session, stream_session
 from rucio.rse import rsemanager as rsemgr
+from rucio.transfertool.transfertool import TransferStatusReport
 from rucio.transfertool.fts3 import FTS3Transfertool
 from rucio.transfertool.globus import GlobusTransferTool
 from rucio.transfertool.mock import MockTransfertool
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Set, Tuple
+    from collections.abc import Callable, Generator, Iterable
+    from typing import Any, Optional
     from sqlalchemy.orm import Session
     from rucio.common.types import InternalAccount
-    from rucio.core.rse import RseData
     from rucio.core.topology import Topology
 
     LoggerFunction = Callable[..., Any]
@@ -61,6 +63,7 @@ Requests accessed by request_id  are covered in the core request.py
 """
 
 REGION_ACCOUNTS = make_region().configure('dogpile.cache.memory', expiration_time=600)
+METRICS = MetricManager(module=__name__)
 
 WEBDAV_TRANSFER_MODE = config_get('conveyor', 'webdav_transfer_mode', False, None)
 
@@ -150,7 +153,7 @@ class DirectTransferDefinition:
                                            protocol_factory=self.protocol_factory,
                                            operation=self.operation_src),
                  src.rse.id,
-                 src.source_ranking)
+                 src.ranking)
                 for src in self.sources
             ]
         return self._legacy_sources
@@ -255,7 +258,9 @@ class StageinTransferDefinition(DirectTransferDefinition):
     """
     def __init__(self, source, destination, rws, protocol_factory, operation_src, operation_dest):
         if not source.rse.is_tape() or destination.rse.is_tape():
-            raise RucioException("Stageing request {} must be from TAPE to DISK rse. Got {} and {}.".format(rws, source, destination))
+            # allow staging_required QoS RSE to be TAPE to TAPE for pin
+            if not destination.rse.attributes.get('staging_required', None):
+                raise RucioException("Stageing request {} must be from TAPE to DISK rse. Got {} and {}.".format(rws, source, destination))
         super().__init__(source, destination, rws, protocol_factory, operation_src, operation_dest)
 
     @property
@@ -275,12 +280,12 @@ class StageinTransferDefinition(DirectTransferDefinition):
                 self.src.rse.name,
                 self.dest_url,  # Source and dest url is the same for stagein requests
                 self.src.rse.id,
-                self.src.source_ranking
+                self.src.ranking
             )]
         return self._legacy_sources
 
 
-def transfer_path_str(transfer_path: "List[DirectTransferDefinition]") -> str:
+def transfer_path_str(transfer_path: "list[DirectTransferDefinition]") -> str:
     """
     an implementation of __str__ for a transfer path, which is a list of direct transfers, so not really an object
     """
@@ -309,8 +314,9 @@ def transfer_path_str(transfer_path: "List[DirectTransferDefinition]") -> str:
 def mark_submitting(
         transfer: "DirectTransferDefinition",
         external_host: str,
+        *,
         logger: "Callable",
-        session: "Optional[Session]" = None,
+        session: "Session",
 ):
     """
     Mark a transfer as submitting
@@ -350,10 +356,12 @@ def mark_submitting(
         raise RequestNotFound("Failed to prepare transfer: request %s does not exist or is not in queued state" % transfer.rws)
 
 
+@transactional_session
 def ensure_db_sources(
-        transfer_path: "List[DirectTransferDefinition]",
+        transfer_path: "list[DirectTransferDefinition]",
+        *,
         logger: "Callable",
-        session: "Optional[Session]" = None,
+        session: "Session",
 ):
     """
     Ensure the needed DB source objects exist
@@ -396,15 +404,25 @@ def ensure_db_sources(
 
 
 @transactional_session
-def set_transfers_state(transfers, state, submitted_at, external_host, external_id, logger, session=None):
+def set_transfers_state(
+        transfers,
+        state: "RequestState",
+        submitted_at: datetime.datetime,
+        external_host: str,
+        external_id: str,
+        transfertool: str,
+        *,
+        session: "Session",
+        logger
+):
     """
     Update the transfer info of a request.
     :param transfers:  Dictionary containing request transfer info.
     :param session:    Database session to use.
     """
 
-    logger(logging.INFO, 'Setting state(%s), external_host(%s) and eid(%s) for transfers: %s',
-           state.name, external_host, external_id, ', '.join(t.rws.request_id for t in transfers))
+    logger(logging.INFO, 'Setting state(%s), transfertool(%s), external_host(%s) and eid(%s) for transfers: %s',
+           state.name, transfertool, external_host, external_id, ', '.join(t.rws.request_id for t in transfers))
     try:
         for transfer in transfers:
             rws = transfer.rws
@@ -418,11 +436,12 @@ def set_transfers_state(transfers, state, submitted_at, external_host, external_
                 synchronize_session=False
             ).values(
                 {
-                    'state': state,
-                    'external_id': external_id,
-                    'external_host': external_host,
-                    'source_rse_id': transfer.src.rse.id,
-                    'submitted_at': submitted_at,
+                    models.Request.state: state,
+                    models.Request.external_id: external_id,
+                    models.Request.external_host: external_host,
+                    models.Request.source_rse_id: transfer.src.rse.id,
+                    models.Request.submitted_at: submitted_at,
+                    models.Request.transfertool: transfertool,
                 }
             )
             rowcount = session.execute(stmt).rowcount
@@ -483,7 +502,49 @@ def set_transfers_state(transfers, state, submitted_at, external_host, external_
 
 
 @transactional_session
-def mark_transfer_lost(request, session=None, logger=logging.log):
+def update_transfer_state(tt_status_report: TransferStatusReport, *, session: "Session", logger=logging.log):
+    """
+    Used by poller and consumer to update the internal state of requests,
+    after the response by the external transfertool.
+
+    :param tt_status_report:      The transfertool status update, retrieved via request.query_request().
+    :param session:               The database session to use.
+    :param logger:                Optional decorated logger that can be passed from the calling daemons or servers.
+    :returns commit_or_rollback:  Boolean.
+    """
+
+    request_id = tt_status_report.request_id
+    try:
+        fields_to_update = tt_status_report.get_db_fields_to_update(session=session, logger=logger)
+        if not fields_to_update:
+            request_core.update_request(request_id, raise_on_missing=True, session=session)
+            return False
+        else:
+            logger(logging.INFO, 'UPDATING REQUEST %s FOR %s with changes: %s' % (str(request_id), tt_status_report, fields_to_update))
+
+            set_request_state(request_id, session=session, **fields_to_update)
+            request = tt_status_report.request(session)
+
+            if tt_status_report.state == RequestState.FAILED:
+                if request_core.is_intermediate_hop(request):
+                    request_core.handle_failed_intermediate_hop(request, session=session)
+
+            request_core.add_monitor_message(
+                new_state=tt_status_report.state,
+                request=request,
+                additional_fields=tt_status_report.get_monitor_msg_fields(session=session, logger=logger),
+                session=session
+            )
+            return True
+    except UnsupportedOperation as error:
+        logger(logging.WARNING, "Request %s doesn't exist - Error: %s" % (request_id, str(error).replace('\n', '')))
+        return False
+    except Exception:
+        logger(logging.CRITICAL, "Exception", exc_info=True)
+
+
+@transactional_session
+def mark_transfer_lost(request, *, session: "Session", logger=logging.log):
     new_state = RequestState.LOST
     reason = "The FTS job lost"
 
@@ -493,48 +554,15 @@ def mark_transfer_lost(request, session=None, logger=logging.log):
     request_core.add_monitor_message(new_state=new_state, request=request, additional_fields={'reason': reason}, session=session)
 
 
+@METRICS.count_it
 @transactional_session
-def set_transfer_update_time(external_host, transfer_id, update_time=datetime.datetime.utcnow(), session=None):
-    """
-    Update the state of a request. Fails silently if the transfer_id does not exist.
-    :param external_host:  Selected external host as string in format protocol://fqdn:port
-    :param transfer_id:    External transfer job id as a string.
-    :param update_time:    Time stamp.
-    :param session:        Database session to use.
-    """
-
-    record_counter('core.request.set_transfer_update_time')
-
-    try:
-        stmt = update(
-            models.Request
-        ).where(
-            models.Request.external_id == transfer_id,
-            models.Request.state == RequestState.SUBMITTED
-        ).execution_options(
-            synchronize_session=False
-        ).values(
-            updated_at=update_time
-        )
-        rowcount = session.executet(stmt).rowcount
-    except IntegrityError as error:
-        raise RucioException(error.args)
-
-    if not rowcount:
-        raise UnsupportedOperation("Transfer %s doesn't exist or its status is not submitted." % transfer_id)
-
-
-@transactional_session
-def touch_transfer(external_host, transfer_id, session=None):
+def touch_transfer(external_host, transfer_id, *, session: "Session"):
     """
     Update the timestamp of requests in a transfer. Fails silently if the transfer_id does not exist.
     :param request_host:   Name of the external host.
     :param transfer_id:    External transfer job id as a string.
     :param session:        Database session to use.
     """
-
-    record_counter('core.request.touch_transfer')
-
     try:
         # don't touch it if it's already touched in 30 seconds
         stmt = update(
@@ -560,37 +588,37 @@ def __create_transfer_definitions(
         topology: "Topology",
         protocol_factory: ProtocolFactory,
         rws: RequestWithSources,
-        sources: "List[RequestSource]",
-        multi_source_sources: "List[RequestSource]",
-        limit_dest_schemes: "List[str]",
+        sources: "list[RequestSource]",
+        max_sources: int,
+        multi_source_sources: "list[RequestSource]",
+        limit_dest_schemes: list[str],
         operation_src: str,
         operation_dest: str,
         domain: str,
-        session: "Optional[Session]" = None,
-) -> "Dict[str, List[DirectTransferDefinition]]":
+        *,
+        session: "Session",
+) -> "dict[str, list[DirectTransferDefinition]]":
     """
     Find the all paths from sources towards the destination of the given transfer request.
     Create the transfer definitions for each point-to-point transfer (multi-source, when possible)
     """
-    inbound_links_by_node = {}
-    shortest_paths = topology.search_shortest_paths(source_rse_ids=[s.rse.id for s in sources], dest_rse_id=rws.dest_rse.id,
-                                                    operation_src=operation_src, operation_dest=operation_dest, domain=domain,
-                                                    limit_dest_schemes=limit_dest_schemes,
-                                                    inbound_links_by_node=inbound_links_by_node, session=session)
+    shortest_paths = topology.search_shortest_paths(src_nodes=[s.rse for s in sources], dst_node=rws.dest_rse,
+                                                    operation_src=operation_src, operation_dest=operation_dest,
+                                                    domain=domain, limit_dest_schemes=limit_dest_schemes, session=session)
 
     transfers_by_source = {}
-    sources_by_rse_id = {s.rse.id: s for s in sources}
-    paths_by_source = {sources_by_rse_id[rse_id]: path for rse_id, path in shortest_paths.items()}
+    sources_by_rse = {s.rse: s for s in sources}
+    paths_by_source = {sources_by_rse[rse]: path for rse, path in shortest_paths.items()}
     for source, list_hops in paths_by_source.items():
         transfer_path = []
         for hop in list_hops:
-            hop_src_rse = topology.rse_collection[hop['source_rse_id']]
-            hop_dst_rse = topology.rse_collection[hop['dest_rse_id']]
+            hop_src_rse = hop['source_rse']
+            hop_dst_rse = hop['dest_rse']
             src = RequestSource(
                 rse_data=hop_src_rse,
                 file_path=source.file_path if hop_src_rse == source.rse else None,
-                source_ranking=source.source_ranking if hop_src_rse == source.rse else 0,
-                distance_ranking=hop['cumulated_distance'] if hop_src_rse == source.rse else hop['hop_distance'],
+                ranking=source.ranking if hop_src_rse == source.rse else 0,
+                distance=hop['cumulated_distance'] if hop_src_rse == source.rse else hop['hop_distance'],
                 scheme=hop['source_scheme'],
             )
             dst = TransferDestination(
@@ -644,14 +672,14 @@ def __create_transfer_definitions(
             # Multiple single-hop DISK rses can be used together in "multi-source" transfers
             #
             # Try adding additional single-hop DISK rses sources to the transfer
-            inbound_links = inbound_links_by_node[transfer_path[0].dst.rse.id]
             main_source_schemes = __add_compatible_schemes(schemes=[transfer_path[0].dst.scheme], allowed_schemes=SUPPORTED_PROTOCOLS)
             added_sources = 0
-            for source in sorted(multi_source_sources, key=lambda s: (-s.source_ranking, s.distance_ranking)):
-                if added_sources >= 5:
+            for source in sorted(multi_source_sources, key=lambda s: (-s.ranking, s.distance)):
+                if added_sources >= max_sources:
                     break
 
-                if source.rse.id not in inbound_links:
+                edge = topology.edge(source.rse, transfer_path[0].dst.rse)
+                if not edge:
                     # There is no direct connection between this source and the destination
                     continue
 
@@ -677,8 +705,8 @@ def __create_transfer_definitions(
                     RequestSource(
                         rse_data=source.rse,
                         file_path=source.file_path,
-                        source_ranking=source.source_ranking,
-                        distance_ranking=inbound_links[source.rse.id],
+                        ranking=source.ranking,
+                        distance=edge.cost,
                         scheme=matching_scheme[1],
                     )
                 )
@@ -688,12 +716,12 @@ def __create_transfer_definitions(
 
 def __create_stagein_definitions(
         rws: RequestWithSources,
-        sources: "List[RequestSource]",
-        limit_dest_schemes: "List[str]",
+        sources: "list[RequestSource]",
+        limit_dest_schemes: list[str],
         operation_src: str,
         operation_dest: str,
         protocol_factory: ProtocolFactory,
-) -> "Dict[str, List[StageinTransferDefinition]]":
+) -> "dict[str, list[StageinTransferDefinition]]":
     """
     for each source, create a single-hop transfer path with a one stageing definition inside
     """
@@ -732,7 +760,7 @@ def get_dsn(scope, name, dsn):
     return 'other'
 
 
-def __filter_multihops_with_intermediate_tape(candidate_paths: "Iterable[List[DirectTransferDefinition]]") -> "Generator[List[DirectTransferDefinition]]":
+def __filter_multihops_with_intermediate_tape(candidate_paths: "Iterable[list[DirectTransferDefinition]]") -> "Generator[list[DirectTransferDefinition]]":
     # Discard multihop transfers which contain a tape source as an intermediate hop
     for path in candidate_paths:
         if any(transfer.src.rse.is_tape_or_staging_required() for transfer in path[1:]):
@@ -742,9 +770,9 @@ def __filter_multihops_with_intermediate_tape(candidate_paths: "Iterable[List[Di
 
 
 def __compress_multihops(
-        candidate_paths: "Iterable[List[DirectTransferDefinition]]",
+        candidate_paths: "Iterable[list[DirectTransferDefinition]]",
         sources: "Iterable[RequestSource]",
-) -> "Generator[List[DirectTransferDefinition]]":
+) -> "Generator[list[DirectTransferDefinition]]":
     # Compress multihop transfers which contain other sources as part of itself.
     # For example: multihop A->B->C and B is a source, compress A->B->C into B->C
     source_rses = {s.rse.id for s in sources}
@@ -763,7 +791,7 @@ def __compress_multihops(
             yield path
 
 
-def __sort_paths(candidate_paths: "Iterable[List[DirectTransferDefinition]]") -> "Generator[List[DirectTransferDefinition]]":
+def __sort_paths(candidate_paths: "Iterable[list[DirectTransferDefinition]]") -> "Generator[list[DirectTransferDefinition]]":
 
     def __transfer_order_key(transfer_path):
         # Reduce the priority of the tape sources. If there are any disk sources,
@@ -771,29 +799,33 @@ def __sort_paths(candidate_paths: "Iterable[List[DirectTransferDefinition]]") ->
         source_ranking_penalty = 1 if transfer_path[0].src.rse.is_tape_or_staging_required() else 0
         # higher source_ranking first,
         # on equal source_ranking, prefer DISK over TAPE
-        # on equal type, prefer lower distance_ranking
+        # on equal type, prefer lower distance
         # on equal distance, prefer single hop
         return (
-            - transfer_path[0].src.source_ranking + source_ranking_penalty,
+            - transfer_path[0].src.ranking + source_ranking_penalty,
             transfer_path[0].src.rse.is_tape_or_staging_required(),  # rely on the fact that False < True
-            transfer_path[0].src.distance_ranking,
+            transfer_path[0].src.distance,
             len(transfer_path) > 1,  # rely on the fact that False < True
         )
 
     yield from sorted(candidate_paths, key=__transfer_order_key)
 
 
+@transactional_session
 def build_transfer_paths(
         topology: "Topology",
+        protocol_factory: "ProtocolFactory",
         requests_with_sources: "Iterable[RequestWithSources]",
-        admin_accounts: "Optional[Set[InternalAccount]]" = None,
-        schemes: "Optional[List[str]]" = None,
-        failover_schemes: "Optional[List[str]]" = None,
-        transfertools: "Optional[List[str]]" = None,
+        admin_accounts: "Optional[set[InternalAccount]]" = None,
+        schemes: "Optional[list[str]]" = None,
+        failover_schemes: "Optional[list[str]]" = None,
+        max_sources: int = 4,
+        transfertools: "Optional[list[str]]" = None,
         requested_source_only: bool = False,
         preparer_mode: bool = False,
+        *,
+        session: "Session",
         logger: "Callable" = logging.log,
-        session: "Optional[Session]" = None,
 ):
     """
     For each request, find all possible transfer paths from its sources, which respect the
@@ -814,8 +846,6 @@ def build_transfer_paths(
     if admin_accounts is None:
         admin_accounts = set()
 
-    protocol_factory = ProtocolFactory()
-
     # Do not print full source RSE list for DIDs which have many sources. Otherwise we fill the monitoring
     # storage with data which has little to no benefit. This log message is unlikely to help debugging
     # transfers issues when there are many sources, but can be very useful for small number of sources.
@@ -825,13 +855,10 @@ def build_transfer_paths(
     reqs_unsupported_transfertool = set()
     for rws in requests_with_sources:
 
-        rws.dest_rse = topology.rse_collection.setdefault(rws.dest_rse.id, rws.dest_rse)
-        rws.dest_rse.ensure_loaded(load_name=True, load_info=True, load_attributes=True, session=session)
-
+        rws.dest_rse.ensure_loaded(load_name=True, load_info=True, load_attributes=True, load_columns=True, session=session)
         all_sources = rws.sources
         for source in all_sources:
-            source.rse = topology.rse_collection.setdefault(source.rse.id, source.rse)
-            source.rse.ensure_loaded(load_name=True, load_info=True, load_attributes=True, session=session)
+            source.rse.ensure_loaded(load_name=True, load_info=True, load_attributes=True, load_columns=True, session=session)
 
         transfer_schemes = schemes
         if rws.previous_attempt_id and failover_schemes:
@@ -847,14 +874,14 @@ def build_transfer_paths(
                rws,
                len(all_sources),
                f' (priority {rws.requested_source.rse})' if requested_source_only and rws.requested_source else '',
-               ','.join('{}:{}:{}'.format(src.rse, src.source_ranking, src.distance_ranking) for src in all_sources[:num_sources_in_logs]),
+               ','.join('{}:{}:{}'.format(src.rse, src.ranking, src.distance) for src in all_sources[:num_sources_in_logs]),
                '... and %d others' % (len(all_sources) - num_sources_in_logs) if len(all_sources) > num_sources_in_logs else '')
 
         # Check if destination is blocked
-        if rws.dest_rse.id in topology.unavailable_write_rses:
+        if not (topology.ignore_availability or rws.dest_rse.columns['availability_write']):
             logger(logging.WARNING, '%s: dst RSE is blocked for write. Will skip the submission of new jobs', rws.request_id)
             continue
-        if rws.account not in admin_accounts and rws.dest_rse.id in topology.restricted_write_rses:
+        if rws.account not in admin_accounts and rws.dest_rse.attributes.get('restricted_write'):
             logger(logging.WARNING, '%s: dst RSE is restricted for write. Will skip the submission', rws.request_id)
             continue
 
@@ -883,9 +910,10 @@ def build_transfer_paths(
             filtered_sources = filter(lambda s: s.rse.id in allowed_source_rses, filtered_sources)
         filtered_sources = filter(lambda s: s.rse.name is not None, filtered_sources)
         if rws.account not in admin_accounts:
-            filtered_sources = filter(lambda s: s.rse.id not in topology.restricted_read_rses, filtered_sources)
+            filtered_sources = filter(lambda s: not s.rse.attributes.get('restricted_read'), filtered_sources)
         # Ignore blocklisted RSEs
-        filtered_sources = filter(lambda s: s.rse.id not in topology.unavailable_read_rses, filtered_sources)
+        if not topology.ignore_availability:
+            filtered_sources = filter(lambda s: s.rse.columns['availability_read'], filtered_sources)
         # For staging requests, the staging_buffer attribute must be correctly set
         if rws.request_type == RequestType.STAGEIN:
             filtered_sources = filter(lambda s: s.rse.attributes.get('staging_buffer') == rws.dest_rse.name, filtered_sources)
@@ -920,6 +948,7 @@ def build_transfer_paths(
             paths = __create_transfer_definitions(topology=topology,
                                                   rws=rws,
                                                   sources=candidate_sources,
+                                                  max_sources=max_sources,
                                                   multi_source_sources=[] if preparer_mode else filtered_sources,
                                                   limit_dest_schemes=[],
                                                   operation_src='third_party_copy_read',
@@ -962,7 +991,7 @@ def build_transfer_paths(
             candidate_paths = __compress_multihops(candidate_paths, all_sources)
         candidate_paths = list(__sort_paths(candidate_paths))
 
-        ordered_sources_log = ','.join(('multihop: ' if len(path) > 1 else '') + '{}:{}:{}'.format(path[0].src.rse, path[0].src.source_ranking, path[0].src.distance_ranking)
+        ordered_sources_log = ','.join(('multihop: ' if len(path) > 1 else '') + '{}:{}:{}'.format(path[0].src.rse, path[0].src.ranking, path[0].src.distance)
                                        for path in candidate_paths[:num_sources_in_logs])
         if len(candidate_paths) > num_sources_in_logs:
             ordered_sources_log += '... and %d others' % (len(candidate_paths) - num_sources_in_logs)
@@ -1013,7 +1042,7 @@ def __add_compatible_schemes(schemes, allowed_schemes):
 
 
 @read_session
-def list_transfer_admin_accounts(session=None) -> "Set[InternalAccount]":
+def list_transfer_admin_accounts(*, session: "Session") -> "set[InternalAccount]":
     """
     List admin accounts and cache the result in memory
     """
@@ -1058,6 +1087,7 @@ def cancel_transfers(transfers_to_cancel, logger=logging.log):
                 logger(logging.WARNING, 'Could not cancel FTS3 transfer %s on %s: %s' % (transfer_id, transfertool_obj, str(error)))
 
 
+@METRICS.count_it
 def cancel_transfer(transfertool_obj, transfer_id):
     """
     Cancel a transfer based on external transfer id.
@@ -1066,7 +1096,6 @@ def cancel_transfer(transfertool_obj, transfer_id):
     :param transfer_id:      External-ID as a 32 character hex string.
     """
 
-    record_counter('core.request.cancel_request_external_id')
     try:
         transfertool_obj.cancel(transfer_ids=[transfer_id])
     except Exception:
@@ -1075,11 +1104,12 @@ def cancel_transfer(transfertool_obj, transfer_id):
 
 @transactional_session
 def prepare_transfers(
-        candidate_paths_by_request_id: "Dict[str, List[List[DirectTransferDefinition]]]",
+        candidate_paths_by_request_id: "dict[str, list[list[DirectTransferDefinition]]]",
         logger: "LoggerFunction" = logging.log,
-        transfertools: "Optional[List[str]]" = None,
-        session: "Optional[Session]" = None,
-) -> "Tuple[List[str], List[str]]":
+        transfertools: "Optional[list[str]]" = None,
+        *,
+        session: "Session",
+) -> tuple[list[str], list[str]]:
     """
     Update transfer requests according to preparer settings.
     """
@@ -1112,48 +1142,41 @@ def prepare_transfers(
             logger(logging.WARNING, '%s: all available sources were filtered', rws)
             continue
 
-        update_dict = {
-            models.Request.state: _throttler_request_state(
+        update_dict: dict[Any, Any] = {
+            models.Request.state.name: _throttler_request_state(
                 activity=rws.activity,
                 source_rse=selected_source.rse,
                 dest_rse=rws.dest_rse,
                 session=session,
             ),
-            models.Request.source_rse_id: selected_source.rse.id,
+            models.Request.source_rse_id.name: selected_source.rse.id,
         }
         if transfertool:
-            update_dict[models.Request.transfertool] = transfertool
+            update_dict[models.Request.transfertool.name] = transfertool
 
-        stmt = update(
-            models.Request
-        ).where(
-            models.Request.id == rws.request_id
-        ).execution_options(
-            synchronize_session=False
-        ).values(
-            update_dict
-        )
-        session.execute(stmt)
+        request_core.update_request(rws.request_id, session=session, **update_dict)
         updated_reqs.append(request_id)
 
     return updated_reqs, reqs_no_transfertool
 
 
+@stream_session
 def applicable_rse_transfer_limits(
         source_rse: "Optional[RseData]" = None,
         dest_rse: "Optional[RseData]" = None,
         activity: "Optional[str]" = None,
-        session: "Optional[Session]" = None,
+        *,
+        session: "Session",
 ):
     """
     Find all RseTransferLimits which must be enforced for transfers between source and destination RSEs for the given activity.
     """
     source_limits = {}
     if source_rse:
-        source_limits = source_rse.ensure_loaded(load_transfer_limits=True, session=session).transfer_limits.get('source', {})
+        source_limits = source_rse.ensure_loaded(load_transfer_limits=True, session=session).transfer_limits.get(TransferLimitDirection.SOURCE, {})
     dest_limits = {}
     if dest_rse:
-        dest_limits = dest_rse.ensure_loaded(load_transfer_limits=True, session=session).transfer_limits.get('destination', {})
+        dest_limits = dest_rse.ensure_loaded(load_transfer_limits=True, session=session).transfer_limits.get(TransferLimitDirection.DESTINATION, {})
 
     if activity is not None:
         limit = source_limits.get(activity)
@@ -1174,7 +1197,7 @@ def applicable_rse_transfer_limits(
         yield limit
 
 
-def _throttler_request_state(activity, source_rse, dest_rse, session: "Optional[Session]" = None) -> RequestState:
+def _throttler_request_state(activity, source_rse, dest_rse, *, session: "Session") -> RequestState:
     """
     Takes request attributes to return a new state for the request
     based on throttler settings. Always returns QUEUED,
@@ -1187,12 +1210,14 @@ def _throttler_request_state(activity, source_rse, dest_rse, session: "Optional[
     return RequestState.WAITING if limit_found else RequestState.QUEUED
 
 
+@read_session
 def get_supported_transfertools(
         source_rse: "RseData",
         dest_rse: "RseData",
-        transfertools: "Optional[List[str]]" = None,
-        session: "Optional[Session]" = None,
-) -> "Set[str]":
+        transfertools: "Optional[list[str]]" = None,
+        *,
+        session: "Session",
+) -> set[str]:
 
     if not transfertools:
         transfertools = list(TRANSFERTOOL_CLASSES_BY_NAME)
