@@ -14,12 +14,13 @@
 # limitations under the License.
 
 import re
+from typing import TYPE_CHECKING
+
 from sqlalchemy.exc import DatabaseError, IntegrityError
 from sqlalchemy.orm.exc import NoResultFound
 
-import rucio.core.config as config_db
 from rucio.common import exception
-from rucio.common.config import config_get_bool
+from rucio.common.config import config_get_bool, config_get
 from rucio.common.types import InternalAccount
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import AccountType, IdentityType
@@ -28,9 +29,12 @@ from rucio.db.sqla.session import read_session, transactional_session
 # Format for long VO names
 LONG_VO_RE = re.compile(r"^[a-zA-Z0-9\.\-]+$")
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 @read_session
-def vo_exists(vo, session=None):
+def vo_exists(vo, *, session: "Session"):
     """
     Verify that the vo exists.
 
@@ -43,7 +47,7 @@ def vo_exists(vo, session=None):
 
 
 @transactional_session
-def add_vo(vo, description, email, session=None):
+def add_vo(vo, description, email, *, session: "Session"):
     """
     Add a VO and setup a new root user.
     New root user will have account name 'root' and a userpass identity with username: 'root@<vo>' and password: 'password'
@@ -85,7 +89,7 @@ def add_vo(vo, description, email, session=None):
 
 
 @read_session
-def list_vos(session=None):
+def list_vos(*, session: "Session"):
     """
     List all the VOs in the db.
 
@@ -110,7 +114,7 @@ def list_vos(session=None):
 
 
 @transactional_session
-def update_vo(vo, parameters, session=None):
+def update_vo(vo, parameters, *, session: "Session"):
     """
     Update VO properties (email, description).
 
@@ -144,6 +148,4 @@ def map_vo(vo):
     # Newline is ignored by regexp if at end of string, so test for that as well.
     if not LONG_VO_RE.match(vo) or '\n' in vo:
         raise exception.RucioException('Invalid characters in VO name.')
-    if not config_db.has_section("vo-map"):
-        return vo  # No mapping config
-    return config_db.get("vo-map", vo, default=vo)
+    return config_get("vo-map", vo, raise_exception=False, default=vo)
