@@ -15,8 +15,9 @@
 
 
 from datetime import datetime
+from typing import TYPE_CHECKING
 
-from sqlalchemy import and_, or_, exists, update
+from sqlalchemy import and_, or_, exists, update, insert
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.sql.expression import select
 
@@ -26,9 +27,12 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import ReplicaState
 from rucio.db.sqla.session import transactional_session
 
+if TYPE_CHECKING:
+    from sqlalchemy.orm import Session
+
 
 @transactional_session
-def add_volatile_replicas(rse_id, replicas, session=None):
+def add_volatile_replicas(rse_id, replicas, *, session: "Session"):
     """
     Bulk add volatile replicas.
 
@@ -48,7 +52,7 @@ def add_volatile_replicas(rse_id, replicas, session=None):
     for replica in replicas:
         file_clause.append(and_(models.DataIdentifier.scope == replica['scope'],
                                 models.DataIdentifier.name == replica['name'],
-                                ~exists(select([1]).prefix_with("/*+ INDEX(REPLICAS REPLICAS_PK) */", dialect='oracle'))
+                                ~exists(select(1).prefix_with("/*+ INDEX(REPLICAS REPLICAS_PK) */", dialect='oracle'))
                                 .where(and_(models.RSEFileAssociation.scope == replica['scope'],
                                             models.RSEFileAssociation.name == replica['name'],
                                             models.RSEFileAssociation.rse_id == rse_id))))
@@ -72,15 +76,26 @@ def add_volatile_replicas(rse_id, replicas, session=None):
                                    models.DataIdentifier.adler32).\
             filter(or_(*file_clause))
 
-        session.bulk_insert_mappings(
-            models.RSEFileAssociation,
-            [{'rse_id': rse_id, 'adler32': adler32, 'state': ReplicaState.AVAILABLE,
-              'scope': scope, 'name': name, 'lock_cnt': 0, 'tombstone': datetime.utcnow(),
-              'bytes': bytes_, 'md5': md5} for scope, name, bytes_, md5, adler32 in file_query])
+        new_replicas = [
+            {
+                'rse_id': rse_id,
+                'adler32': adler32,
+                'state': ReplicaState.AVAILABLE,
+                'scope': scope,
+                'name': name,
+                'lock_cnt': 0,
+                'tombstone': datetime.utcnow(),
+                'bytes': bytes_,
+                'md5': md5
+            }
+            for scope, name, bytes_, md5, adler32 in file_query
+        ]
+        if new_replicas:
+            session.execute(insert(models.RSEFileAssociation), new_replicas)
 
 
 @transactional_session
-def delete_volatile_replicas(rse_id, replicas, session=None):
+def delete_volatile_replicas(rse_id, replicas, *, session: "Session"):
     """
     Bulk delete volatile replicas.
 
