@@ -20,6 +20,7 @@ import functools
 import logging
 import threading
 import time
+from typing import TYPE_CHECKING
 
 import rucio.db.sqla.util
 from rucio.common import exception
@@ -27,7 +28,12 @@ from rucio.common.logging import setup_logging
 from rucio.core.replica import get_cleaned_updated_collection_replicas, update_collection_replica
 from rucio.daemons.common import run_daemon
 
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
 graceful_stop = threading.Event()
+DAEMON_NAME = 'abacus-collection-replica'
 
 
 def collection_replica_update(once=False, limit=1000, sleep_time=10):
@@ -37,8 +43,7 @@ def collection_replica_update(once=False, limit=1000, sleep_time=10):
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
-        executable='abacus-collection-replica',
-        logger_prefix='collection_replica_update',
+        executable=DAEMON_NAME,
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -77,7 +82,7 @@ def run_once(heartbeat_handler, limit, **_kwargs):
     return must_sleep
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -89,7 +94,7 @@ def run(once=False, threads=1, sleep_time=10, limit=1000):
     """
     Starts up the Abacus-Collection-Replica threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
