@@ -13,16 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import queue as Queue
 import bz2
 import glob
 import logging
 import os
-import select
-import sys
-
+import queue as Queue
 from datetime import datetime
 from datetime import timedelta
+
+import select
+
 from rucio.common import config
 from rucio.common.dumper import LogPipeHandler
 from rucio.common.dumper import mkdir
@@ -33,8 +33,8 @@ from rucio.common.utils import chunks
 from rucio.core.quarantined_replica import add_quarantined_replicas
 from rucio.core.replica import declare_bad_file_replicas, list_replicas
 from rucio.core.rse import get_rse_usage, get_rse_id
-from rucio.daemons.auditor.hdfs import ReplicaFromHDFS
 from rucio.daemons.auditor import srmdumps
+from rucio.daemons.auditor.hdfs import ReplicaFromHDFS
 from rucio.db.sqla.constants import BadFilesStatus
 
 
@@ -155,7 +155,7 @@ def process_output(output, sanity_check=True, compress=True):
     rse = os.path.basename(output[:output.rfind('_')])
     rse_id = get_rse_id(rse=rse)
     usage = get_rse_usage(rse_id=rse_id, source='rucio')[0]
-    threshold = config.config_get('auditor', 'threshold', False, 0.2)
+    threshold = config.config_get_float('auditor', 'threshold', False, 0.2)
 
     # Perform a basic sanity check by comparing the number of entries
     # with the total number of files on the RSE.  If the percentage is
@@ -224,16 +224,13 @@ def check(queue, retry, terminate, logpipe, cache_dir, results_dir, keep_dumps,
             if output:
                 process_output(output)
         except:
-            success = False
-            class_, desc = sys.exc_info()[0:2]
-        else:
-            success = True
-        finally:
             elapsed = (datetime.now() - start).total_seconds() / 60
-            if success:
-                logger.info('SUCCESS checking "%s" in %d minutes', rse, elapsed)
-            else:
-                logger.error('Check of "%s" failed in %d minutes, %d remaining attemps: (%s: %s)', rse, elapsed, attemps, class_.__name__, desc)
+            logger.error('Check of "%s" failed in %d minutes, %d remaining attemps', rse, elapsed, attemps, exc_info=True)
+            success = False
+        else:
+            elapsed = (datetime.now() - start).total_seconds() / 60
+            logger.info('SUCCESS checking "%s" in %d minutes', rse, elapsed)
+            success = True
 
         if not keep_dumps:
             remove = glob.glob(os.path.join(cache_dir, 'replicafromhdfs_{0}_*'.format(rse)))
