@@ -1,20 +1,18 @@
-'''
- Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
- Licensed under the Apache License, Version 2.0 (the "License");
- You may not use this file except in compliance with the License.
- You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
-
- Authors:
- - Fernando Lopez, <felopez@cern.ch>, 2015
- - Dimitrios Christidis, <dimitrios.christidis@cern.ch>, 2020
-
- PY3K COMPATIBLE
-'''
-
-from rucio.common.dumper import DUMPS_CACHE_DIR
-from rucio.common.dumper import temp_file
-from rucio.common.dumper.data_models import Replica
 import hashlib
 import logging
 import os
@@ -23,6 +21,10 @@ import shutil
 import subprocess
 import tempfile
 
+from rucio.common.dumper import DUMPS_CACHE_DIR
+from rucio.common.dumper import temp_file
+from rucio.common.dumper.data_models import Replica
+
 
 def _hdfs_get(src_url, dst_path):
     cmd = ['hadoop', 'fs', '-get', src_url, dst_path]
