@@ -30,18 +30,23 @@ from rucio.client.uploadclient import UploadClient
 from rucio.common import exception
 from rucio.common.config import config_get, config_get_int, config_get_bool
 from rucio.common.logging import setup_logging
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.types import InternalScope
 from rucio.common.utils import execute, generate_uuid
-from rucio.core import monitor
+from rucio.core.monitor import MetricManager
 from rucio.core.scope import list_scopes
 from rucio.core.vo import map_vo
 from rucio.daemons.common import run_daemon
 
-
 if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
     from rucio.daemons.common import HeartbeatHandler
 
+METRICS = MetricManager(module=__name__)
 graceful_stop = threading.Event()
+DAEMON_NAME = "automatix"
 
 
 def get_data_distribution(inputfile: str):
@@ -52,7 +57,7 @@ def get_data_distribution(inputfile: str):
     for key in data:
         probability += data[key]["probability"]
         probabilities[key] = probability
-    for key in probabilities:
+    for key in list(probabilities):
         probabilities[key] = float(probabilities[key]) / probability
     return probabilities, data
 
@@ -88,9 +93,9 @@ def generate_didname(metadata: dict, dsn: str, did_type: str) -> str:
     fields = pattern.split(separator)
     file_name = ""
     for field in fields:
-        if field == "date":
-            field_str = str(datetime.now().date())
-        elif field == "did_prefix":
+        if field == 'date':
+            field_str = str(datetime.utcnow().date())
+        elif field == 'did_prefix':
             field_str = did_prefix
         elif field == "dsn":
             field_str = dsn
@@ -118,8 +123,7 @@ def automatix(inputfile: str, sleep_time: int, once: bool = False) -> None:
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
-        executable="automatix",
-        logger_prefix="automatix",
+        executable=DAEMON_NAME,
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -173,9 +177,9 @@ def run_once(heartbeat_handler: "HeartbeatHandler", inputfile: str, **_kwargs) -
     probabilities, data = get_data_distribution(inputfile)
     logger(logging.DEBUG, "Probabilities %s", probabilities)
 
-    cycle_timer = monitor.Timer()
+    cycle_stopwatch = Stopwatch()
     for rse in rses:
-        timer = monitor.Timer()
+        stopwatch = Stopwatch()
         _, _, logger = heartbeat_handler.live()
         tmpdir = tempfile.mkdtemp()
         logger(logging.INFO, "Running on RSE %s", rse)
@@ -227,9 +231,9 @@ def run_once(heartbeat_handler: "HeartbeatHandler", inputfile: str, **_kwargs) -
         ret = upload_client.upload(files)
         if ret == 0:
             logger(logging.INFO, "%s sucessfully registered" % dsn)
-            monitor.record_counter(name="automatix.addnewdataset.done", delta=1)
-            monitor.record_counter(name="automatix.addnewfile.done", delta=nbfiles)
-            timer.record('automatix.datasetinjection')
+            METRICS.counter(name="addnewdataset.done").inc()
+            METRICS.counter(name="addnewfile.done").inc(nbfiles)
+            METRICS.timer(name='datasetinjection').observe(stopwatch.elapsed)
         else:
             logger(logging.INFO, "Error uploading files")
         for physical_fname in physical_fnames:
@@ -238,7 +242,7 @@ def run_once(heartbeat_handler: "HeartbeatHandler", inputfile: str, **_kwargs) -
     logger(
         logging.INFO,
         "It took %f seconds to upload one dataset on %s",
-        cycle_timer.elapsed,
+        cycle_stopwatch.elapsed,
         str(rses),
     )
     return True
@@ -253,7 +257,7 @@ def run(
     """
     Starts up the automatix threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException(
@@ -270,7 +274,7 @@ def run(
         [thread.join(timeout=3.14) for thread in threads]
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
