@@ -15,13 +15,13 @@
 
 import functools
 import logging
-import math
 import re
 import threading
+from collections.abc import Callable
 from datetime import datetime
 from typing import TYPE_CHECKING
-from typing import Tuple, Dict, Callable
 
+import math
 from sqlalchemy.exc import DatabaseError
 
 import rucio.db.sqla.util
@@ -38,15 +38,18 @@ from rucio.daemons.common import run_daemon
 from rucio.db.sqla.constants import BadFilesStatus, BadPFNStatus, ReplicaState
 from rucio.db.sqla.session import get_session
 
-
 if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
     from rucio.daemons.common import HeartbeatHandler
     from rucio.common.types import InternalAccount
 
 graceful_stop = threading.Event()
+DAEMON_NAME = 'minos'
 
 
-def __classify_bad_pfns(pfns: list) -> Tuple[Dict, Dict]:
+def __classify_bad_pfns(pfns: list) -> tuple[dict, dict]:
     """
     Function that takes a list of PFNs and classify them in 2 dictionaries : bad_replicas and temporary_unvailables
     :param pfns: List of PFNs
@@ -132,7 +135,7 @@ def __update_temporary_unavailable(chunk: list, reason: str, expires_at: datetim
                 logger(logging.INFO, '%s is in unavailable state. Will be removed from the list of bad PFNs', str(rep['pfn']))
                 bulk_delete_bad_pfns(pfns=[rep['pfn']], session=None)
             # If the expiration date of the TEMPORARY_UNAVAILABLE is in the past, it is removed from the bad PFNs table
-            elif expires_at < datetime.now():
+            elif expires_at < datetime.utcnow():
                 logger(logging.INFO, 'PFN %s expiration time (%s) is older than now and is not in unavailable state. Removing the PFNs from bad_pfns', str(rep['pfn']), expires_at)
                 bulk_delete_bad_pfns(pfns=[rep['pfn']], session=None)
             # Else update everything in the same transaction
@@ -162,8 +165,7 @@ def minos(bulk: int = 1000, once: bool = False, sleep_time: int = 60) -> None:
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
-        executable='minos',
-        logger_prefix='minos',
+        executable=DAEMON_NAME,
         partition_wait_time=10,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -282,7 +284,7 @@ def run(threads: int = 1, bulk: int = 100, once: bool = False, sleep_time: int =
     """
     Starts up the minos threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException('Database was not updated, daemon won\'t start')
@@ -302,7 +304,7 @@ def run(threads: int = 1, bulk: int = 100, once: bool = False, sleep_time: int =
             thread_list = [thread.join(timeout=3.14) for thread in thread_list if thread and thread.is_alive()]
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
