@@ -15,12 +15,12 @@
 
 import functools
 import logging
-import math
 import re
 import threading
 import traceback
 from typing import TYPE_CHECKING
 
+import math
 from sqlalchemy.exc import DatabaseError
 
 import rucio.db.sqla.util
@@ -35,11 +35,14 @@ from rucio.daemons.common import run_daemon
 from rucio.db.sqla.constants import BadFilesStatus, ReplicaState
 from rucio.db.sqla.session import get_session
 
-
 if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
     from rucio.daemons.common import HeartbeatHandler
 
 graceful_stop = threading.Event()
+DAEMON_NAME = 'minos-temporary-expiration'
 
 
 def minos_tu_expiration(bulk: int = 1000, once: bool = False, sleep_time: int = 60) -> None:
@@ -54,8 +57,7 @@ def minos_tu_expiration(bulk: int = 1000, once: bool = False, sleep_time: int =
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
-        executable='minos-temporary-expiration',
-        logger_prefix='minos-temporary-expiration',
+        executable=DAEMON_NAME,
         partition_wait_time=10,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -144,7 +146,7 @@ def run(threads: int = 1, bulk: int = 100, once: bool = False, sleep_time: int =
     """
     Starts up the minos threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
@@ -164,7 +166,7 @@ def run(threads: int = 1, bulk: int = 100, once: bool = False, sleep_time: int =
             thread_list = [thread.join(timeout=3.14) for thread in thread_list if thread and thread.is_alive()]
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
