@@ -14,18 +14,14 @@
 # limitations under the License.
 
 import logging
-
 from datetime import datetime, date, timedelta
 from string import Template
-from sqlalchemy.orm import aliased
+
+from requests import get
 from sqlalchemy import func, and_, or_, cast, BigInteger
+from sqlalchemy.orm import aliased
 from sqlalchemy.sql.expression import case, select
 
-from rucio.core.lock import get_dataset_locks
-from rucio.core.rule import get_rule, add_rule, update_rule
-from rucio.core.rse_expression_parser import parse_expression
-from rucio.core.rse import list_rse_attributes, get_rse_name, get_rse_vo
-from rucio.core.rse_selector import RSESelector
 from rucio.common.config import config_get, config_get_int, config_get_bool
 from rucio.common.exception import (
     InsufficientTargetRSEs,
@@ -34,11 +30,14 @@ from rucio.common.exception import (
     InsufficientAccountLimit,
 )
 from rucio.common.types import InternalAccount, InternalScope
-
-from rucio.db.sqla.session import transactional_session, read_session
+from rucio.core.lock import get_dataset_locks
+from rucio.core.rse import list_rse_attributes, get_rse_name, get_rse_vo
+from rucio.core.rse_expression_parser import parse_expression
+from rucio.core.rse_selector import RSESelector
+from rucio.core.rule import get_rule, add_rule, update_rule
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType, RuleState, RuleGrouping, LockState
-from requests import get
+from rucio.db.sqla.session import transactional_session, read_session
 
 
 @transactional_session
@@ -49,7 +48,8 @@ def rebalance_rule(
     priority,
     source_replica_expression="*\\bb8-enabled=false",
     comment=None,
-    session=None,
+    *,
+    session,
 ):
     """
     Rebalance a replication rule to a new RSE
@@ -251,7 +251,7 @@ def _list_rebalance_rule_candidates_dump(rse_id, mode=None, logger=logging.log):
 
 
 @transactional_session
-def list_rebalance_rule_candidates(rse_id, mode=None, session=None):
+def list_rebalance_rule_candidates(rse_id, mode=None, *, session=None):
     """
     List the rebalance rule candidates based on the agreed on specification
     :param rse_id:       RSE of the source.
@@ -303,7 +303,7 @@ def list_rebalance_rule_candidates(rse_id, mode=None, session=None):
         expiration_time=3600,
     )
     if min_created_days > 0:
-        min_created_days = datetime.now() - timedelta(days=min_created_days)
+        min_created_days = datetime.utcnow() - timedelta(days=min_created_days)
         rule_clause.append(models.ReplicationRule.created_at < min_created_days)
 
     # Only move rules which are owned by <allowed_accounts> (coma separated accounts, e.g. panda,root,ddmadmin,jdoe)
@@ -396,7 +396,7 @@ def list_rebalance_rule_candidates(rse_id, mode=None, session=None):
     # Now build the query
     external_dsl = aliased(models.DatasetLock)
     count_locks = (
-        select([func.count()])
+        select(func.count())
         .where(
             and_(
                 external_dsl.scope == models.DatasetLock.scope,
@@ -416,15 +416,13 @@ def list_rebalance_rule_candidates(rse_id, mode=None, session=None):
             models.DataIdentifier.bytes,
             models.DataIdentifier.length,
             case(
-                [
-                    (
-                        or_(
-                            models.DatasetLock.length < 1,
-                            models.DatasetLock.length.is_(None),
-                        ),
-                        0,
-                    )
-                ],
+                (
+                    or_(
+                        models.DatasetLock.length < 1,
+                        models.DatasetLock.length.is_(None),
+                    ),
+                    0,
+                ),
                 else_=cast(
                     models.DatasetLock.bytes / models.DatasetLock.length, BigInteger
                 ),
@@ -446,15 +444,13 @@ def list_rebalance_rule_candidates(rse_id, mode=None, session=None):
         .filter(and_(*did_clause))
         .filter(
             case(
-                [
-                    (
-                        or_(
-                            models.DatasetLock.length < 1,
-                            models.DatasetLock.length.is_(None),
-                        ),
-                        0,
-                    )
-                ],
+                (
+                    or_(
+                        models.DatasetLock.length < 1,
+                        models.DatasetLock.length.is_(None),
+                    ),
+                    0,
+                ),
                 else_=cast(
                     models.DatasetLock.bytes / models.DatasetLock.length, BigInteger
                 ),
@@ -465,15 +461,13 @@ def list_rebalance_rule_candidates(rse_id, mode=None, session=None):
     )
     summary = query.order_by(
         case(
-            [
-                (
-                    or_(
-                        models.DatasetLock.length < 1,
-                        models.DatasetLock.length.is_(None),
-                    ),
-                    0,
-                )
-            ],
+            (
+                or_(
+                    models.DatasetLock.length < 1,
+                    models.DatasetLock.length.is_(None),
+                ),
+                0,
+            ),
             else_=cast(
                 models.DatasetLock.bytes / models.DatasetLock.length, BigInteger
             ),
@@ -493,6 +487,7 @@ def select_target_rse(
     other_rses=[],
     exclude_expression=None,
     force_expression=None,
+    *,
     session=None,
 ):
     """
@@ -592,6 +587,7 @@ def rebalance_rse(
     mode=None,
     priority=3,
     source_replica_expression="*\\bb8-enabled=false",
+    *,
     session=None,
     logger=logging.log,
 ):
@@ -665,7 +661,7 @@ def rebalance_rse(
                 if not dry_run:
                     child_rule_id = rebalance_rule(
                         parent_rule=rule,
-                        activity="Data rebalancing",
+                        activity="Data Rebalancing",
                         rse_expression=target_rse_exp,
                         priority=priority,
                         source_replica_expression=source_replica_expression,
@@ -727,7 +723,7 @@ def rebalance_rse(
 
 
 @read_session
-def get_active_locks(session=None):
+def get_active_locks(*, session=None):
     locks_dict = {}
     rule_ids = (
         session.query(models.ReplicationRule.id)
