@@ -19,13 +19,12 @@ This script is to be used to background rebalance ATLAS t2 datadisks
 
 from sqlalchemy import or_
 
-from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rse import get_rse_usage, get_rse_attribute
+from rucio.core.rse_expression_parser import parse_expression
 from rucio.daemons.bb8.common import rebalance_rse
 from rucio.db.sqla import models
-from rucio.db.sqla.session import get_session
 from rucio.db.sqla.constants import RuleState
-
+from rucio.db.sqla.session import get_session
 
 tolerance = 0.1
 max_total_rebalance_volume = 200 * 1E12
@@ -109,13 +108,13 @@ for src in rses_over_ratio:
 
 print('Excluding RSEs as desetinations which are blocklisted:')
 for des in rses_under_ratio:
-    if des['availability'] != 7:
+    if not (des['availability_read'] and des['availability_write'] and des['availability_delete']):
         print('  %s' % (des['rse']))
         rses_under_ratio.remove(des)
 
 print('Excluding RSEs as sources which are blocklisted:')
 for src in rses_over_ratio:
-    if src['availability'] != 7:
+    if not (src['availability_read'] and src['availability_write'] and src['availability_delete']):
         print('  %s' % (src['rse']))
         rses_over_ratio.remove(src)
 
