@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,15 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2015-2021
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2017-2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 '''
 Dynamic data placement daemon.
@@ -30,14 +21,15 @@ import logging
 from datetime import datetime
 from hashlib import md5
 from json import dumps
+from queue import Queue
 from threading import Event, Thread
 from time import sleep
+from typing import TYPE_CHECKING
 from uuid import uuid4
 
 from requests import post
 from requests.auth import HTTPBasicAuth
 from requests.exceptions import RequestException
-from six import string_types
 
 import rucio.db.sqla.util
 from rucio.client import Client
@@ -49,12 +41,12 @@ from rucio.daemons.c3po.collectors.free_space import FreeSpaceCollector
 from rucio.daemons.c3po.collectors.jedi_did import JediDIDCollector
 from rucio.daemons.c3po.collectors.workload import WorkloadCollector
 
-try:
-    from Queue import Queue
-except ImportError:
-    from queue import Queue
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
 
 GRACEFUL_STOP = Event()
+DAEMON_NAME = 'c3po'
 
 
 def read_free_space(once=False, thread=0, waiting_time=1800, sleep_time=10):
@@ -214,7 +206,7 @@ def place_replica(once=False,
 
             for _ in range(0, len_dids):
                 did = did_queue.get()
-                if isinstance(did[0], string_types):
+                if isinstance(did[0], str):
                     did[0] = InternalScope(did[0], vo=vo)
                 for algorithm, instance in instances.items():
                     logging.info('(%s:%s) Retrieved %s:%s from queue. Run placement algorithm' % (algorithm, instance_id, did[0], did[1]))
@@ -262,7 +254,7 @@ def place_replica(once=False,
         logging.critical(e)
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -293,7 +285,7 @@ def run(once=False,
     """
     Starts up the main thread
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
