@@ -22,20 +22,27 @@ import logging
 import threading
 import time
 from traceback import format_exc
+from typing import TYPE_CHECKING
 
 import rucio.db.sqla.util
 from rucio.common import exception
-from rucio.common.config import config_get, config_get_int, config_get_bool
+from rucio.common.config import config_get, config_get_int, config_get_bool, config_get_list
 from rucio.common.logging import setup_logging, formatted_logger
-from rucio.common.stomp_utils import get_stomp_brokers
+from rucio.common.stomp_utils import StompConnectionManager
 from rucio.common.types import InternalScope
-from rucio.core.monitor import record_counter
+from rucio.core.monitor import MetricManager
 from rucio.core.rse import get_rse_id
 from rucio.core.volatile_replica import add_volatile_replicas, delete_volatile_replicas
 
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
 logging.getLogger("stomp").setLevel(logging.CRITICAL)
 
+METRICS = MetricManager(module=__name__)
 GRACEFUL_STOP = threading.Event()
+DAEMON_NAME = 'cache-consumer'
 
 
 class AMQConsumer(object):
@@ -51,22 +58,22 @@ class AMQConsumer(object):
         self.__conn = conn
         self.__logger = logger
 
+    @METRICS.count_it
     def on_heartbeat_timeout(self):
-        record_counter('daemons.cache.consumer.heartbeat.lost')
         self.__conn.disconnect()
 
+    @METRICS.count_it
     def on_error(self, frame):
         """
         on_error
         """
-        record_counter('daemons.cache.consumer.error')
         self.__logger(logging.ERROR, 'Message receive error: [%s] %s' % (self.__broker, frame.body))
 
+    @METRICS.count_it
     def on_message(self, frame):
         """
         on_message
         """
-        record_counter('daemons.cache.consumer2.message')
         try:
             msg = json.loads(frame.body)
             self.__logger(logging.DEBUG, 'Message received: %s ' % msg)
@@ -99,22 +106,13 @@ def consumer(id_, num_thread=1):
     Main loop to consume messages from the Rucio Cache producer.
     """
 
-    prepend_str = 'cache-consumer '
-    logger = formatted_logger(logging.log, prepend_str + '%s')
+    logger = formatted_logger(logging.log, DAEMON_NAME + ' %s')
 
     logger(logging.INFO, 'Rucio Cache consumer starting')
 
-    try:
-        brokers_alias = [b.strip() for b in config_get('messaging-cache', 'brokers').split(',')]
-    except:
-        raise Exception('Could not load rucio cache brokers from configuration')
-
-    use_ssl = True
-    try:
-        use_ssl = config_get_bool('messaging-cache', 'use_ssl')
-    except Exception:
-        pass
+    brokers = config_get_list('messaging-cache', 'brokers')
 
+    use_ssl = config_get_bool('messaging-cache', 'use_ssl', default=True, raise_exception=False)
     if not use_ssl:
         username = config_get('messaging-cache', 'username')
         password = config_get('messaging-cache', 'password')
@@ -127,15 +125,18 @@ def consumer(id_, num_thread=1):
     ssl_key_file = config_get('messaging-cache', 'ssl_key_file', raise_exception=False)
     ssl_cert_file = config_get('messaging-cache', 'ssl_cert_file', raise_exception=False)
 
-    conns = get_stomp_brokers(brokers=brokers_alias,
-                              port=port,
-                              use_ssl=use_ssl,
-                              vhost=vhost,
-                              reconnect_attempts=reconnect_attempts,
-                              ssl_key_file=ssl_key_file,
-                              ssl_cert_file=ssl_cert_file,
-                              timeout=None,
-                              logger=logger)
+    stomp_conn_mngr = StompConnectionManager()
+    conns, _ = stomp_conn_mngr.re_configure(
+        brokers=brokers,
+        port=port,
+        use_ssl=use_ssl,
+        vhost=vhost,
+        reconnect_attempts=reconnect_attempts,
+        ssl_key_file=ssl_key_file,
+        ssl_cert_file=ssl_cert_file,
+        timeout=None,
+        logger=logger
+    )
 
     logger(logging.INFO, 'consumer started')
 
@@ -145,7 +146,7 @@ def consumer(id_, num_thread=1):
                 host_port = conn.transport._Transport__host_and_ports[0]
 
                 logger(logging.INFO, 'connecting to %s' % host_port[0])
-                record_counter('daemons.messaging.cache.reconnect.{host}', labels={'host': host_port[0]})
+                METRICS.counter('reconnect.{host}').labels(host=host_port[0]).inc()
                 conn.set_listener('rucio-cache-consumer', AMQConsumer(broker=host_port, conn=conn, logger=logger))
                 if not use_ssl:
                     conn.connect(username, password)
@@ -156,17 +157,11 @@ def consumer(id_, num_thread=1):
         time.sleep(1)
 
     logger(logging.INFO, 'graceful stop requested')
-
-    for conn in conns:
-        try:
-            conn.disconnect()
-        except:
-            pass
-
+    stomp_conn_mngr.disconnect()
     logger(logging.INFO, 'graceful stop done')
 
 
-def stop():
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -178,7 +173,7 @@ def run(num_thread=1):
     """
     Starts up the rucio cache consumer thread
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
