@@ -23,174 +23,225 @@ import logging
 import os
 import re
 import threading
+from types import FrameType
+from typing import TYPE_CHECKING, Optional
+from urllib.parse import urlparse
 
 from dogpile.cache.api import NoValue
 from sqlalchemy.exc import DatabaseError
 
 import rucio.db.sqla.util
 from rucio.common.cache import make_region_memcached
-from rucio.common.exception import DatabaseException, ConfigNotFound, UnsupportedOperation, ReplicaNotFound, RequestNotFound, RSEProtocolNotSupported
+from rucio.common.config import config_get_list, config_get_bool
+from rucio.common.exception import DatabaseException, UnsupportedOperation, ReplicaNotFound, RequestNotFound, RSEProtocolNotSupported
 from rucio.common.logging import setup_logging
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.types import InternalAccount
 from rucio.common.utils import chunks
 from rucio.core import request as request_core, replica as replica_core
-from rucio.core.config import items
-from rucio.core.monitor import record_counter, Timer
+from rucio.core.monitor import MetricManager
 from rucio.core.rse import list_rses
-from rucio.daemons.common import run_daemon
+from rucio.core.transfer import ProtocolFactory
+from rucio.core.topology import Topology, ExpiringObjectCache
+from rucio.daemons.common import db_workqueue, ProducerConsumerDaemon
 from rucio.db.sqla.constants import RequestState, RequestType, ReplicaState, BadFilesStatus
 from rucio.db.sqla.session import transactional_session
-from rucio.rse import rsemanager
 
-from urllib.parse import urlparse
+if TYPE_CHECKING:
+    from rucio.daemons.common import HeartbeatHandler
 
-graceful_stop = threading.Event()
+GRACEFUL_STOP = threading.Event()
 
-region = make_region_memcached(expiration_time=900)
+REGION = make_region_memcached(expiration_time=900)
+METRICS = MetricManager(module=__name__)
+DAEMON_NAME = 'conveyor-finisher'
+FAILED_DURING_SUBMISSION_DELAY = datetime.timedelta(minutes=120)
 
 
-def run_once(bulk, db_bulk, suspicious_patterns, retry_protocol_mismatches, heartbeat_handler, activity):
+def _fetch_requests(
+        db_bulk,
+        set_last_processed_by: bool,
+        cached_topology,
+        heartbeat_handler,
+        activity,
+):
     worker_number, total_workers, logger = heartbeat_handler.live()
 
+    logger(logging.DEBUG, 'Working on activity %s', activity)
+
+    topology = cached_topology.get() if cached_topology else Topology()
+
+    get_requests_fnc = functools.partial(
+        request_core.get_and_mark_next,
+        rse_collection=topology,
+        request_type=[RequestType.TRANSFER, RequestType.STAGEIN, RequestType.STAGEOUT],
+        processed_by=heartbeat_handler.short_executable if set_last_processed_by else None,
+        limit=db_bulk,
+        total_workers=total_workers,
+        worker_number=worker_number,
+        mode_all=True,
+        include_dependent=False,
+        hash_variable='rule_id',
+    )
+    reqs = get_requests_fnc(
+        state=[
+            RequestState.DONE,
+            RequestState.FAILED,
+            RequestState.LOST,
+            RequestState.SUBMISSION_FAILED,
+            RequestState.NO_SOURCES,
+            RequestState.ONLY_TAPE_SOURCES,
+            RequestState.MISMATCH_SCHEME
+        ],
+    )
+    reqs.extend(
+        get_requests_fnc(
+            state=[RequestState.SUBMITTING],
+            older_than=datetime.datetime.utcnow() - FAILED_DURING_SUBMISSION_DELAY
+        )
+    )
+
+    must_sleep = False
+    if len(reqs) < db_bulk / 2:
+        logger(logging.INFO, "Only %s transfers, which is less than half of the bulk %s", len(reqs), db_bulk)
+        must_sleep = True
+    return must_sleep, (reqs, topology)
+
+
+def _handle_requests(
+        batch,
+        bulk,
+        suspicious_patterns,
+        retry_protocol_mismatches,
+        *,
+        logger=logging.log,
+):
+    reqs, topology = batch
+    if not reqs:
+        return
+
     try:
-        logger(logging.DEBUG, 'Working on activity %s', activity)
+        logger(logging.DEBUG, 'Updating %i requests', len(reqs))
 
-        with Timer('daemons.conveyor.finisher.get_next'):
-            reqs = request_core.get_next(request_type=[RequestType.TRANSFER, RequestType.STAGEIN, RequestType.STAGEOUT],
-                                         state=[RequestState.DONE, RequestState.FAILED,
-                                                RequestState.LOST, RequestState.SUBMITTING,
-                                                RequestState.SUBMISSION_FAILED, RequestState.NO_SOURCES,
-                                                RequestState.ONLY_TAPE_SOURCES, RequestState.MISMATCH_SCHEME],
-                                         limit=db_bulk,
-                                         older_than=datetime.datetime.utcnow(),
-                                         total_workers=total_workers,
-                                         worker_number=worker_number,
-                                         mode_all=True,
-                                         include_dependent=False,
-                                         hash_variable='rule_id')
-
-        if reqs:
-            logger(logging.DEBUG, 'Updating %i requests for activity %s', len(reqs), activity)
-
-        timer = Timer()
+        total_stopwatch = Stopwatch()
 
         for chunk in chunks(reqs, bulk):
             try:
-                worker_number, total_workers, logger = heartbeat_handler.live()
-                with Timer('daemons.conveyor.finisher.handle_requests_time', divisor=len(chunk) or 1):
-                    __handle_requests(chunk, suspicious_patterns, retry_protocol_mismatches, logger=logger)
-                record_counter('daemons.conveyor.finisher.handle_requests', delta=len(chunk))
+                stopwatch = Stopwatch()
+                _finish_requests(topology, chunk, suspicious_patterns, retry_protocol_mismatches, logger=logger)
+                METRICS.timer('handle_requests_time').observe(stopwatch.elapsed / (len(chunk) or 1))
+                METRICS.counter('handle_requests').inc(len(chunk))
             except Exception as error:
                 logger(logging.WARNING, '%s', str(error))
 
-        timer.stop()
-
-        if reqs:
-            logger(logging.DEBUG, 'Finish to update %s finished requests for activity %s in %s seconds', len(reqs), activity, timer.elapsed)
+        logger(logging.DEBUG, 'Finish to update %s finished requests in %s seconds', len(reqs), total_stopwatch.elapsed)
 
     except (DatabaseException, DatabaseError) as error:
         if re.match('.*ORA-00054.*', error.args[0]) or re.match('.*ORA-00060.*', error.args[0]) or 'ERROR 1205 (HY000)' in error.args[0]:
             logger(logging.WARNING, 'Lock detected when handling request - skipping: %s', str(error))
         else:
             raise
-    return True
 
 
-def finisher(once=False, sleep_time=60, activities=None, bulk=100, db_bulk=1000, partition_wait_time=10):
+def finisher(
+        once=False,
+        sleep_time=60,
+        activities=None,
+        bulk=100,
+        db_bulk=1000,
+        partition_wait_time=10,
+        cached_topology=None,
+        total_threads=1,
+):
     """
     Main loop to update the replicas and rules based on finished requests.
     """
-    try:
-        conveyor_config = {item[0]: item[1] for item in items('conveyor')}
-    except ConfigNotFound:
-        logging.log(logging.INFO, 'No configuration found for conveyor')
-        conveyor_config = {}
-
     # Get suspicious patterns
-    suspicious_patterns = conveyor_config.get('suspicious_pattern', [])
-    if suspicious_patterns:
-        pattern = str(suspicious_patterns)
-        patterns = pattern.split(",")
-        suspicious_patterns = [re.compile(pat.strip()) for pat in patterns]
+    suspicious_patterns = config_get_list('conveyor', 'suspicious_pattern', default=[])
+    suspicious_patterns = [re.compile(pat.strip()) for pat in suspicious_patterns]
     logging.log(logging.DEBUG, "Suspicious patterns: %s" % [pat.pattern for pat in suspicious_patterns])
 
-    retry_protocol_mismatches = conveyor_config.get('retry_protocol_mismatches', False)
+    retry_protocol_mismatches = config_get_bool('conveyor', 'retry_protocol_mismatches', default=False)
 
-    logger_prefix = executable = 'conveyor-finisher'
+    executable = DAEMON_NAME
     if activities:
         activities.sort()
         executable += '--activities ' + str(activities)
 
-    run_daemon(
+    @db_workqueue(
         once=once,
-        graceful_stop=graceful_stop,
+        graceful_stop=GRACEFUL_STOP,
         executable=executable,
-        logger_prefix=logger_prefix,
         partition_wait_time=partition_wait_time,
         sleep_time=sleep_time,
-        run_once_fnc=functools.partial(
-            run_once,
-            bulk=bulk,
-            db_bulk=db_bulk,
-            suspicious_patterns=suspicious_patterns,
-            retry_protocol_mismatches=retry_protocol_mismatches,
-        ),
         activities=activities,
     )
+    def _db_producer(*, activity: str, heartbeat_handler: "HeartbeatHandler"):
+        return _fetch_requests(
+            db_bulk=db_bulk,
+            cached_topology=cached_topology,
+            activity=activity,
+            set_last_processed_by=not once,
+            heartbeat_handler=heartbeat_handler,
+        )
+
+    def _consumer(batch):
+        return _handle_requests(
+            batch=batch,
+            bulk=bulk,
+            suspicious_patterns=suspicious_patterns,
+            retry_protocol_mismatches=retry_protocol_mismatches,
+        )
+
+    ProducerConsumerDaemon(
+        producers=[_db_producer],
+        consumers=[_consumer for _ in range(total_threads)],
+        graceful_stop=GRACEFUL_STOP,
+    ).run()
 
 
-def stop(signum=None, frame=None):
+def stop(signum: Optional[int] = None, frame: Optional[FrameType] = None) -> None:
     """
     Graceful exit.
     """
 
-    graceful_stop.set()
+    GRACEFUL_STOP.set()
 
 
 def run(once=False, total_threads=1, sleep_time=60, activities=None, bulk=100, db_bulk=1000):
     """
     Starts up the conveyer threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException('Database was not updated, daemon won\'t start')
 
-    if once:
-        logging.log(logging.INFO, 'executing one finisher iteration only')
-        finisher(once=once, activities=activities, bulk=bulk, db_bulk=db_bulk)
-
-    else:
-
-        logging.log(logging.INFO, 'starting finisher threads')
-        threads = [threading.Thread(target=finisher, kwargs={'sleep_time': sleep_time,
-                                                             'activities': activities,
-                                                             'db_bulk': db_bulk,
-                                                             'bulk': bulk}) for _ in range(0, total_threads)]
-
-        [thread.start() for thread in threads]
-
-        logging.log(logging.INFO, 'waiting for interrupts')
-
-        # Interruptible joins require a timeout.
-        while threads:
-            threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.is_alive()]
+    cached_topology = ExpiringObjectCache(ttl=300, new_obj_fnc=lambda: Topology())
+    finisher(
+        once=once,
+        activities=activities,
+        bulk=bulk,
+        db_bulk=db_bulk,
+        sleep_time=sleep_time,
+        cached_topology=cached_topology,
+        total_threads=total_threads
+    )
 
 
-def __handle_requests(reqs, suspicious_patterns, retry_protocol_mismatches, logger=logging.log):
+def _finish_requests(topology: "Topology", reqs, suspicious_patterns, retry_protocol_mismatches, logger=logging.log):
     """
     Used by finisher to handle terminated requests,
 
     :param reqs:                         List of requests.
     :param suspicious_patterns:          List of suspicious patterns.
     :param retry_protocol_mismatches:    Boolean to retry the transfer in case of protocol mismatch.
-    :param prepend_str: String to prepend to logging.
     """
 
     failed_during_submission = [RequestState.SUBMITTING, RequestState.SUBMISSION_FAILED, RequestState.LOST]
     failed_no_submission_attempts = [RequestState.NO_SOURCES, RequestState.ONLY_TAPE_SOURCES, RequestState.MISMATCH_SCHEME]
     undeterministic_rses = __get_undeterministic_rses(logger=logger)
-    rses_info, protocols = {}, {}
+    protocol_factory = ProtocolFactory()
     replicas = {}
     for req in reqs:
         try:
@@ -211,14 +262,11 @@ def __handle_requests(reqs, suspicious_patterns, retry_protocol_mismatches, logg
 
                 # for TAPE, replica path is needed
                 if req['request_type'] in (RequestType.TRANSFER, RequestType.STAGEIN) and req['dest_rse_id'] in undeterministic_rses:
-                    if req['dest_rse_id'] not in rses_info:
-                        rses_info[req['dest_rse_id']] = rsemanager.get_rse_info(rse_id=req['dest_rse_id'])
+                    dst_rse = topology[req['dest_rse_id']].ensure_loaded(load_info=True)
                     pfn = req['dest_url']
                     scheme = urlparse(pfn).scheme
-                    dest_rse_id_scheme = '%s_%s' % (req['dest_rse_id'], scheme)
-                    if dest_rse_id_scheme not in protocols:
-                        protocols[dest_rse_id_scheme] = rsemanager.create_protocol(rses_info[req['dest_rse_id']], 'write', scheme)
-                    path = protocols[dest_rse_id_scheme].parse_pfns([pfn])[pfn]['path']
+                    protocol = protocol_factory.protocol(dst_rse, scheme, 'write')
+                    path = protocol.parse_pfns([pfn])[pfn]['path']
                     replica['path'] = os.path.join(path, os.path.basename(pfn))
 
                 # replica should not be added to replicas until all info are filled
@@ -227,13 +275,11 @@ def __handle_requests(reqs, suspicious_patterns, retry_protocol_mismatches, logg
             # Standard failure from the transfer tool
             elif req['state'] == RequestState.FAILED:
                 __check_suspicious_files(req, suspicious_patterns, logger=logger)
-                timer = Timer()
                 try:
                     if request_core.should_retry_request(req, retry_protocol_mismatches):
                         new_req = request_core.requeue_and_archive(req, source_ranking_update=True, retry_protocol_mismatches=retry_protocol_mismatches, logger=logger)
                         # should_retry_request and requeue_and_archive are not in one session,
                         # another process can requeue_and_archive and this one will return None.
-                        timer.record('daemons.conveyor.common.update_request_state.request_requeue_and_archive')
                         logger(logging.WARNING, 'REQUEUED DID %s:%s REQUEST %s AS %s TRY %s' % (req['scope'],
                                                                                                 req['name'],
                                                                                                 req['request_id'],
@@ -251,14 +297,9 @@ def __handle_requests(reqs, suspicious_patterns, retry_protocol_mismatches, logg
 
             # All other failures
             elif req['state'] in failed_during_submission or req['state'] in failed_no_submission_attempts:
-                if req['state'] in failed_during_submission and req['updated_at'] > (datetime.datetime.utcnow() - datetime.timedelta(minutes=120)):
-                    # To prevent race conditions
-                    continue
                 try:
-                    timer = Timer()
                     if request_core.should_retry_request(req, retry_protocol_mismatches):
                         new_req = request_core.requeue_and_archive(req, source_ranking_update=False, retry_protocol_mismatches=retry_protocol_mismatches, logger=logger)
-                        timer.record('daemons.conveyor.common.update_request_state.request_requeue_and_archive')
                         logger(logging.WARNING, 'REQUEUED SUBMITTING DID %s:%s REQUEST %s AS %s TRY %s' % (req['scope'],
                                                                                                            req['name'],
                                                                                                            req['request_id'],
@@ -291,12 +332,12 @@ def __get_undeterministic_rses(logger=logging.log):
     :returns:  List of undeterministc rses
     """
     key = 'undeterministic_rses'
-    result = region.get(key)
+    result = REGION.get(key)
     if isinstance(result, NoValue):
         rses_list = list_rses(filters={'deterministic': False})
         result = [rse['id'] for rse in rses_list]
         try:
-            region.set(key, result)
+            REGION.set(key, result)
         except Exception as error:
             logger(logging.WARNING, "Failed to set dogpile cache, error: %s", str(error))
     return result
@@ -340,7 +381,6 @@ def __handle_terminated_replicas(replicas, logger=logging.log):
     Used by finisher to handle available and unavailable replicas.
 
     :param replicas: List of replicas.
-    :param prepend_str: String to prepend to logging.
     """
 
     for req_type in replicas:
@@ -374,7 +414,7 @@ def __handle_terminated_replicas(replicas, logger=logging.log):
 
 
 @transactional_session
-def __update_bulk_replicas(replicas, session=None, logger=logging.log):
+def __update_bulk_replicas(replicas, *, session, logger=logging.log):
     """
     Used by finisher to handle available and unavailable replicas blongs to same rule in bulk way.
 
@@ -396,7 +436,7 @@ def __update_bulk_replicas(replicas, session=None, logger=logging.log):
 
 
 @transactional_session
-def __update_replica(replica, session=None, logger=logging.log):
+def __update_replica(replica, *, session, logger=logging.log):
     """
     Used by finisher to update a replica to a finished state.
 
