@@ -17,13 +17,14 @@
 Conveyor is a daemon to manage file transfers.
 """
 
-import datetime
 import json
 import logging
 import socket
 import threading
 import time
 import traceback
+from types import FrameType
+from typing import Optional
 
 import stomp
 
@@ -32,34 +33,33 @@ from rucio.common import exception
 from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.logging import setup_logging
 from rucio.common.policy import get_policy
-from rucio.core import request as request_core
-from rucio.core.monitor import record_counter
-from rucio.core.transfer import set_transfer_update_time
+from rucio.core import transfer as transfer_core
+from rucio.core.monitor import MetricManager
 from rucio.daemons.common import HeartbeatHandler
 from rucio.db.sqla.session import transactional_session
 from rucio.transfertool.fts3 import FTS3CompletionMessageTransferStatusReport
 
 logging.getLogger("stomp").setLevel(logging.CRITICAL)
 
-graceful_stop = threading.Event()
+METRICS = MetricManager(module=__name__)
+GRACEFUL_STOP = threading.Event()
+DAEMON_NAME = 'conveyor-receiver'
 
 
 class Receiver(object):
 
-    def __init__(self, broker, id_, total_threads, full_mode=False, all_vos=False):
+    def __init__(self, broker, id_, total_threads, all_vos=False):
         self.__all_vos = all_vos
         self.__broker = broker
         self.__id = id_
         self.__total_threads = total_threads
-        self.__full_mode = full_mode
 
+    @METRICS.count_it
     def on_error(self, frame):
-        record_counter('daemons.conveyor.receiver.error')
         logging.error('[%s] %s' % (self.__broker, frame.body))
 
+    @METRICS.count_it
     def on_message(self, frame):
-        record_counter('daemons.conveyor.receiver.message_all')
-
         msg = json.loads(frame.body)
 
         if not self.__all_vos:
@@ -72,12 +72,12 @@ class Receiver(object):
            and str(msg['job_metadata']['issuer']) == str('rucio'):
 
             if 'job_state' in msg.keys() and (str(msg['job_state']) != str('ACTIVE') or msg.get('job_multihop', False) is True):
-                record_counter('daemons.conveyor.receiver.message_rucio')
+                METRICS.counter('message_rucio').inc()
 
                 self._perform_request_update(msg)
 
     @transactional_session
-    def _perform_request_update(self, msg, session=None, logger=logging.log):
+    def _perform_request_update(self, msg, *, session=None, logger=logging.log):
         external_host = msg.get('endpnt', None)
         request_id = msg['file_metadata'].get('request_id', None)
         try:
@@ -85,28 +85,18 @@ class Receiver(object):
             if tt_status_report.get_db_fields_to_update(session=session, logger=logger):
                 logging.info('RECEIVED %s', tt_status_report)
 
-                if self.__full_mode:
-                    ret = request_core.update_request_state(tt_status_report, session=session, logger=logger)
-                    record_counter('daemons.conveyor.receiver.update_request_state.{updated}', labels={'updated': ret})
-                else:
-                    try:
-                        logging.debug("Update request %s update time" % request_id)
-                        set_transfer_update_time(external_host, tt_status_report.external_id, datetime.datetime.utcnow() - datetime.timedelta(hours=24), session=session)
-                        record_counter('daemons.conveyor.receiver.set_transfer_update_time')
-                    except Exception as error:
-                        logging.debug("Failed to update transfer's update time: %s" % str(error))
+                ret = transfer_core.update_transfer_state(tt_status_report, session=session, logger=logger)
+                METRICS.counter('update_request_state.{updated}').labels(updated=ret).inc()
         except Exception:
             logging.critical(traceback.format_exc())
 
 
-def receiver(id_, total_threads=1, full_mode=False, all_vos=False):
+def receiver(id_, total_threads=1, all_vos=False):
     """
     Main loop to consume messages from the FTS3 producer.
     """
 
-    logging.info('receiver starting in full mode: %s' % full_mode)
-
-    logger_prefix = executable = 'conveyor-receiver'
+    logging.info('receiver starting')
 
     brokers_alias = []
     brokers_resolved = []
@@ -142,25 +132,23 @@ def receiver(id_, total_threads=1, full_mode=False, all_vos=False):
     for broker in brokers_resolved:
         if not use_ssl:
             logging.info('setting up username/password authentication: %s' % broker)
-            con = stomp.Connection12(host_and_ports=[(broker, port)],
-                                     use_ssl=False,
-                                     vhost=vhost,
-                                     reconnect_attempts_max=999)
         else:
             logging.info('setting up ssl cert/key authentication: %s' % broker)
-            con = stomp.Connection12(host_and_ports=[(broker, port)],
-                                     use_ssl=True,
-                                     ssl_key_file=config_get('messaging-fts3', 'ssl_key_file'),
-                                     ssl_cert_file=config_get('messaging-fts3', 'ssl_cert_file'),
-                                     vhost=vhost,
-                                     reconnect_attempts_max=999)
+        con = stomp.Connection12(host_and_ports=[(broker, port)],
+                                 vhost=vhost,
+                                 reconnect_attempts_max=999)
+        if use_ssl:
+            con.set_ssl(
+                key_file=config_get('messaging-fts3', 'ssl_key_file'),
+                cert_file=config_get('messaging-fts3', 'ssl_cert_file'),
+            )
         conns.append(con)
 
     logging.info('receiver started')
 
-    with HeartbeatHandler(executable=executable, renewal_interval=30, logger_prefix=logger_prefix) as heartbeat_handler:
+    with HeartbeatHandler(executable=DAEMON_NAME, renewal_interval=30) as heartbeat_handler:
 
-        while not graceful_stop.is_set():
+        while not GRACEFUL_STOP.is_set():
 
             _, _, logger = heartbeat_handler.live()
 
@@ -168,11 +156,10 @@ def receiver(id_, total_threads=1, full_mode=False, all_vos=False):
 
                 if not conn.is_connected():
                     logger(logging.INFO, 'connecting to %s' % conn.transport._Transport__host_and_ports[0][0])
-                    record_counter('daemons.messaging.fts3.reconnect.{host}', labels={'host': conn.transport._Transport__host_and_ports[0][0].split('.')[0]})
+                    METRICS.counter('reconnect.{host}').labels(host=conn.transport._Transport__host_and_ports[0][0].split('.')[0]).inc()
 
                     conn.set_listener('rucio-messaging-fts3', Receiver(broker=conn.transport._Transport__host_and_ports[0],
-                                                                       id_=id_, total_threads=total_threads,
-                                                                       full_mode=full_mode, all_vos=all_vos))
+                                                                       id_=id_, total_threads=total_threads, all_vos=all_vos))
                     if not use_ssl:
                         conn.connect(username, password, wait=True)
                     else:
@@ -189,26 +176,25 @@ def receiver(id_, total_threads=1, full_mode=False, all_vos=False):
                 pass
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
 
-    graceful_stop.set()
+    GRACEFUL_STOP.set()
 
 
-def run(once=False, total_threads=1, full_mode=False):
+def run(once=False, total_threads=1):
     """
     Starts up the receiver thread
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
 
     logging.info('starting receiver thread')
     threads = [threading.Thread(target=receiver, kwargs={'id_': i,
-                                                         'full_mode': full_mode,
                                                          'total_threads': total_threads}) for i in range(0, total_threads)]
 
     [thread.start() for thread in threads]
