@@ -17,98 +17,97 @@
 Conveyor throttler is a daemon to manage rucio internal queue.
 """
 import logging
-import math
 import threading
 import traceback
 from collections import defaultdict
+from types import FrameType
+from typing import TYPE_CHECKING, Optional
 
+import math
 from sqlalchemy import null
 
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.logging import setup_logging
-from rucio.core.monitor import record_counter, record_gauge
-from rucio.core.request import get_request_stats, release_all_waiting_requests, release_waiting_requests_fifo, release_waiting_requests_grouped_fifo
-from rucio.core.transfer import applicable_rse_transfer_limits
+from rucio.core.monitor import MetricManager
+from rucio.core.request import (get_request_stats, release_all_waiting_requests, release_waiting_requests_fifo,
+                                release_waiting_requests_grouped_fifo, set_transfer_limit_stats, re_sync_all_transfer_limits)
 from rucio.core.rse import RseCollection
-from rucio.daemons.common import run_daemon
-from rucio.db.sqla.constants import RequestState
+from rucio.core.transfer import applicable_rse_transfer_limits
+from rucio.daemons.common import db_workqueue, ProducerConsumerDaemon
+from rucio.db.sqla.constants import RequestState, TransferLimitDirection
 
-graceful_stop = threading.Event()
+if TYPE_CHECKING:
+    from rucio.daemons.common import HeartbeatHandler
+
+GRACEFUL_STOP = threading.Event()
+METRICS = MetricManager(module=__name__)
+DAEMON_NAME = 'conveyor-throttler'
 
 
-def throttler(once=False, sleep_time=600, partition_wait_time=10):
+def throttler(
+        once=False,
+        sleep_time=600,
+        partition_wait_time=10
+):
     """
     Main loop to check rse transfer limits.
     """
 
     logging.info('Throttler starting')
 
-    logger_prefix = executable = 'conveyor-throttler'
-
-    run_daemon(
+    @db_workqueue(
         once=once,
-        graceful_stop=graceful_stop,
-        executable=executable,
-        logger_prefix=logger_prefix,
+        graceful_stop=GRACEFUL_STOP,
+        executable=DAEMON_NAME,
         partition_wait_time=partition_wait_time,
-        sleep_time=sleep_time,
-        run_once_fnc=run_once,
-        activities=None,
-    )
+        sleep_time=sleep_time)
+    def _db_producer(*, activity: str, heartbeat_handler: "HeartbeatHandler"):
+        worker_number, total_workers, logger = heartbeat_handler.live()
+        if worker_number != 0:
+            logger(logging.INFO, 'Throttler thread id is not 0, will sleep. Only thread 0 will work')
+            return True, None
+
+        re_sync_all_transfer_limits()
+        rse_collection = RseCollection()
+        release_groups = _get_request_stats(rse_collection, logger=logger)
+        return True, release_groups
+
+    def _consumer(release_groups):
+        if release_groups is None:
+            return
+        logger = logging.log
+        logger(logging.INFO, "Throttler - schedule requests")
+        try:
+            _handle_requests(release_groups, logger=logger)
+        except Exception:
+            logger(logging.CRITICAL, "Failed to schedule requests, error: %s" % (traceback.format_exc()))
+
+    ProducerConsumerDaemon(
+        producers=[_db_producer],
+        consumers=[_consumer],
+        graceful_stop=GRACEFUL_STOP,
+    ).run()
 
 
-def stop(signum=None, frame=None):
+def stop(signum: Optional[int] = None, frame: Optional[FrameType] = None) -> None:
     """
     Graceful exit.
     """
 
-    graceful_stop.set()
+    GRACEFUL_STOP.set()
 
 
 def run(once=False, sleep_time=600):
     """
     Starts up the conveyer threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
 
-    if once:
-        logging.info('running throttler one iteration only')
-        throttler(once=True, sleep_time=sleep_time)
-    else:
-        threads = []
-        logging.info('starting throttler thread')
-        throttler_thread = threading.Thread(target=throttler, kwargs={'once': once, 'sleep_time': sleep_time})
-        threads.append(throttler_thread)
-        [thread.start() for thread in threads]
-
-        logging.info('waiting for interrupts')
-
-        # Interruptible joins require a timeout.
-        while threads:
-            threads = [thread.join(timeout=3.14) for thread in threads if thread and thread.is_alive()]
-
-
-def run_once(worker_number=0, logger=logging.log, session=None, heartbeat_handler=None, **kwargs):
-    """
-    Schedule requests
-    """
-    if heartbeat_handler:
-        worker_number, total_workers, logger = heartbeat_handler.live()
-    if worker_number != 0:
-        logger(logging.INFO, 'Throttler thread id is not 0, will sleep. Only thread 0 will work')
-        return True
-    logger(logging.INFO, "Throttler - schedule requests")
-    try:
-        rse_collection = RseCollection()
-        release_groups = _get_request_stats(rse_collection, logger=logger, session=session)
-        _release_requests(rse_collection, release_groups, logger=logger, session=session)
-    except Exception:
-        logger(logging.CRITICAL, "Failed to schedule requests, error: %s" % (traceback.format_exc()))
-    return True
+    throttler(once=once, sleep_time=sleep_time)
 
 
 class RequestGrouper:
@@ -135,14 +134,14 @@ class RequestGrouper:
         src_info = self.rse_stats[source_rse]
         dst_info = self.rse_stats[dest_rse]
 
-        if dest_rse and not dest_rse.columns.availability & 2:
+        if dest_rse and not dest_rse.columns['availability_write']:
             src_info.unavailable_destinations.add(dest_rse)
-        if source_rse and not source_rse.columns.availability & 4:
+        if source_rse and not source_rse.columns['availability_read']:
             dst_info.unavailable_sources.add(source_rse)
         for limit_stat in applicable_limits:
             limit = limit_stat['limit']
             limit_is_per_activity = limit['activity'] is not None
-            if source_rse and source_rse.id == limit['rse_id']:
+            if source_rse and limit['direction'] == TransferLimitDirection.SOURCE:
                 src_info.has_any_per_activity_limit |= limit_is_per_activity
                 dst_info.sources_with_limits.add(source_rse)
                 dst_info.any_source_has_per_activity_limit |= limit_is_per_activity
@@ -193,7 +192,7 @@ class RequestGrouper:
         return merged_groups
 
 
-def _get_request_stats(rse_collection: RseCollection, logger=logging.log, session=None):
+def _get_request_stats(rse_collection: RseCollection, *, logger=logging.log):
     """
     Group waiting requests into arbitrary groups for bulk handling.
     The current grouping (source rse + dest rse + activity) was dictated
@@ -215,7 +214,6 @@ def _get_request_stats(rse_collection: RseCollection, logger=logging.log, sessio
                RequestState.SUBMITTING,
                RequestState.SUBMITTED,
                RequestState.WAITING],
-        session=session,
     )
 
     # for each active limit, compute how many waiting and active transfers are currently in the database
@@ -228,19 +226,27 @@ def _get_request_stats(rse_collection: RseCollection, logger=logging.log, sessio
         counter = db_stat.counter
         activity = db_stat.activity
 
-        dest_rse = rse_collection[db_stat.dest_rse_id].ensure_loaded(load_transfer_limits=True, load_name=True, load_columns=True, session=session)
+        try:
+            dest_rse = rse_collection[db_stat.dest_rse_id].ensure_loaded(load_transfer_limits=True, load_name=True, load_columns=True)
+        except exception.RSENotFound:
+            logger(logging.INFO, "Destination RSE {} not found. Probably deleted.", db_stat.dest_rse_id)
+            continue
         source_rse = None
         if db_stat.source_rse_id:
-            source_rse = rse_collection[db_stat.source_rse_id].ensure_loaded(load_transfer_limits=True, load_name=True, load_columns=True, session=session)
+            try:
+                source_rse = rse_collection[db_stat.source_rse_id].ensure_loaded(load_transfer_limits=True, load_name=True, load_columns=True)
+            except exception.RSENotFound:
+                logger(logging.INFO, "Source RSE {} not found. Probably deleted.", db_stat.source_rse_id)
+                continue
 
-        source_limits = list(applicable_rse_transfer_limits(activity=activity, source_rse=source_rse, session=session))
-        dest_limits = list(applicable_rse_transfer_limits(activity=activity, dest_rse=dest_rse, session=session))
+        source_limits = list(applicable_rse_transfer_limits(activity=activity, source_rse=source_rse))
+        dest_limits = list(applicable_rse_transfer_limits(activity=activity, dest_rse=dest_rse))
         limits = source_limits + dest_limits
 
         if counter and (limits or state == RequestState.WAITING):
             applicable_limits = []
             for limit in limits:
-                limit_stat = limit_stats.setdefault(id(limit), {})
+                limit_stat = limit_stats.setdefault(limit['id'], {})
                 applicable_limits.append(limit_stat)
 
                 if not limit_stat:
@@ -288,8 +294,8 @@ def _get_request_stats(rse_collection: RseCollection, logger=logging.log, sessio
 
         max_transfers = limit['max_transfers']
 
-        rse = rse_collection[limit['rse_id']]
-        log_str = f'limit {"from" if limit["direction"] == "source" else "to"} {rse.name} activity {limit["activity"]}'
+        rse_expression = limit['rse_expression']
+        log_str = f'limit {"from" if limit["direction"] == "source" else "to"} {rse_expression} activity {limit["activity"]}'
 
         if max_transfers is None:
             # The limit was explicitly set to NULL. Release all waiting requests.
@@ -305,14 +311,17 @@ def _get_request_stats(rse_collection: RseCollection, logger=logging.log, sessio
         stat['residual_capacity'] = residual_capacity
 
         activity = limit['activity'] or 'all_activities'
-        record_gauge('daemons.conveyor.throttler.rse_transfer_limits.{activity}.{rse}.{limit_attr}', residual_capacity, labels={'activity': activity, 'rse': rse.name, 'limit_attr': 'residual_capacity'})
-        record_gauge('daemons.conveyor.throttler.rse_transfer_limits.{activity}.{rse}.{limit_attr}', max_transfers, labels={'activity': activity, 'rse': rse.name, 'limit_attr': 'max_transfers'})
-        record_gauge('daemons.conveyor.throttler.rse_transfer_limits.{activity}.{rse}.{limit_attr}', active, labels={'activity': activity, 'rse': rse.name, 'limit_attr': 'active'})
-        record_gauge('daemons.conveyor.throttler.rse_transfer_limits.{activity}.{rse}.{limit_attr}', waiting, labels={'activity': activity, 'rse': rse.name, 'limit_attr': 'waiting'})
+        METRICS.gauge('rse_transfer_limits.{activity}.{rse}.{limit_attr}').labels(activity=activity, rse=rse_expression, limit_attr='residual_capacity').set(residual_capacity)
+        METRICS.gauge('rse_transfer_limits.{activity}.{rse}.{limit_attr}').labels(activity=activity, rse=rse_expression, limit_attr='max_transfers').set(max_transfers)
+        METRICS.gauge('rse_transfer_limits.{activity}.{rse}.{limit_attr}').labels(activity=activity, rse=rse_expression, limit_attr='active').set(active)
+        METRICS.gauge('rse_transfer_limits.{activity}.{rse}.{limit_attr}').labels(activity=activity, rse=rse_expression, limit_attr='waiting').set(waiting)
 
         if waiting:
             logger(logging.DEBUG, "%s: can release %s out of %s waiting requests", log_str, residual_capacity, waiting)
 
+        if waiting != limit['waitings'] or active != limit['transfers']:
+            set_transfer_limit_stats(limit['id'], waitings=waiting, transfers=active)
+
         for account, to_release_for_account in _split_threshold_per_account(stat['accounts'], total_to_release=residual_capacity):
             stat['accounts'][account]['residual_capacity'] = to_release_for_account
 
@@ -382,7 +391,7 @@ def _combine_limits(applicable_limits):
     return to_release, strategy, volume, deadline
 
 
-def _release_requests(rse_collection: RseCollection, release_groups, logger, session):
+def _handle_requests(release_groups, logger):
     """
     Release (set to queued state) waiting requests in groups defined by release_groups.
 
@@ -394,9 +403,9 @@ def _release_requests(rse_collection: RseCollection, release_groups, logger, ses
     for (source_rse, dest_rse, activity), applicable_limits in release_groups.items():
 
         # Skip if dest_rse is blocklisted for write or src_rse is blocklisted for read
-        if dest_rse and not dest_rse.columns.availability & 2:
+        if dest_rse and not dest_rse.columns['availability_write']:
             continue
-        if source_rse and not source_rse.columns.availability & 4:
+        if source_rse and not source_rse.columns['availability_read']:
             continue
 
         source_rse_id = source_rse.id if source_rse else None
@@ -412,7 +421,7 @@ def _release_requests(rse_collection: RseCollection, release_groups, logger, ses
             total_released = 0
         elif to_release == math.inf:
             logger(logging.DEBUG, "will release all waiting requests%s", log_str)
-            total_released = release_all_waiting_requests(dest_rse_id=dest_rse_id, source_rse_id=source_rse_id, activity=activity, session=session)
+            total_released = release_all_waiting_requests(dest_rse_id=dest_rse_id, source_rse_id=source_rse_id, activity=activity)
         elif strategy == 'grouped_fifo':
             logger(logging.DEBUG, "will release %s remaining requests%s", to_release, log_str)
             additional_kwargs = {}
@@ -424,7 +433,6 @@ def _release_requests(rse_collection: RseCollection, release_groups, logger, ses
                 source_rse_id=source_rse_id,
                 dest_rse_id=dest_rse_id,
                 count=to_release,
-                session=session,
                 **additional_kwargs,
             )
         else:
@@ -447,7 +455,6 @@ def _release_requests(rse_collection: RseCollection, release_groups, logger, ses
                     count=to_release_account,
                     activity=activity,
                     account=account,
-                    session=session
                 )
                 total_released += nb_released
 
@@ -456,6 +463,6 @@ def _release_requests(rse_collection: RseCollection, release_groups, logger, ses
 
         if total_released:
             for limit_stat in applicable_limits:
-                rse = rse_collection[limit_stat['limit']['rse_id']]
+                rse_expression = limit_stat['limit']['rse_expression']
                 limit_stat['stat']['residual_capacity'] -= total_released
-                record_counter('daemons.conveyor.throttler.released_waiting_requests.{activity}.{rse}', total_released, labels={'activity': activity, 'rse': rse.name})
+                METRICS.counter('released_waiting_requests.{activity}.{rse}').labels(activity=activity, rse=rse_expression).inc(total_released)
