@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +12,13 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 import logging
 import os
 import socket
 import threading
 import time
+from typing import TYPE_CHECKING
 
 import rucio.db.sqla.util
 from rucio.common import exception
@@ -31,7 +27,12 @@ from rucio.common.utils import get_thread_with_periodic_running_function
 from rucio.core.did import create_reports
 from rucio.core.heartbeat import live, die, sanity_check
 
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
 graceful_stop = threading.Event()
+DAEMON_NAME = 'rucio-follower'
 
 
 def aggregate_events(once=False):
@@ -44,10 +45,10 @@ def aggregate_events(once=False):
     hostname = socket.gethostname()
     pid = os.getpid()
     current_thread = threading.current_thread()
-    live(executable='rucio-follower', hostname=hostname, pid=pid, thread=current_thread)
+    live(executable=DAEMON_NAME, hostname=hostname, pid=pid, thread=current_thread)
 
     while not graceful_stop.is_set():
-        heartbeat = live(executable='rucio-follower', hostname=hostname, pid=pid, thread=current_thread)
+        heartbeat = live(executable=DAEMON_NAME, hostname=hostname, pid=pid, thread=current_thread)
         # Create a report of events and send a mail to the corresponding account.
         start_time = time.time()
         create_reports(total_workers=heartbeat['nr_threads'] - 1,
@@ -58,11 +59,11 @@ def aggregate_events(once=False):
             break
 
     logging.info('follower: graceful stop requested')
-    die(executable='rucio-follower', hostname=hostname, pid=pid, thread=current_thread)
+    die(executable=DAEMON_NAME, hostname=hostname, pid=pid, thread=current_thread)
     logging.info('follower: graceful stop done')
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -73,13 +74,13 @@ def run(once=False, threads=1):
     """
     Starts up the follower threads
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
 
     hostname = socket.gethostname()
-    sanity_check(executable='rucio-follower', hostname=hostname)
+    sanity_check(executable=DAEMON_NAME, hostname=hostname)
 
     if once:
         logging.info("executing one follower iteration only")
