@@ -26,6 +26,7 @@ import time
 import traceback
 from typing import TYPE_CHECKING
 
+import rucio.core.rse as rse_core
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.config import config_get_bool
@@ -34,23 +35,27 @@ from rucio.common.exception import (SourceNotFound, ServiceUnavailable,
                                     RSENotFound, VONotFound)
 from rucio.common.logging import setup_logging
 from rucio.core.message import add_message
-from rucio.core.monitor import record_counter
+from rucio.core.monitor import MetricManager
 from rucio.core.quarantined_replica import (list_quarantined_replicas,
                                             delete_quarantined_replicas,
                                             list_rses_with_quarantined_replicas)
-import rucio.core.rse as rse_core
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.vo import list_vos
 from rucio.daemons.common import run_daemon
 from rucio.rse import rsemanager as rsemgr
 
 if TYPE_CHECKING:
-    from typing import Sequence, Optional
+    from collections.abc import Sequence
+    from types import FrameType
+    from typing import Optional
+
     from rucio.daemons.common import HeartbeatHandler
 
 logging.getLogger("requests").setLevel(logging.CRITICAL)
 
+METRICS = MetricManager(module=__name__)
 GRACEFUL_STOP = threading.Event()
+DAEMON_NAME = 'dark-reaper'
 
 
 def reaper(
@@ -60,15 +65,13 @@ def reaper(
         scheme: "Optional[str]" = None,
         sleep_time: int = 300,
 ):
-    executable = 'dark-reaper'
+    executable = DAEMON_NAME
     if rses:
         executable += ' '.join(sys.argv[1:])
-    logger_prefix = 'dark-reaper'
     run_daemon(
         once=once,
         graceful_stop=GRACEFUL_STOP,
         executable=executable,
-        logger_prefix=logger_prefix,
         partition_wait_time=10,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -131,7 +134,7 @@ def run_once(
                     logger(logging.INFO, 'Deletion ATTEMPT of %s:%s as %s on %s', scope, replica['name'], pfn, rse)
                     start = time.time()
                     prot.delete(pfn)
-                    record_counter('daemons.dark_reaper.deleted_replicas')
+                    METRICS.counter('deleted_replicas').inc()
                     duration = time.time() - start
                     logger(logging.INFO, 'Deletion SUCCESS of %s:%s as %s on %s in %s seconds', scope, replica['name'], pfn, rse, duration)
                     payload = {'scope': scope,
@@ -183,7 +186,7 @@ def run_once(
     return must_sleep
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -205,7 +208,7 @@ def run(total_workers=1, chunk_size=100, once=False, rses=[], scheme=None,
     :param vos: VOs on which to look for RSEs. Only used in multi-VO mode.
                 If None, we either use all VOs if run from "def", or the current VO otherwise.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise exception.DatabaseException('Database was not updated, daemon won\'t start')
