@@ -19,35 +19,35 @@ Reaper is a daemon to manage file deletion.
 
 import functools
 import logging
-import os
 import random
 import threading
 import time
 import traceback
 from configparser import NoOptionError, NoSectionError
 from datetime import datetime, timedelta
-from math import log2
 from typing import TYPE_CHECKING
 
-from dogpile.cache.api import NO_VALUE
+from dogpile.cache.api import NoValue
+from math import log2
 from sqlalchemy.exc import DatabaseError, IntegrityError
 
 import rucio.db.sqla.util
-from rucio.common.config import config_get, config_get_bool
 from rucio.common.cache import make_region_memcached
+from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.exception import (DatabaseException, RSENotFound,
                                     ReplicaUnAvailable, ReplicaNotFound, ServiceUnavailable,
                                     RSEAccessDenied, ResourceTemporaryUnavailable, SourceNotFound,
-                                    VONotFound)
+                                    VONotFound, RSEProtocolNotSupported)
 from rucio.common.logging import setup_logging
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.types import InternalAccount
 from rucio.common.utils import chunks
-from rucio.core import monitor
 from rucio.core.credential import get_signed_url
 from rucio.core.heartbeat import list_payload_counts
 from rucio.core.message import add_message
+from rucio.core.monitor import MetricManager
 from rucio.core.oidc import get_token_for_account_operation
-from rucio.core.replica import list_and_mark_unlocked_replicas, list_and_mark_unlocked_replicas_no_temp_table, delete_replicas
+from rucio.core.replica import list_and_mark_unlocked_replicas, delete_replicas
 from rucio.core.rse import list_rses, RseData
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import get_evaluation_backlog
@@ -56,17 +56,18 @@ from rucio.daemons.common import run_daemon
 from rucio.rse import rsemanager as rsemgr
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Optional, Tuple
+    from collections.abc import Callable
+    from types import FrameType
+    from typing import Any, Optional
+
     from rucio.daemons.common import HeartbeatHandler
 
 GRACEFUL_STOP = threading.Event()
-
+METRICS = MetricManager(module=__name__)
 REGION = make_region_memcached(expiration_time=600)
+DAEMON_NAME = 'reaper'
 
-DELETION_COUNTER = monitor.MultiCounter(prom='rucio_daemons_reaper_deletion_done', statsd='reaper.deletion.done',
-                                        documentation='Number of deleted replicas')
-EXCLUDED_RSE_GAUGE = monitor.MultiGauge('daemons.reaper.excluded_rses',
-                                        documentation='Temporarly excluded RSEs', labelnames=('rse',))
+EXCLUDED_RSE_GAUGE = METRICS.gauge('excluded_rses.{rse}', documentation='Temporarly excluded RSEs')
 
 
 def get_rses_to_process(rses, include_rses, exclude_rses, vos):
@@ -96,13 +97,12 @@ def get_rses_to_process(rses, include_rses, exclude_rses, vos):
             vos = [v['vo'] for v in list_vos()]
         logging.log(logging.INFO, 'Reaper: This instance will work on VO%s: %s' % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
 
-    pid = os.getpid()
-    cache_key = 'rses_to_process_%s' % pid
+    cache_key = 'rses_to_process_1%s2%s3%s' % (str(rses), str(include_rses), str(exclude_rses))
     if multi_vo:
         cache_key += '@%s' % '-'.join(vo for vo in vos)
 
     result = REGION.get(cache_key)
-    if result is not NO_VALUE:
+    if not isinstance(result, NoValue):
         return result
 
     all_rses = []
@@ -143,19 +143,19 @@ def delete_from_storage(heartbeat_handler, hb_payload, replicas, prot, rse_info,
         for replica in replicas:
             # Physical deletion
             _, _, logger = heartbeat_handler.live(payload=hb_payload)
+            stopwatch = Stopwatch()
+            deletion_dict = {'scope': replica['scope'].external,
+                             'name': replica['name'],
+                             'rse': rse_name,
+                             'file-size': replica['bytes'],
+                             'bytes': replica['bytes'],
+                             'url': replica['pfn'],
+                             'protocol': prot.attributes['scheme'],
+                             'datatype': replica['datatype']}
             try:
-                deletion_dict = {'scope': replica['scope'].external,
-                                 'name': replica['name'],
-                                 'rse': rse_name,
-                                 'file-size': replica['bytes'],
-                                 'bytes': replica['bytes'],
-                                 'url': replica['pfn'],
-                                 'protocol': prot.attributes['scheme'],
-                                 'datatype': replica['datatype']}
                 if replica['scope'].vo != 'def':
                     deletion_dict['vo'] = replica['scope'].vo
                 logger(logging.DEBUG, 'Deletion ATTEMPT of %s:%s as %s on %s', replica['scope'], replica['name'], replica['pfn'], rse_name)
-                timer = monitor.Timer()
                 # For STAGING RSEs, no physical deletion
                 if is_staging:
                     logger(logging.WARNING, 'Deletion STAGING of %s:%s as %s on %s, will only delete the catalog and not do physical deletion', replica['scope'], replica['name'], replica['pfn'], rse_name)
@@ -174,9 +174,8 @@ def delete_from_storage(heartbeat_handler, hb_payload, replicas, prot, rse_info,
                 else:
                     logger(logging.WARNING, 'Deletion UNAVAILABLE of %s:%s as %s on %s', replica['scope'], replica['name'], replica['pfn'], rse_name)
 
-                timer.stop()
-                timer.record('daemons.reaper.delete.{scheme}.{rse}', labels={'scheme': prot.attributes['scheme'], 'rse': rse_name})
-                duration = timer.elapsed
+                duration = stopwatch.elapsed
+                METRICS.timer('delete.{scheme}.{rse}').labels(scheme=prot.attributes['scheme'], rse=rse_name).observe(duration)
 
                 deleted_files.append({'scope': replica['scope'], 'name': replica['name']})
 
@@ -185,7 +184,7 @@ def delete_from_storage(heartbeat_handler, hb_payload, replicas, prot, rse_info,
                 logger(logging.INFO, 'Deletion SUCCESS of %s:%s as %s on %s in %.2f seconds', replica['scope'], replica['name'], replica['pfn'], rse_name, duration)
 
             except SourceNotFound:
-                duration = timer.elapsed
+                duration = stopwatch.elapsed
                 err_msg = 'Deletion NOTFOUND of %s:%s as %s on %s in %.2f seconds' % (replica['scope'], replica['name'], replica['pfn'], rse_name, duration)
                 logger(logging.WARNING, '%s', err_msg)
                 deletion_dict['reason'] = 'File Not Found'
@@ -194,7 +193,7 @@ def delete_from_storage(heartbeat_handler, hb_payload, replicas, prot, rse_info,
                 deleted_files.append({'scope': replica['scope'], 'name': replica['name']})
 
             except (ServiceUnavailable, RSEAccessDenied, ResourceTemporaryUnavailable) as error:
-                duration = timer.elapsed
+                duration = stopwatch.elapsed
                 logger(logging.WARNING, 'Deletion NOACCESS of %s:%s as %s on %s: %s in %.2f', replica['scope'], replica['name'], replica['pfn'], rse_name, str(error), duration)
                 deletion_dict['reason'] = str(error)
                 deletion_dict['duration'] = duration
@@ -203,11 +202,13 @@ def delete_from_storage(heartbeat_handler, hb_payload, replicas, prot, rse_info,
                 if noaccess_attempts >= auto_exclude_threshold:
                     logger(logging.INFO, 'Too many (%d) NOACCESS attempts for %s. RSE will be temporarly excluded.', noaccess_attempts, rse_name)
                     REGION.set('temporary_exclude_%s' % rse_id, True)
+                    METRICS.gauge('excluded_rses.{rse}').labels(rse=rse_name).set(1)
+
                     EXCLUDED_RSE_GAUGE.labels(rse=rse_name).set(1)
                     break
 
             except Exception as error:
-                duration = timer.elapsed
+                duration = stopwatch.elapsed
                 logger(logging.CRITICAL, 'Deletion CRITICAL of %s:%s as %s on %s in %.2f seconds : %s', replica['scope'], replica['name'], replica['pfn'], rse_name, duration, str(traceback.format_exc()))
                 deletion_dict['reason'] = str(error)
                 deletion_dict['duration'] = duration
@@ -239,18 +240,22 @@ def delete_from_storage(heartbeat_handler, hb_payload, replicas, prot, rse_info,
     return deleted_files
 
 
-def _rse_deletion_hostname(rse: RseData) -> "Optional[str]":
+def _rse_deletion_hostname(rse: RseData, scheme: "Optional[str]") -> "Optional[str]":
     """
     Retrieves the hostname of the default deletion protocol
     """
     rse.ensure_loaded(load_info=True)
     for prot in rse.info['protocols']:
-        if prot['domains']['wan']['delete'] == 1:
-            return prot['hostname']
+        if scheme:
+            if prot['scheme'] == scheme and prot['domains']['wan']['delete'] != 0:
+                return prot['hostname']
+        else:
+            if prot['domains']['wan']['delete'] == 1:
+                return prot['hostname']
     return None
 
 
-def get_max_deletion_threads_by_hostname(hostname):
+def get_max_deletion_threads_by_hostname(hostname: str) -> int:
     """
     Internal method to check RSE usage and limits.
 
@@ -259,12 +264,12 @@ def get_max_deletion_threads_by_hostname(hostname):
     :returns: The maximum deletion thread for the SE.
     """
     result = REGION.get('max_deletion_threads_%s' % hostname)
-    if result is NO_VALUE:
+    if isinstance(result, NoValue):
         try:
-            max_deletion_thread = config_get('reaper', 'max_deletion_threads_%s' % hostname)
+            max_deletion_thread = config_get_int('reaper', 'max_deletion_threads_%s' % hostname)
         except (NoOptionError, NoSectionError, RuntimeError):
             try:
-                max_deletion_thread = config_get('reaper', 'nb_workers_by_hostname')
+                max_deletion_thread = config_get_int('reaper', 'nb_workers_by_hostname')
             except (NoOptionError, NoSectionError, RuntimeError):
                 max_deletion_thread = 5
         REGION.set('max_deletion_threads_%s' % hostname, max_deletion_thread)
@@ -304,19 +309,19 @@ def __try_reserve_worker_slot(heartbeat_handler: "HeartbeatHandler", rse: RseDat
     return rse_hostname_key
 
 
-def __check_rse_usage_cached(rse: RseData, greedy: bool = False, logger: "Callable[..., Any]" = logging.log) -> 'Tuple[int, bool]':
+def __check_rse_usage_cached(rse: RseData, greedy: bool = False, logger: "Callable[..., Any]" = logging.log) -> tuple[int, bool]:
     """
     Wrapper around __check_rse_usage which manages the cache entry.
     """
     cache_key = 'rse_usage_%s' % rse.id
     result = REGION.get(cache_key)
-    if result is NO_VALUE:
+    if isinstance(result, NoValue):
         result = __check_rse_usage(rse=rse, greedy=greedy, logger=logger)
         REGION.set(cache_key, result)
     return result
 
 
-def __check_rse_usage(rse: RseData, greedy: bool = False, logger: "Callable[..., Any]" = logging.log) -> 'Tuple[int, bool]':
+def __check_rse_usage(rse: RseData, greedy: bool = False, logger: "Callable[..., Any]" = logging.log) -> tuple[int, bool]:
     """
     Internal method to check RSE usage and limits.
 
@@ -327,65 +332,49 @@ def __check_rse_usage(rse: RseData, greedy: bool = False, logger: "Callable[...,
     :returns: needed_free_space, only_delete_obsolete.
     """
 
+    needed_free_space = 0
     # First of all check if greedy mode is enabled for this RSE
     if greedy:
         return 1000000000000, False
 
     rse.ensure_loaded(load_limits=True, load_usage=True, load_attributes=True)
+    available_sources = {}
+    available_sources['total'] = {key['source']: key['total'] for key in rse.usage}
+    available_sources['used'] = {key['source']: key['used'] for key in rse.usage}
 
     # Get RSE limits
     min_free_space = rse.limits.get('MinFreeSpace', 0)
 
-    # Check from which sources to get used and total spaces
-    # Default is storage
+    # Check from which sources to get used and total spaces (default storage)
+    # If specified sources do not exist, only delete obsolete
     source_for_total_space = rse.attributes.get('source_for_total_space', 'storage')
+    if source_for_total_space not in available_sources['total']:
+        logger(logging.WARNING, 'RSE: %s, \'%s\' requested for source_for_total_space but cannot be found. Will only delete obsolete',
+               rse.name, source_for_total_space)
+        return 0, True
     source_for_used_space = rse.attributes.get('source_for_used_space', 'storage')
+    if source_for_used_space not in available_sources['used']:
+        logger(logging.WARNING, 'RSE: %s, \'%s\' requested for source_for_used_space but cannot be found. Will only delete obsolete',
+               rse.name, source_for_used_space)
+        return 0, True
 
     logger(logging.DEBUG, 'RSE: %s, source_for_total_space: %s, source_for_used_space: %s',
            rse.name, source_for_total_space, source_for_used_space)
 
-    # Get total, used and obsolete space
-    total_space_entry = None
-    used_space_entry = None
-    obsolete_entry = None
-    for entry in rse.usage:
-        if total_space_entry and used_space_entry and obsolete_entry:
-            break
-
-        entry_source = entry['source']
-        if not total_space_entry and entry_source == source_for_total_space:
-            total_space_entry = entry
-        if not used_space_entry and entry_source == source_for_used_space:
-            used_space_entry = entry
-        if not obsolete_entry and entry_source == 'obsolete':
-            obsolete_entry = entry
-
-    obsolete = 0
-    if obsolete_entry:
-        obsolete = obsolete_entry['used']
-
-    # If no information is available about disk space, do nothing except if there are replicas with Epoch tombstone
-    needed_free_space = 0
-    if not total_space_entry:
-        if not obsolete:
-            return needed_free_space, False
-        return obsolete, True
-    if not used_space_entry:
-        return needed_free_space, False
-
-    # Extract the total and used space
-    total, used = total_space_entry['total'], used_space_entry['used']
+    # Get total and used space
+    total = available_sources['total'][source_for_total_space]
+    used = available_sources['used'][source_for_used_space]
 
     free = total - used
     if min_free_space:
         needed_free_space = min_free_space - free
 
     # If needed_free_space negative, nothing to delete except if some Epoch tombstoned replicas
-    if needed_free_space <= 0:
-        return obsolete, True
-    else:
+    if needed_free_space > 0:
         return needed_free_space, False
 
+    return 0, True
+
 
 def reaper(rses, include_rses, exclude_rses, vos=None, chunk_size=100, once=False, greedy=False,
            scheme=None, delay_seconds=0, sleep_time=60, auto_exclude_threshold=100, auto_exclude_timeout=600):
@@ -407,7 +396,6 @@ def reaper(rses, include_rses, exclude_rses, vos=None, chunk_size=100, once=Fals
     :param auto_exclude_timeout:   Timeout for temporarily excluded RSEs.
     """
 
-    executable = 'reaper'
     oidc_account = config_get('reaper', 'oidc_account', False, 'root')
     oidc_scope = config_get('reaper', 'oidc_scope', False, 'delete')
     oidc_audience = config_get('reaper', 'oidc_audience', False, 'rse')
@@ -415,8 +403,7 @@ def reaper(rses, include_rses, exclude_rses, vos=None, chunk_size=100, once=Fals
     run_daemon(
         once=once,
         graceful_stop=GRACEFUL_STOP,
-        executable=executable,
-        logger_prefix=executable,
+        executable=DAEMON_NAME,
         partition_wait_time=0 if once else 10,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -448,11 +435,11 @@ def run_once(rses, include_rses, exclude_rses, vos, chunk_size, greedy, scheme,
     logger(logging.INFO, 'Reaper started')
 
     # try to get auto exclude parameters from the config table. Otherwise use CLI parameters.
-    auto_exclude_threshold = config_get('reaper', 'auto_exclude_threshold', default=auto_exclude_threshold, raise_exception=False)
-    auto_exclude_timeout = config_get('reaper', 'auto_exclude_timeout', default=auto_exclude_timeout, raise_exception=False)
+    auto_exclude_threshold = config_get_int('reaper', 'auto_exclude_threshold', default=auto_exclude_threshold, raise_exception=False)
+    auto_exclude_timeout = config_get_int('reaper', 'auto_exclude_timeout', default=auto_exclude_timeout, raise_exception=False)
     # Check if there is a Judge Evaluator backlog
-    max_evaluator_backlog_count = config_get('reaper', 'max_evaluator_backlog_count', default=None, raise_exception=False)
-    max_evaluator_backlog_duration = config_get('reaper', 'max_evaluator_backlog_duration', default=None, raise_exception=False)
+    max_evaluator_backlog_count = config_get_int('reaper', 'max_evaluator_backlog_count', default=None, raise_exception=False)
+    max_evaluator_backlog_duration = config_get_int('reaper', 'max_evaluator_backlog_duration', default=None, raise_exception=False)
     if max_evaluator_backlog_count or max_evaluator_backlog_duration:
         backlog = get_evaluation_backlog()
         count_is_hit = max_evaluator_backlog_count and backlog[0] and backlog[0] > max_evaluator_backlog_count
@@ -514,7 +501,7 @@ def _run_once(rses_to_process, chunk_size, greedy, scheme,
     tot_needed_free_space = 0
     for rse in rses_to_process:
         # Check if RSE is blocklisted
-        if rse.columns['availability'] % 2 == 0:
+        if not rse.columns['availability_delete']:
             logger(logging.DEBUG, 'RSE %s is blocklisted for delete', rse.name)
             continue
         rse.ensure_loaded(load_attributes=True)
@@ -543,13 +530,13 @@ def _run_once(rses_to_process, chunk_size, greedy, scheme,
     paused_rses = []
     for rse, needed_free_space, only_delete_obsolete, enable_greedy in rses_with_params:
         result = REGION.get('pause_deletion_%s' % rse.id, expiration_time=120)
-        if result is not NO_VALUE:
+        if not isinstance(result, NoValue):
             paused_rses.append(rse.name)
             logger(logging.DEBUG, 'Not enough replicas to delete on %s during the previous cycle. Deletion paused for a while', rse.name)
             continue
 
         result = REGION.get('temporary_exclude_%s' % rse.id, expiration_time=auto_exclude_timeout)
-        if result is not NO_VALUE:
+        if not isinstance(result, NoValue):
             logger(logging.WARNING, 'Too many failed attempts for %s in last cycle. RSE is temporarly excluded.', rse.name)
             EXCLUDED_RSE_GAUGE.labels(rse=rse.name).set(1)
             continue
@@ -560,9 +547,12 @@ def _run_once(rses_to_process, chunk_size, greedy, scheme,
             percent = needed_free_space / tot_needed_free_space * 100
         logger(logging.DEBUG, 'Working on %s. Percentage of the total space needed %.2f', rse.name, percent)
 
-        rse_hostname = _rse_deletion_hostname(rse)
+        rse_hostname = _rse_deletion_hostname(rse, scheme)
         if not rse_hostname:
-            logger(logging.WARNING, 'No default delete protocol for %s', rse.name)
+            if scheme:
+                logger(logging.WARNING, 'Protocol %s not supported on %s', scheme, rse.name)
+            else:
+                logger(logging.WARNING, 'No default delete protocol for %s', rse.name)
             REGION.set('pause_deletion_%s' % rse.id, True)
             continue
 
@@ -574,24 +564,15 @@ def _run_once(rses_to_process, chunk_size, greedy, scheme,
         # List and mark BEING_DELETED the files to delete
         del_start_time = time.time()
         try:
-            use_temp_tables = config_get_bool('core', 'use_temp_tables', default=False)
-            with monitor.Timer('reaper.list_unlocked_replicas'):
+            with METRICS.timer('list_unlocked_replicas'):
                 if only_delete_obsolete:
                     logger(logging.DEBUG, 'Will run list_and_mark_unlocked_replicas on %s. No space needed, will only delete EPOCH tombstoned replicas', rse.name)
-                if use_temp_tables:
-                    replicas = list_and_mark_unlocked_replicas(limit=chunk_size,
-                                                               bytes_=needed_free_space,
-                                                               rse_id=rse.id,
-                                                               delay_seconds=delay_seconds,
-                                                               only_delete_obsolete=only_delete_obsolete,
-                                                               session=None)
-                else:
-                    replicas = list_and_mark_unlocked_replicas_no_temp_table(limit=chunk_size,
-                                                                             bytes_=needed_free_space,
-                                                                             rse_id=rse.id,
-                                                                             delay_seconds=delay_seconds,
-                                                                             only_delete_obsolete=only_delete_obsolete,
-                                                                             session=None)
+                replicas = list_and_mark_unlocked_replicas(limit=chunk_size,
+                                                           bytes_=needed_free_space,
+                                                           rse_id=rse.id,
+                                                           delay_seconds=delay_seconds,
+                                                           only_delete_obsolete=only_delete_obsolete,
+                                                           session=None)
             logger(logging.DEBUG, 'list_and_mark_unlocked_replicas on %s for %s bytes in %s seconds: %s replicas', rse.name, needed_free_space, time.time() - del_start_time, len(replicas))
             if (len(replicas) == 0 and enable_greedy) or (len(replicas) < chunk_size and not enable_greedy):
                 logger(logging.DEBUG, 'Not enough replicas to delete on %s (%s requested vs %s returned). Will skip any new attempts on this RSE until next cycle', rse.name, chunk_size, len(replicas))
@@ -639,10 +620,11 @@ def _run_once(rses_to_process, chunk_size, greedy, scheme,
 
                 # Then finally delete the replicas
                 del_start = time.time()
-                with monitor.Timer('reaper.delete_replicas'):
-                    delete_replicas(rse_id=rse.id, files=deleted_files)
+                delete_replicas(rse_id=rse.id, files=deleted_files)
                 logger(logging.DEBUG, 'delete_replicas successed on %s : %s replicas in %s seconds', rse.name, len(deleted_files), time.time() - del_start)
-                DELETION_COUNTER.inc(len(deleted_files))
+                METRICS.counter('deletion.done').inc(len(deleted_files))
+        except RSEProtocolNotSupported:
+            logger(logging.WARNING, 'Protocol %s not supported on %s', scheme, rse.name)
         except Exception:
             logger(logging.CRITICAL, 'Exception', exc_info=True)
 
@@ -653,7 +635,7 @@ def _run_once(rses_to_process, chunk_size, greedy, scheme,
     return rses_with_more_work
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -682,7 +664,7 @@ def run(threads=1, chunk_size=100, once=False, greedy=False, rses=None, scheme=N
     :param auto_exclude_threshold: Number of service unavailable exceptions after which the RSE gets temporarily excluded.
     :param auto_exclude_timeout:   Timeout for temporarily excluded RSEs.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException('Database was not updated, daemon won\'t start')
