@@ -20,35 +20,43 @@ or if they are the last remaining copy, on how many suspicious replicas are on a
 Consequently, automatic replica recovery is triggered via necromancer daemon, which creates a rule for such bad replica(s).
 """
 
+import functools
+import json
 import logging
+import re
 import threading
 import time
 from datetime import datetime, timedelta
-from sys import argv
-from typing import List, Any, Optional
-
-import functools
-import json
+from typing import TYPE_CHECKING, Any, Optional
 
 import rucio.db.sqla.util
-from rucio.db.sqla.util import get_db_time
 from rucio.common.config import config_get_bool
+from rucio.common.constants import SuspiciousAvailability
 from rucio.common.exception import DatabaseException, VONotFound
 from rucio.common.logging import setup_logging
 from rucio.common.types import InternalAccount
-from rucio.common.constants import SuspiciousAvailability
 from rucio.core.did import get_metadata
-from rucio.core.replica import list_replicas, get_suspicious_files, add_bad_pfns, declare_bad_file_replicas, get_suspicious_reason
+from rucio.core.replica import (
+    add_bad_pfns,
+    declare_bad_file_replicas,
+    get_suspicious_files,
+    get_suspicious_reason,
+    list_replicas,
+)
+from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rule import add_rule
 from rucio.core.vo import list_vos
-from rucio.core.rse_expression_parser import parse_expression
 from rucio.daemons.common import run_daemon
+from rucio.db.sqla.util import get_db_time
 
+if TYPE_CHECKING:
+    from types import FrameType
 
 GRACEFUL_STOP = threading.Event()
+DAEMON_NAME = 'suspicious-replica-recoverer'
 
 
-def declare_suspicious_replicas_bad(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: Optional[List[str]] = None, limit_suspicious_files_on_rse: int = 5, sleep_time: int = 3600, active_mode: bool = False) -> None:
+def declare_suspicious_replicas_bad(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: Optional[list[str]] = None, limit_suspicious_files_on_rse: int = 5, json_file_name: str = "/opt/rucio/etc/suspicious_replica_recoverer.json", sleep_time: int = 3600, active_mode: bool = False) -> None:
     """
     Main loop to check for available replicas which are labeled as suspicious.
 
@@ -76,8 +84,7 @@ def declare_suspicious_replicas_bad(once: bool = False, younger_than: int = 3, n
     run_daemon(
         once=once,
         graceful_stop=GRACEFUL_STOP,
-        executable=argv[0],
-        logger_prefix='replica_recoverer',
+        executable=DAEMON_NAME,
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -86,14 +93,13 @@ def declare_suspicious_replicas_bad(once: bool = False, younger_than: int = 3, n
             nattempts=nattempts,
             vos=vos,
             limit_suspicious_files_on_rse=limit_suspicious_files_on_rse,
+            json_file_name=json_file_name,
             active_mode=active_mode
         ),
     )
 
 
-def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Optional[List[str]], limit_suspicious_files_on_rse: int, active_mode: int, **_kwargs) -> bool:
-    # assembling the worker name identifier ('executable') including the rses from <rse_expression>
-    # in order to have the possibility to detect a start of a second instance with the same set of RSES
+def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Optional[list[str]], limit_suspicious_files_on_rse: int, json_file_name: str, active_mode: int, **_kwargs) -> bool:
 
     worker_number, total_workers, logger = heartbeat_handler.live()
 
@@ -114,27 +120,25 @@ def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Opt
             vos = [v['vo'] for v in list_vos()]
         logger(logging.INFO, 'This instance will work on VO%s: %s' % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
 
-    # make an initial heartbeat - expected only one replica-recoverer thread on one node
-    # heartbeat mechanism is used in this daemon only for information purposes
-    # (due to expected low load, the actual DB query does not filter the result based on worker number)
-
     start = time.time()
 
     try:
-        json_file = open("/opt/rucio/etc/suspicious_replica_recoverer.json")
+        json_file = open(json_file_name, mode="r")
+        logger(logging.INFO, "JSON file has been opened.")
     except:
-        logger(logging.WARNING, "An error occured whilst trying to open the JSON file.")
+        logger(logging.WARNING, "An error occured while trying to open the JSON file.")
         must_sleep = True
         return must_sleep
 
     try:
         json_data = json.load(json_file)
-    except ValueError:
-        logger(logging.WARNING, "No JSON object could be decoded.")
+    except ValueError as e:
+        logger(logging.WARNING, "No JSON object could be decoded. Error: %s", e)
+        return True
 
     # Checking that the json file is formatedd properly.
     for i, entry in enumerate(json_data):
-        if "datatype" not in entry or "action" not in entry:
+        if ("datatype" not in entry or "scope" not in entry or "action" not in entry):
             logger(logging.ERROR, 'Entry %s in the json file is incomplete (missing either "datatype" or "action").', i)
             break
 
@@ -196,8 +200,8 @@ def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Opt
                 logger(logging.DEBUG, '%s', i)
 
             # RSEs that aren't available shouldn't have suspicious replicas showing up. Skip to next RSE.
-            if (rse['availability'] not in {4, 5, 6, 7}) and ((len(suspicious_replicas_avail_elsewhere) > 0) or (len(suspicious_replicas_last_copy) > 0)):
-                logger(logging.WARNING, "%s is not available (availability: %s), yet it has suspicious replicas. Please investigate. \n", rse_expr, rse['availability'])
+            if not rse['availability_read'] and ((len(suspicious_replicas_avail_elsewhere) > 0) or (len(suspicious_replicas_last_copy) > 0)):
+                logger(logging.WARNING, "%s is not available (availability: %s), yet it has suspicious replicas. Please investigate. \n", rse_expr, rse['availability_read'])
                 continue
 
             if suspicious_replicas_avail_elsewhere:
@@ -338,7 +342,9 @@ def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Opt
             # Get the rse_id by going to one of the suspicious replicas from that RSE and reading it from there
             rse_id = list(recoverable_replicas[vo][rse_key].values())[0]['rse_id']
             for replica_key in list(recoverable_replicas[vo][rse_key].keys()):
-                file_metadata = get_metadata(recoverable_replicas[vo][rse_key][replica_key]["scope"], recoverable_replicas[vo][rse_key][replica_key]["name"])
+                file_scope = recoverable_replicas[vo][rse_key][replica_key]["scope"]
+                file_name = recoverable_replicas[vo][rse_key][replica_key]["name"]
+                file_metadata = get_metadata(file_scope, file_name)
                 recoverable_replicas[vo][rse_key][replica_key]["datatype"] = str(file_metadata["datatype"])
                 if recoverable_replicas[vo][rse_key][replica_key]['available_elsewhere'] is True:
                     # Replicas with other copies on at least one other RSE can safely be labeled as bad
@@ -346,7 +352,7 @@ def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Opt
                     # Remove replica from dictionary
                     del recoverable_replicas[vo][rse_key][replica_key]
                 elif recoverable_replicas[vo][rse_key][replica_key]['available_elsewhere'] is False:
-                    if (recoverable_replicas[vo][rse_key][replica_key]['name'].startswith("log.")) or (recoverable_replicas[vo][rse_key][replica_key]['name'].startswith("user")):
+                    if (file_name.startswith("log.")) or (file_name.startswith("user")):
                         # Don't keep log files or user files
                         files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
                         del recoverable_replicas[vo][rse_key][replica_key]
@@ -354,33 +360,61 @@ def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Opt
                         # Deal with replicas based on their metadata.
                         if file_metadata["datatype"] is None:  # "None" type has no function "split()"
                             files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
+                            logger(logging.WARNING, "RSE: %s, replica name %s, surl %s: Replica does not have a data type associated with it. No action will be taken.",
+                                   rse_key, replica_key, recoverable_replicas[vo][rse_key][replica_key]['surl'])
                             continue
-                        for i in json_data:
-                            if i["datatype"] == file_metadata["datatype"].split("_")[-1]:
-                                action = i["action"]
-                                if action == "ignore":
-                                    files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
-                                elif action == "declare bad":
-                                    suspicious_reason = get_suspicious_reason(recoverable_replicas[vo][rse_key][replica_key]["rse_id"],
-                                                                              recoverable_replicas[vo][rse_key][replica_key]["scope"],
-                                                                              recoverable_replicas[vo][rse_key][replica_key]["name"],
-                                                                              nattempts)
-                                    for reason in suspicious_reason:
-                                        if "auditor" in reason["reason"].lower():
-                                            auditor += 1
-                                            files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
-                                            break
-                                        elif "checksum" in reason["reason"].lower():
-                                            checksum += 1
-                                            files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
-                                            break
-                                else:
-                                    logger(logging.WARNING, "RSE: %s, replica name %s, surl %s: Match for the metadata 'datatype' (%s) of replica found in json file, but no match for 'action' (%s)",
-                                           rse_key, replica_key, recoverable_replicas[vo][rse_key][replica_key]['surl'], i["datatype"], i["action"])
-                                break
-                            else:
-                                # If no policy has been set, default to ignoring the file (no action taken).
+
+                        file_metadata_datatype = str(file_metadata["datatype"])
+                        file_metadata_scope = str(file_metadata["scope"])
+                        action = ""
+                        if file_metadata_datatype:
+                            # Some files don't have a datatype. They should be ignored.
+                            for policy in json_data:
+                                match_scope = False
+                                match_datatype = False
+
+                                if not policy.get("scope", []):
+                                    match_scope = True
+                                for scope in policy.get("scope", []):
+                                    if re.match(scope, file_metadata_scope):
+                                        match_scope = True
+                                        break
+
+                                if not policy.get("datatype", []):
+                                    match_datatype = True
+                                for datatype in policy.get("datatype", []):
+                                    if re.match(datatype, file_metadata_datatype):
+                                        match_datatype = True
+                                        break
+
+                                if match_scope and match_datatype:
+                                    action = policy["action"]
+                                    logger(logging.INFO, "The action that will be performed is %s", action)
+                                    break
+
+                        if not action:
+                            logger(logging.WARNING, "No recognised actions (ignore/declare bad) found in policy file (etc/suspicious_replica_recoverer.json). Replica will be ignored by default.")
+
+                        if action:
+                            if action == "ignore":
                                 files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
+                            elif action == "declare bad":
+                                suspicious_reason = get_suspicious_reason(recoverable_replicas[vo][rse_key][replica_key]["rse_id"],
+                                                                          file_scope,
+                                                                          file_name,
+                                                                          nattempts)
+                                for reason in suspicious_reason:
+                                    if "auditor" in reason["reason"].lower():
+                                        auditor += 1
+                                        files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
+                                        break
+                                    elif "checksum" in reason["reason"].lower():
+                                        checksum += 1
+                                        files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
+                                        break
+                        else:
+                            # If no policy has been set, default to ignoring the file (no action taken).
+                            files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
 
             logger(logging.INFO, '(%s) Remaining replicas (pfns) that will be ignored:', rse_key)
             for i in files_to_be_ignored:
@@ -412,11 +446,11 @@ def run_once(heartbeat_handler: Any, younger_than: int, nattempts: int, vos: Opt
     return must_sleep
 
 
-def run(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: List[str] = None, limit_suspicious_files_on_rse: int = 5, sleep_time: int = 3600, active_mode: bool = False) -> None:
+def run(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: list[str] = None, limit_suspicious_files_on_rse: int = 5, json_file_name: str = "/opt/rucio/etc/suspicious_replica_recoverer.json", sleep_time: int = 3600, active_mode: bool = False) -> None:
     """
     Starts up the Suspicious-Replica-Recoverer threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException('Database was not updated, daemon won\'t start')
@@ -435,6 +469,7 @@ def run(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: Lis
                                  'nattempts': nattempts,
                                  'vos': vos,
                                  'limit_suspicious_files_on_rse': limit_suspicious_files_on_rse,
+                                 'json_file_name': json_file_name,
                                  'sleep_time': sleep_time,
                                  'active_mode': active_mode})
     t.start()
@@ -445,7 +480,7 @@ def run(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: Lis
         t.join(timeout=3.14)
 
 
-def stop():
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
