@@ -24,32 +24,38 @@ import logging
 import os
 import re
 import socket
-import time
 import threading
+import time
 import traceback
-
 from datetime import datetime
+from typing import TYPE_CHECKING
+
+from sqlalchemy.exc import DatabaseError, IntegrityError
+from sqlalchemy.orm.exc import FlushError
 
 from rucio.common import exception
 from rucio.common.logging import formatted_logger, setup_logging
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
-from rucio.core.monitor import record_gauge, record_counter
+from rucio.core.monitor import MetricManager
 from rucio.core.quarantined_replica import add_quarantined_replicas
 from rucio.core.replica import __exist_replicas, update_replicas_states
 from rucio.core.rse import list_rses, get_rse_id
-from rucio.rse.rsemanager import lfns2pfns, get_rse_info, parse_pfns
-
 # FIXME: these are needed by local version of declare_bad_file_replicas()
 # TODO: remove after move of this code to core/replica.py - see https://github.com/rucio/rucio/pull/5068
 from rucio.db.sqla import models
-from rucio.db.sqla.session import transactional_session
 from rucio.db.sqla.constants import (ReplicaState, BadFilesStatus)
-from sqlalchemy.exc import DatabaseError, IntegrityError
-from sqlalchemy.orm.exc import FlushError
+from rucio.db.sqla.session import transactional_session
+from rucio.rse.rsemanager import lfns2pfns, get_rse_info, parse_pfns
 
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
+METRICS = MetricManager(module=__name__)
 graceful_stop = threading.Event()
+DAEMON_NAME = 'storage-consistency-actions'
 
 # FIXME: declare_bad_file_replicas will be used directly from core/replica.py when handling of DID is added there
 # TODO: remove after move to core/replica.py
@@ -57,7 +63,7 @@ graceful_stop = threading.Event()
 
 @transactional_session
 def declare_bad_file_replicas(dids, rse_id, reason, issuer,
-                              status=BadFilesStatus.BAD, scheme=None, session=None):
+                              status=BadFilesStatus.BAD, scheme=None, *, session=None):
     """
     Declare a list of bad replicas.
 
@@ -369,10 +375,8 @@ def process_dark_files(path, scope, rse, latest_run, max_dark_fraction,
 # Labels for the Prometheus counters/gauges
     labels = {'rse': rse}
 
-    record_gauge('storage.consistency.actions_dark_files_found',
-                 confirmed_dark_files, labels=labels)
-    record_gauge('storage.consistency.actions_dark_files_confirmed',
-                 confirmed_dark_files, labels=labels)
+    METRICS.gauge('actions_dark_files_found.{rse}').labels(**labels).set(dark_files)
+    METRICS.gauge('actions_dark_files_confirmed.{rse}').labels(**labels).set(confirmed_dark_files)
 
     deleted_files = 0
     if confirmed_dark_files / max_files_at_site < max_dark_fraction or force_proceed is True:
@@ -399,9 +403,7 @@ def process_dark_files(path, scope, rse, latest_run, max_dark_fraction,
                              'path': paths[url]['path'] + paths[url]['name']}]
                 add_quarantined_replicas(rse_id, replicas, session=None)
                 deleted_files += 1
-                labels = {'rse': rse}
-                record_counter('storage.consistency.actions_dark_files_deleted_counter',
-                               delta=1, labels=labels)
+                METRICS.counter('actions_dark_files_deleted_counter.{rse}').labels(**labels).inc()
 
 # Update the stats
         t1 = time.time()
@@ -413,9 +415,6 @@ def process_dark_files(path, scope, rse, latest_run, max_dark_fraction,
             "status": "done"
         })
         stats[stats_key] = cc_stats
-        record_gauge('storage.consistency.actions_dark_files_deleted',
-                     deleted_files, labels=labels)
-
     else:
         darkperc = 100. * confirmed_dark_files / max_files_at_site
         logger(logging.WARNING, '\n ATTENTION: Too many DARK files! (%3.2f%%) \n\
@@ -432,7 +431,7 @@ def process_dark_files(path, scope, rse, latest_run, max_dark_fraction,
             "aborted_reason": "%3.2f%% dark" % darkperc,
         })
         stats[stats_key] = cc_stats
-        record_gauge('storage.consistency.actions_dark_files_deleted', 0, labels=labels)
+    METRICS.gauge('actions_dark_files_deleted.{rse}').labels(**labels).set(deleted_files)
 
 
 def process_miss_files(path, scope, rse, latest_run, max_miss_fraction,
@@ -477,12 +476,12 @@ def process_miss_files(path, scope, rse, latest_run, max_miss_fraction,
     logger(logging.INFO, 'max_miss_fraction configured for this RSE (in %%): %f' % max_miss_fraction)
 
     labels = {'rse': rse}
-    record_gauge('storage.consistency.actions_miss_files_found', miss_files, labels=labels)
+    METRICS.gauge('actions_miss_files_found.{rse}').labels(**labels).set(miss_files)
 
+    invalidated_files = 0
     if miss_files / max_files_at_site < max_miss_fraction or force_proceed is True:
         logger(logging.INFO, 'Can proceed with missing files retransfer')
 
-        invalidated_files = 0
         issuer = InternalAccount('root')
         with open(latest_miss, 'r') as csvfile:
             reader = csv.reader(csvfile)
@@ -496,8 +495,7 @@ def process_miss_files(path, scope, rse, latest_run, max_miss_fraction,
                 declare_bad_file_replicas(dids=dids, rse_id=rse_id, reason=reason,
                                           issuer=issuer)
                 invalidated_files += 1
-                record_counter('storage.consistency.actions_miss_files_to_retransfer_counter',
-                               delta=1, labels=labels)
+                METRICS.counter('actions_miss_files_to_retransfer_counter.{rse}').labels(**labels).inc()
 
 # TODO: The stats updating can be refactored in a future version of the Stats class.
 # See: https://github.com/rucio/rucio/pull/5120#discussion_r792688019
@@ -511,8 +509,6 @@ def process_miss_files(path, scope, rse, latest_run, max_miss_fraction,
                 "status": "done"
             })
             stats[stats_key] = cc_stats
-            record_gauge('storage.consistency.actions_miss_files_to_retransfer',
-                         invalidated_files, labels=labels)
 
     else:
         missperc = 100. * miss_files / max_files_at_site
@@ -530,8 +526,7 @@ def process_miss_files(path, scope, rse, latest_run, max_miss_fraction,
             "aborted_reason": "%3.2f%% miss" % missperc,
         })
         stats[stats_key] = cc_stats
-        record_gauge('storage.consistency.actions_miss_files_to_retransfer',
-                     0, labels=labels)
+    METRICS.gauge('actions_miss_files_to_retransfer.{rse}').labels(**labels).set(invalidated_files)
 
 
 def deckard(scope, rse, dark_min_age, dark_threshold_percent, miss_threshold_percent,
@@ -653,8 +648,7 @@ def actions_loop(once, scope, rses, sleep_time, dark_min_age, dark_threshold_per
     pid = os.getpid()
     current_thread = threading.current_thread()
 
-    executable = 'storage-consistency-actions'
-    heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
+    heartbeat = live(executable=DAEMON_NAME, hostname=hostname, pid=pid, thread=current_thread)
 
     # Make an initial heartbeat
     # so that all storage-consistency-actions have the correct worker number on the next try
@@ -668,7 +662,7 @@ def actions_loop(once, scope, rses, sleep_time, dark_min_age, dark_threshold_per
 
     while not graceful_stop.is_set():
         try:
-            heartbeat = live(executable=executable, hostname=hostname, pid=pid,
+            heartbeat = live(executable=DAEMON_NAME, hostname=hostname, pid=pid,
                              thread=current_thread)
             logger(logging.INFO, 'heartbeat? %s' % heartbeat)
 
@@ -690,10 +684,10 @@ def actions_loop(once, scope, rses, sleep_time, dark_min_age, dark_threshold_per
         if once:
             break
 
-    die(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
+    die(executable=DAEMON_NAME, hostname=hostname, pid=pid, thread=current_thread)
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -707,7 +701,7 @@ def run(once=False, scope=None, rses=None, sleep_time=60, default_dark_min_age=2
     Starts up the Consistency-Actions.
     """
 
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     prefix = 'storage-consistency-actions (run())'
     logger = formatted_logger(logging.log, prefix + '%s')
@@ -733,9 +727,8 @@ def run(once=False, scope=None, rses=None, sleep_time=60, default_dark_min_age=2
      \n Scanner files path: %s ' % (rses, once, sleep_time, dark_min_age, dark_threshold_percent,
            miss_threshold_percent, force_proceed, scanner_files_path))
 
-    executable = 'storage-consistency-actions'
     hostname = socket.gethostname()
-    sanity_check(executable=executable, hostname=hostname)
+    sanity_check(executable=DAEMON_NAME, hostname=hostname)
 
 # It was decided that for the time being this daemon is best executed in a single thread
 # TODO: If this decicion is reversed in the future, the following line should be removed.
