@@ -17,39 +17,44 @@
 This daemon consumes tracer messages from ActiveMQ and updates the atime for replicas.
 """
 
+import functools
 import logging
 import re
 from configparser import NoOptionError, NoSectionError
 from datetime import datetime
-import functools
 from json import loads as jloads, dumps as jdumps
+from queue import Queue
 from threading import Event, Thread
 from time import time
-from typing import Dict
+from typing import TYPE_CHECKING
 
 import rucio.db.sqla.util
-from rucio.daemons.common import HeartbeatHandler, run_daemon
-from rucio.common.config import config_get, config_get_bool, config_get_int
+from rucio.common.config import config_get, config_get_bool, config_get_int, config_get_list
 from rucio.common.exception import RSENotFound, DatabaseException
 from rucio.common.logging import setup_logging
-from rucio.common.stomp_utils import get_stomp_brokers
+from rucio.common.stomp_utils import StompConnectionManager
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.core.did import touch_dids, list_parent_dids
 from rucio.core.lock import touch_dataset_locks
-from rucio.core.monitor import record_counter, Timer
+from rucio.core.monitor import MetricManager
 from rucio.core.replica import touch_replica, touch_collection_replicas, declare_bad_file_replicas
 from rucio.core.rse import get_rse_id
+from rucio.daemons.common import HeartbeatHandler, run_daemon
 from rucio.db.sqla.constants import DIDType, BadFilesStatus
 
-from queue import Queue
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
 
 
 logging.getLogger("stomp").setLevel(logging.CRITICAL)
 
+METRICS = MetricManager(module=__name__)
 graceful_stop = Event()
 
 
-class AMQConsumer(object):
+class AMQConsumer:
     """ActiveMQ message consumer"""
 
     def __init__(self, broker, conn, queue, chunksize, subscription_id, excluded_usrdns, dataset_queue, bad_files_patterns, logger=logging.log):
@@ -68,24 +73,23 @@ class AMQConsumer(object):
         self.__bad_files_patterns = bad_files_patterns
         self.__logger = logger
 
+    @METRICS.count_it
     def on_heartbeat_timeout(self):
-        record_counter('daemons.tracer.kronos.heartbeat.lost')
         self.__conn.disconnect()
 
+    @METRICS.count_it
     def on_error(self, frame):
-        record_counter('daemons.tracer.kronos.error')
         self.__logger(logging.ERROR, 'Message receive error: [%s] %s' % (self.__broker, frame.body))
 
+    @METRICS.count_it
     def on_message(self, frame):
-        record_counter('daemons.tracer.kronos.reports')
-
         appversion = 'dq2'
         msg_id = frame.headers['message-id']
         if 'appversion' in frame.headers:
             appversion = frame.headers['appversion']
 
         if 'resubmitted' in frame.headers:
-            record_counter('daemons.tracer.kronos.received_resubmitted')
+            METRICS.counter('received_resubmitted').inc()
 
         try:
             if appversion == 'dq2':
@@ -96,7 +100,7 @@ class AMQConsumer(object):
         except Exception:
             # message is corrupt, not much to do here
             # send count to graphite, send ack to broker and return
-            record_counter('daemons.tracer.kronos.json_error')
+            METRICS.counter('json_error').inc()
             self.__logger(logging.ERROR, 'json error', exc_info=True)
             self.__conn.ack(msg_id, self.__subscription_id)
             return
@@ -148,11 +152,11 @@ class AMQConsumer(object):
 
                 # check if scope in report. if not skip this one.
                 if 'scope' not in report:
-                    record_counter('daemons.tracer.kronos.missing_scope')
+                    METRICS.counter('missing_scope').inc()
                     if report['eventType'] != 'touch':
                         continue
                 else:
-                    record_counter('daemons.tracer.kronos.with_scope')
+                    METRICS.counter('with_scope').inc()
                     report['scope'] = InternalScope(report['scope'], report['vo'])
 
                 # handle all events starting with get* and download and touch events.
@@ -160,25 +164,25 @@ class AMQConsumer(object):
                     continue
                 if report['eventType'].endswith('_es'):
                     continue
-                record_counter('daemons.tracer.kronos.total_get')
+                METRICS.counter('total_get').inc()
                 if report['eventType'] == 'get':
-                    record_counter('daemons.tracer.kronos.dq2clients')
+                    METRICS.counter('dq2clients').inc()
                 elif report['eventType'] == 'get_sm' or report['eventType'] == 'sm_get':
                     if report['eventVersion'] == 'aCT':
-                        record_counter('daemons.tracer.kronos.panda_production_act')
+                        METRICS.counter('panda_production_act').inc()
                     else:
-                        record_counter('daemons.tracer.kronos.panda_production')
+                        METRICS.counter('panda_production').inc()
                 elif report['eventType'] == 'get_sm_a' or report['eventType'] == 'sm_get_a':
                     if report['eventVersion'] == 'aCT':
-                        record_counter('daemons.tracer.kronos.panda_analysis_act')
+                        METRICS.counter('panda_analysis_act').inc()
                     else:
-                        record_counter('daemons.tracer.kronos.panda_analysis')
+                        METRICS.counter('panda_analysis').inc()
                 elif report['eventType'] == 'download':
-                    record_counter('daemons.tracer.kronos.rucio_download')
+                    METRICS.counter('rucio_download').inc()
                 elif report['eventType'] == 'touch':
-                    record_counter('daemons.tracer.kronos.rucio_touch')
+                    METRICS.counter('rucio_touch').inc()
                 else:
-                    record_counter('daemons.tracer.kronos.other_get')
+                    METRICS.counter('other_get').inc()
 
                 if report['eventType'] == 'download' or report['eventType'] == 'touch':
                     report['usrdn'] = report['account']
@@ -208,7 +212,7 @@ class AMQConsumer(object):
                             rse_id = get_rse_id(rse=rse, vo=report['vo'])
                         except RSENotFound:
                             self.__logger(logging.WARNING, "Cannot lookup rse_id for %s. Will skip this report.", rse)
-                            record_counter('daemons.tracer.kronos.rse_not_found')
+                            METRICS.counter('rse_not_found').inc()
                             continue
                         replicas.append({'name': report['filename'], 'scope': report['scope'], 'rse': rse, 'rse_id': rse_id, 'accessed_at': datetime.utcfromtimestamp(report['traceTimeentryUnix']),
                                          'traceTimeentryUnix': report['traceTimeentryUnix'], 'eventVersion': report['eventVersion']})
@@ -224,7 +228,7 @@ class AMQConsumer(object):
                             rse_id = get_rse_id(rse=rse, vo=report['vo'])
                         except RSENotFound:
                             self.__logger(logging.WARNING, "Cannot lookup rse_id for %s.", rse)
-                            record_counter('daemons.tracer.kronos.rse_not_found')
+                            METRICS.counter('rse_not_found').inc()
                     if 'datasetScope' in report:
                         self.__dataset_queue.put({'scope': InternalScope(report['datasetScope'], vo=report['vo']),
                                                   'name': report['dataset'],
@@ -242,7 +246,7 @@ class AMQConsumer(object):
 
             except (KeyError, AttributeError):
                 self.__logger(logging.ERROR, "Cannot handle report.", exc_info=True)
-                record_counter('daemons.tracer.kronos.report_error')
+                METRICS.counter('report_error').inc()
                 continue
             except Exception:
                 self.__logger(logging.ERROR, "Exception", exc_info=True)
@@ -259,7 +263,7 @@ class AMQConsumer(object):
                         rse_id = get_rse_id(rse=rse, vo=report['vo'])
                     except RSENotFound:
                         self.__logger(logging.WARNING, "Cannot lookup rse_id for %s. Will skip this report.", rse)
-                        record_counter('daemons.tracer.kronos.rse_not_found')
+                        METRICS.counter('rse_not_found').inc()
                         continue
                     self.__dataset_queue.put({'scope': did['scope'], 'name': did['name'], 'did_type': did['type'], 'rse_id': rse_id, 'accessed_at': datetime.utcfromtimestamp(report['traceTimeentryUnix'])})
 
@@ -268,8 +272,8 @@ class AMQConsumer(object):
 
         self.__logger(logging.DEBUG, "trying to update replicas: %s", replicas)
 
+        stopwatch = Stopwatch()
         try:
-            timer = Timer()
             for replica in replicas:
                 # if touch replica hits a locked row put the trace back into queue for later retry
                 if not touch_replica(replica):
@@ -284,42 +288,37 @@ class AMQConsumer(object):
                     if replica['scope'].vo != 'def':
                         resubmit['vo'] = replica['scope'].vo
                     self.__conn.send(body=jdumps(resubmit), destination=self.__queue, headers={'appversion': 'rucio', 'resubmitted': '1'})
-                    record_counter('daemons.tracer.kronos.sent_resubmitted')
-            timer.record('daemons.tracer.kronos.update_atime')
+                    METRICS.counter('sent_resubmitted').inc()
+            METRICS.timer('update_atime').observe(stopwatch.elapsed)
         except Exception:
             self.__logger(logging.ERROR, "Cannot update replicas.", exc_info=True)
-            record_counter('daemons.tracer.kronos.update_error')
+            METRICS.counter('update_error').inc()
 
-        record_counter('daemons.tracer.kronos.updated_replicas')
+        METRICS.counter('updated_replicas').inc()
 
 
 def kronos_file(once: bool = False, dataset_queue: Queue = None, sleep_time: int = 60):
     """
     Main loop to consume tracer reports.
     """
-    return_values = {}
+    stomp_conn_mngr = StompConnectionManager()
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
         executable='kronos-file',
-        logger_prefix='kronos-file',
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
             run_once_kronos_file,
-            return_values=return_values,
+            stomp_conn_mngr=stomp_conn_mngr,
             dataset_queue=dataset_queue,
             sleep_time=sleep_time,
         )
     )
-    for conn in return_values['conns']:
-        try:
-            conn.disconnect()
-        except Exception:
-            pass
+    stomp_conn_mngr.disconnect()
 
 
-def run_once_kronos_file(heartbeat_handler: HeartbeatHandler, return_values: Dict, dataset_queue: Queue, sleep_time: int, **kwargs):
+def run_once_kronos_file(heartbeat_handler: HeartbeatHandler, stomp_conn_mngr: StompConnectionManager, dataset_queue: Queue, sleep_time: int, **kwargs):
     """
     Run the amq consumer once.
     """
@@ -341,10 +340,8 @@ def run_once_kronos_file(heartbeat_handler: HeartbeatHandler, return_values: Dic
     except Exception as error:
         logger.error(f'Failed to get bad_file_patterns {str(error)}')
         bad_files_patterns = []
-    try:
-        use_ssl = config_get_bool('tracer-kronos', 'use_ssl')
-    except Exception:
-        pass
+
+    use_ssl = config_get_bool('tracer-kronos', 'use_ssl', default=True, raise_exception=False)
     if not use_ssl:
         username = config_get('tracer-kronos', 'username')
         password = config_get('tracer-kronos', 'password')
@@ -352,12 +349,13 @@ def run_once_kronos_file(heartbeat_handler: HeartbeatHandler, return_values: Dic
     excluded_usrdns = set(config_get('tracer-kronos', 'excluded_usrdns').split(','))
     vhost = config_get('tracer-kronos', 'broker_virtual_host', raise_exception=False)
 
-    brokers_alias = [b.strip() for b in config_get('tracer-kronos', 'brokers').split(',')]
+    brokers_alias = config_get_list('tracer-kronos', 'brokers')
     port = config_get_int('tracer-kronos', 'port')
     reconnect_attempts = config_get_int('tracer-kronos', 'reconnect_attempts')
     ssl_key_file = config_get('tracer-kronos', 'ssl_key_file', raise_exception=False)
     ssl_cert_file = config_get('tracer-kronos', 'ssl_cert_file', raise_exception=False)
-    conns = return_values['conns'] = get_stomp_brokers(
+
+    created_conns, _ = stomp_conn_mngr.re_configure(
         brokers=brokers_alias,
         port=port,
         use_ssl=use_ssl,
@@ -366,13 +364,14 @@ def run_once_kronos_file(heartbeat_handler: HeartbeatHandler, return_values: Dic
         ssl_key_file=ssl_key_file,
         ssl_cert_file=ssl_cert_file,
         timeout=sleep_time,
-        logger=logger
+        heartbeats=(0, 5000),
+        logger=logger,
     )
 
-    for conn in conns:
+    for conn in created_conns:
         if not conn.is_connected():
             logger(logging.INFO, 'connecting to %s' % str(conn.transport._Transport__host_and_ports[0]))
-            record_counter('daemons.tracer.kronos.reconnect.{host}', labels={'host': conn.transport._Transport__host_and_ports[0][0]})
+            METRICS.counter('reconnect.{host}').labels(host=conn.transport._Transport__host_and_ports[0][0]).inc()
             conn.set_listener('rucio-tracer-kronos', AMQConsumer(broker=conn.transport._Transport__host_and_ports[0],
                                                                  conn=conn,
                                                                  queue=config_get('tracer-kronos', 'queue'),
@@ -395,7 +394,6 @@ def kronos_dataset(dataset_queue: Queue, once: bool = False, sleep_time: int = 6
         once=once,
         graceful_stop=graceful_stop,
         executable='kronos-dataset',
-        logger_prefix='kronos-dataset',
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -479,7 +477,7 @@ def run_once_kronos_dataset(dataset_queue: Queue, return_values: dict, heartbeat
     logger(logging.INFO, 'update done for %d collection replicas, %d failed (%ds)' % (total, failed, time() - start))
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -490,7 +488,7 @@ def run(once=False, threads=1, sleep_time_datasets=60, sleep_time_files=60):
     """
     Starts up the consumer threads
     """
-    setup_logging()
+    setup_logging(process_name='tracer-kronos')
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException('Database was not updated, daemon won\'t start')
