@@ -18,12 +18,12 @@ import logging
 import re
 import threading
 import time
+from collections.abc import Callable
 from datetime import datetime
 from json import loads, dumps
-from typing import TYPE_CHECKING, List, Dict, Callable
+from typing import TYPE_CHECKING
 
 import rucio.db.sqla.util
-from rucio.db.sqla.constants import DIDType, SubscriptionState
 from rucio.common.config import config_get
 from rucio.common.exception import (
     DatabaseException,
@@ -39,22 +39,29 @@ from rucio.common.exception import (
     SubscriptionNotFound,
 )
 from rucio.common.logging import setup_logging
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.types import InternalAccount
 from rucio.common.utils import chunks
-from rucio.core import monitor
 from rucio.core.did import list_new_dids, set_new_dids, get_metadata
+from rucio.core.monitor import MetricManager
 from rucio.core.rse import list_rses, rse_exists, get_rse_id, list_rse_attributes
 from rucio.core.rse_expression_parser import parse_expression
 from rucio.core.rse_selector import resolve_rse_expression
 from rucio.core.rule import add_rule, list_rules, get_rule
 from rucio.core.subscription import list_subscriptions, update_subscription
 from rucio.daemons.common import run_daemon
+from rucio.db.sqla.constants import DIDType, SubscriptionState
 
 if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
     from rucio.daemons.common import HeartbeatHandler
     from rucio.common.types import InternalScope
 
+METRICS = MetricManager(module=__name__)
 graceful_stop = threading.Event()
+DAEMON_NAME = "transmogrifier"
 
 RULES_COMMENT_LENGTH = 255
 
@@ -122,7 +129,7 @@ def __split_rule_select_rses(
     copies: int,
     blocklisted_rse_id: list,
     logger: "Callable",
-) -> List[Dict]:
+) -> tuple[list, bool, bool]:
     """
     Internal method to create a list of RSEs that match RSE expression for subscriptions with split_rule.
 
@@ -136,8 +143,10 @@ def __split_rule_select_rses(
     :param copies: The number of copies.
     :param blocklisted_rse_id: The list of blocklisted_rse_id.
     :param logger: The logger.
-    :return: A tuple of list selected_rses, preferred_rses, preferred_unmatched.
+    :return: A tuple with list selected_rses, and 2 booleans create_rule, wont_reevaluate.
     """
+    wont_reevaluate = False
+    create_rule = True
     preferred_rses = set()
     for rule in list_rules(
         filters={
@@ -152,46 +161,59 @@ def __split_rule_select_rses(
         ):
             preferred_rses.add(rse_dict["rse"])
     preferred_rses = list(preferred_rses)
+    preferred_unmatched = list()
+    selected_rses = list()
 
-    try:
-        (selected_rses, preferred_unmatched,) = resolve_rse_expression(
-            rse_expression,
-            account,
-            weight=weight,
-            copies=copies,
-            size=0,
-            preferred_rses=preferred_rses,
-            blocklist=blocklisted_rse_id,
-        )
-
-    except (
-        InsufficientTargetRSEs,
-        InsufficientAccountLimit,
-        InvalidRuleWeight,
-        RSEOverQuota,
-    ) as error:
-        logger(
-            logging.WARNING,
-            'Problem getting RSEs for subscription "%s" for account %s : %s. Try including blocklisted sites'
-            % (
-                subscription_name,
+    for attempt in range(0, 2):
+        # First attempt excludes blocklisted RSEs
+        # Second attempt includes blocklisted RSEs
+        try:
+            (selected_rses, preferred_unmatched,) = resolve_rse_expression(
+                rse_expression,
                 account,
-                str(error),
-            ),
-        )
-        # Now including the blocklisted sites
-        (selected_rses, preferred_unmatched,) = resolve_rse_expression(
-            rse_expression,
-            account,
-            weight=weight,
-            copies=copies,
-            size=0,
-            preferred_rses=preferred_rses,
-        )
-    return selected_rses, preferred_rses, preferred_unmatched
+                weight=weight,
+                copies=copies,
+                size=0,
+                preferred_rses=preferred_rses,
+                blocklist=blocklisted_rse_id,
+            )
+            wont_reevaluate = True
+            break
+        except (
+            InsufficientTargetRSEs,
+            InsufficientAccountLimit,
+            InvalidRuleWeight,
+            RSEOverQuota,
+        ) as error:
+            logger(
+                logging.WARNING,
+                'Problem getting RSEs for subscription "%s" for account %s : %s. %s'
+                % (
+                    subscription_name,
+                    account,
+                    str(error),
+                    'Try including blocklisted sites' if attempt == 0 else 'Skipping rule creation.'
+                ),
+            )
+            # Now including the blocklisted sites
+            blocklisted_rse_id = []
+            METRICS.counter(name="addnewrule.errortype.{exception}").labels(exception=str(error.__class__.__name__)).inc()
+            wont_reevaluate = True
+        except Exception as error:
+            logger(
+                logging.ERROR,
+                "Problem resolving RSE expression %s : %s"
+                % (
+                    rse_expression,
+                    str(error),
+                )
+            )
+    if len(preferred_rses) - len(preferred_unmatched) >= copies:
+        create_rule = False
+    return selected_rses, create_rule, wont_reevaluate
 
 
-def get_subscriptions(logger: "Callable" = logging.log) -> List[Dict]:
+def get_subscriptions(logger: Callable = logging.log) -> list[dict]:
     """
     A method to extract the list of active subscriptions and exclued the one that have bad RSE expression.
     :param logger: The logger.
@@ -359,7 +381,7 @@ def __is_matching_subscription(subscription, did, metadata):
     return True
 
 
-def select_algorithm(algorithm: str, rule_ids: list, params: dict) -> dict:
+def select_algorithm(algorithm: str, rule_ids: list, params: dict, logger: "Callable") -> dict:
     """
     Method used in case of chained subscriptions
 
@@ -369,15 +391,15 @@ def select_algorithm(algorithm: str, rule_ids: list, params: dict) -> dict:
     :param params: Dictionary of rules parameters to be used by the algorithm
     """
     selected_rses = {}
-    if algorithm == "associated_site":
-        for rule_id in rule_ids:
-            rule = get_rule(rule_id)
-            logging.debug("In select_algorithm, %s", str(rule))
-            rse = rule["rse_expression"]
-            vo = rule["account"].vo
-            if rse_exists(rse, vo=vo):
-                rse_id = get_rse_id(rse, vo=vo)
-                rse_attributes = list_rse_attributes(rse_id)
+    for rule_id in rule_ids:
+        rule = get_rule(rule_id)
+        logging.debug("In select_algorithm, %s", str(rule))
+        rse = rule["rse_expression"]
+        vo = rule["account"].vo
+        if rse_exists(rse, vo=vo):
+            rse_id = get_rse_id(rse, vo=vo)
+            rse_attributes = list_rse_attributes(rse_id)
+            if algorithm == "associated_site":
                 associated_sites = rse_attributes.get("associated_sites", None)
                 associated_site_idx = params.get("associated_site_idx", None)
                 if not associated_site_idx:
@@ -395,14 +417,40 @@ def select_algorithm(algorithm: str, rule_ids: list, params: dict) -> dict:
                         "source_replica_expression": rse,
                         "weight": None,
                     }
-            else:
-                raise SubscriptionWrongParameter(
-                    "Algorithm associated_site only works with split_rule"
-                )
-            if rule["copies"] != 1:
-                raise SubscriptionWrongParameter(
-                    "Algorithm associated_site only works with split_rule"
+            if algorithm == "exclude_site":
+                site = rse_attributes.get("site", None)
+                rse_expression = params['rse_expression'] + '\\site=%s' % site
+                (
+                    selected_rses,
+                    create_rule,
+                    wont_reevaluate,
+                ) = __split_rule_select_rses(
+                    subscription_id=params["subscription_id"],
+                    subscription_name=params["subscription_name"],
+                    scope=rule["scope"],
+                    name=rule["name"],
+                    account=rule.get("account"),
+                    weight=rule.get("weight"),
+                    rse_expression=rse_expression,
+                    copies=rule.get('copies'),
+                    blocklisted_rse_id=params['blocklisted_rse_id'],
+                    logger=logger,
                 )
+                dict_selected_rses = {}
+                for entry in selected_rses:
+                    dict_selected_rses[entry] = {
+                        "source_replica_expression": rse,
+                        "weight": None,
+                    }
+                selected_rses = dict_selected_rses
+        else:
+            raise SubscriptionWrongParameter(
+                "Algorithm %s only works with split_rule" % algorithm
+            )
+        if rule["copies"] != 1:
+            raise SubscriptionWrongParameter(
+                "Algorithm %s only works with split_rule" % algorithm
+            )
     return selected_rses
 
 
@@ -419,8 +467,7 @@ def transmogrifier(bulk: int = 5, once: bool = False, sleep_time: int = 60) -> N
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
-        executable="transmogrifier",
-        logger_prefix="transmogrifier",
+        executable=DAEMON_NAME,
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -433,7 +480,7 @@ def transmogrifier(bulk: int = 5, once: bool = False, sleep_time: int = 60) -> N
 def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> bool:
 
     worker_number, total_workers, logger = heartbeat_handler.live()
-    timer = monitor.Timer()
+    stopwatch = Stopwatch()
     blocklisted_rse_id = [rse["id"] for rse in list_rses({"availability_write": False})]
     identifiers = []
     #  List all the active subscriptions
@@ -495,6 +542,10 @@ def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> boo
                     if chained_idx:
                         #  In the case of chained subscription, don't use rseselector but use the rses returned by the algorithm
                         params = {}
+                        params['rse_expression'] = rule_dict.get("rse_expression")
+                        params['subscription_id'] = subscription["id"]
+                        params['subscription_name'] = subscription["name"]
+                        params['blocklisted_rse_id'] = blocklisted_rse_id
                         if rule_dict.get("associated_site_idx", None):
                             params["associated_site_idx"] = rule_dict.get(
                                 "associated_site_idx", None
@@ -506,56 +557,38 @@ def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> boo
                         )
                         algorithm = rule_dict.get("algorithm", None)
                         selected_rses = select_algorithm(
-                            algorithm, created_rules[chained_idx], params
+                            algorithm,
+                            created_rules[chained_idx],
+                            params,
+                            logger
                         )
                         copies = 1
                     elif split_rule:
-                        try:
-                            (
-                                selected_rses,
-                                preferred_rses,
-                                preferred_unmatched,
-                            ) = __split_rule_select_rses(
-                                subscription_id=subscription["id"],
-                                subscription_name=subscription["name"],
-                                scope=did["scope"],
-                                name=did["name"],
-                                account=rule_dict.get("account"),
-                                weight=weight,
-                                rse_expression=rule_dict.get("rse_expression"),
-                                copies=copies,
-                                blocklisted_rse_id=blocklisted_rse_id,
-                                logger=logger,
-                            )
-                            copies = 1
-                        except (
-                            InsufficientTargetRSEs,
-                            InsufficientAccountLimit,
-                            InvalidRuleWeight,
-                            RSEOverQuota,
-                        ) as error:
-                            logger(
-                                logging.WARNING,
-                                'Problem getting RSEs for subscription "%s" for account %s : %s. Skipping rule creation.'
-                                % (
-                                    subscription["name"],
-                                    rule_dict.get("account"),
-                                    str(error),
-                                ),
-                            )
-                            monitor.record_counter(
-                                name="transmogrifier.addnewrule.errortype.{exception}",
-                                labels={"exception": str(error.__class__.__name__)},
-                            )
-                            # The DID won't be reevaluated at the next cycle
-                            did_success = did_success and True
-                            continue
-
-                        if len(preferred_rses) - len(preferred_unmatched) >= copies:
+                        (
+                            selected_rses,
+                            create_rule,
+                            wont_reevaluate,
+                        ) = __split_rule_select_rses(
+                            subscription_id=subscription["id"],
+                            subscription_name=subscription["name"],
+                            scope=did["scope"],
+                            name=did["name"],
+                            account=rule_dict.get("account"),
+                            weight=weight,
+                            rse_expression=rule_dict.get("rse_expression"),
+                            copies=copies,
+                            blocklisted_rse_id=blocklisted_rse_id,
+                            logger=logger,
+                        )
+                        copies = 1
+                        if not create_rule:
                             continue
+                        # The DID won't be reevaluated at the next cycle
+                        did_success = did_success and wont_reevaluate
 
                     nb_rule = 0
                     #  Try to create the rule
+                    logger(logging.DEBUG, 'selected_rses : %s' % selected_rses)
                     try:
                         for rse in selected_rses:
                             if isinstance(selected_rses, dict):
@@ -601,17 +634,8 @@ def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> boo
                             if split_rule:
                                 success = True
 
-                        monitor.record_counter(
-                            name="transmogrifier.addnewrule.done",
-                            delta=nb_rule,
-                        )
-                        monitor.record_counter(
-                            name="transmogrifier.addnewrule.activity.{activity}",
-                            delta=nb_rule,
-                            labels={
-                                "activity": "".join(rule_dict.get("activity").split())
-                            },
-                        )
+                        METRICS.counter("addnewrule.done").inc(nb_rule)
+                        METRICS.counter("addnewrule.activity.{activity}").labels(activity="".join(rule_dict.get("activity").split())).inc(nb_rule)
                         success = True
                     except (
                         InvalidReplicationRule,
@@ -623,16 +647,10 @@ def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> boo
                         # Errors that won't be retried
                         success = True
                         logger(logging.ERROR, str(error))
-                        monitor.record_counter(
-                            name="transmogrifier.addnewrule.errortype.{exception}",
-                            labels={"exception": str(error.__class__.__name__)},
-                        )
+                        METRICS.counter("addnewrule.errortype.{exception}").labels(exception=str(error.__class__.__name__)).inc()
                     except Exception:
                         # Errors that will be retried
-                        monitor.record_counter(
-                            name="transmogrifier.addnewrule.errortype.{exception}",
-                            labels={"exception": "unknown"},
-                        )
+                        METRICS.counter("addnewrule.errortype.{exception}").labels(exception="unknown").inc()
                         logger(logging.ERROR, "Unexpected error", exc_info=True)
 
                     did_success = did_success and success
@@ -655,14 +673,12 @@ def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> boo
 
         if did_success:
             if did["did_type"] == str(DIDType.FILE):
-                monitor.record_counter(name="transmogrifier.did.file.processed")
+                METRICS.counter(name="files_processed").inc()
             elif did["did_type"] == str(DIDType.DATASET):
-                monitor.record_counter(name="transmogrifier.did.dataset.processed")
+                METRICS.counter(name="datasets_processed").inc()
             elif did["did_type"] == str(DIDType.CONTAINER):
-                monitor.record_counter(
-                    name="transmogrifier.did.container.processed", delta=1
-                )
-            monitor.record_counter(name="transmogrifier.did.processed", delta=1)
+                METRICS.counter(name="containers_processed").inc()
+            METRICS.counter(name="dids_processed").inc()
             identifiers.append(
                 {
                     "scope": did["scope"],
@@ -672,26 +688,26 @@ def run_once(heartbeat_handler: "HeartbeatHandler", bulk: int, **_kwargs) -> boo
             )
 
     #  Mark the DIDs as processed
-    flag_timer = monitor.Timer()
+    flag_stopwatch = Stopwatch()
     for identifier in chunks(identifiers, 100):
         set_new_dids(identifier, None)
-    logger(logging.DEBUG, "Time to set the new flag : %f" % flag_timer.elapsed)
+    logger(logging.DEBUG, "Time to set the new flag : %f" % flag_stopwatch.elapsed)
 
-    timer.stop()
+    stopwatch.stop()
 
     for sub in subscriptions:
         update_subscription(
             name=sub["name"],
             account=sub["account"],
-            metadata={"last_processed": datetime.now()},
+            metadata={"last_processed": datetime.utcnow()},
         )
     logger(
         logging.INFO,
-        "It took %f seconds to process %i DIDs" % (timer.elapsed, len(identifiers)),
+        "It took %f seconds to process %i DIDs" % (stopwatch.elapsed, len(identifiers)),
     )
     logger(logging.DEBUG, "DIDs processed : %s" % (str(identifiers)))
-    monitor.record_counter(name="transmogrifier.job.done", delta=1)
-    timer.record("transmogrifier.job.duration")
+    METRICS.counter(name="transmogrifier.job.done").inc(1)
+    METRICS.timer("job.duration").observe(stopwatch.elapsed)
     must_sleep = True
     return must_sleep
 
@@ -702,7 +718,7 @@ def run(
     """
     Starts up the transmogrifier threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException("Database was not updated, daemon won't start")
@@ -730,7 +746,7 @@ def run(
             ]
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
