@@ -25,7 +25,7 @@ from copy import deepcopy
 from datetime import datetime, timedelta
 from random import randint
 from re import match
-from typing import Tuple, Dict
+from typing import TYPE_CHECKING
 
 from sqlalchemy.exc import DatabaseError
 
@@ -35,25 +35,29 @@ from rucio.common.logging import setup_logging
 from rucio.common.types import InternalAccount
 from rucio.common.utils import chunks
 from rucio.core.did import list_expired_dids, delete_dids
-from rucio.core.monitor import record_counter
+from rucio.core.monitor import MetricManager
 from rucio.daemons.common import run_daemon, HeartbeatHandler
 
+if TYPE_CHECKING:
+    from types import FrameType
+    from typing import Optional
+
 logging.getLogger("requests").setLevel(logging.CRITICAL)
 
+METRICS = MetricManager(module=__name__)
 graceful_stop = threading.Event()
+DAEMON_NAME = 'undertaker'
 
 
 def undertaker(once: bool = False, sleep_time: int = 60, chunk_size: int = 10):
     """
     Main loop to select and delete dids.
     """
-    executable = 'undertaker'
     paused_dids = {}  # {(scope, name): datetime}
     run_daemon(
         once=once,
         graceful_stop=graceful_stop,
-        executable=executable,
-        logger_prefix=executable,
+        executable=DAEMON_NAME,
         partition_wait_time=1,
         sleep_time=sleep_time,
         run_once_fnc=functools.partial(
@@ -64,7 +68,7 @@ def undertaker(once: bool = False, sleep_time: int = 60, chunk_size: int = 10):
     )
 
 
-def run_once(paused_dids: Dict[Tuple, datetime], chunk_size: int, heartbeat_handler: HeartbeatHandler, **_kwargs):
+def run_once(paused_dids: dict[tuple, datetime], chunk_size: int, heartbeat_handler: HeartbeatHandler, **_kwargs):
     worker_number, total_workers, logger = heartbeat_handler.live()
 
     try:
@@ -88,14 +92,14 @@ def run_once(paused_dids: Dict[Tuple, datetime], chunk_size: int, heartbeat_hand
                 logger(logging.INFO, 'Receive %s dids to delete', len(chunk))
                 delete_dids(dids=chunk, account=InternalAccount('root', vo='def'), expire_rules=True)
                 logger(logging.INFO, 'Delete %s dids', len(chunk))
-                record_counter(name='undertaker.delete_dids', delta=len(chunk))
+                METRICS.counter(name='undertaker.delete_dids').inc(len(chunk))
             except RuleNotFound as error:
                 logger(logging.ERROR, error)
             except (DatabaseException, DatabaseError, UnsupportedOperation) as e:
                 if match('.*ORA-00054.*', str(e.args[0])) or match('.*55P03.*', str(e.args[0])) or match('.*3572.*', str(e.args[0])):
                     for did in chunk:
                         paused_dids[(did['scope'], did['name'])] = datetime.utcnow() + timedelta(seconds=randint(600, 2400))
-                    record_counter('undertaker.delete_dids.exceptions.{exception}', labels={'exception': 'LocksDetected'})
+                    METRICS.counter('delete_dids.exceptions.{exception}').labels(exception='LocksDetected').inc()
                     logger(logging.WARNING, 'Locks detected for chunk')
                 else:
                     logger(logging.ERROR, 'Got database error %s.', str(e))
@@ -103,7 +107,7 @@ def run_once(paused_dids: Dict[Tuple, datetime], chunk_size: int, heartbeat_hand
         logging.critical(traceback.format_exc())
 
 
-def stop(signum=None, frame=None):
+def stop(signum: "Optional[int]" = None, frame: "Optional[FrameType]" = None) -> None:
     """
     Graceful exit.
     """
@@ -114,7 +118,7 @@ def run(once: bool = False, total_workers: int = 1, chunk_size: int = 10, sleep_
     """
     Starts up the undertaker threads.
     """
-    setup_logging()
+    setup_logging(process_name=DAEMON_NAME)
 
     if rucio.db.sqla.util.is_old_db():
         raise DatabaseException('Database was not updated, daemon won\'t start')
