@@ -21,12 +21,7 @@ import sqlalchemy as sa
 from alembic import context
 from alembic.op import (add_column, create_primary_key, create_table, create_unique_constraint,
                         drop_column, drop_constraint, drop_table, bulk_insert)
-
-
-def String(*arg, **kw):
-    kw['convert_unicode'] = True
-    return sa.String(*arg, **kw)
-
+from sqlalchemy import String
 
 # Alembic revision identifiers
 revision = 'a118956323f8'
