@@ -13,27 +13,38 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import copy
+import logging
 import os
 import sys
-
-from functools import wraps
-from inspect import isgeneratorfunction
-from retrying import retry
-from threading import Lock
+from datetime import datetime, timedelta
+from functools import update_wrapper
+from inspect import isgeneratorfunction, getfullargspec
 from os.path import basename
+from time import sleep
+from threading import Lock
+from typing import TYPE_CHECKING
 
-from sqlalchemy import create_engine, event
-from sqlalchemy.exc import DatabaseError, DisconnectionError, OperationalError, TimeoutError
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import sessionmaker, scoped_session
+from sqlalchemy import create_engine, event, MetaData, text
+from sqlalchemy.exc import DatabaseError, DisconnectionError, OperationalError, TimeoutError, SQLAlchemyError
+from sqlalchemy.orm import DeclarativeBase, sessionmaker, scoped_session, Session
 from sqlalchemy.pool import QueuePool, SingletonThreadPool, NullPool
 
 from rucio.common.config import config_get
 from rucio.common.exception import RucioException, DatabaseException, InputValidationError
 from rucio.common.extra import import_extras
+from rucio.common.utils import retrying
 
 EXTRA_MODULES = import_extras(['MySQLdb', 'pymysql'])
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+    from typing import Optional, ParamSpec, TypeVar
+
+    P = ParamSpec('P')
+    R = TypeVar('R')
+
+
 try:
     main_script = os.path.basename(sys.argv[0])
     CURRENT_COMPONENT = main_script.split('-')[1]
@@ -49,15 +60,16 @@ try:
 except:
     pass
 
-BASE = declarative_base()
 DEFAULT_SCHEMA_NAME = config_get(DATABASE_SECTION, 'schema',
                                  raise_exception=False, default=None, check_config_table=False)
-if DEFAULT_SCHEMA_NAME:
-    BASE.metadata.schema = DEFAULT_SCHEMA_NAME
-
+_METADATA = MetaData(schema=DEFAULT_SCHEMA_NAME)
 _MAKER, _ENGINE, _LOCK = None, None, Lock()
 
 
+class BASE(DeclarativeBase):
+    metadata = _METADATA
+
+
 def _fk_pragma_on_connect(dbapi_con, con_record):
     # Hack for previous versions of sqlite3
     try:
@@ -200,7 +212,9 @@ def get_engine():
                 params[param] = param_type(config_get(DATABASE_SECTION, param, check_config_table=False))
             except:
                 pass
-        _ENGINE = create_engine(sql_connection, **params)
+        # Using sqlAlchemy 2.0 with future=True.
+        # if backing up from 2.0, need to remove future=True .
+        _ENGINE = create_engine(sql_connection, future=True, **params)
         if 'mysql' in sql_connection:
             event.listen(_ENGINE, 'checkout', mysql_ping_listener)
         elif 'postgresql' in sql_connection:
@@ -247,7 +261,8 @@ def get_maker():
     global _MAKER, _ENGINE
     assert _ENGINE
     if not _MAKER:
-        _MAKER = sessionmaker(bind=_ENGINE, autocommit=False, autoflush=True, expire_on_commit=True)
+        # turn on sqlAlchemy 2.0 with future=True.
+        _MAKER = sessionmaker(bind=_ENGINE, autocommit=False, autoflush=True, expire_on_commit=True, future=True)
     return _MAKER
 
 
@@ -268,6 +283,27 @@ def get_session():
     return session
 
 
+def wait_for_database(timeout: int = 60, interval: int = 2, *, logger=logging.log):
+    """ Wait for the database for a specific amount of time """
+
+    end_time = datetime.utcnow() + timedelta(seconds=timeout)
+    while True:
+        try:
+            session = get_session()
+            if session.bind.dialect.name == 'oracle':
+                session.execute(text('select 1 from dual'))
+            else:
+                session.execute(text('select 1'))
+            session.close()
+            break
+        except SQLAlchemyError as e:
+            logger(logging.WARNING, 'Still waiting for database: %s', e)
+            if datetime.utcnow() >= end_time:
+                raise
+
+        sleep(interval)
+
+
 def retry_if_db_connection_error(exception):
     """Return True if error in connecting to db."""
     if isinstance(exception, (OperationalError, DatabaseException)):
@@ -284,7 +320,41 @@ def retry_if_db_connection_error(exception):
     return False
 
 
-def read_session(function):
+def _update_session_wrapper(wrapper, wrapped):
+    """
+    In addition to the work done by functools.update_wrapper, this function also preservers
+    the signature of the initial function. With the exception that the 'session' parameter
+    is overriden to have a default value of 'None'.
+
+    wrapper is the function to be updated
+    wrapped is the original function
+
+    To simplify the implementation of this function, we require 'session' be a
+    keyword-only argument in the wrapped function.
+    """
+    try:
+        arg_spec = getfullargspec(wrapped)
+        arg_spec.kwonlyargs.index('session')
+    except ValueError:
+        # We require decorated functions to have a 'session' keyword-only attribute.
+        # re-raise ValueError if not
+        raise
+
+    update_wrapper(wrapper, wrapped)
+
+    wrapper.__defaults__ = copy.copy(wrapped.__defaults__)
+    wrapper.__kwdefaults__ = copy.copy(wrapped.__kwdefaults__)
+    wrapper.__annotations__["session"] = "Optional[Session]"
+
+    # Set the default of the keyword-only attribute 'session' to None
+    if not wrapped.__kwdefaults__:
+        wrapper.__kwdefaults__ = {'session': None}
+    elif 'session' not in wrapped.__kwdefaults__:
+        wrapper.__kwdefaults__['session'] = None
+    return wrapper
+
+
+def read_session(function: "Callable[P, R]"):
     '''
     decorator that set the session variable to use inside a function.
     With that decorator it's possible to use the session variable like if a global variable session is declared.
@@ -293,20 +363,19 @@ def read_session(function):
     This is useful if only SELECTs and the like are being done; anything involving
     INSERTs, UPDATEs etc should use transactional_session.
     '''
-    @retry(retry_on_exception=retry_if_db_connection_error,
-           wait_fixed=500,
-           stop_max_attempt_number=2,
-           wrap_exception=False)
-    @wraps(function)
-    def new_funct(*args, **kwargs):
+    @retrying(retry_on_exception=retry_if_db_connection_error,
+              wait_fixed=500,
+              stop_max_attempt_number=2)
+    def new_funct(*args: "P.args", session: "Optional[Session]" = None, **kwargs):  # pylint:disable=missing-kwoa
         if isgeneratorfunction(function):
             raise RucioException('read_session decorator should not be used with generator. Use stream_session instead.')
 
-        if not kwargs.get('session'):
-            session = get_session()
+        if not session:
+            session_scoped = get_session()
+            session = session_scoped()
+            session.begin()
             try:
-                kwargs['session'] = session
-                return function(*args, **kwargs)
+                return function(*args, session=session, **kwargs)
             except TimeoutError as error:
                 session.rollback()  # pylint: disable=maybe-no-member
                 raise DatabaseException(str(error))
@@ -317,16 +386,15 @@ def read_session(function):
                 session.rollback()  # pylint: disable=maybe-no-member
                 raise
             finally:
-                session.remove()
+                session_scoped.remove()
         try:
-            return function(*args, **kwargs)
+            return function(*args, session=session, **kwargs)
         except Exception:
             raise
-    new_funct.__doc__ = function.__doc__
-    return new_funct
+    return _update_session_wrapper(new_funct, function)
 
 
-def stream_session(function):
+def stream_session(function: "Callable[P, R]"):
     '''
     decorator that set the session variable to use inside a function.
     With that decorator it's possible to use the session variable like if a global variable session is declared.
@@ -335,21 +403,20 @@ def stream_session(function):
     This is useful if only SELECTs and the like are being done; anything involving
     INSERTs, UPDATEs etc should use transactional_session.
     '''
-    @retry(retry_on_exception=retry_if_db_connection_error,
-           wait_fixed=500,
-           stop_max_attempt_number=2,
-           wrap_exception=False)
-    @wraps(function)
-    def new_funct(*args, **kwargs):
+    @retrying(retry_on_exception=retry_if_db_connection_error,
+              wait_fixed=500,
+              stop_max_attempt_number=2)
+    def new_funct(*args: "P.args", session: "Optional[Session]" = None, **kwargs):  # pylint:disable=missing-kwoa
 
         if not isgeneratorfunction(function):
             raise RucioException('stream_session decorator should be used only with generator. Use read_session instead.')
 
-        if not kwargs.get('session'):
-            session = get_session()
+        if not session:
+            session_scoped = get_session()
+            session = session_scoped()
+            session.begin()
             try:
-                kwargs['session'] = session
-                for row in function(*args, **kwargs):
+                for row in function(*args, session=session, **kwargs):
                     yield row
             except TimeoutError as error:
                 session.rollback()  # pylint: disable=maybe-no-member
@@ -361,31 +428,30 @@ def stream_session(function):
                 session.rollback()  # pylint: disable=maybe-no-member
                 raise
             finally:
-                session.remove()
+                session_scoped.remove()
         else:
             try:
-                for row in function(*args, **kwargs):
+                for row in function(*args, session=session, **kwargs):
                     yield row
             except:
                 raise
-    new_funct.__doc__ = function.__doc__
-    return new_funct
+    return _update_session_wrapper(new_funct, function)
 
 
-def transactional_session(function):
+def transactional_session(function: "Callable[P, R]"):
     '''
     decorator that set the session variable to use inside a function.
     With that decorator it's possible to use the session variable like if a global variable session is declared.
 
     session is a sqlalchemy session, and you can get one calling get_session().
     '''
-    @wraps(function)
-    def new_funct(*args, **kwargs):
-        if not kwargs.get('session'):
-            session = get_session()
+    def new_funct(*args: "P.args", session: "Optional[Session]" = None, **kwargs):  # pylint:disable=missing-kwoa
+        if not session:
+            session_scoped = get_session()
+            session = session_scoped()
+            session.begin()
             try:
-                kwargs['session'] = session
-                result = function(*args, **kwargs)
+                result = function(*args, session=session, **kwargs)
                 session.commit()  # pylint: disable=maybe-no-member
             except TimeoutError as error:
                 session.rollback()  # pylint: disable=maybe-no-member
@@ -397,9 +463,8 @@ def transactional_session(function):
                 session.rollback()  # pylint: disable=maybe-no-member
                 raise
             finally:
-                session.remove()  # pylint: disable=maybe-no-member
+                session_scoped.remove()  # pylint: disable=maybe-no-member
         else:
-            result = function(*args, **kwargs)
+            result = function(*args, session=session, **kwargs)
         return result
-    new_funct.__doc__ = function.__doc__
-    return new_funct
+    return _update_session_wrapper(new_funct, function)
