@@ -26,7 +26,7 @@ from dogpile.cache.api import NoValue
 from sqlalchemy import func, inspect, Column, PrimaryKeyConstraint
 from sqlalchemy.dialects.postgresql.base import PGInspector
 from sqlalchemy.exc import IntegrityError, DatabaseError
-from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.orm import declarative_base
 from sqlalchemy.schema import CreateSchema, MetaData, Table, CreateTable, DropTable, ForeignKeyConstraint, DropConstraint
 from sqlalchemy.sql.ddl import DropSchema
 from sqlalchemy.sql.expression import select, text
@@ -37,7 +37,6 @@ from rucio.common.config import config_get, config_get_list
 from rucio.common.schema import get_schema_value
 from rucio.common.types import InternalAccount
 from rucio.common.utils import generate_uuid
-from rucio.core.account_counter import create_counters_for_new_account
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import AccountStatus, AccountType, IdentityType
 from rucio.db.sqla.session import get_engine, get_session, get_dump_engine
@@ -59,7 +58,9 @@ def build_database():
     if schema:
         print('Schema set in config, trying to create schema:', schema)
         try:
-            engine.execute(CreateSchema(schema))
+            with engine.connect() as conn:
+                with conn.begin():
+                    conn.execute(CreateSchema(schema))
         except Exception as e:
             print('Cannot create schema, please validate manually if schema creation is needed, continuing:', e)
 
@@ -97,7 +98,8 @@ def drop_everything():
 
     # the transaction only applies if the DB supports
     # transactional DDL, i.e. Postgresql, MS SQL Server
-    with engine.begin() as conn:
+
+    with engine.connect() as conn:
 
         inspector = inspect(conn)  # type: Union[Inspector, PGInspector]
 
@@ -128,19 +130,17 @@ def drop_everything():
 def create_base_vo():
     """ Creates the base VO """
 
-    s = get_session()
+    session_scoped = get_session()
 
     vo = models.VO(vo='def', description='Default base VO', email='N/A')
-
-    s.add_all([vo])
-    s.commit()
+    with session_scoped() as s:
+        with s.begin():
+            s.add_all([vo])
 
 
-def create_root_account(create_counters=True):
+def create_root_account():
     """
     Inserts the default root account to an existing database. Make sure to change the default password later.
-
-    :param create_counters: If True, create counters for the new account at existing RSEs.
     """
 
     multi_vo = bool(config_get('common', 'multi_vo', False, False))
@@ -148,7 +148,7 @@ def create_root_account(create_counters=True):
     up_id = 'ddmlab'
     up_pwd = 'secret'
     up_email = 'ph-adp-ddm-lab@cern.ch'
-    x509_id = '/C=CH/ST=Geneva/O=CERN/OU=PH-ADP-CO/CN=DDMLAB Client Certificate/emailAddress=ph-adp-ddm-lab@cern.ch'
+    x509_id = 'emailAddress=ph-adp-ddm-lab@cern.ch,CN=DDMLAB Client Certificate,OU=PH-ADP-CO,O=CERN,ST=Geneva,C=CH'
     x509_email = 'ph-adp-ddm-lab@cern.ch'
     gss_id = 'ddmlab@CERN.CH'
     gss_email = 'ph-adp-ddm-lab@cern.ch'
@@ -173,7 +173,7 @@ def create_root_account(create_counters=True):
         pass
         # print 'Config values are missing (check rucio.cfg{.template}). Using hardcoded defaults.'
 
-    s = get_session()
+    session_scoped = get_session()
 
     if multi_vo:
         access = 'super_root'
@@ -200,46 +200,45 @@ def create_root_account(create_counters=True):
     identity4 = models.Identity(identity=ssh_id, identity_type=IdentityType.SSH, email=ssh_email)
     iaa4 = models.IdentityAccountAssociation(identity=identity4.identity, identity_type=identity4.identity_type, account=account.account, is_default=True)
 
-    # Account counters
-    if create_counters:
-        create_counters_for_new_account(account=account.account, session=s)
-
-    # Apply
-    for identity in [identity1, identity2, identity3, identity4]:
-        try:
-            s.add(identity)
-            s.commit()
-        except IntegrityError:
-            # Identities may already be in the DB when running multi-VO conversion
-            s.rollback()
-    s.add(account)
-    s.commit()
-    s.add_all([iaa1, iaa2, iaa3, iaa4])
-    s.commit()
+    with session_scoped() as s:
+        s.begin()
+        # Apply
+        for identity in [identity1, identity2, identity3, identity4]:
+            try:
+                s.add(identity)
+                s.commit()
+            except IntegrityError:
+                # Identities may already be in the DB when running multi-VO conversion
+                s.rollback()
+        s.add(account)
+        s.flush()
+        s.add_all([iaa1, iaa2, iaa3, iaa4])
+        s.commit()
 
 
 def get_db_time():
     """ Gives the utc time on the db. """
-    s = get_session()
+    session_scoped = get_session()
     try:
         storage_date_format = None
-        if s.bind.dialect.name == 'oracle':
-            query = select([text("sys_extract_utc(systimestamp)")])
-        elif s.bind.dialect.name == 'mysql':
-            query = select([text("utc_timestamp()")])
-        elif s.bind.dialect.name == 'sqlite':
-            query = select([text("datetime('now', 'utc')")])
+        if session_scoped.bind.dialect.name == 'oracle':
+            query = select(text("sys_extract_utc(systimestamp)"))
+        elif session_scoped.bind.dialect.name == 'mysql':
+            query = select(text("utc_timestamp()"))
+        elif session_scoped.bind.dialect.name == 'sqlite':
+            query = select(text("datetime('now', 'utc')"))
             storage_date_format = '%Y-%m-%d  %H:%M:%S'
         else:
-            query = select([func.current_date()])
+            query = select(func.current_date())
 
-        for now, in s.execute(query):
+        session = session_scoped()
+        session.begin()
+        for now, in session.execute(query):
             if storage_date_format:
                 return datetime.strptime(now, storage_date_format)
             return now
-
     finally:
-        s.remove()
+        session_scoped.remove()
 
 
 def get_count(q):
@@ -262,15 +261,20 @@ def is_old_db():
     schema = config_get('database', 'schema', raise_exception=False)
 
     # checks if alembic is being used by looking up the AlembicVersion table
-    if not get_engine().has_table(models.AlembicVersion.__tablename__, schema):
+    inspector = inspect(get_engine())
+    if not inspector.has_table(models.AlembicVersion.__tablename__, schema):
         return False
 
-    s = get_session()
-    query = s.query(models.AlembicVersion.version_num)
-    return query.count() != 0 and str(query.first()[0]) != alembicrevision.ALEMBIC_REVISION
+    session_scoped = get_session()
+    with session_scoped() as s:
+        with s.begin():
+            # query = s.query(models.AlembicVersion.version_num)
+            query = s.execute(select(models.AlembicVersion)).scalars().all()
+            # return query.count() != 0 and str(query.first()[0]) != alembicrevision.ALEMBIC_REVISION
+            return (len(query) != 0 and str(query[0].version_num) != alembicrevision.ALEMBIC_REVISION)
 
 
-def json_implemented(session=None):
+def json_implemented(*, session=None):
     """
     Checks if the database on the current server installation can support json fields.
 
