@@ -17,12 +17,11 @@ import copy
 from sqlalchemy import and_, delete, exists, select
 from sqlalchemy.orm import aliased
 
-from rucio.core import config as config_db
+from rucio.core import config as core_config
 from rucio.core.vo import map_vo
 from rucio.db.sqla import models
 from rucio.db.sqla.session import transactional_session, get_session
-from rucio.tests.common import get_long_vo
-
+from .common import get_long_vo
 
 # Functions containing server-only includes that can't be included in client tests
 # For each table, get the foreign key constraints from all other tables towards this table.
@@ -62,7 +61,7 @@ def _dependency_paths(stack, nb_times_in_stack, cur_table):
 
 
 @transactional_session
-def cleanup_db_deps(model, select_rows_stmt, session=None):
+def cleanup_db_deps(model, select_rows_stmt, *, session=None):
     """
     Removes rows which have foreign key constraints pointing to rows
     selected by `select_rows_stmt` in `model`. The deletion is transitive.
@@ -99,7 +98,7 @@ def cleanup_db_deps(model, select_rows_stmt, session=None):
             ).where(
                 exists(
                     select(
-                        [1]
+                        1
                     ).where(
                         and_(*filters)
                     ).where(
@@ -121,8 +120,8 @@ def reset_config_table():
     db_session = get_session()
     db_session.query(models.Config).delete()
     db_session.commit()
-    config_db.set("vo-map", "testvo1", "tst")
-    config_db.set("vo-map", "testvo2", "ts2")
+    core_config.set("vo-map", "testvo1", "tst")
+    core_config.set("vo-map", "testvo2", "ts2")
 
 
 def get_vo():
