@@ -15,32 +15,33 @@
 
 import datetime
 import json
-import functools
 import logging
+import pathlib
 import traceback
 import uuid
-from typing import Any, Callable, Dict, Tuple, TYPE_CHECKING
+from collections.abc import Callable
+from configparser import NoOptionError, NoSectionError
+from json import loads
+from typing import Any, Optional, TYPE_CHECKING
 from urllib.parse import urlparse
 
 import requests
-from configparser import NoOptionError, NoSectionError
-from json import loads
+from dogpile.cache.api import NoValue
 from requests.adapters import ReadTimeout
 from requests.packages.urllib3 import disable_warnings  # pylint: disable=import-error
 
-from dogpile.cache.api import NoValue
-
 from rucio.common.cache import make_region_memcached
-from rucio.common.config import config_get, config_get_bool
+from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.constants import FTS_JOB_TYPE, FTS_STATE, FTS_COMPLETE_STATE
 from rucio.common.exception import TransferToolTimeout, TransferToolWrongAnswer, DuplicateFileTransferSubmission
+from rucio.common.stopwatch import Stopwatch
 from rucio.common.utils import APIEncoder, chunks, PREFERRED_CHECKSUM
+from rucio.core.monitor import MetricManager
+from rucio.core.oidc import get_token_for_account_operation
 from rucio.core.request import get_source_rse, get_transfer_error
 from rucio.core.rse import get_rse_supported_checksums_from_attributes
-from rucio.core.oidc import get_token_for_account_operation
-from rucio.core.monitor import record_counter, Timer, MultiCounter
-from rucio.transfertool.transfertool import Transfertool, TransferToolBuilder, TransferStatusReport
 from rucio.db.sqla.constants import RequestState
+from rucio.transfertool.transfertool import Transfertool, TransferToolBuilder, TransferStatusReport
 
 if TYPE_CHECKING:
     from rucio.core.transfer import DirectTransferDefinition
@@ -50,32 +51,131 @@ logging.getLogger("requests").setLevel(logging.CRITICAL)
 disable_warnings()
 
 REGION_SHORT = make_region_memcached(expiration_time=900)
+METRICS = MetricManager(module=__name__)
 
-SUBMISSION_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_submission', statsd='transfertool.fts3.{host}.submission.{state}',
-                                  documentation='Number of transfers submitted', labelnames=('state', 'host'))
-CANCEL_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_cancel', statsd='transfertool.fts3.{host}.cancel.{state}',
-                              documentation='Number of cancelled transfers', labelnames=('state', 'host'))
-UPDATE_PRIORITY_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_update_priority', statsd='transfertool.fts3.{host}.update_priority.{state}',
-                                       documentation='Number of priority updates', labelnames=('state', 'host'))
-QUERY_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_query', statsd='transfertool.fts3.{host}.query.{state}',
-                             documentation='Number of queried transfers', labelnames=('state', 'host'))
-WHOAMI_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_whoami', statsd='transfertool.fts3.{host}.whoami.{state}',
-                              documentation='Number of whoami requests', labelnames=('state', 'host'))
-VERSION_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_version', statsd='transfertool.fts3.{host}.version.{state}',
-                               documentation='Number of version requests', labelnames=('state', 'host'))
-BULK_QUERY_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_bulk_query', statsd='transfertool.fts3.{host}.bulk_query.{state}',
-                                  documentation='Number of bulk queries', labelnames=('state', 'host'))
-QUERY_DETAILS_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_query_details', statsd='transfertool.fts3.{host}.query_details.{state}',
-                                     documentation='Number of detailed status queries', labelnames=('state', 'host'))
+SUBMISSION_COUNTER = METRICS.counter(name='{host}.submission.{state}',
+                                     documentation='Number of transfers submitted', labelnames=('state', 'host'))
+CANCEL_COUNTER = METRICS.counter(name='{host}.cancel.{state}',
+                                 documentation='Number of cancelled transfers', labelnames=('state', 'host'))
+UPDATE_PRIORITY_COUNTER = METRICS.counter(name='{host}.update_priority.{state}',
+                                          documentation='Number of priority updates', labelnames=('state', 'host'))
+QUERY_COUNTER = METRICS.counter(name='{host}.query.{state}',
+                                documentation='Number of queried transfers', labelnames=('state', 'host'))
+WHOAMI_COUNTER = METRICS.counter(name='{host}.whoami.{state}',
+                                 documentation='Number of whoami requests', labelnames=('state', 'host'))
+VERSION_COUNTER = METRICS.counter(name='{host}.version.{state}',
+                                  documentation='Number of version requests', labelnames=('state', 'host'))
+BULK_QUERY_COUNTER = METRICS.counter(name='{host}.bulk_query.{state}',
+                                     documentation='Number of bulk queries', labelnames=('state', 'host'))
+QUERY_DETAILS_COUNTER = METRICS.counter(name='{host}.query_details.{state}',
+                                        documentation='Number of detailed status queries', labelnames=('state', 'host'))
 
 ALLOW_USER_OIDC_TOKENS = config_get_bool('conveyor', 'allow_user_oidc_tokens', False, False)
 REQUEST_OIDC_SCOPE = config_get('conveyor', 'request_oidc_scope', False, 'fts:submit-transfer')
 REQUEST_OIDC_AUDIENCE = config_get('conveyor', 'request_oidc_audience', False, 'fts:example')
+REWRITE_HTTPS_TO_DAVS = config_get_bool('transfers', 'rewrite_https_to_davs', default=False)
+VO_CERTS_PATH = config_get('conveyor', 'vo_certs_path', False, None)
 
 # https://fts3-docs.web.cern.ch/fts3-docs/docs/state_machine.html
 FINAL_FTS_JOB_STATES = (FTS_STATE.FAILED, FTS_STATE.CANCELED, FTS_STATE.FINISHED, FTS_STATE.FINISHEDDIRTY)
 FINAL_FTS_FILE_STATES = (FTS_STATE.FAILED, FTS_STATE.CANCELED, FTS_STATE.FINISHED, FTS_STATE.NOT_USED)
 
+# In a multi-hop transfer, we must compute a checksum validation strategy valid for the whole path.
+# This state-machine defines how strategies of hops are merged into a path-wide strategy.
+# For example, if HOP1 supports only validation of checksum at source while HOP2 only
+# supports validation at destination, the strategy for the whole path MUST be "none". Otherwise,
+# transfers will fail when FTS will try to validate the checksum.
+PATH_CHECKSUM_VALIDATION_STRATEGY: dict[tuple[str, str], str] = {
+    ('both', 'both'): 'both',
+    ('both', 'target'): 'target',
+    ('both', 'source'): 'source',
+    ('both', 'none'): 'none',
+    ('target', 'both'): 'target',
+    ('target', 'target'): 'target',
+    ('target', 'source'): 'none',
+    ('target', 'none'): 'none',
+    ('source', 'both'): 'source',
+    ('source', 'target'): 'none',
+    ('source', 'source'): 'source',
+    ('source', 'none'): 'none',
+    ('none', 'both'): 'none',
+    ('none', 'target'): 'none',
+    ('none', 'source'): 'none',
+    ('none', 'none'): 'none',
+}
+
+_SCITAGS_NEXT_REFRESH = datetime.datetime.utcnow()
+_SCITAGS_EXP_ID = None
+_SCITAGS_ACTIVITY_IDS = {}
+
+
+def _scitags_ids(logger: Callable[..., Any] = logging.log) -> "tuple[int | None, dict[str, int]]":
+    """
+    Re-fetch if needed and return the scitags ids
+    """
+    enabled = config_get_bool('packet-marking', 'enabled', default=False)
+    if not enabled:
+        return None, {}
+
+    now = datetime.datetime.utcnow()
+    global _SCITAGS_ACTIVITY_IDS
+    global _SCITAGS_EXP_ID
+    global _SCITAGS_NEXT_REFRESH
+    if _SCITAGS_NEXT_REFRESH < now:
+        exp_name = config_get('packet-marking', 'exp_name', default='')
+        fetch_url = config_get('packet-marking', 'fetch_url', default='https://www.scitags.org/api.json')
+        fetch_interval = config_get_int('packet-marking', 'fetch_interval', default=datetime.timedelta(hours=48).seconds)
+        fetch_timeout = config_get_int('packet-marking', 'fetch_timeout', default=5)
+
+        _SCITAGS_NEXT_REFRESH = now + datetime.timedelta(seconds=fetch_interval)
+
+        if exp_name:
+            had_exception = False
+            exp_id = None
+            activity_ids = {}
+            try:
+                result = requests.get(fetch_url, timeout=fetch_timeout)
+                if result and result.status_code == 200:
+                    marks = result.json()
+                    for experiment in marks.get('experiments', []):
+                        if experiment.get('expName') == exp_name:
+                            exp_id = experiment.get('expId')
+                            for activity_dict in experiment.get('activities', []):
+                                activity_name = activity_dict.get('activityName')
+                                activity_id = activity_dict.get('activityId')
+                                if activity_name and activity_id:
+                                    activity_ids[activity_name] = int(activity_id)
+                            break
+            except (requests.exceptions.RequestException, TypeError, ValueError):
+                had_exception = True
+                logger(logging.WARNING, 'Failed to fetch the scitags markings', exc_info=True)
+
+            if had_exception:
+                # Retry quicker after fetch errors
+                _SCITAGS_NEXT_REFRESH = min(_SCITAGS_NEXT_REFRESH, now + datetime.timedelta(minutes=5))
+            else:
+                _SCITAGS_EXP_ID = exp_id
+                _SCITAGS_ACTIVITY_IDS = activity_ids
+
+    return _SCITAGS_EXP_ID, _SCITAGS_ACTIVITY_IDS
+
+
+def _pick_cert_file(vo: "Optional[str]") -> "Optional[str]":
+    cert = None
+    if vo:
+        vo_cert = config_get('vo_certs', vo, False, None)
+        if vo_cert:
+            cert = vo_cert
+        elif VO_CERTS_PATH:
+            vo_cert = pathlib.Path(VO_CERTS_PATH) / vo
+            if vo_cert.exists():
+                cert = str(vo_cert)
+    if not cert:
+        usercert = config_get('conveyor', 'usercert', False, None)
+        if usercert:
+            cert = usercert
+    return cert
+
 
 def _configured_source_strategy(activity: str, logger: Callable[..., Any]) -> str:
     """
@@ -114,13 +214,11 @@ def oidc_supported(transfer_hop) -> bool:
     return True
 
 
-def _checksum_validation_strategy(
-    transfer: "DirectTransferDefinition",
-    logger: Callable[..., Any],
-) -> Tuple[str, str]:
+def _available_checksums(
+        transfer: "DirectTransferDefinition",
+) -> tuple[set[str], set[str]]:
     """
-    Compute the checksum validation strategy (none, source, destination or both) and the checksum
-    to use (in the format accepted by FTS) from the attributes of the source and destination RSE.
+    Get checksums which can be used for file validation on the source and the destination RSE
     """
     src_attributes = transfer.src.rse.attributes
     if src_attributes.get('verify_checksum', True):
@@ -134,20 +232,73 @@ def _checksum_validation_strategy(
     else:
         dst_checksums = set()
 
+    return src_checksums, dst_checksums
+
+
+def _hop_checksum_validation_strategy(
+        transfer: "DirectTransferDefinition",
+        logger: Callable[..., Any],
+) -> tuple[str, set[str]]:
+    """
+    Compute the checksum validation strategy (none, source, destination or both) depending
+    on available source and destination checksums for a single hop transfer
+    """
+    src_checksums, dst_checksums = _available_checksums(transfer)
     intersection = src_checksums.intersection(dst_checksums)
 
     if intersection:
         strategy, possible_checksums = 'both', intersection
     elif dst_checksums:
         # The prioritization of destination over source here is desired, not random
-        logger(logging.INFO, 'No common checksum method. Verifying destination only.')
+        logger(logging.INFO, f'No common checksum method for {transfer}. Verifying destination only.')
         strategy, possible_checksums = 'target', dst_checksums
     elif src_checksums:
-        logger(logging.INFO, 'No common checksum method. Verifying source only.')
+        logger(logging.INFO, f'No common checksum method for {transfer}. Verifying source only.')
         strategy, possible_checksums = 'source', src_checksums
     else:
-        logger(logging.INFO, 'No common checksum method. Not verifying source nor destination.')
+        logger(logging.INFO, f'No common checksum method for {transfer}. Not verifying source nor destination.')
         strategy, possible_checksums = 'none', set()
+    return strategy, possible_checksums
+
+
+def _path_checksum_validation_strategy(
+        transfer_path: "list[DirectTransferDefinition]",
+        logger: Callable[..., Any],
+) -> str:
+    """
+    Compute the checksum validation strategy for the whole transfer path.
+    """
+
+    path_strategy = 'both'
+    for transfer_hop in transfer_path:
+        hop_strategy, _ = _hop_checksum_validation_strategy(transfer_hop, logger)
+
+        path_strategy = PATH_CHECKSUM_VALIDATION_STRATEGY.get((path_strategy, hop_strategy), 'none')
+
+    return path_strategy
+
+
+def _pick_fts_checksum(
+        transfer: "DirectTransferDefinition",
+        path_strategy: "str",
+) -> Optional[str]:
+    """
+    Pick the checksum to use for validating file integrity on this particular transfer hop.
+    This function will only work correctly for values of 'path_strategy' which are
+    valid for the englobing multi-hop transfer path.
+
+    Returns the checksum as a string in the format expected by the FTS bulks submission API.
+    """
+    src_checksums, dst_checksums = _available_checksums(transfer)
+
+    if path_strategy == 'both':
+        possible_checksums = src_checksums.intersection(dst_checksums)
+    elif path_strategy == 'target':
+        possible_checksums = dst_checksums
+    elif path_strategy == 'source':
+        possible_checksums = src_checksums
+    else:
+        possible_checksums = set()
 
     checksum_to_use = None
     for checksum_name in possible_checksums:
@@ -159,58 +310,71 @@ def _checksum_validation_strategy(
         if checksum_name == PREFERRED_CHECKSUM:
             break
 
-    return strategy, checksum_to_use
+    return checksum_to_use
 
 
-def job_params_for_fts_transfer(transfer, bring_online, default_lifetime, archive_timeout_override, max_time_in_queue, logger, multihop=False):
+def build_job_params(transfer_path, bring_online, default_lifetime, archive_timeout_override, max_time_in_queue, logger):
     """
     Prepare the job parameters which will be passed to FTS transfertool
     """
 
+    # The last hop is the main request (the one which triggered the whole transfer),
+    # so most attributes will come from it
+    last_hop = transfer_path[-1]
+    first_hop = transfer_path[0]
+
     overwrite, bring_online_local = True, None
-    if transfer.src.rse.is_tape_or_staging_required():
+    if first_hop.src.rse.is_tape_or_staging_required():
+        # Activate bring_online if it was requested by first hop
+        # We don't allow multihop via a tape, so bring_online should not be set on any other hop
         bring_online_local = bring_online
-    if transfer.dst.rse.is_tape():
+    if last_hop.dst.rse.is_tape():
         overwrite = False
 
     # Get dest space token
-    dest_protocol = transfer.protocol_factory.protocol(transfer.dst.rse, transfer.dst.scheme, transfer.operation_dest)
+    dest_protocol = last_hop.protocol_factory.protocol(last_hop.dst.rse, last_hop.dst.scheme, last_hop.operation_dest)
     dest_spacetoken = None
     if dest_protocol.attributes and 'extended_attributes' in dest_protocol.attributes and \
             dest_protocol.attributes['extended_attributes'] and 'space_token' in dest_protocol.attributes['extended_attributes']:
         dest_spacetoken = dest_protocol.attributes['extended_attributes']['space_token']
-    src_spacetoken = None
 
-    strict_copy = transfer.dst.rse.attributes.get('strict_copy', False)
-    archive_timeout = transfer.dst.rse.attributes.get('archive_timeout', None)
+    strict_copy = last_hop.dst.rse.attributes.get('strict_copy', False)
+    archive_timeout = last_hop.dst.rse.attributes.get('archive_timeout', None)
 
-    verify_checksum, _checksum_to_use = _checksum_validation_strategy(transfer, logger=logger)
-
-    job_params = {'account': transfer.rws.account,
-                  'verify_checksum': verify_checksum,
-                  'copy_pin_lifetime': transfer.rws.attributes.get('lifetime', default_lifetime),
+    job_params = {'account': last_hop.rws.account,
+                  'verify_checksum': _path_checksum_validation_strategy(transfer_path, logger=logger),
+                  'copy_pin_lifetime': last_hop.rws.attributes.get('lifetime', default_lifetime),
                   'bring_online': bring_online_local,
                   'job_metadata': {
                       'issuer': 'rucio',
-                      'multi_sources': True if len(transfer.legacy_sources) > 1 else False,
+                      'multi_sources': False,
                   },
-                  'overwrite': transfer.rws.attributes.get('overwrite', overwrite),
-                  'priority': transfer.rws.priority}
+                  'overwrite': last_hop.rws.attributes.get('overwrite', overwrite),
+                  'priority': last_hop.rws.priority}
 
-    if multihop:
+    if len(transfer_path) > 1:
         job_params['multihop'] = True
         job_params['job_metadata']['multihop'] = True
+    elif len(last_hop.legacy_sources) > 1:
+        job_params['job_metadata']['multi_sources'] = True
     if strict_copy:
         job_params['strict_copy'] = strict_copy
     if dest_spacetoken:
         job_params['spacetoken'] = dest_spacetoken
-    if src_spacetoken:
-        job_params['source_spacetoken'] = src_spacetoken
-    if transfer.use_ipv4:
+    if last_hop.use_ipv4:
         job_params['ipv4'] = True
         job_params['ipv6'] = False
 
-    if archive_timeout and transfer.dst.rse.is_tape():
+    # assume s3alternate True (path-style URL S3 RSEs)
+    job_params['s3alternate'] = True
+    src_rse_s3_url_style = first_hop.src.rse.attributes.get('s3_url_style', None)
+    if src_rse_s3_url_style == "host":
+        job_params['s3alternate'] = False
+    dst_rse_s3_url_style = last_hop.dst.rse.attributes.get('s3_url_style', None)
+    if dst_rse_s3_url_style == "host":
+        job_params['s3alternate'] = False
+
+    if archive_timeout and last_hop.dst.rse.is_tape():
         try:
             archive_timeout = int(archive_timeout)
             if archive_timeout_override is None:
@@ -221,13 +385,24 @@ def job_params_for_fts_transfer(transfer, bring_online, default_lifetime, archiv
             job_params['dst_file_report'] = True
             logger(logging.DEBUG, 'Added archive timeout to transfer.')
         except ValueError:
-            logger(logging.WARNING, 'Could not set archive_timeout for %s. Must be integer.', transfer)
+            logger(logging.WARNING, 'Could not set archive_timeout for %s. Must be integer.', last_hop)
             pass
     if max_time_in_queue:
-        if transfer.rws.activity in max_time_in_queue:
-            job_params['max_time_in_queue'] = max_time_in_queue[transfer.rws.activity]
+        if last_hop.rws.activity in max_time_in_queue:
+            job_params['max_time_in_queue'] = max_time_in_queue[last_hop.rws.activity]
         elif 'default' in max_time_in_queue:
             job_params['max_time_in_queue'] = max_time_in_queue['default']
+
+    overwrite_hop = True
+    for transfer_hop in transfer_path[:-1]:
+        # Only allow overwrite if all hops in multihop allow it
+        h_overwrite = transfer_hop.rws.attributes.get('overwrite', True)
+        job_params['overwrite'] = h_overwrite and job_params['overwrite']
+        # Allow overwrite_hop if all intermediate hops allow it (ignoring the last hop)
+        overwrite_hop = h_overwrite and overwrite_hop
+    if not job_params['overwrite'] and overwrite_hop:
+        job_params['overwrite_hop'] = overwrite_hop
+
     return job_params
 
 
@@ -247,52 +422,35 @@ def bulk_group_transfers(transfer_paths, policy='rule', group_bulk=200, source_s
     """
 
     grouped_transfers = {}
-    grouped_jobs = []
+    fts_jobs = []
 
-    _build_job_params = functools.partial(job_params_for_fts_transfer,
-                                          bring_online=bring_online,
-                                          default_lifetime=default_lifetime,
-                                          archive_timeout_override=archive_timeout_override,
-                                          max_time_in_queue=max_time_in_queue,
-                                          logger=logger)
     for transfer_path in transfer_paths:
-        if len(transfer_path) > 1:
-            # for multihop transfers, all the path is submitted as a separate job
-            job_params = _build_job_params(transfer_path[-1], multihop=True)
-
-            overwrite_hop = True
-            for transfer in transfer_path[:-1]:
-                hop_params = _build_job_params(transfer, multihop=True)
-                # Only allow overwrite if all transfers in multihop allow it
-                job_params['overwrite'] = hop_params['overwrite'] and job_params['overwrite']
-                # Allow overwrite_hop if all intermediate hops allow it (ignoring the last hop)
-                overwrite_hop = hop_params['overwrite'] and overwrite_hop
-                # Activate bring_online if it was requested by first hop (it is a multihop starting at a tape)
-                # We don't allow multihop via a tape, so bring_online should not be set on any other hop
-                if transfer is transfer_path[0] and hop_params['bring_online']:
-                    job_params['bring_online'] = hop_params['bring_online']
-            if not job_params['overwrite'] and overwrite_hop:
-                job_params['overwrite_hop'] = overwrite_hop
-
-            group_key = 'multihop_%s' % transfer_path[-1].rws.request_id
-            grouped_transfers[group_key] = {'transfers': transfer_path[0:group_bulk], 'job_params': job_params}
-        elif len(transfer_path[0].legacy_sources) > 1:
-            # for multi-source transfers, no bulk submission.
-            transfer = transfer_path[0]
-            grouped_jobs.append({'transfers': [transfer], 'job_params': _build_job_params(transfer)})
+        job_params = build_job_params(
+            transfer_path=transfer_path,
+            bring_online=bring_online,
+            default_lifetime=default_lifetime,
+            archive_timeout_override=archive_timeout_override,
+            max_time_in_queue=max_time_in_queue,
+            logger=logger
+        )
+        if job_params['job_metadata'].get('multi_sources') or job_params['job_metadata'].get('multihop'):
+            # for multi-hop and multi-source transfers, no bulk submission.
+            fts_jobs.append({'transfers': transfer_path[0:group_bulk], 'job_params': job_params})
         else:
             # it's a single-hop, single-source, transfer. Hence, a candidate for bulk submission.
             transfer = transfer_path[0]
-            job_params = _build_job_params(transfer)
 
             # we cannot group transfers together if their job_key differ
-            job_key = '%s,%s,%s,%s,%s,%s,%s,%s' % (job_params['verify_checksum'], job_params.get('spacetoken', None),
-                                                   job_params['copy_pin_lifetime'],
-                                                   job_params['bring_online'], job_params['job_metadata'],
-                                                   job_params.get('source_spacetoken', None),
-                                                   job_params['overwrite'], job_params['priority'])
-            if 'max_time_in_queue' in job_params:
-                job_key = job_key + ',%s' % job_params['max_time_in_queue']
+            job_key = '%s,%s,%s,%s,%s,%s,%s,%s' % (
+                job_params['verify_checksum'],
+                job_params.get('spacetoken', ''),
+                job_params['copy_pin_lifetime'],
+                job_params['bring_online'],
+                job_params['job_metadata'],
+                job_params['overwrite'],
+                job_params['priority'],
+                job_params.get('max_time_in_queue', '')
+            )
 
             # Additionally, we don't want to group transfers together if their policy_key differ
             policy_key = ''
@@ -321,9 +479,9 @@ def bulk_group_transfers(transfer_paths, policy='rule', group_bulk=200, source_s
     for group in grouped_transfers.values():
         job_params = group['job_params']
         for transfer_paths in chunks(group['transfers'], group_bulk):
-            grouped_jobs.append({'transfers': transfer_paths, 'job_params': job_params})
+            fts_jobs.append({'transfers': transfer_paths, 'job_params': job_params})
 
-    return grouped_jobs
+    return fts_jobs
 
 
 class Fts3TransferStatusReport(TransferStatusReport):
@@ -446,8 +604,8 @@ class Fts3TransferStatusReport(TransferStatusReport):
         return False
 
     @classmethod
-    def _is_recoverable_fts_overwrite_error(cls, request: Dict[str, Any], reason: str,
-                                            file_metadata: Dict[str, Any]) -> bool:
+    def _is_recoverable_fts_overwrite_error(cls, request: dict[str, Any], reason: str,
+                                            file_metadata: dict[str, Any]) -> bool:
         """
         Verify the special case when FTS cannot copy a file because destination exists and overwrite is disabled,
         but the destination file is actually correct.
@@ -461,18 +619,15 @@ class Fts3TransferStatusReport(TransferStatusReport):
             return False
         dst_file = file_metadata.get('dst_file', {})
         dst_type = file_metadata.get('dst_type', None)
-        record_counter('transfertool.fts3.overwrite.check.{rsetype}.{rse}',
-                       labels={'rse': file_metadata["dst_rse"], 'rsetype': dst_type})
+        METRICS.counter('overwrite.check.{rsetype}.{rse}').labels(rse=file_metadata["dst_rse"], rsetype=dst_type).inc()
 
         if 'Destination file exists and overwrite is not enabled' in (reason or ''):
             if cls._dst_file_set_and_file_correct(request, dst_file):
                 if dst_type == 'DISK' or dst_file.get('file_on_tape'):
-                    record_counter('transfertool.fts3.overwrite.ok.{rsetype}.{rse}',
-                                   labels={'rse': file_metadata["dst_rse"], 'rsetype': dst_type})
+                    METRICS.counter('overwrite.ok.{rsetype}.{rse}').labels(rse=file_metadata["dst_rse"], rsetype=dst_type).inc()
                     return True
 
-        record_counter('transfertool.fts3.overwrite.fail.{rsetype}.{rse}',
-                       labels={'rse': file_metadata["dst_rse"], 'rsetype': dst_type})
+        METRICS.counter('overwrite.fail.{rsetype}.{rse}').labels(rse=file_metadata["dst_rse"], rsetype=dst_type).inc()
         return False
 
 
@@ -500,7 +655,7 @@ class FTS3CompletionMessageTransferStatusReport(Fts3TransferStatusReport):
         reason = fts_message.get('t__error_message', None)
         # job_state = fts_message.get('t_final_transfer_state', None)
         new_state = None
-        if str(fts_message['t_final_transfer_state']) == FTS_COMPLETE_STATE.OK:  # pylint:disable=no-member
+        if str(fts_message['t_final_transfer_state']) == FTS_COMPLETE_STATE.OK and not fts_message.get('is_archiving'):  # pylint:disable=no-member
             new_state = RequestState.DONE
         elif str(fts_message['t_final_transfer_state']) == FTS_COMPLETE_STATE.ERROR:
             request = self.request(session)
@@ -582,7 +737,7 @@ class FTS3ApiTransferStatusReport(Fts3TransferStatusReport):
         if file_state_is_final:
             if file_state == FTS_STATE.FINISHED:
                 new_state = RequestState.DONE
-            elif file_state == FTS_STATE.FAILED and job_state == FTS_STATE.FAILED or \
+            elif file_state == FTS_STATE.FAILED and job_state_is_final or \
                     file_state == FTS_STATE.FAILED and not self._multi_sources:  # for multi-source transfers we must wait for the job to be in a final state
                 if self._is_recoverable_fts_overwrite_error(self.request(session), reason, self._file_metadata):
                     new_state = RequestState.DONE
@@ -661,10 +816,6 @@ class FTS3Transfertool(Transfertool):
         self.default_lifetime = default_lifetime
         self.archive_timeout_override = archive_timeout_override
 
-        usercert = config_get('conveyor', 'usercert', False, None)
-        if vo:
-            usercert = config_get('vo_certs', vo, False, usercert)
-
         # token for OAuth 2.0 OIDC authorization scheme (working only with dCache + davs/https protocols as of Sep 2019)
         self.token = None
         if oidc_account:
@@ -688,12 +839,15 @@ class FTS3Transfertool(Transfertool):
                 self.verify = False
                 self.headers['Authorization'] = 'Bearer ' + self.token
             else:
-                self.cert = (usercert, usercert)
+                cert = _pick_cert_file(vo=vo)
+                self.cert = (cert, cert)
                 self.verify = False
         else:
             self.cert = None
             self.verify = True  # True is the default setting of a requests.* method
 
+        self.scitags_exp_id, self.scitags_activity_ids = _scitags_ids(logger=logger)
+
     @classmethod
     def _pick_fts_servers(cls, source_rse: "RseData", dest_rse: "RseData"):
         """
@@ -758,9 +912,9 @@ class FTS3Transfertool(Transfertool):
         )
         return jobs
 
-    def __file_from_transfer(self, transfer):
+    def _file_from_transfer(self, transfer, job_params):
         rws = transfer.rws
-        _verify_checksum, checksum_to_use = _checksum_validation_strategy(transfer, logger=self.logger)
+        checksum_to_use = _pick_fts_checksum(transfer, path_strategy=job_params['verify_checksum'])
         t_file = {
             'sources': [s[1] for s in transfer.legacy_sources],
             'destinations': [transfer.dest_url],
@@ -785,6 +939,10 @@ class FTS3Transfertool(Transfertool):
             'selection_strategy': self.source_strategy if self.source_strategy else _configured_source_strategy(transfer.rws.activity, logger=self.logger),
             'activity': rws.activity
         }
+        if isinstance(self.scitags_exp_id, int):
+            activity_id = self.scitags_activity_ids.get(rws.activity)
+            if isinstance(activity_id, int):
+                t_file['scitag'] = self.scitags_exp_id << 6 | activity_id
         return t_file
 
     def submit(self, transfers, job_params, timeout=None):
@@ -798,34 +956,30 @@ class FTS3Transfertool(Transfertool):
         """
         files = []
         for transfer in transfers:
-            if isinstance(transfer, dict):
-                # Compatibility with scripts form /tools which directly use transfertools and pass a dict to it instead of transfer definitions
-                # TODO: ensure that those scripts are still used and get rid of this compatibility otherwise
-                files.append(transfer)
-                continue
-
-            files.append(self.__file_from_transfer(transfer))
+            files.append(self._file_from_transfer(transfer, job_params))
 
         # FTS3 expects 'davs' as the scheme identifier instead of https
         for transfer_file in files:
             if not transfer_file['sources'] or transfer_file['sources'] == []:
                 raise Exception('No sources defined')
 
-            new_src_urls = []
-            new_dst_urls = []
-            for url in transfer_file['sources']:
-                if url.startswith('https'):
-                    new_src_urls.append(':'.join(['davs'] + url.split(':')[1:]))
-                else:
-                    new_src_urls.append(url)
-            for url in transfer_file['destinations']:
-                if url.startswith('https'):
-                    new_dst_urls.append(':'.join(['davs'] + url.split(':')[1:]))
-                else:
-                    new_dst_urls.append(url)
+            # TODO: remove the following logic in rucio 1.31
+            if REWRITE_HTTPS_TO_DAVS:
+                new_src_urls = []
+                new_dst_urls = []
+                for url in transfer_file['sources']:
+                    if url.startswith('https'):
+                        new_src_urls.append(':'.join(['davs'] + url.split(':')[1:]))
+                    else:
+                        new_src_urls.append(url)
+                for url in transfer_file['destinations']:
+                    if url.startswith('https'):
+                        new_dst_urls.append(':'.join(['davs'] + url.split(':')[1:]))
+                    else:
+                        new_dst_urls.append(url)
 
-            transfer_file['sources'] = new_src_urls
-            transfer_file['destinations'] = new_dst_urls
+                transfer_file['sources'] = new_src_urls
+                transfer_file['destinations'] = new_dst_urls
 
         transfer_id = None
         expected_transfer_id = None
@@ -841,8 +995,8 @@ class FTS3Transfertool(Transfertool):
         params_str = json.dumps(params_dict, cls=APIEncoder)
 
         post_result = None
+        stopwatch = Stopwatch()
         try:
-            timer = Timer()
             post_result = requests.post('%s/jobs' % self.external_host,
                                         verify=self.verify,
                                         cert=self.cert,
@@ -850,7 +1004,7 @@ class FTS3Transfertool(Transfertool):
                                         headers=self.headers,
                                         timeout=timeout)
             labels = {'host': self.__extract_host(self.external_host)}
-            timer.record('transfertool.fts3.submit_transfer.{host}', divisor=len(files), labels=labels)
+            METRICS.timer('submit_transfer.{host}').labels(**labels).observe(stopwatch.elapsed / (len(files) or 1))
         except ReadTimeout as error:
             raise TransferToolTimeout(error)
         except json.JSONDecodeError as error:
@@ -874,7 +1028,7 @@ class FTS3Transfertool(Transfertool):
 
         if not transfer_id:
             raise TransferToolWrongAnswer('No transfer id returned by %s' % self.external_host)
-        timer.record('core.request.submit_transfers_fts3', divisor=len(transfers))
+        METRICS.timer('submit_transfers_fts3').observe(stopwatch.elapsed / (len(transfers) or 1))
         return transfer_id
 
     def cancel(self, transfer_ids, timeout=None):
@@ -1026,7 +1180,7 @@ class FTS3Transfertool(Transfertool):
                                timeout=timeout)
 
         if jobs is None:
-            record_counter('transfertool.fts3.{host}.bulk_query.failure', labels={'host': self.__extract_host(self.external_host)})
+            BULK_QUERY_COUNTER.labels(state='failure', host=self.__extract_host(self.external_host)).inc()
             for transfer_id in requests_by_eid:
                 responses[transfer_id] = Exception('Transfer information returns None: %s' % jobs)
         elif jobs.status_code in (200, 207, 404):
