@@ -16,8 +16,8 @@
 import logging
 
 from rucio.common.utils import chunks
-from rucio.transfertool.transfertool import Transfertool, TransferToolBuilder, TransferStatusReport
 from rucio.db.sqla.constants import RequestState
+from rucio.transfertool.transfertool import Transfertool, TransferToolBuilder, TransferStatusReport
 from .globus_library import bulk_submit_xfer, submit_xfer, bulk_check_xfers
 
 
@@ -82,6 +82,7 @@ class GlobusTransferTool(Transfertool):
     """
 
     external_name = 'globus'
+    required_rse_attrs = ('globus_endpoint_id', )
 
     def __init__(self, external_host, logger=logging.log, group_bulk=200, group_policy='single'):
         """
@@ -99,9 +100,7 @@ class GlobusTransferTool(Transfertool):
     @classmethod
     def submission_builder_for_path(cls, transfer_path, logger=logging.log):
         hop = transfer_path[0]
-        source_globus_endpoint_id = hop.src.rse.attributes.get('globus_endpoint_id', None)
-        dest_globus_endpoint_id = hop.dst.rse.attributes.get('globus_endpoint_id', None)
-        if not source_globus_endpoint_id or not dest_globus_endpoint_id:
+        if not cls.can_perform_transfer(hop.src.rse, hop.dst.rse):
             logger(logging.WARNING, "Source or destination globus_endpoint_id not set. Skipping {}".format(hop))
             return [], None
 
@@ -192,38 +191,10 @@ class GlobusTransferTool(Transfertool):
         return response
 
     def bulk_update(self, resps, request_ids):
-        """
-        bulk update request_state for globus transfers
-
-        :param resps: dictionary containing task IDs and current status
-        # TODO: do we need request IDs?
-        :param request_ids original list of rucio request IDs
-
-        counter = 0
-        for task_id in resps:
-            requests = get_requests_by_transfer(external_host=None, transfer_id=task_id, session=None)
-            logging.debug('requests: %s' % requests)
-            for request_id in requests:
-                transfer_core.update_transfer_state(external_host=None, transfer_id=request_id, state=resps[task_id][file_state])
-                counter += 1
-
-        return counter
-        """
         pass
 
     def cancel(self):
         pass
 
-    def query(self, transfer_ids, details=False, timeout=None):
-        """
-        Query the status of a transfer in Globus Online.
-
-        :param transfer_ids: transfer identifiers as list of strings.
-        :param details:      Switch if detailed information should be listed.
-        :param timeout:      Timeout in seconds.
-        :returns:            Transfer status information as a list of dictionaries.
-        """
-        pass
-
     def update_priority(self):
         pass
