@@ -14,11 +14,14 @@
 # limitations under the License.
 
 import logging
-
 from abc import ABCMeta, abstractmethod
+from typing import TYPE_CHECKING
 
 from rucio.core.request import get_request
 
+if TYPE_CHECKING:
+    from rucio.core.rse import RseData
+
 
 class TransferToolBuilder(object):
     """
@@ -112,6 +115,9 @@ class Transfertool(object, metaclass=ABCMeta):
     Interface definition of the Rucio transfertool
     """
 
+    external_name = ''
+    required_rse_attrs = ('globus_endpoint_id', )
+
     def __init__(self, external_host, logger=logging.log):
         """
         Initializes the transfertool
@@ -123,9 +129,20 @@ class Transfertool(object, metaclass=ABCMeta):
         self.logger = logger
 
     def __str__(self):
-        return self.external_host
+        return self.external_host if self.external_host is not None else self.__class__.__name__
+
+    @classmethod
+    def can_perform_transfer(cls, source_rse: "RseData", dest_rse: "RseData"):
+        """
+        Return True if this transfertool is able to perform a transfer between the given source and destination rses
+        """
+        if (
+                all(source_rse.attributes.get(attribute) is not None for attribute in cls.required_rse_attrs)
+                and all(dest_rse.attributes.get(attribute) is not None for attribute in cls.required_rse_attrs)
+        ):
+            return True
+        return False
 
-    @staticmethod
     @classmethod
     def submission_builder_for_path(cls, transfer_path, logger=logging.log):
         """
@@ -162,14 +179,12 @@ class Transfertool(object, metaclass=ABCMeta):
         pass
 
     @abstractmethod
-    def query(self, transfer_ids, details=False, timeout=None):
+    def bulk_query(self, requests_by_eid, timeout=None) -> dict[str, dict[str, TransferStatusReport]]:
         """
-        Query the status of transfers in the transfertool.
+        Query the status of a bulk of transfers in FTS3 via JSON.
 
-        :param transfer_ids: List of transfertool internal identifiers as a string.
-        :param details:      Switch if detailed information should be listed.
-        :param timeout:      Timeout in seconds.
-        :returns:            Transfer status information as list of dictionaries.
+        :param requests_by_eid: dictionary {external_id1: {request_id1: request1, ...}, ...} of request to be queried
+        :returns: Transfer status information as a dictionary.
         """
         pass
 
