@@ -16,7 +16,7 @@
 from datetime import datetime
 from json import dumps
 
-from flask import Flask, Blueprint, Response, request, redirect, jsonify
+from flask import Flask, Response, request, redirect, jsonify
 
 from rucio.api.account import add_account, del_account, get_account_info, list_accounts, list_identities, \
     list_account_attributes, add_account_attribute, del_account_attribute, update_account, get_usage_history
@@ -28,7 +28,8 @@ from rucio.api.scope import add_scope, get_scopes
 from rucio.common.exception import AccountNotFound, Duplicate, AccessDenied, RuleNotFound, RSENotFound, \
     IdentityError, CounterNotFound, ScopeNotFound, InvalidObject
 from rucio.common.utils import APIEncoder, render_json
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     try_stream, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
@@ -130,7 +131,6 @@ class Attributes(ErrorHandlingMethodView):
             description: Attribute already exists
         """
         parameters = json_parameters()
-        key = param_get(parameters, 'key', default=key)
         value = param_get(parameters, 'value')
         try:
             add_account_attribute(key=key, value=value, account=account, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
@@ -347,8 +347,6 @@ class AccountParameter(ErrorHandlingMethodView):
             if isinstance(value, datetime):
                 accdict[key] = value.strftime('%Y-%m-%dT%H:%M:%S')
 
-        del accdict['_sa_instance_state']
-
         return Response(render_json(**accdict), content_type="application/json")
 
     def put(self, account):
@@ -513,6 +511,8 @@ class Account(ErrorHandlingMethodView):
                       email:
                         description: The email.
                         type: string
+          401:
+            description: Invalid Auth Token
         """
 
         def generate(_filter, vo):
@@ -1035,7 +1035,7 @@ class GlobalUsage(ErrorHandlingMethodView):
 
 
 def blueprint(with_doc=False):
-    bp = Blueprint('accounts', __name__, url_prefix='/accounts')
+    bp = AuthenticatedBlueprint('accounts', __name__, url_prefix='/accounts')
 
     attributes_view = Attributes.as_view('attributes')
     bp.add_url_rule('/<account>/attr/', view_func=attributes_view, methods=['get', ])
@@ -1077,7 +1077,6 @@ def blueprint(with_doc=False):
         bp.add_url_rule('', view_func=account_view, methods=['get', ])
     bp.add_url_rule('/', view_func=account_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
