@@ -15,10 +15,11 @@
 
 from json import dumps
 
-from flask import Flask, Blueprint, request
+from flask import Flask, request
 
 from rucio.api.did import list_archive_content
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     parse_scope_name, try_stream, generate_http_error_flask, ErrorHandlingMethodView
 
 
@@ -52,19 +53,19 @@ class Archive(ErrorHandlingMethodView):
                     properties:
                       scope:
                         description: The scope of the did.
-                        type: str
+                        type: string
                       name:
                         description: The name of the did.
-                        type: str
+                        type: string
                       bytes:
                         description: The number of bytes.
-                        type: int
+                        type: integer
                       adler32:
                         description: The adler32 checksum.
-                        type: str
+                        type: string
                       md5:
                         description: The md5 checksum.
-                        type: str
+                        type: string
           400:
             description: Invalid value
           406:
@@ -83,12 +84,11 @@ class Archive(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('archives', __name__, url_prefix='/archives')
+    bp = AuthenticatedBlueprint('archives', __name__, url_prefix='/archives')
 
     archive_view = Archive.as_view('archive')
     bp.add_url_rule('/<path:scope_name>/files', view_func=archive_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
