@@ -17,30 +17,29 @@ import itertools
 import json
 import logging
 import re
+from configparser import NoOptionError, NoSectionError
 from functools import wraps
 from time import time
 from typing import TYPE_CHECKING
 
 import flask
-import typing
 from flask.views import MethodView
 from werkzeug.datastructures import Headers
 from werkzeug.exceptions import HTTPException
 from werkzeug.wrappers import Request, Response
 
 from rucio.api.authentication import validate_auth_token
+from rucio.common import config
 from rucio.common.exception import DatabaseException, RucioException, CannotAuthenticate, UnsupportedRequestedContentType
 from rucio.common.schema import get_schema_value
 from rucio.common.utils import generate_uuid, render_json
 from rucio.core.vo import map_vo
-from rucio.common import config
-from configparser import NoOptionError, NoSectionError
-
 
 if TYPE_CHECKING:
-    from typing import Optional, Union, Dict, Sequence, Tuple, Callable, Any, List
+    from collections.abc import Callable, Iterable, Sequence
+    from typing import Optional, Union, Any
 
-    HeadersType = Union[Headers, Dict[str, str], Sequence[Tuple[str, str]]]
+    HeadersType = Union[Headers, dict[str, str], Sequence[tuple[str, str]]]
 
 
 class CORSMiddleware(object):
@@ -49,10 +48,10 @@ class CORSMiddleware(object):
     This middleware intercepts the preflight OPTIONS requests and returns a 200 OK response.
     """
 
-    def __init__(self, app: flask.Flask) -> typing.NoReturn:
+    def __init__(self, app: flask.Flask) -> None:
         self.app = app
 
-    def __call__(self, environ: typing.Dict, start_response: typing.Callable) -> typing.Union[Response, typing.Iterable[bytes]]:
+    def __call__(self, environ: dict, start_response: 'Callable') -> 'Union[Response, Iterable[bytes]]':
         request: Request = Request(environ)
 
         if request.environ.get('REQUEST_METHOD') == 'OPTIONS':
@@ -141,15 +140,14 @@ def request_auth_env():
 
     try:
         auth = validate_auth_token(auth_token)
+    except CannotAuthenticate:
+        return generate_http_error_flask(401, CannotAuthenticate.__name__, 'Cannot authenticate with given credentials')
     except RucioException as error:
         return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
     except Exception:
         logging.exception('Internal error in validate_auth_token')
         return 'Internal Error', 500
 
-    if auth is None:
-        return generate_http_error_flask(401, CannotAuthenticate.__name__, 'Cannot authenticate with given credentials')
-
     flask.request.environ['vo'] = auth.get('vo', 'def')
     flask.request.environ['issuer'] = auth.get('account')
     flask.request.environ['identity'] = auth.get('identity')
@@ -301,7 +299,7 @@ def generate_http_error_flask(
         raise
 
 
-def json_parameters(json_loads: "Callable[[str], Any]" = json.loads, optional: "bool" = False) -> "Dict":
+def json_parameters(json_loads: "Callable[[str], Any]" = json.loads, optional: bool = False) -> dict:
     """
     Returns the JSON parameters from the current request's body as dict.
     """
@@ -312,7 +310,7 @@ def json_parameters(json_loads: "Callable[[str], Any]" = json.loads, optional: "
     return json_parse(types=(dict, ), json_loads=json_loads, **kwargs)
 
 
-def json_list(json_loads: "Callable[[str], Any]" = json.loads, optional: "bool" = False) -> "List":
+def json_list(json_loads: "Callable[[str], Any]" = json.loads, optional: bool = False) -> list:
     """
     Returns the JSON array from the current request's body as list.
     """
@@ -323,14 +321,14 @@ def json_list(json_loads: "Callable[[str], Any]" = json.loads, optional: "bool"
     return json_parse(types=(list, ), json_loads=json_loads, **kwargs)
 
 
-def json_parse(types: "Tuple", json_loads: "Callable[[str], Any]" = json.loads, **kwargs):
+def json_parse(types: tuple, json_loads: "Callable[[str], Any]" = json.loads, **kwargs):
     def clstostr(cls):
         if cls.__name__ == "dict":
             return "dictionary"
         else:
             return cls.__name__
 
-    def typestostr(_types: "Tuple"):
+    def typestostr(_types: tuple):
         return " or ".join(map(clstostr, _types))
 
     data = flask.request.get_data(as_text=True)
@@ -357,7 +355,7 @@ def json_parse(types: "Tuple", json_loads: "Callable[[str], Any]" = json.loads,
         )
 
 
-def param_get(parameters: "Dict", name: "str", **kwargs):
+def param_get(parameters: dict, name: str, **kwargs):
     if 'default' in kwargs:
         return parameters.get(name, kwargs['default'])
     else:
