@@ -15,13 +15,14 @@
 
 from typing import TYPE_CHECKING
 
-from flask import Flask, Blueprint, request
+from flask import Flask, request
 from werkzeug.datastructures import Headers
 
 from rucio.api.credential import get_signed_url
 from rucio.common.exception import CannotAuthenticate
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
 from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, extract_vo, \
-    generate_http_error_flask, ErrorHandlingMethodView, request_auth_env, response_headers
+    generate_http_error_flask, ErrorHandlingMethodView, response_headers
 
 if TYPE_CHECKING:
     from typing import Optional
@@ -44,14 +45,40 @@ class SignURL(ErrorHandlingMethodView):
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :status 200: OK
+        ---
+        summary: Cross-Site Scripting
+        description: Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Credentials
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+                description: The http origin.
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+                description: The http access controll request headers.
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+                description: The allowed methods.
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+                description: If credentials are allowed.
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+                description: The exposed access controll header.
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
@@ -166,7 +193,7 @@ class SignURL(ErrorHandlingMethodView):
 
 
 def blueprint(with_doc=False):
-    bp = Blueprint('credentials', __name__, url_prefix='/credentials')
+    bp = AuthenticatedBlueprint('credentials', __name__, url_prefix='/credentials')
 
     signurl_view = SignURL.as_view('signurl')
     bp.add_url_rule('/signurl', view_func=signurl_view, methods=['get', 'options'])
@@ -174,7 +201,6 @@ def blueprint(with_doc=False):
         # yes, /signur ~= '/signurl?$'
         bp.add_url_rule('/signur', view_func=signurl_view, methods=['get', 'options'])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
 
     return bp
