@@ -14,23 +14,23 @@
 # limitations under the License.
 
 import ast
-
 from json import dumps
 
-from flask import Flask, Blueprint, Response, request
+from flask import Flask, Response, request
 
-from rucio.api.did import add_did, add_dids, list_content, list_content_history, list_dids, list_dids_extended, \
+from rucio.api.did import add_did, add_dids, list_content, list_content_history, list_dids, \
     list_files, scope_list, get_did, set_metadata, get_metadata, get_metadata_bulk, set_status, attach_dids, \
     detach_dids, attach_dids_to_dids, get_dataset_by_guid, list_parent_dids, create_did_sample, list_new_dids, \
     resurrect, get_users_following_did, remove_did_from_followed, add_did_to_followed, delete_metadata, \
     set_metadata_bulk, set_dids_metadata_bulk
 from rucio.api.rule import list_replication_rules, list_associated_replication_rules_for_file
-from rucio.common.exception import ScopeNotFound, DataIdentifierNotFound, DataIdentifierAlreadyExists, \
+from rucio.common.exception import ScopeNotFound, DatabaseException, DataIdentifierNotFound, DataIdentifierAlreadyExists, \
     DuplicateContent, AccessDenied, KeyNotFound, Duplicate, InvalidValueForKey, UnsupportedStatus, \
     UnsupportedOperation, RSENotFound, RuleNotFound, InvalidMetadata, InvalidPath, FileAlreadyExists, InvalidObject, FileConsistencyMismatch
 from rucio.common.utils import render_json, APIEncoder
 from rucio.db.sqla.constants import DIDType
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     parse_scope_name, try_stream, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, json_list, param_get, json_parse
 
 
@@ -242,132 +242,6 @@ class Search(ErrorHandlingMethodView):
             return generate_http_error_flask(404, error)
 
 
-class SearchExtended(ErrorHandlingMethodView):
-
-    @check_accept_header_wrapper_flask(['application/x-json-stream'])
-    def get(self, scope):
-        """
-        ---
-        summary: List Data identifier with plugin metadata
-        description: List all data identifiers in a scope which match a given metadata. Extended Version to included meteadata from various plugins.
-        tags:
-          - Data Identifiers
-        parameters:
-        - name: scope
-          in: path
-          description: The scope of the data identifiers.
-          schema:
-            type: string
-          style: simple
-        - name: type
-          in: query
-          description: The did type to search for.
-          schema:
-            type: string
-            enum: ['all', 'collection', 'container', 'dataset', 'file']
-            default: 'collection'
-        - name: limit
-          in: query
-          description: The maximum number od dids returned.
-          schema:
-            type: integer
-        - name: long
-          in: query
-          description: Provides a longer output, otherwise just prints names.
-          schema:
-            type: boolean
-            default: false
-        - name: recursive
-          in: query
-          description: Recursively list chilred.
-          schema:
-            type: boolean
-        - name: created_before
-          in: query
-          description: Date string in RFC-1123 format where the creation date was earlier.
-          schema:
-            type: string
-        - name: created_after
-          in: query
-          description: Date string in RFC-1123 format where the creation date was later.
-          schema:
-            type: string
-        - name: length
-          in: query
-          description:  Exact number of attached DIDs.
-          schema:
-            type: integer
-        - name: length.gt
-          in: query
-          description: Number of attached DIDs greater than.
-          schema:
-            type: integer
-        - name: length.lt
-          in: query
-          description: Number of attached DIDs less than.
-          schema:
-            type: integer
-        - name: length.gte
-          in: query
-          description: Number of attached DIDs greater than or equal to
-          schema:
-            type: integer
-        - name: length.lte
-          in: query
-          description: Number of attached DIDs less than or equal to.
-          schema:
-            type: integer
-        - name: name
-          in: query
-          description: Name or pattern of a did.
-          schema:
-            type: string
-        responses:
-          200:
-            description: OK
-            content:
-              application/x-json-stream:
-                schema:
-                  description: Line separated name of DIDs or dictionaries of DIDs for long option.
-                  type: array
-                  items:
-                    type: object
-                    description: the name of a DID or a dictionarie of a DID for long option.
-          401:
-            description: Invalid Auth Token
-          404:
-            description: Invalid key in filter.
-          406:
-            description: Not acceptable
-          409:
-            description: Wrong did type
-        """
-        filters = request.args.get('filters', default=None)
-        if filters is not None:
-            filters = ast.literal_eval(filters)
-        else:
-            # backwards compatability for created*, length* and name filters passed through as request args
-            filters = {}
-            for arg, value in request.args.copy().items():
-                if arg not in ['type', 'limit', 'long', 'recursive']:
-                    filters[arg] = value
-            filters = [filters]
-
-        did_type = request.args.get('type', default=None)
-        limit = request.args.get('limit', default=None)
-        long = request.args.get('long', type=['True', '1'].__contains__, default=False)
-        recursive = request.args.get('recursive', type='True'.__eq__, default=False)
-        try:
-            def generate(vo):
-                for did in list_dids_extended(scope=scope, filters=filters, did_type=did_type, limit=limit, long=long, recursive=recursive, vo=vo):
-                    yield dumps(did) + '\n'
-            return try_stream(generate(vo=request.environ.get('vo')))
-        except UnsupportedOperation as error:
-            return generate_http_error_flask(409, error)
-        except KeyNotFound as error:
-            return generate_http_error_flask(404, error)
-
-
 class BulkDIDS(ErrorHandlingMethodView):
 
     @check_accept_header_wrapper_flask(['application/json'])
@@ -781,6 +655,15 @@ class DIDs(ErrorHandlingMethodView):
             return generate_http_error_flask(409, error)
         except AccessDenied as error:
             return generate_http_error_flask(401, error)
+        except DatabaseException as error:
+            if 'DELETED_DIDS_PK violated' in str(error):
+                return generate_http_error_flask(
+                    status_code=406,
+                    exc=error.__class__.__name__,
+                    exc_msg=str('A deleted DID {} with scope {} is reused'.format(name, scope))
+                )
+            else:
+                return generate_http_error_flask(406, error)
 
         return 'Created', 201
 
@@ -1309,7 +1192,7 @@ class Meta(ErrorHandlingMethodView):
           description: The plugin to use.
           schema:
             type: string
-          default: DID_COLUMN
+            default: DID_COLUMN
         responses:
           200:
             description: OK
@@ -1355,7 +1238,7 @@ class Meta(ErrorHandlingMethodView):
           content:
             'application/json':
               schema:
-                type:
+                type: object
                 required:
                 - meta
                 properties:
@@ -1485,13 +1368,13 @@ class SingleMeta(ErrorHandlingMethodView):
           content:
             'application/json':
               schema:
-                type:
+                type: object
                 required:
                 - value
                 properties:
                   value:
                     description: The value to set.
-                    type: AnyValue
+                    type: object
         responses:
           201:
             description: Created
@@ -1667,7 +1550,7 @@ class BulkMeta(ErrorHandlingMethodView):
           content:
             'application/x-json-stream':
               schema:
-                type:
+                type: object
                 required:
                 - dids
                 properties:
@@ -1847,7 +1730,7 @@ class GUIDLookup(ErrorHandlingMethodView):
             return generate_http_error_flask(404, error)
 
 
-class Sample(ErrorHandlingMethodView):
+class SampleLegacy(ErrorHandlingMethodView):
 
     def post(self, input_scope, input_name, output_scope, output_name, nbfiles):
         """
@@ -1924,6 +1807,81 @@ class Sample(ErrorHandlingMethodView):
         return 'Created', 201
 
 
+class Sample(ErrorHandlingMethodView):
+
+    def post(self):
+        """
+        ---
+        summary: Create sample
+        description: Creates a sample from an input collection.
+        tags:
+          - Data Identifiers
+        requestBody:
+          description: Parameters (source and destination) for the files in the sample to be created
+          content:
+            'application/json':
+              schema:
+                type: object
+                required:
+                - input_scope
+                - input_name
+                - output_scope
+                - output_name
+                - nbfiles
+                properties:
+                  input_scope:
+                    description: The input scope.
+                    type: string
+                  input_name:
+                    description: The input name.
+                    type: string
+                  output_scope:
+                    description: The output scope.
+                    type: string
+                  output_name:
+                    description: The output name.
+                    type: string
+                  nbfiles:
+                    description: The number of files to register in the output dataset.
+                    type: string
+        responses:
+          201:
+            description: OK
+            content:
+              application/json:
+                schema:
+                  type: string
+                  enum: ["Created"]
+          401:
+            description: Invalid Auth Token
+          404:
+            description: Not found
+          406:
+            description: Not acceptable
+          409:
+            description: Duplication
+        """
+        parameters = json_parameters()
+        try:
+            create_did_sample(
+                input_scope=parameters['input_scope'],
+                input_name=parameters['input_name'],
+                output_scope=parameters['output_scope'],
+                output_name=parameters['output_name'],
+                issuer=request.environ.get('issuer'),
+                nbfiles=parameters['nbfiles'],
+                vo=request.environ.get('vo'),
+            )
+        except DataIdentifierNotFound as error:
+            return generate_http_error_flask(404, error)
+        except (DuplicateContent, DataIdentifierAlreadyExists, UnsupportedOperation) as error:
+            return generate_http_error_flask(409, error)
+        except AccessDenied as error:
+            return generate_http_error_flask(401, error)
+
+        return 'Created', 201
+
+
 class NewDIDs(ErrorHandlingMethodView):
 
     @check_accept_header_wrapper_flask(['application/x-json-stream'])
@@ -2155,7 +2113,7 @@ class Follow(ErrorHandlingMethodView):
           content:
             'application/json':
               schema:
-                type:
+                type: object
                 required:
                 - account
                 properties:
@@ -2189,7 +2147,7 @@ class Follow(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('dids', __name__, url_prefix='/dids')
+    bp = AuthenticatedBlueprint('dids', __name__, url_prefix='/dids')
 
     scope_view = Scope.as_view('scope')
     bp.add_url_rule('/<scope>/', view_func=scope_view, methods=['get', ])
@@ -2197,10 +2155,8 @@ def blueprint():
     bp.add_url_rule('/<guid>/guid', view_func=guid_lookup_view, methods=['get', ])
     search_view = Search.as_view('search')
     bp.add_url_rule('/<scope>/dids/search', view_func=search_view, methods=['get', ])
-    search_extended_view = SearchExtended.as_view('search_extended')
-    bp.add_url_rule('/<scope>/dids/search_extended', view_func=search_extended_view, methods=['get', ])
     dids_view = DIDs.as_view('dids')
-    bp.add_url_rule('/<path:scope_name>/status', view_func=dids_view, methods=['put', ])
+    bp.add_url_rule('/<path:scope_name>/status', view_func=dids_view, methods=['put', 'get'])
     files_view = Files.as_view('files')
     bp.add_url_rule('/<path:scope_name>/files', view_func=files_view, methods=['get', ])
     attachment_history_view = AttachmentHistory.as_view('attachment_history')
@@ -2224,8 +2180,10 @@ def blueprint():
     bp.add_url_rule('/<path:scope_name>', view_func=dids_view, methods=['get', 'post'])
     bulkdids_view = BulkDIDS.as_view('bulkdids')
     bp.add_url_rule('', view_func=bulkdids_view, methods=['post', ])
-    sample_view = Sample.as_view('sample')
-    bp.add_url_rule('/<input_scope>/<input_name>/<output_scope>/<output_name>/<nbfiles>/sample', view_func=sample_view, methods=['post', ])
+    sample_view_legacy = SampleLegacy.as_view('sample')
+    bp.add_url_rule('/<input_scope>/<input_name>/<output_scope>/<output_name>/<nbfiles>/sample', view_func=sample_view_legacy, methods=['post', ])
+    sample_view = Sample.as_view('sample_new')
+    bp.add_url_rule('/sample', view_func=sample_view, methods=['post', ])
     attachements_view = Attachments.as_view('attachments')
     bp.add_url_rule('/attachments', view_func=attachements_view, methods=['post', ])
     new_dids_view = NewDIDs.as_view('new_dids')
@@ -2235,7 +2193,6 @@ def blueprint():
     bulkmeta_view = BulkMeta.as_view('bulkmeta')
     bp.add_url_rule('/bulkmeta', view_func=bulkmeta_view, methods=['post', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
