@@ -13,10 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from flask import Flask, Blueprint, request, jsonify
+from flask import Flask, request, jsonify
 
 from rucio.api.identity import add_identity, add_account_identity, list_accounts_for_identity
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     ErrorHandlingMethodView
 
 
@@ -199,7 +200,7 @@ class Accounts(ErrorHandlingMethodView):
     """ Retrieve list of accounts mapped to an identity. """
 
     @check_accept_header_wrapper_flask(['application/json'])
-    def get(self, identity_key, type):
+    def get(self, identity_key, type_):
         """
         ---
         summary: List
@@ -235,12 +236,12 @@ class Accounts(ErrorHandlingMethodView):
           401:
             description: Not acceptable
         """
-        accounts = list_accounts_for_identity(identity_key, type)
+        accounts = list_accounts_for_identity(identity_key, type_)
         return jsonify(accounts)
 
 
 def blueprint():
-    bp = Blueprint('identities', __name__, url_prefix='/identities')
+    bp = AuthenticatedBlueprint('identities', __name__, url_prefix='/identities')
 
     userpass_view = UserPass.as_view('userpass')
     bp.add_url_rule('/<account>/userpass', view_func=userpass_view, methods=['put', ])
@@ -251,7 +252,6 @@ def blueprint():
     accounts_view = Accounts.as_view('accounts')
     bp.add_url_rule('/<identity_key>/<type>/accounts', view_func=accounts_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
