@@ -13,11 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from flask import Flask, Blueprint, request
+from flask import Flask, request
 
 from rucio.api.importer import import_data
 from rucio.common.utils import parse_response
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, ErrorHandlingMethodView, json_parameters
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, ErrorHandlingMethodView, json_parameters
 
 
 class Import(ErrorHandlingMethodView):
@@ -59,29 +60,12 @@ class Import(ErrorHandlingMethodView):
                         description: Distance for two rses.
                         type: object
                         properties:
-                          ranking:
+                          distance:
                             description: The distance between the rses.
                             type: integer
-                          agis_distance:
-                            description: The agis distance between the rses.
-                            type: integer
-                          geoip_distance:
-                            description: The geoip distance between the rses.
-                            type: integer
-                          active:
-                            description: Active FTS transfer.
-                            type: integer
-                          submitted:
-                            description: Submitted FTS transfer.
-                            type: integer
-                          transfer_speed:
-                            description: FTS transfer speed.
-                            type: integer
-                          finished:
-                            description: Finished FTS transfer.
-                            type: integer
-                          failed:
-                            description: Failed fts transfer.
+                          ranking:
+                            deprecated: true
+                            description: Same as distance
                             type: integer
                   accounts:
                     description: Account data.
@@ -130,7 +114,7 @@ class Import(ErrorHandlingMethodView):
 
 
 def blueprint(with_doc=False):
-    bp = Blueprint('import', __name__, url_prefix='/import')
+    bp = AuthenticatedBlueprint('import', __name__, url_prefix='/import')
 
     import_view = Import.as_view('scope')
     if not with_doc:
@@ -138,7 +122,6 @@ def blueprint(with_doc=False):
         bp.add_url_rule('', view_func=import_view, methods=['post', ])
     bp.add_url_rule('/', view_func=import_view, methods=['post', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
