@@ -15,13 +15,14 @@
 
 from json import dumps
 
-from flask import Flask, Blueprint, Response, request
+from flask import Flask, Response, request
 
 from rucio.api.lifetime_exception import list_exceptions, add_exception, update_exception
 from rucio.common.exception import LifetimeExceptionNotFound, UnsupportedOperation, InvalidObject, AccessDenied, \
     LifetimeExceptionDuplicate
 from rucio.common.utils import APIEncoder
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     try_stream, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
@@ -296,14 +297,13 @@ class LifetimeExceptionId(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('lifetime_exceptions', __name__, url_prefix='/lifetime_exceptions')
+    bp = AuthenticatedBlueprint('lifetime_exceptions', __name__, url_prefix='/lifetime_exceptions')
 
     lifetime_exception_view = LifetimeException.as_view('lifetime_exception')
     bp.add_url_rule('/', view_func=lifetime_exception_view, methods=['get', 'post'])
     lifetime_exception_id_view = LifetimeExceptionId.as_view('lifetime_exception_id')
     bp.add_url_rule('/<exception_id>', view_func=lifetime_exception_id_view, methods=['get', 'put'])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
