@@ -13,11 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from flask import Flask, Blueprint, request, jsonify
+from flask import Flask, request, jsonify
 
 from rucio.api.meta import add_key, add_value, list_keys, list_values
 from rucio.common.exception import Duplicate, InvalidValueForKey, KeyNotFound, UnsupportedValueType, UnsupportedKeyType
-from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, request_auth_env, response_headers, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, response_headers, \
     generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
@@ -166,7 +167,7 @@ class Values(ErrorHandlingMethodView):
           content:
             application/json:
               schema:
-                type:
+                type: object
                 required:
                 - value
                 properties:
@@ -206,7 +207,7 @@ class Values(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('meta', __name__, url_prefix='/meta')
+    bp = AuthenticatedBlueprint('meta', __name__, url_prefix='/meta')
 
     meta_view = Meta.as_view('meta')
     bp.add_url_rule('/', view_func=meta_view, methods=['get', ])
@@ -214,7 +215,6 @@ def blueprint():
     values_view = Values.as_view('values')
     bp.add_url_rule('/<key>/', view_func=values_view, methods=['get', 'post'])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
