@@ -19,22 +19,23 @@ from json import dumps, loads
 from urllib.parse import parse_qs, unquote
 from xml.sax.saxutils import escape
 
-from flask import Flask, Blueprint, Response, request
+from flask import Flask, Response, request
 
+from rucio.api.quarantined_replica import quarantine_file_replicas
 from rucio.api.replica import add_replicas, list_replicas, list_dataset_replicas, list_dataset_replicas_bulk, \
     delete_replicas, get_did_from_pfns, update_replicas_states, declare_bad_file_replicas, add_bad_dids, add_bad_pfns, \
     get_suspicious_files, declare_suspicious_file_replicas, list_bad_replicas_status, get_bad_replicas_summary, \
     list_datasets_per_rse, set_tombstone, list_dataset_replicas_vp
-from rucio.api.quarantined_replica import quarantine_file_replicas
-from rucio.common.config import config_get
+from rucio.common.config import config_get, config_get_int
 from rucio.common.constants import SUPPORTED_PROTOCOLS
 from rucio.common.exception import AccessDenied, DataIdentifierAlreadyExists, InvalidType, DataIdentifierNotFound, \
     Duplicate, InvalidPath, ResourceTemporaryUnavailable, RSENotFound, ReplicaNotFound, InvalidObject, ScopeNotFound, ReplicaIsLocked
 from rucio.common.utils import parse_response, APIEncoder, render_json_list
 from rucio.core.replica_sorter import sort_replicas
 from rucio.db.sqla.constants import BadFilesStatus
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
 from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, try_stream, parse_scope_name, \
-    request_auth_env, response_headers, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
+    response_headers, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
 def _sorted_with_priorities(replicas, sorted_pfns, limit=None):
@@ -652,7 +653,7 @@ class ListReplicas(ErrorHandlingMethodView):
             signature_lifetime = param_get(parameters, 'signature_lifetime')
         else:
             # hardcoded default of 10 minutes if config is not parseable
-            signature_lifetime = config_get('credentials', 'signature_lifetime', raise_exception=False, default=600)
+            signature_lifetime = config_get_int('credentials', 'signature_lifetime', raise_exception=False, default=600)
         resolve_archives = param_get(parameters, 'resolve_archives', default=True)
         resolve_parents = param_get(parameters, 'resolve_parents', default=False)
         updated_after = param_get(parameters, 'updated_after', default=None)
@@ -688,7 +689,8 @@ class ListReplicas(ErrorHandlingMethodView):
                                            all_states=all_states,
                                            rse_expression=rse_expression,
                                            client_location=client_location,
-                                           domain=domain, signature_lifetime=signature_lifetime,
+                                           domain=domain,
+                                           signature_lifetime=signature_lifetime,
                                            resolve_archives=resolve_archives,
                                            resolve_parents=resolve_parents,
                                            nrandom=nrandom,
@@ -770,10 +772,10 @@ class ReplicasDIDs(ErrorHandlingMethodView):
                       properties:
                         scope:
                           description: The scope of the DID.
-                          type: str
+                          type: string
                         name:
                           description: The name of the DID.
-                          type: str
+                          type: string
           400:
             description: Cannot decode json parameter list.
           401:
@@ -1108,7 +1110,7 @@ class BadReplicasStates(ErrorHandlingMethodView):
                   description: A list of all result replicas.
                   type: array
                   items:
-                    oneof:
+                    oneOf:
                       - type: object
                         properties:
                           scope:
@@ -1798,7 +1800,7 @@ class Tombstone(ErrorHandlingMethodView):
 
 
 def blueprint(with_doc=False):
-    bp = Blueprint('replicas', __name__, url_prefix='/replicas')
+    bp = AuthenticatedBlueprint('replicas', __name__, url_prefix='/replicas')
 
     list_replicas_view = ListReplicas.as_view('list_replicas')
     bp.add_url_rule('/list', view_func=list_replicas_view, methods=['post', ])
@@ -1853,7 +1855,6 @@ def blueprint(with_doc=False):
         bp.add_url_rule('/tombstone/', view_func=set_tombstone_view, methods=['post', ])
         bp.add_url_rule('/quarantine/', view_func=quarantine_replicas_view, methods=['post', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
