@@ -16,15 +16,16 @@
 import json
 
 import flask
-from flask import Flask, Blueprint, Response
+from flask import Flask, Response
 
 from rucio.api import request
 from rucio.common.exception import RequestNotFound
 from rucio.common.utils import APIEncoder, render_json
-from rucio.core.rse import get_rses_with_attribute_value, get_rse_name
+from rucio.core.rse import get_rses_with_attribute_value
 from rucio.db.sqla.constants import RequestState
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
 from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, parse_scope_name, try_stream, \
-    request_auth_env, response_headers, generate_http_error_flask, ErrorHandlingMethodView
+    response_headers, generate_http_error_flask, ErrorHandlingMethodView
 
 
 class RequestGet(ErrorHandlingMethodView):
@@ -62,7 +63,7 @@ class RequestGet(ErrorHandlingMethodView):
                   properties:
                     id:
                       description: The id of the request.
-                      type: strig
+                      type: string
                     request_type:
                       description: The request type.
                       type: string
@@ -227,7 +228,7 @@ class RequestHistoryGet(ErrorHandlingMethodView):
                   properties:
                     id:
                       description: The id of the request.
-                      type: strig
+                      type: string
                     request_type:
                       description: The request type.
                       type: string
@@ -424,7 +425,7 @@ class RequestList(ErrorHandlingMethodView):
                     properties:
                       id:
                         description: The id of the request.
-                        type: strig
+                        type: string
                       request_type:
                         description: The request type.
                         type: string
@@ -561,21 +562,20 @@ class RequestList(ErrorHandlingMethodView):
         src_rses = []
         dst_rses = []
         if src_site:
-            src_rses = get_rses_with_attribute_value(key='site', value=src_site, lookup_key='site', vo=flask.request.environ.get('vo'))
+            src_rses = get_rses_with_attribute_value(key='site', value=src_site, vo=flask.request.environ.get('vo'))
             if not src_rses:
                 return generate_http_error_flask(404, 'NotFound', f'Could not resolve site name {src_site} to RSE')
-            src_rses = [get_rse_name(rse['rse_id']) for rse in src_rses]
-            dst_rses = get_rses_with_attribute_value(key='site', value=dst_site, lookup_key='site', vo=flask.request.environ.get('vo'))
+            src_rses = [rse['rse_name'] for rse in src_rses]
+            dst_rses = get_rses_with_attribute_value(key='site', value=dst_site, vo=flask.request.environ.get('vo'))
             if not dst_rses:
                 return generate_http_error_flask(404, 'NotFound', f'Could not resolve site name {dst_site} to RSE')
-            dst_rses = [get_rse_name(rse['rse_id']) for rse in dst_rses]
+            dst_rses = [rse['rse_name'] for rse in dst_rses]
         else:
             dst_rses = [dst_rse]
             src_rses = [src_rse]
 
         def generate(issuer, vo):
             for result in request.list_requests(src_rses, dst_rses, states, issuer=issuer, vo=vo):
-                del result['_sa_instance_state']
                 yield render_json(**result) + '\n'
 
         return try_stream(generate(issuer=flask.request.environ.get('issuer'), vo=flask.request.environ.get('vo')))
@@ -639,13 +639,13 @@ class RequestHistoryList(ErrorHandlingMethodView):
           description: The offset of the list.
           schema:
             type: integer
-          default: 0
+            default: 0
         - name: limit
           in: query
           description: The maximum number of items to return.
           schema:
             type: integer
-          default: 100
+            default: 100
         responses:
           200:
             description: OK
@@ -660,7 +660,7 @@ class RequestHistoryList(ErrorHandlingMethodView):
                     properties:
                       id:
                         description: The id of the request.
-                        type: strig
+                        type: string
                       request_type:
                         description: The request type.
                         type: string
@@ -799,28 +799,27 @@ class RequestHistoryList(ErrorHandlingMethodView):
         src_rses = []
         dst_rses = []
         if src_site:
-            src_rses = get_rses_with_attribute_value(key='site', value=src_site, lookup_key='site', vo=flask.request.environ.get('vo'))
+            src_rses = get_rses_with_attribute_value(key='site', value=src_site, vo=flask.request.environ.get('vo'))
             if not src_rses:
                 return generate_http_error_flask(404, 'NotFound', f'Could not resolve site name {src_site} to RSE')
-            src_rses = [get_rse_name(rse['rse_id']) for rse in src_rses]
-            dst_rses = get_rses_with_attribute_value(key='site', value=dst_site, lookup_key='site', vo=flask.request.environ.get('vo'))
+            src_rses = [rse['rse_name'] for rse in src_rses]
+            dst_rses = get_rses_with_attribute_value(key='site', value=dst_site, vo=flask.request.environ.get('vo'))
             if not dst_rses:
                 return generate_http_error_flask(404, 'NotFound', f'Could not resolve site name {dst_site} to RSE')
-            dst_rses = [get_rse_name(rse['rse_id']) for rse in dst_rses]
+            dst_rses = [rse['rse_name'] for rse in dst_rses]
         else:
             dst_rses = [dst_rse]
             src_rses = [src_rse]
 
         def generate(issuer, vo):
             for result in request.list_requests_history(src_rses, dst_rses, states, issuer=issuer, vo=vo, offset=offset, limit=limit):
-                del result['_sa_instance_state']
                 yield render_json(**result) + '\n'
 
         return try_stream(generate(issuer=flask.request.environ.get('issuer'), vo=flask.request.environ.get('vo')))
 
 
 def blueprint():
-    bp = Blueprint('requests', __name__, url_prefix='/requests')
+    bp = AuthenticatedBlueprint('requests', __name__, url_prefix='/requests')
 
     request_get_view = RequestGet.as_view('request_get')
     bp.add_url_rule('/<path:scope_name>/<rse>', view_func=request_get_view, methods=['get', ])
@@ -831,7 +830,6 @@ def blueprint():
     request_history_list_view = RequestHistoryList.as_view('request_history_list')
     bp.add_url_rule('/history/list', view_func=request_history_list_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
