@@ -15,7 +15,7 @@
 
 from json import dumps
 
-from flask import Flask, Blueprint, Response, request, jsonify
+from flask import Flask, Response, request, jsonify
 
 from rucio.api.account_limit import get_rse_account_usage
 from rucio.api.rse import add_rse, update_rse, list_rses, del_rse, add_rse_attribute, list_rse_attributes, \
@@ -25,9 +25,10 @@ from rucio.api.rse import add_rse, update_rse, list_rses, del_rse, add_rse_attri
 from rucio.common.exception import Duplicate, AccessDenied, RSENotFound, RSEOperationNotSupported, \
     RSEProtocolNotSupported, InvalidObject, RSEProtocolDomainNotSupported, RSEProtocolPriorityError, \
     InvalidRSEExpression, RSEAttributeNotFound, CounterNotFound, InvalidPath, ReplicaNotFound, InputValidationError
-from rucio.common.utils import render_json, APIEncoder
+from rucio.common.utils import Availability, render_json, APIEncoder
 from rucio.rse import rsemanager
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     try_stream, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
@@ -42,15 +43,12 @@ class RSEs(ErrorHandlingMethodView):
         description: Lists all RSEs.
         tags:
           - Rucio Storage Elements
-        requestBody:
-          content:
-            application/json:
-              schema:
-                type: object
-                properties:
-                  expression:
-                    description: An RSE expression.
-                    type: string
+        parameters:
+        - name: expression
+          in: query
+          description: RSE expression to select RSEs.
+          schema:
+            type: string
         responses:
           200:
             description: OK
@@ -110,6 +108,16 @@ class RSEs(ErrorHandlingMethodView):
                       availability:
                         description: The availability of the rse.
                         type: integer
+                        deprecated: true
+                      availability_read:
+                        description: If the RSE is readable.
+                        type: integer
+                      availability_write:
+                        description: If the RSE is writable.
+                        type: integer
+                      availability_delete:
+                        description: If the RSE is deletable.
+                        type: integer
                       usage:
                         description: The usage of the rse.
                         type: integer
@@ -137,6 +145,7 @@ class RSEs(ErrorHandlingMethodView):
         else:
             def generate(vo):
                 for rse in list_rses(vo=vo):
+                    rse['availability'] = Availability(rse['availability_read'], rse['availability_write'], rse['availability_delete']).integer
                     yield render_json(**rse) + '\n'
 
             return try_stream(generate(vo=request.environ.get('vo')))
@@ -198,16 +207,26 @@ class RSE(ErrorHandlingMethodView):
                     enum: ["DISK", "TAPE"]
                   latitute:
                     description: The latitute of the RSE.
-                    type: float
+                    type: number
                   longitude:
                     description: The longitude of the RSE.
-                    type: float
+                    type: number
                   ASN:
                     description: The access service network of the RSE.
                     type: string
                   availability:
                     description: The availability of the RSE.
                     type: integer
+                    deprecated: true
+                  availability_read:
+                    description: If the RSE is readable.
+                    type: boolean
+                  availability_write:
+                    description: If the RSE is writable.
+                    type: boolean
+                  availability_delete:
+                    description: If the RSE is deletable.
+                    type: boolean
         responses:
           201:
             description: OK
@@ -239,12 +258,25 @@ class RSE(ErrorHandlingMethodView):
             'latitude': None,
             'longitude': None,
             'ASN': None,
-            'availability': None,
+            'availability_read': None,
+            'availability_write': None,
+            'availability_delete': None,
         }
         if request.get_data(as_text=True):
             parameters = json_parameters()
+
+            if "availability" in parameters and ("availability_read" in parameters or "availability_write" in parameters or "availability_delete" in parameters):
+                return generate_http_error_flask(422, InputValidationError("'availability' can not be specified with 'availability_read', 'availability_write' or 'availability_delete'"))  # noqa: E501
+
+            if "availability" in parameters:
+                availability = Availability.from_integer(parameters["availability"])
+                kwargs["availability_read"] = availability.read
+                kwargs["availability_write"] = availability.write
+                kwargs["availability_delete"] = availability.delete
+
             for keyword in kwargs.keys():
                 kwargs[keyword] = param_get(parameters, keyword, default=kwargs[keyword])
+
         kwargs['issuer'] = request.environ.get('issuer')
         kwargs['vo'] = request.environ.get('vo')
         try:
@@ -280,7 +312,7 @@ class RSE(ErrorHandlingMethodView):
               schema:
                 type: object
                 properties:
-                  availability_raed:
+                  availability_read:
                     description: The vailability of the RSE.
                     type: boolean
                   availability_write:
@@ -316,10 +348,10 @@ class RSE(ErrorHandlingMethodView):
                     enum: ["DISK", "TAPE"]
                   latitute:
                     description: The latitute of the RSE.
-                    type: float
+                    type: number
                   longitude:
                     description: The longitude of the RSE.
-                    type: float
+                    type: number
         responses:
           201:
             description: OK
@@ -410,16 +442,25 @@ class RSE(ErrorHandlingMethodView):
                       enum: ["DISK", "TAPE"]
                     latitute:
                       description: The latitute of the RSE.
-                      type: float
+                      type: number
                     longitude:
                       description: The longitude of the RSE.
-                      type: float
+                      type: number
                     ASN:
                       description: The access service network of the RSE.
                       type: string
                     availability:
                       description: The availability of the RSE.
                       type: integer
+                      deprecated: true
+                    availability_read:
+                      description: If the RSE is readable.
+                      type: integer
+                    availability_write:
+                      description: If the RSE is writable.
+                      type: integer
+                    availability_delete:
+                      description: If the RSE is deletable.
           401:
             description: Invalid Auth Token
           404:
@@ -428,8 +469,9 @@ class RSE(ErrorHandlingMethodView):
             description: Not acceptable
         """
         try:
-            rse_prop = get_rse(rse=rse, vo=request.environ.get('vo'))
-            return Response(render_json(**rse_prop), content_type="application/json")
+            rse = get_rse(rse=rse, vo=request.environ.get('vo'))
+            rse['availability'] = Availability(rse['availability_read'], rse['availability_write'], rse['availability_delete']).integer
+            return Response(render_json(**rse), content_type="application/json")
         except RSENotFound as error:
             return generate_http_error_flask(404, error)
 
@@ -721,6 +763,9 @@ class ProtocolList(ErrorHandlingMethodView):
                                     description: The delete value of the lan protocol.
                                     type: integer
                               wan:
+                                description: The wan domain
+                                type: object
+                                properties:
                                   read:
                                     description: The read value of the wan protocol.
                                     type: integer
@@ -761,7 +806,7 @@ class LFNS2PFNS(ErrorHandlingMethodView):
     """ Translate one-or-more LFNs to corresponding PFNs. """
 
     @check_accept_header_wrapper_flask(['application/json'])
-    def get(self, rse, scheme=None):
+    def get(self, rse):
         """
         ---
         summary: Translate LFNs to PFNs
@@ -775,13 +820,6 @@ class LFNS2PFNS(ErrorHandlingMethodView):
           schema:
             type: string
           style: simple
-        - name: schema
-          in: path
-          description: The protocol identifier.
-          schema:
-            type: string
-          style: simple
-          required: False
         - name: lfn
           in: query
           description: The lfns of the request.
@@ -1061,6 +1099,9 @@ class Protocol(ErrorHandlingMethodView):
                                     description: The delete value of the lan protocol.
                                     type: integer
                               wan:
+                                description: The wan domain
+                                type: object
+                                properties:
                                   read:
                                     description: The read value of the wan protocol.
                                     type: integer
@@ -1222,6 +1263,9 @@ class Protocol(ErrorHandlingMethodView):
                                     description: The delete value of the lan protocol.
                                     type: integer
                               wan:
+                                description: The wan domain
+                                type: object
+                                properties:
                                   read:
                                     description: The read value of the wan protocol.
                                     type: integer
@@ -1777,14 +1821,12 @@ class Distance(ErrorHandlingMethodView):
                       dest_rse_id:
                         description: The destination rse id.
                         type: string
+                      distance:
+                        description: The distance between RSEs.
+                        type: integer
                       ranking:
-                        description: The ranking.
-                        type: integer
-                      agis_distance:
-                        description: The agis distance.
-                        type: integer
-                      geoip_distance:
-                        description: The geo ip distance.
+                        deprecated: true
+                        description: Same as distance.
                         type: integer
           401:
             description: Invalid Auth Token
@@ -1825,29 +1867,12 @@ class Distance(ErrorHandlingMethodView):
               schema:
                 type: object
                 properties:
-                  ranking:
-                    description: The ranking of the distance.
-                    type: integer
                   distance:
-                    description: The distance between the Rses.
+                    description: The distance between RSEs.
                     type: integer
-                  geoip_distance:
-                    description: The geoip distance between the Rses.
-                    type: integer
-                  active:
-                    description: If the distance is active.
-                    type: boolean
-                  submitted:
-                    description: If the distance is submitted.
-                    type: boolean
-                  finished:
-                    description: If the distance is finished.
-                    type: boolean
-                  failed:
-                    description: If the distance failed.
-                    type: boolean
-                  transfer_speed:
-                    description: The transferspeed between the Rses.
+                  ranking:
+                    deprecated: true
+                    description: Same as distance.
                     type: integer
         responses:
           201:
@@ -1865,26 +1890,18 @@ class Distance(ErrorHandlingMethodView):
             description: Not acceptable
         """
         parameters = json_parameters()
-        kwargs = {
-            'ranking': None,
-            'distance': None,
-            'geoip_distance': None,
-            'active': None,
-            'submitted': None,
-            'finished': None,
-            'failed': None,
-            'transfer_speed': None,
-        }
-        for keyword in kwargs.keys():
-            kwargs[keyword] = param_get(parameters, keyword, default=kwargs[keyword])
+
+        distance = param_get(parameters, 'distance', default=None)
+        if distance is None:
+            distance = param_get(parameters, 'ranking', default=None)
 
         try:
             add_distance(
                 source=source,
                 destination=destination,
+                distance=distance,
                 issuer=request.environ.get('issuer'),
                 vo=request.environ.get('vo'),
-                **kwargs,
             )
         except AccessDenied as error:
             return generate_http_error_flask(401, error)
@@ -1921,29 +1938,12 @@ class Distance(ErrorHandlingMethodView):
               schema:
                 type: object
                 properties:
+                  distance:
+                    description: The distance between the RSEs.
+                    type: integer
                   ranking:
-                    description: The ranking of the distance.
-                    type: integer
-                  agis_distance:
-                    description: The distance between the Rses.
-                    type: integer
-                  geoip_distance:
-                    description: The geoip distance between the Rses.
-                    type: integer
-                  active:
-                    description: If the distance is active.
-                    type: boolean
-                  submitted:
-                    description: If the distance is submitted.
-                    type: boolean
-                  finished:
-                    description: If the distance is finished.
-                    type: boolean
-                  failed:
-                    description: If the distance failed.
-                    type: boolean
-                  transfer_speed:
-                    description: The transferspeed between the Rses.
+                    deprecated: true
+                    description: Same as distance.
                     type: integer
         responses:
           201:
@@ -1961,13 +1961,18 @@ class Distance(ErrorHandlingMethodView):
             description: Not acceptable
         """
         parameters = json_parameters()
+
+        distance = param_get(parameters, 'distance', default=None)
+        if distance is None:
+            distance = param_get(parameters, 'ranking', default=None)
+
         try:
             update_distance(
                 source=source,
                 destination=destination,
+                distance=distance,
                 issuer=request.environ.get('issuer'),
                 vo=request.environ.get('vo'),
-                parameters=parameters,
             )
         except AccessDenied as error:
             return generate_http_error_flask(401, error)
@@ -2157,7 +2162,7 @@ class QoSPolicy(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('rses', __name__, url_prefix='/rses')
+    bp = AuthenticatedBlueprint('rses', __name__, url_prefix='/rses')
 
     attributes_view = Attributes.as_view('attributes')
     bp.add_url_rule('/<rse>/attr/<key>', view_func=attributes_view, methods=['post', 'delete'])
@@ -2188,7 +2193,6 @@ def blueprint():
     rses_view = RSEs.as_view('rses')
     bp.add_url_rule('/', view_func=rses_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
