@@ -14,20 +14,22 @@
 # limitations under the License.
 
 from json import dumps
+from typing import Any
 
-from flask import Flask, Blueprint, request, Response
+from flask import Flask, request, Response
 
 from rucio.api.lock import get_replica_locks_for_rule_id
 from rucio.api.rule import add_replication_rule, delete_replication_rule, get_replication_rule, \
     update_replication_rule, reduce_replication_rule, list_replication_rule_history, \
     list_replication_rule_full_history, list_replication_rules, examine_replication_rule, move_replication_rule
-from rucio.common.exception import InsufficientAccountLimit, RuleNotFound, AccessDenied, InvalidRSEExpression, \
+from rucio.common.exception import InputValidationError, InsufficientAccountLimit, RuleNotFound, AccessDenied, InvalidRSEExpression, \
     InvalidReplicationRule, DataIdentifierNotFound, InsufficientTargetRSEs, ReplicationRuleCreationTemporaryFailed, \
     InvalidRuleWeight, StagingAreaRuleRequiresLifetime, DuplicateRule, InvalidObject, AccountNotFound, \
     RuleReplaceFailed, ScratchDiskLifetimeConflict, ManualRuleApprovalBlocked, UnsupportedOperation
 from rucio.common.utils import render_json, APIEncoder
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
 from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, parse_scope_name, try_stream, \
-    request_auth_env, response_headers, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
+    response_headers, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
 class Rule(ErrorHandlingMethodView):
@@ -150,14 +152,15 @@ class Rule(ErrorHandlingMethodView):
             description: No rule found for the given id
        """
         parameters = json_parameters()
-        options = param_get(parameters, 'options')
+        options: dict[str, Any] = param_get(parameters, 'options')
         try:
             update_replication_rule(rule_id=rule_id, options=options, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccessDenied as error:
             return generate_http_error_flask(401, error)
         except (RuleNotFound, AccountNotFound) as error:
             return generate_http_error_flask(404, error)
-        except (ScratchDiskLifetimeConflict, UnsupportedOperation) as error:
+        except (ScratchDiskLifetimeConflict,
+                UnsupportedOperation, InputValidationError) as error:
             return generate_http_error_flask(409, error)
 
         return '', 200
@@ -252,7 +255,7 @@ class AllRule(ErrorHandlingMethodView):
                     items:
                       type: string
                   account:
-                    descipriton: The account of the issuer.
+                    description: The account of the issuer.
                     type: string
                   copies:
                     description: The number of replicas.
@@ -622,7 +625,7 @@ class RuleHistory(ErrorHandlingMethodView):
                         description: The date of the update.
                       state:
                         type: string
-                        descipriton: The state of the update.
+                        description: The state of the update.
                       locks_ok_cnt:
                         type: integer
                         description: The number of locks which are ok.
@@ -688,10 +691,10 @@ class RuleHistoryFull(ErrorHandlingMethodView):
                         description: The rse expression.
                       state:
                         type: string
-                        descipriton: The state of the update.
+                        description: The state of the update.
                       account:
                         type: string
-                        descipriton: The account who initiated the change.
+                        description: The account who initiated the change.
                       locks_ok_cnt:
                         type: integer
                         description: The number of locks which are ok.
@@ -791,7 +794,7 @@ class RuleAnalysis(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('rules', __name__, url_prefix='/rules')
+    bp = AuthenticatedBlueprint('rules', __name__, url_prefix='/rules')
 
     rule_view = Rule.as_view('rule')
     bp.add_url_rule('/<rule_id>', view_func=rule_view, methods=['get', 'put', 'delete'])
@@ -810,7 +813,6 @@ def blueprint():
     rule_analysis_view = RuleAnalysis.as_view('rule_analysis')
     bp.add_url_rule('/<rule_id>/analysis', view_func=rule_analysis_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
