@@ -13,10 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from flask import Flask, Blueprint, request
+from flask import Flask, request
 
 from rucio.api.temporary_did import add_temporary_dids
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, ErrorHandlingMethodView, json_list
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, ErrorHandlingMethodView, json_list
 
 
 class BulkDIDS(ErrorHandlingMethodView):
@@ -98,12 +99,11 @@ class BulkDIDS(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('tmp_dids', __name__, url_prefix='/tmp_dids')
+    bp = AuthenticatedBlueprint('tmp_dids', __name__, url_prefix='/tmp_dids')
 
     bulk_dids_view = BulkDIDS.as_view('bulk_dids')
     bp.add_url_rule('', view_func=bulk_dids_view, methods=['post', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
