@@ -13,12 +13,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from flask import Flask, Blueprint, request
+from flask import Flask, request
 
 from rucio.api.vo import add_vo, list_vos, recover_vo_root_identity, update_vo
 from rucio.common.exception import AccessDenied, AccountNotFound, Duplicate, VONotFound, UnsupportedOperation
 from rucio.common.utils import render_json
-from rucio.web.rest.flaskapi.v1.common import request_auth_env, response_headers, check_accept_header_wrapper_flask, \
+from rucio.web.rest.flaskapi.authenticated_bp import AuthenticatedBlueprint
+from rucio.web.rest.flaskapi.v1.common import response_headers, check_accept_header_wrapper_flask, \
     try_stream, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
@@ -27,16 +28,45 @@ class VOs(ErrorHandlingMethodView):
 
     @check_accept_header_wrapper_flask(['application/x-json-stream'])
     def get(self):
-        """ List all VOs.
-
-        .. :quickref: VOs; List all VOs.
-
-        :resheader Content-Type: application/x-json-stream
-        :status 200: VOs found.
-        :status 401: Invalid Auth Token.
-        :status 409: Unsupported operation.
-        :returns: A list containing all VOs.
+        """
+        ---
+        summary: List VOs
+        tags:
+          - VO
+        responses:
+          200:
+            description: OK
+            content:
+              application/json:
+                schema:
+                  type: array
+                  items:
+                    type: object
+                    properties:
+                      vo:
+                        description: The vo.
+                        type: string
+                      description:
+                        description: The description of the vo.
+                        type: string
+                      email:
+                        description: The email for the vo.
+                        type: string
+                      created_at:
+                        description: The date the vo was created.
+                        type: string
+                        format: date-time
+                      updated_at:
+                        description: The date the vo was updated.
+                        type: string
+                        format: date-time
 
+          406:
+            description: Not Acceptable
+          401:
+            description: Invalid Auth Token
+          409:
+            description: Unsupported operation.
         """
         try:
             def generate(issuer, vo):
@@ -54,16 +84,37 @@ class VO(ErrorHandlingMethodView):
     """ Add and update a VO. """
 
     def post(self, vo):
-        """ Add a VO with a given name.
-
-        .. :quickref: VO; Add a VOs.
-
-        :param vo: VO to be added.
-        :<json string description: Desciption of VO.
-        :<json string email: Admin email for VO.
-        :status 201: VO created successfully.
-        :status 401: Invalid Auth Token.
-        :status 409: Unsupported operation.
+        """
+        ---
+        summary: Add VO
+        tags:
+          - VO
+        parameters:
+        - name: vo
+          in: path
+          description: The vo to add.
+          schema:
+            type: string
+          style: simple
+        requestBody:
+          content:
+            application/json:
+              schema:
+                type: object
+                properties:
+                  description:
+                    description: The description of the VO.
+                    type: string
+                  email:
+                    description: The admin email associated with the VO.
+                    type: string
+        responses:
+          201:
+            description: OK
+          401:
+            description: Invalid Auth Token
+          409:
+            description: Unsupported operation.
         """
         parameters = json_parameters(optional=True)
         kwargs = {'description': None, 'email': None}
@@ -82,17 +133,39 @@ class VO(ErrorHandlingMethodView):
         return 'Created', 201
 
     def put(self, vo):
-        """ Update the details for a given VO
-
-        .. :quickref: VO; Update a VOs.
-
-        :param vo: VO to be updated.
-        :<json string description: Desciption of VO.
-        :<json string email: Admin email for VO.
-        :status 200: VO updated successfully.
-        :status 401: Invalid Auth Token.
-        :status 404: VO not found.
-        :status 409: Unsupported operation.
+        """
+        ---
+        summary: Update VO
+        tags:
+          - VO
+        parameters:
+        - name: vo
+          in: path
+          description: The vo to add.
+          schema:
+            type: string
+          style: simple
+        requestBody:
+          content:
+            application/json:
+              schema:
+                type: object
+                properties:
+                  description:
+                    description: The description of the VO.
+                    type: string
+                  email:
+                    description: The admin email associated with the VO.
+                    type: string
+        responses:
+          200:
+            description: OK
+          401:
+            description: Invalid Auth Token
+          404:
+            description: VO not found.
+          409:
+            description: Unsupported operation.
         """
         parameters = json_parameters()
         try:
@@ -111,20 +184,52 @@ class RecoverVO(ErrorHandlingMethodView):
     """ Recover root identity for a VO. """
 
     def post(self, vo):
-        """ Recover root identity for a given VO
-
-        .. :quickref: RecoverVO; Recover VO root identity.
-
-        :param vo: VO to be recovered.
-        :<json string identity: Identity key to use.
-        :<json string authtype: Type of identity.
-        :<json string email: Admin email for VO.
-        :<json string email: Password for identity.
-        :<json bool default: Whether to use identity as account default.
-        :status 201: VO recovered successfully.
-        :status 401: Invalid Auth Token.
-        :status 404: Account not found.
-        :status 409: Unsupported operation.
+        """
+        ---
+        summary: Recover Root Identity
+        tags:
+          - VO
+        parameters:
+        - name: vo
+          in: path
+          description: The vo to add.
+          schema:
+            type: string
+          style: simple
+        requestBody:
+          content:
+            application/json:
+              schema:
+                type: object
+                required:
+                - identity
+                - authtype
+                - email
+                properties:
+                  identity:
+                    description: Identity key to use.
+                    type: string
+                  authtype:
+                    description: The authtype of the account.
+                    type: string
+                  email:
+                    description: The admin email for the vo.
+                    type: string
+                  password:
+                    description: Password for identity.
+                    type: string
+                  default:
+                    description: Whether to use identity as account default.
+                    type: boolean
+        responses:
+          201:
+            description: OK
+          401:
+            description: Invalid Auth Token
+          404:
+            description: Account not found.
+          409:
+            description: Unsupported operation.
         """
         parameters = json_parameters()
         identity = param_get(parameters, 'identity')
@@ -155,7 +260,7 @@ class RecoverVO(ErrorHandlingMethodView):
 
 
 def blueprint():
-    bp = Blueprint('vos', __name__, url_prefix='/vos')
+    bp = AuthenticatedBlueprint('vos', __name__, url_prefix='/vos')
 
     recover_view = RecoverVO.as_view('recover')
     bp.add_url_rule('/<vo>/recover', view_func=recover_view, methods=['post', ])
@@ -164,7 +269,6 @@ def blueprint():
     vos_view = VOs.as_view('vos')
     bp.add_url_rule('/', view_func=vos_view, methods=['get', ])
 
-    bp.before_request(request_auth_env)
     bp.after_request(response_headers)
     return bp
 
